/*---------------------------------------------------------------------------
 
  FILENAME:
        alarms.c
 
  PURPOSE:
        Provide the wview alarms generator entry point.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        01/10/04        M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/*  ... Library include files
*/
#include <radsystem.h>

/*  ... Local include files
*/
#include <dbfiles.h>
#include <alarms.h>

static USHORT DF_START_FRAME[4] = 
{
    0xF388, 
    0xC6A2, 
    0xDADA, 
    0xE7CF
};
#define DF_START_FRAME_LEN          8


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
static WVIEW_ALARM_WORK alarmsWork;


/* ... methods
*/

static int executeScript (WVIEW_ALARM *alarm)
{
    int             retVal;
    char            *args[5];
    char            path[512];
    char            type[16];
    char            thresh[64];
    char            trigVal[64];

    strcpy (path, alarm->scriptToRun);
    args[0] = path;

    sprintf (type, "%d", alarm->type);
    args[1] = type;

    sprintf (thresh, "%.3f", alarm->bound);
    args[2] = thresh;

    sprintf (trigVal, "%.3f", alarm->triggerValue);
    args[3] = trigVal;

    args[4] = 0;

    retVal = fork ();
    if (retVal == -1)
    {
        return -1;
    }
    else if (retVal == 0)
    {
        // we are the child!
        execv (path, args);
    }
    else
    {
        // we are the parent, pause a bit to let the script run 
        // and avoid a process "storm"
        radUtilsSleep (50);                 // 50 ms
    }

    return 0;
}

static int readAlarmsConfFile (char *filename)
{
    WVIEW_ALARM     *alarm;
    FILE            *file;
    int             i, done, numAlarms = 0;
    char            *token;
    char            temp[1536];

    file = fopen (filename, "r");
    if (file == NULL)
    {
        radMsgLog (PRI_HIGH, "readAlarmsConfFile: %s does not exist!",
                   filename);
        return ERROR_ABORT;
    }

    while (fgets (temp, 1536, file) != NULL)
    {
        if (temp[0] == ' ' || temp[0] == '\n' || temp[0] == '#')
        {
            // comment or whitespace
            continue;
        }

        token = strtok (temp, " \t");
        if (token == NULL)
        {
            continue;
        }

        // check for units designation first
        if (!strncmp (token, "STATION", 7))
        {
            // this is the designator
            if (!strcmp (token, "STATION_METRIC"))
            {
                // we are a metric station!
                alarmsWork.isMetric = 1;
            }

            continue;
        }

        alarm = (WVIEW_ALARM *) malloc (sizeof (*alarm));
        if (alarm == NULL)
        {
            for (alarm = (WVIEW_ALARM *)radListRemoveFirst (&alarmsWork.alarmList);
                 alarm != NULL;
                 alarm = (WVIEW_ALARM *)radListRemoveFirst (&alarmsWork.alarmList))
            {
                free (alarm);
            }

            fclose (file);
            return ERROR;
        }
        memset (alarm, 0, sizeof (*alarm));


        // parse the mother...

        // get the type
        if (!strcmp (token, "Barometer"))
        {
            alarm->type = Barometer;
        }
        else if (!strcmp (token, "InsideTemp"))
        {
            alarm->type = InsideTemp;
        }
        else if (!strcmp (token, "InsideHumidity"))
        {
            alarm->type = InsideHumidity;
        }
        else if (!strcmp (token, "OutsideTemp"))
        {
            alarm->type = OutsideTemp;
        }
        else if (!strcmp (token, "WindSpeed"))
        {
            alarm->type = WindSpeed;
        }
        else if (!strcmp (token, "TenMinuteAvgWindSpeed"))
        {
            alarm->type = TenMinuteAvgWindSpeed;
        }
        else if (!strcmp (token, "WindDirection"))
        {
            alarm->type = WindDirection;
        }
        else if (!strcmp (token, "OutsideHumidity"))
        {
            alarm->type = OutsideHumidity;
        }
        else if (!strcmp (token, "RainRate"))
        {
            alarm->type = RainRate;
        }
        else if (!strcmp (token, "StormRain"))
        {
            alarm->type = StormRain;
        }
        else if (!strcmp (token, "DayRain"))
        {
            alarm->type = DayRain;
        }
        else if (!strcmp (token, "MonthRain"))
        {
            alarm->type = MonthRain;
        }
        else if (!strcmp (token, "YearRain"))
        {
            alarm->type = YearRain;
        }
        else if (!strcmp (token, "TxBatteryStatus"))
        {
            alarm->type = TxBatteryStatus;
        }
        else if (!strcmp (token, "ConsoleBatteryVoltage"))
        {
            alarm->type = ConsoleBatteryVoltage;
        }
        else if (!strcmp (token, "DewPoint"))
        {
            alarm->type = DewPoint;
        }
        else if (!strcmp (token, "WindChill"))
        {
            alarm->type = WindChill;
        }
        else if (!strcmp (token, "HeatIndex"))
        {
            alarm->type = HeatIndex;
        }
        else if (!strcmp (token, "Radiation"))
        {
            alarm->type = Radiation;
        }
        else if (!strcmp (token, "UV"))
        {
            alarm->type = UV;
        }
        else if (!strcmp (token, "ET"))
        {
            alarm->type = ET;
        }
        else if (!strcmp (token, "ExtraTemp1"))
        {
            alarm->type = ExtraTemp1;
        }
        else if (!strcmp (token, "ExtraTemp2"))
        {
            alarm->type = ExtraTemp2;
        }
        else if (!strcmp (token, "ExtraTemp3"))
        {
            alarm->type = ExtraTemp3;
        }
        else if (!strcmp (token, "SoilTemp1"))
        {
            alarm->type = SoilTemp1;
        }
        else if (!strcmp (token, "SoilTemp2"))
        {
            alarm->type = SoilTemp2;
        }
        else if (!strcmp (token, "SoilTemp3"))
        {
            alarm->type = SoilTemp3;
        }
        else if (!strcmp (token, "SoilTemp4"))
        {
            alarm->type = SoilTemp4;
        }
        else if (!strcmp (token, "LeafTemp1"))
        {
            alarm->type = LeafTemp1;
        }
        else if (!strcmp (token, "LeafTemp2"))
        {
            alarm->type = LeafTemp2;
        }
        else if (!strcmp (token, "ExtraHumid1"))
        {
            alarm->type = ExtraHumid1;
        }
        else if (!strcmp (token, "ExtraHumid2"))
        {
            alarm->type = ExtraHumid2;
        }
        else
        {
            free (alarm);
            radMsgLog (PRI_MEDIUM, "invalid alarm definition - skipping...",
                       token);
            continue;
        }

        //  do the max/min flag
        token = strtok (NULL, " \t");
        if (token == NULL)
        {
            free (alarm);
            radMsgLog (PRI_MEDIUM, "invalid alarm definition - skipping...",
                       token);
            continue;
        }
        alarm->isMax = atoi (token);

        //  now the bounding value
        token = strtok (NULL, " \t");
        if (token == NULL)
        {
            free (alarm);
            radMsgLog (PRI_MEDIUM, "invalid alarm definition - skipping...",
                       token);
            continue;
        }
        alarm->bound = (float)atof (token);

        //  now the abatement seconds
        token = strtok (NULL, " \t");
        if (token == NULL)
        {
            free (alarm);
            radMsgLog (PRI_MEDIUM, "invalid alarm definition - skipping...",
                       token);
            continue;
        }
        alarm->abateSecs = atoi (token);

        //  finally, the alarm script
        token = strtok (NULL, " \t\n");
        if (token == NULL)
        {
            free (alarm);
            radMsgLog (PRI_MEDIUM, "invalid alarm definition - skipping...",
                       token);
            continue;
        }
        strncpy (alarm->scriptToRun, token, WVIEW_ALARM_SCRIPT_LENGTH);

        radListAddToEnd (&alarmsWork.alarmList, (NODE_PTR)alarm);
    }

    fclose (file);
    return (radListGetNumberOfNodes (&alarmsWork.alarmList));
}

static int waitForWviewDaemon (void)
{
    WVIEW_MSG_REQUEST   msg;
    char                srcQName[QUEUE_NAME_LENGTH+1];
    UINT                msgType;
    UINT                length;
    void                *recvBfr;
    int                 retVal, done = FALSE;

    // enable message reception from the radlib router for the archive path
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_ARCHIVE_PATH);

    msg.requestType = WVIEW_RQST_TYPE_ARCHIVE_PATH;

    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_REQUEST, &msg, sizeof(msg)) == ERROR)
    {
        radMsgLog (PRI_HIGH, "waitForWviewDaemon: radMsgRouterMessageSend failed!");
        radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
        return ERROR;
    }

    // now wait for the response here
    while (!done)
    {
        radUtilsSleep (50);

        if ((retVal = radQueueRecv (radProcessQueueGetID (),
                                    srcQName,
                                    &msgType,
                                    &recvBfr,
                                    &length))
            == FALSE)
        {
            continue;
        }
        else if (retVal == ERROR)
        {
            radMsgLog (PRI_STATUS, "waitForWviewDaemon: queue is closed!");
            radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
            return ERROR;
        }

        // is this what we want?
        if (msgType == WVIEW_MSG_TYPE_ARCHIVE_PATH)
        {
            // yes!
            done = TRUE;
        }

        // release the received buffer
        radBufferRls (recvBfr);
        }

    // disable message reception from the radlib router for the archive path
    radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
    
    return OK;
}

static void processAlarms (LOOP_PKT *loopData)
{
    WVIEW_ALARM_CLIENT  *client, *oldClient;
    WVIEW_ALARM         *alarm;
    float               tempFloat;

    // give priority to our socket clients
    for (client = (WVIEW_ALARM_CLIENT *) radListGetFirst (&alarmsWork.clientList);
         client != NULL;
         client = (WVIEW_ALARM_CLIENT *) radListGetNext (&alarmsWork.clientList, 
                                                         (NODE_PTR)client))
    {
        // write the frame start so clients may sync to the beginning of each
        // data update
        if (radSocketWriteExact (client->client, 
                                 (void *)DF_START_FRAME, 
                                 DF_START_FRAME_LEN)
            != DF_START_FRAME_LEN)
        {
            // write error, bail on this guy
            radMsgLog (PRI_HIGH, "dataFeed: write error to client %s:%d - closing socket...",
                       radSocketGetHost (client->client),
                       radSocketGetPort (client->client));
            radSocketDestroy (client->client);
            oldClient = client;
            client = (WVIEW_ALARM_CLIENT *) 
                        radListGetPrevious (&alarmsWork.clientList, (NODE_PTR)client);
            radListRemove (&alarmsWork.clientList, (NODE_PTR)oldClient);
            free (oldClient);
            continue;
        }

        // write out the loop data
        if (radSocketWriteExact (client->client, 
                                 loopData, 
                                 sizeof (*loopData))
            != sizeof (*loopData))
        {
            // write error, bail on this guy
            radMsgLog (PRI_HIGH, "dataFeed: write error to client %s:%d - closing socket...",
                       radSocketGetHost (client->client),
                       radSocketGetPort (client->client));
            radSocketDestroy (client->client);
            oldClient = client;
            client = (WVIEW_ALARM_CLIENT *) 
                        radListGetPrevious (&alarmsWork.clientList, (NODE_PTR)client);
            radListRemove (&alarmsWork.clientList, (NODE_PTR)oldClient);
            free (oldClient);
            continue;
        }
    }


    // now to process the local types
    for (alarm = (WVIEW_ALARM *) radListGetFirst (&alarmsWork.alarmList);
         alarm != NULL;
         alarm = (WVIEW_ALARM *) radListGetNext (&alarmsWork.alarmList, 
                                                 (NODE_PTR)alarm))
    {
        // first check to see if we are in abatement
        if (alarm->triggered)
        {
            if ((radTimeGetSECSinceEpoch () - alarm->abateStart) < alarm->abateSecs)
            {
                // abatement - go to the next alarm
                continue;
            }
            else
            {
                // clear trigger for future alarms
                alarm->triggered = FALSE;
            }
        }

        // switch on alarm type
        switch (alarm->type)
        {
            case Barometer:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertBPInchesToMillibars(loopData->barometer);
                }
                else
                {
                    tempFloat = loopData->barometer;
                }
                break;

            case InsideTemp:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius(loopData->inTemp);
                }
                else
                {
                    tempFloat = loopData->inTemp;
                }
                break;

            case InsideHumidity:
                tempFloat = (float)loopData->inHumidity;
                break;

            case OutsideTemp:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius(loopData->outTemp);
                }
                else
                {
                    tempFloat = loopData->outTemp;
                }
                break;

            case WindSpeed:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertMphToKph((float)loopData->windSpeed);
                }
                else
                {
                    tempFloat = (float)loopData->windSpeed;
                }
                break;

            case TenMinuteAvgWindSpeed:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertMphToKph((float)loopData->tenMinuteAvgWindSpeed);
                }
                else
                {
                    tempFloat = (float)loopData->tenMinuteAvgWindSpeed;
                }
                break;

            case WindDirection:
                tempFloat = (float)loopData->windDir;
                break;

            case OutsideHumidity:
                tempFloat = (float)loopData->outHumidity;
                break;

            case RainRate:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertRainInchesToCentimeters(loopData->rainRate);
                }
                else
                {
                    tempFloat = loopData->rainRate;
                }
                break;

            case StormRain:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertRainInchesToCentimeters(loopData->stormRain);
                }
                else
                {
                    tempFloat = loopData->stormRain;
                }
                break;

            case DayRain:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertRainInchesToCentimeters(loopData->dayRain);
                }
                else
                {
                    tempFloat = loopData->dayRain;
                }
                break;

            case MonthRain:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertRainInchesToCentimeters(loopData->monthRain);
                }
                else
                {
                    tempFloat = loopData->monthRain;
                }
                break;

            case YearRain:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertRainInchesToCentimeters(loopData->yearRain);
                }
                else
                {
                    tempFloat = loopData->yearRain;
                }
                break;

            case TxBatteryStatus:
                tempFloat = (float)loopData->txBatteryStatus;
                break;

            case ConsoleBatteryVoltage:
                tempFloat = (float)loopData->consBatteryVoltage;
                break;

            case DewPoint:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius(loopData->dewpoint);
                }
                else
                {
                    tempFloat = loopData->dewpoint;
                }
                break;

            case WindChill:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius(loopData->windchill);
                }
                else
                {
                    tempFloat = loopData->windchill;
                }
                break;

            case HeatIndex:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius(loopData->heatindex);
                }
                else
                {
                    tempFloat = loopData->heatindex;
                }
                break;

            case Radiation:
                tempFloat = (float)loopData->radiation;
                break;

            case UV:
                tempFloat = (float)loopData->UV/10.0;
                break;

            case ET:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertRainInchesToCentimeters(loopData->dayET);
                }
                else
                {
                    tempFloat = loopData->dayET;
                }
                break;

            case ExtraTemp1:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius((float)loopData->extraTemp1 - 90);
                }
                else
                {
                    tempFloat = (float)loopData->extraTemp1 - 90;
                }
                break;

            case ExtraTemp2:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius((float)loopData->extraTemp2 - 90);
                }
                else
                {
                    tempFloat = (float)loopData->extraTemp2 - 90;
                }
                break;

            case ExtraTemp3:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius((float)loopData->extraTemp3 - 90);
                }
                else
                {
                    tempFloat = (float)loopData->extraTemp3 - 90;
                }
                break;

            case SoilTemp1:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius((float)loopData->soilTemp1 - 90);
                }
                else
                {
                    tempFloat = (float)loopData->soilTemp1 - 90;
                }
                break;

            case SoilTemp2:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius((float)loopData->soilTemp2 - 90);
                }
                else
                {
                    tempFloat = (float)loopData->soilTemp2 - 90;
                }
                break;

            case SoilTemp3:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius((float)loopData->soilTemp3 - 90);
                }
                else
                {
                    tempFloat = (float)loopData->soilTemp3 - 90;
                }
                break;

            case SoilTemp4:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius((float)loopData->soilTemp4 - 90);
                }
                else
                {
                    tempFloat = (float)loopData->soilTemp4 - 90;
                }
                break;

            case LeafTemp1:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius((float)loopData->leafTemp1 - 90);
                }
                else
                {
                    tempFloat = (float)loopData->leafTemp1 - 90;
                }
                break;

            case LeafTemp2:
                if (alarmsWork.isMetric)
                {
                    tempFloat = wvutilsConvertFahrenToCelsius((float)loopData->leafTemp2 - 90);
                }
                else
                {
                    tempFloat = (float)loopData->leafTemp2 - 90;
                }
                break;

            case ExtraHumid1:
                tempFloat = (float)loopData->extraHumid1;
                break;

            case ExtraHumid2:
                tempFloat = (float)loopData->extraHumid2;
                break;

            default:
                // no match, blow it off
                continue;
        }

        // see if we tripped the breaker here
        if (alarm->isMax)
        {
            if (tempFloat >= alarm->bound)
            {
                // we did!
                alarm->triggered = TRUE;
                alarm->triggerValue = tempFloat;
                alarm->abateStart = radTimeGetSECSinceEpoch ();

                // run user script here
                if (executeScript (alarm) != 0)
                {
                    radMsgLog (PRI_MEDIUM, 
                               "processAlarms: script %d failed",
                               alarm->scriptToRun);
                }
            }
        }
        else
        {
            if (tempFloat <= alarm->bound)
            {
                // we did!
                alarm->triggered = TRUE;
                alarm->triggerValue = tempFloat;
                alarm->abateStart = radTimeGetSECSinceEpoch ();

                // run user script here
                if (executeScript (alarm) != 0)
                {
                    radMsgLog (PRI_MEDIUM, 
                               "processAlarms: script %d failed",
                               alarm->scriptToRun);
                }
            }
        }
    }

    return;
}

/*  ... system initialization
*/
static int alarmsSysInit (WVIEW_ALARM_WORK *work)
{
    char            devPath[256], temp[64];
    struct stat     fileData;

    /*  ... exit if our config file is not there
    */
    sprintf (work->configFile, "%s/%s", WVIEW_CONFIG_DIR, ALARMS_CONFIG_FILENAME);
    if (stat (work->configFile, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_ALARMS, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "%s does not exist - exiting...\n",
                   work->configFile);
        radMsgLogExit ();
        return ERROR_ABORT;
    }

    /*  ... check for our daemon's pid file, don't run if it isn't there
    */
    sprintf (devPath, "%s/%s", WVIEW_RUN_DIR, WVD_LOCK_FILE_NAME);
    if (stat (devPath, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_ALARMS, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "wview daemon lock file %s does not exist - aborting!",
                   devPath);
        radMsgLogExit ();
        return ERROR;
    }

    sprintf (work->pidFile, "%s/%s", WVIEW_RUN_DIR, ALARMS_LOCK_FILE_NAME);
    sprintf (work->fifoFile, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_ALARMS);
    sprintf (work->daemonQname, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_DAEMON);
    sprintf (work->wviewdir, "%s", WVIEW_RUN_DIR);

    /*  ... check for our pid file, don't run if it IS there
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        radMsgLogInit (PROC_NAME_ALARMS, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "lock file %s exists, older copy may be running - aborting!",
                   work->pidFile);
        radMsgLogExit ();
        return ERROR;
    }

    return OK;
}

/*  ... system exit
*/
static int alarmsSysExit (WVIEW_ALARM_WORK *work)
{
    struct stat     fileData;

    /*  ... delete our pid file
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        unlink (work->pidFile);
    }

    return OK;
}

static void defaultSigHandler (int signum)
{
    int         retVal;

    switch (signum)
    {
        case SIGHUP:
            // user wants us to change the verbosity setting
            retVal = wvutilsToggleVerbosity ();
            radMsgLog (PRI_STATUS, "wvalarmd: SIGHUP - toggling log verbosity %s",
                       ((retVal == 0) ? "OFF" : "ON"));

            signal (signum, defaultSigHandler);
            return;

        case SIGPIPE:
            // we have a far end socket disconnection, we'll handle it in the
            // "read/write" code
            signal (signum, defaultSigHandler);
            break;

        case SIGCHLD:
            wait (NULL);
            signal (signum, defaultSigHandler);
            break;

        case SIGILL:
        case SIGBUS:
        case SIGFPE:
        case SIGSEGV:
        case SIGXFSZ:
        case SIGSYS:
            // unrecoverable signal - we must exit right now!
            radMsgLog (PRI_CATASTROPHIC, "wvalarmd: recv sig %d: bailing out!", signum);
            abort ();

        default:
            // can we allow the process to exit normally?
            if (!alarmsWork.inMainLoop)
            {
                // NO! - we gotta bail here!
                radMsgLog (PRI_HIGH, "wvalarmd: recv sig %d: exiting now!", signum);
                if (alarmsWork.dataFeedServer)
                    radSocketDestroy (alarmsWork.dataFeedServer);

                radMsgRouterExit ();
                alarmsSysExit (&alarmsWork);
                radProcessExit ();
                radSystemExit (WVIEW_SYSTEM_ID);
                exit (1);                
            }

            // we can allow the process to exit normally...
            radMsgLog (PRI_HIGH, "wvalarmd: recv sig %d: exiting!", signum);

            alarmsWork.exiting = TRUE;
            radProcessSetExitFlag ();

            signal (signum, defaultSigHandler);
            break;
    }

    return;
}

static void msgHandler
(
    char                    *srcQueueName,
    UINT                    msgType,
    void                    *msg,
    UINT                    length,
    void                    *userData
)
{
    LOOP_PKT                *loopData;

    if (msgType == WVIEW_MSG_TYPE_LOOP_DATA_SVC)
    {
        loopData = &(((WVIEW_MSG_LOOP_DATA *)msg)->loopData);

        // process alarms
        processAlarms (loopData);
    }

    return;
}

static void evtHandler
(
    UINT        eventsRx,
    UINT        rxData,
    void        *userData
)
{
    return;
}

static void timerHandler (void *parm)
{
    return;
}

static void dataFeedAccept (int fd, void *userData)
{
    RADSOCK_ID          newConnection;
    WVIEW_ALARM_CLIENT  *client;

    newConnection = radSocketServerAcceptConnection (alarmsWork.dataFeedServer);
    if (newConnection == NULL)
    {
        radMsgLog (PRI_MEDIUM, "dataFeed: accept connection failed!");
        return;
    }

    // stick him on the data feed client list
    client = (WVIEW_ALARM_CLIENT *) malloc (sizeof (*client));
    if (client == NULL)
    {
        radMsgLog (PRI_MEDIUM, "dataFeedAccept: malloc failed!");
        radSocketDestroy (newConnection);
        return;
    }
    memset (client, 0, sizeof (*client));

    client->client = newConnection;

    radListAddToEnd (&alarmsWork.clientList, (NODE_PTR)client);
    radMsgLog (PRI_STATUS, "dataFeed: client %s:%d accepted...",
               radSocketGetHost (client->client),
               radSocketGetPort (client->client));

    return;
}


/*  ... the main entry point for the alarm process
*/
int main (int argc, char *argv[])
{
    void            (*alarmHandler)(int);
    int             retVal;
    FILE            *pidfile;

    memset (&alarmsWork, 0, sizeof (alarmsWork));
    radListReset (&alarmsWork.alarmList);
    radListReset (&alarmsWork.clientList);

    /*  ... initialize some system stuff first
    */
    retVal = alarmsSysInit (&alarmsWork);
    if (retVal == ERROR)
    {
        radMsgLogInit (PROC_NAME_ALARMS, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "wvalarmd init failed!\n");
        radMsgLogExit ();
        exit (1);
    }
    else if (retVal == ERROR_ABORT)
    {
        exit (2);
    }


    /*  ... call the global radlib system init function
    */
    if (radSystemInit (WVIEW_SYSTEM_ID) == ERROR)
    {
        radMsgLogInit (PROC_NAME_ALARMS, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "radSystemInit failed!");
        radMsgLogExit ();
        exit (1);
    }


    /*  ... call the radlib process init function
    */
    if (radProcessInit (PROC_NAME_ALARMS,
                        alarmsWork.fifoFile,
                        PROC_NUM_TIMERS_ALARMS,
                        TRUE,                      // TRUE for daemon
                        msgHandler,
                        evtHandler,
                        NULL)
        == ERROR)
    {
        printf ("\nradProcessInit failed: %s\n\n", PROC_NAME_ALARMS);
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    alarmsWork.myPid = getpid ();
    pidfile = fopen (alarmsWork.pidFile, "w");
    if (pidfile == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "lock file create failed!\n");
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    fprintf (pidfile, "%d", getpid ());
    fclose (pidfile);


    alarmHandler = radProcessSignalGetHandler (SIGALRM);
    radProcessSignalCatchAll (defaultSigHandler);
    radProcessSignalCatch (SIGALRM, alarmHandler);


    // get the wview verbosity setting
    if (wvutilsSetVerbosity (WV_VERBOSE_WVALARMD) == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "wvutilsSetVerbosity failed!\n");
        alarmsSysExit (&alarmsWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }


    // grab all of our alarm definitions from the config file
    if (readAlarmsConfFile (alarmsWork.configFile) < 0)
    {
        radMsgLog (PRI_HIGH, "readAlarmsConfFile failed - does wvalarm.conf exist?");
        alarmsSysExit (&alarmsWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        radMsgLog (PRI_STATUS, 
                   "alarms: added %d alarm definitions from wvalarm.conf",
                   radListGetNumberOfNodes (&alarmsWork.alarmList));
    }


    // wait a bit here before continuing
    radUtilsSleep (500);


    //  register with the radlib message router
    if (radMsgRouterInit (WVIEW_RUN_DIR) == ERROR)
    {
        radMsgLog (PRI_HIGH, "radMsgRouterInit failed!");
        alarmsSysExit (&alarmsWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }


    // wait for the wview daemon to be ready
    if (waitForWviewDaemon () == ERROR)
    {
        radMsgLog (PRI_HIGH, "waitForWviewDaemon failed");
        radMsgRouterExit ();
        alarmsSysExit (&alarmsWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    // finally, initialize our data feed socket
    alarmsWork.dataFeedServer = radSocketServerCreate (WVALARM_LISTEN_PORT);
    if (alarmsWork.dataFeedServer == NULL)
    {
        radMsgLog (PRI_HIGH, "radSocketServerCreate failed...");
        radMsgRouterExit ();
        alarmsSysExit (&alarmsWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    // add it to our descriptors of interest
    if (radProcessIORegisterDescriptor (radSocketGetDescriptor (alarmsWork.dataFeedServer),
                                        dataFeedAccept,
                                        NULL)
            == ERROR)
    {
        radMsgLog (PRI_HIGH, "radProcessIORegisterDescriptor failed...");
        radSocketDestroy (alarmsWork.dataFeedServer);
        radMsgRouterExit ();
        alarmsSysExit (&alarmsWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    // enable message reception from the radlib router for loop data
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_LOOP_DATA_SVC);


    // enter normal processing
    alarmsWork.inMainLoop = TRUE;
    radMsgLog (PRI_STATUS, "running...");

    while (!alarmsWork.exiting)
    {
        // wait on something interesting
        if (radProcessWait (0) == ERROR)
        {
            alarmsWork.exiting = TRUE;
        }
    }


    radMsgLog (PRI_STATUS, "exiting normally...");

    radSocketDestroy (alarmsWork.dataFeedServer);
    radMsgRouterExit ();
    alarmsSysExit (&alarmsWork);
    radProcessExit ();
    radSystemExit (WVIEW_SYSTEM_ID);
    exit (0);
}

