/*---------------------------------------------------------------------
 
 FILE NAME:
        datafeedClient.c
 
 PURPOSE:
        Example wview alarm data feed client.
 
 REVISION HISTORY:
    Date        Programmer  Revision    Function
    06/05/2005  M.S. Teel   0           Original
 
 ASSUMPTIONS:
 None.
 
------------------------------------------------------------------------*/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include <sys/stat.h>

#include <radsysdefs.h>
#include <radmsgLog.h>
#include <radsocket.h>


/*  ... global references
*/

/*  ... local memory
*/
#define WV_DATAFEED_PORT            11011

static USHORT DF_START_FRAME[4] = 
{
    0xF388, 0xC6A2, 0xDADA, 0xE7CF
};
#define DF_START_FRAME_LEN          8


// LOOP_PKT - define the LOOP (current readings) data format for IPM msgs and
//            station sensor polling; those required for auto-archive generation 
//            and HILOW are marked "!" and MUST be provided by the station
typedef struct
{
    // minimum required fields - station must provide these
    float               barometer;              /* ! inches               */
    float               stationPressure;        /* ! inches               */
    float               inTemp;                 /* ! degrees F            */
    float               outTemp;                /* ! degrees F            */
    USHORT              inHumidity;             /* ! percent              */
    USHORT              outHumidity;            /* ! percent              */
    USHORT              windSpeed;              /* ! mph                  */
    USHORT              windDir;                /* ! degrees              */
    USHORT              windGust;               /* ! mph                  */
    USHORT              windGustDir;            /* ! degrees              */
    float               rainRate;               /* ! in/hr                */
    float               sampleRain;             /* ! inches               */
    float               sampleET;               /* ! ET                   */
    USHORT              radiation;              /* ! watts/m^3            */
    USHORT              UV;                     /* ! UV index * 10        */
    float               dewpoint;               /* ! degrees F            */
    float               windchill;              /* ! degrees F            */
    float               heatindex;              /* ! degrees F            */

    // computed values - station should not mess with these
    float               stormRain;              /* inches                 */
    time_t              stormStart;             /* time_t                 */
    float               dayRain;                /* inches                 */
    float               monthRain;              /* inches                 */
    float               yearRain;               /* inches                 */
    float               dayET;                  /* inches                 */
    float               intervalAvgWCHILL;      /* degrees F              */
    short               intervalAvgWSPEED;      /* mph                    */
    USHORT              yearRainMonth;          /* 1-12 Rain Start Month  */
    
    // --- The following may or may not be supported for a given station ---

    // Vantage Pro
    USHORT              rxCheckPercent;         /* 0 - 100                */
    USHORT              tenMinuteAvgWindSpeed;  /* mph                    */
    USHORT              forecastIcon;           /* VP only                */
    USHORT              forecastRule;           /* VP only                */
    USHORT              txBatteryStatus;        /* VP only                */
    USHORT              consBatteryVoltage;     /* VP only                */
    UCHAR               extraTemp1;             /* degrees F + 90         */
    UCHAR               extraTemp2;             /* degrees F + 90         */
    UCHAR               extraTemp3;             /* degrees F + 90         */
    UCHAR               soilTemp1;              /* degrees F + 90         */
    UCHAR               soilTemp2;              /* degrees F + 90         */
    UCHAR               soilTemp3;              /* degrees F + 90         */
    UCHAR               soilTemp4;              /* degrees F + 90         */
    UCHAR               leafTemp1;              /* degrees F + 90         */
    UCHAR               leafTemp2;              /* degrees F + 90         */
    UCHAR               extraHumid1;            /* percent                */
    UCHAR               extraHumid2;            /* percent                */
    UCHAR               soilMoist1;
    UCHAR               soilMoist2;
    UCHAR               leafWet1;
    UCHAR               leafWet2;
    UCHAR               pad1;
    
    // Vaisala WXT-510
    float               wxt510Hail;             /* inches                 */
    float               wxt510Hailrate;         /* in/hr                  */
    float               wxt510HeatingTemp;      /* degrees F              */
    float               wxt510HeatingVoltage;   /* volts                  */
    float               wxt510SupplyVoltage;    /* volts                  */
    float               wxt510ReferenceVoltage; /* volts                  */

    }__attribute__ ((packed)) LOOP_PKT;


/*  ... methods
*/
    
static int syncStartOfFrame (RADSOCK_ID socket)
{
    USHORT          start;

    if (radSocketReadExact (socket, (void *)&start, sizeof (USHORT)) != sizeof (USHORT))
    {
        radMsgLog (PRI_HIGH, "datafeedClient: socket read error - abort!\n");
        return ERROR;
    }
    else if (start != DF_START_FRAME[0])
    {
        return FALSE;
    }

    if (radSocketReadExact (socket, (void *)&start, sizeof (USHORT)) != sizeof (USHORT))
    {
        radMsgLog (PRI_HIGH, "datafeedClient: socket read error - abort!\n");
        return ERROR;
    }
    else if (start != DF_START_FRAME[1])
    {
        return FALSE;
    }
 
    if (radSocketReadExact (socket, (void *)&start, sizeof (USHORT)) != sizeof (USHORT))
    {
        radMsgLog (PRI_HIGH, "datafeedClient: socket read error - abort!\n");
        return ERROR;
    }
    else if (start != DF_START_FRAME[2])
    {
        return FALSE;
    }
    
    if (radSocketReadExact (socket, (void *)&start, sizeof (USHORT)) != sizeof (USHORT))
    {
        radMsgLog (PRI_HIGH, "datafeedClient: socket read error - abort!\n");
        return ERROR;
    }
    else if (start != DF_START_FRAME[3])
    {
        return FALSE;
    }

    return TRUE;
}

/*  ... the entry point - if no hostname or IP is given, localhost is used
*/
int main (int argc, char *argv[])
{
    RADSOCK_ID      socket;
    int             retVal, done = FALSE;
    char            temp[128];
    LOOP_PKT        data;

    radMsgLogInit ("dataFeed", FALSE, TRUE);

    if (argc < 2)
        strcpy (temp, "localhost");
    else
        strcpy (temp, argv[1]);
    
    socket = radSocketClientCreate (temp, WV_DATAFEED_PORT);
    if (socket == NULL)
    {
        radMsgLog (PRI_HIGH, "datafeedClient: failed to connect to server!");
        exit (1);
    }
    
    /* now loop, waiting to get the start frame sequence */
    while (!done)
    {
        /* try to find the start frame (this blocks if the socket is empty) */
        retVal = syncStartOfFrame (socket);
        if (retVal == TRUE)
        {
            /* OK, we have an update coming (this may block) */
            if (radSocketReadExact (socket, (void *)&data, sizeof (data)) 
                != sizeof (data))
            {
                radMsgLog (PRI_HIGH, "datafeedClient: socket read error - abort!");
                done = TRUE;
                continue;
            }
            
            /* could transpose from integral to floating point 
               representation here */
            
            
            /* process the data ... for example, will just log receipt */
            radMsgLog (PRI_STATUS, "dataFeedClient:%s:%d:received update: %.1f",
                       radSocketGetHost (socket),
                       radSocketGetPort (socket),
                       data.outTemp);
        }
        else if (retVal == ERROR)
        {
            /* problems! - bail out */
            radMsgLog (PRI_HIGH, "datafeedClient: socket error during sync");
            done = TRUE;
        }
        else
        {
            continue;
        }
    }
    
    radSocketDestroy (socket);
    radMsgLogExit ();
    exit (0);
}

