/*---------------------------------------------------------------------------
 
  FILENAME:
        dbdatabase.c
 
  PURPOSE:
        Provide the SQL database archiving API methods.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        4/18/2005       M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

//  ... System header files
#include <radmsgLog.h>


//  ... Local header files
#include <dbdatabase.h>


//  ... local memory

static DBDATABASE_WORK          dbdbWork;
static int                      dbTableExists;
static char                     dupQuery[256];
static int                      archiveExtendedIndex = 18;  // where extended begins
static DB_ARCHIVE_DESCRIPTION   archiveDescription[] = 
{
    {"RecordTime",   FIELD_DATETIME | FIELD_PRI_KEY | FIELD_UNIQUE_INDEX | FIELD_NOT_NULL, 16},
    {"ArcInt",       FIELD_INT,                                           0},
    {"OutTemp",      FIELD_FLOAT,                                         0},
    {"HiOutTemp",    FIELD_FLOAT,                                         0},
    {"LowOutTemp",   FIELD_FLOAT,                                         0},
    {"InTemp",       FIELD_FLOAT,                                         0},
    {"Barometer",    FIELD_FLOAT,                                         0},
    {"OutHumid",     FIELD_FLOAT,                                         0},
    {"InHumid",      FIELD_FLOAT,                                         0},
    {"Rain",         FIELD_FLOAT,                                         0},
    {"HiRainRate",   FIELD_FLOAT,                                         0},
    {"WindSpeed",    FIELD_FLOAT,                                         0},
    {"HiWindSpeed",  FIELD_FLOAT,                                         0},
    {"WindDir",      FIELD_INT,                                           0},
    {"HiWindDir",    FIELD_INT,                                           0},
    {"Dewpoint",     FIELD_FLOAT,                                         0},
    {"WindChill",    FIELD_FLOAT,                                         0},
    {"HeatIndex",    FIELD_FLOAT,                                         0},
    {"solarRad",     FIELD_INT,                                           0},
    {"hiSolarRad",   FIELD_INT,                                           0},
    {"UV",           FIELD_FLOAT,                                         0},
    {"hiUV",         FIELD_FLOAT,                                         0},
    {"ET",           FIELD_FLOAT,                                         0},
    {"leafTemp1",    FIELD_INT,                                           0},
    {"leafTemp2",    FIELD_INT,                                           0},
    {"leafTemp3",    FIELD_INT,                                           0},
    {"leafTemp4",    FIELD_INT,                                           0},
    {"leafWet1",     FIELD_INT,                                           0},
    {"leafWet2",     FIELD_INT,                                           0},
    {"leafWet3",     FIELD_INT,                                           0},
    {"leafWet4",     FIELD_INT,                                           0},
    {"soilTemp1",    FIELD_INT,                                           0},
    {"soilTemp2",    FIELD_INT,                                           0},
    {"soilTemp3",    FIELD_INT,                                           0},
    {"soilTemp4",    FIELD_INT,                                           0},
    {"soilTemp5",    FIELD_INT,                                           0},
    {"soilTemp6",    FIELD_INT,                                           0},
    {"soilMoist1",   FIELD_INT,                                           0},
    {"soilMoist2",   FIELD_INT,                                           0},
    {"soilMoist3",   FIELD_INT,                                           0},
    {"soilMoist4",   FIELD_INT,                                           0},
    {"soilMoist5",   FIELD_INT,                                           0},
    {"soilMoist6",   FIELD_INT,                                           0},
    {"extraHumid1",  FIELD_INT,                                           0},
    {"extraHumid2",  FIELD_INT,                                           0},
    {"extraTemp1",   FIELD_INT,                                           0},
    {"extraTemp2",   FIELD_INT,                                           0},
    {"extraTemp3",   FIELD_INT,                                           0},
    {NULL, 0, 0}
};


//  ... define methods here

int dbdatabaseInit 
(
    int             isEnabled,
    int             useExtended,
    char            *host,
    char            *username,
    char            *password,
    char            *databaseName,
    char            *tableName,
    int             isMetricUnits
)
{
    dbTableExists = FALSE;

    memset (&dbdbWork, 0, sizeof (dbdbWork));
    
    dbdbWork.isEnabled = isEnabled;
    dbdbWork.isExtended = useExtended;
    
    if (!dbdbWork.isEnabled)
        return OK;

    if (host == NULL)
        strcpy (dbdbWork.host, "NA");
    else
        strcpy (dbdbWork.host, host);
        
    strcpy (dbdbWork.username, username);
    strcpy (dbdbWork.password, password);
    strcpy (dbdbWork.databaseName, databaseName);
    strcpy (dbdbWork.tableName, tableName);
    dbdbWork.isMetricUnits = isMetricUnits;

    return OK;
}

int dbdatabaseOpen (void)
{
    char            *host;
    int             i, retVal, maxLoop;
    ROW_ID          row;

    if (!dbdbWork.isEnabled)
        return ERROR;

    if (dbdbWork.dbId != NULL)
    {
        // try to close it
        raddatabaseClose (dbdbWork.dbId);
        dbdbWork.dbId = NULL;
    }

    if (!strcmp (dbdbWork.host, "NA"))
        host = NULL;
    else
        host = dbdbWork.host;
    
    // does the database exist?
    if ((dbdbWork.dbId = raddatabaseOpen (host,
                                          dbdbWork.username,
                                          dbdbWork.password,
                                          dbdbWork.databaseName))
        == NULL)
    {
        radMsgLog (PRI_MEDIUM, "dbdatabaseOpen: %s databaseOpen failed!",
                   dbdbWork.databaseName);
        return ERROR;
    }

    // open the table (or create it)
    if (!dbTableExists)
    {
        if (!raddatabaseTableIfExists (dbdbWork.dbId, dbdbWork.tableName))
        {
            row = raddatabaseRowDescriptionCreate ();
            if (row == NULL)
            {
                radMsgLog (PRI_MEDIUM, "dbdatabaseOpen: databaseRowDescriptionCreate failed!");
                raddatabaseClose (dbdbWork.dbId);
                dbdbWork.dbId = NULL;
                return ERROR;
            }
            
            if (dbdbWork.isExtended)
                maxLoop = 255;
            else
                maxLoop = archiveExtendedIndex;
            
            // loop through adding all defined fields
            for (i = 0; i < maxLoop && archiveDescription[i].name != NULL; i ++)
            {
                retVal = raddatabaseRowDescriptionAddField (row, 
                                                            archiveDescription[i].name, 
                                                            archiveDescription[i].type, 
                                                            archiveDescription[i].maxLength);
                if (retVal == ERROR)
                {
                    radMsgLog (PRI_MEDIUM, "dbdatabaseOpen: "
                               "RowDescriptionAddField failed!");
                    raddatabaseRowDescriptionDelete (row);
                    raddatabaseClose (dbdbWork.dbId);
                    dbdbWork.dbId = NULL;
                    return ERROR;
                }
            }
    
            //  ... now try to create the table
            retVal = raddatabaseTableCreate (dbdbWork.dbId, dbdbWork.tableName, row);
            if (retVal == ERROR)
            {
                radMsgLog (PRI_MEDIUM, "dbdatabaseOpen: "
                           "raddatabaseTableCreate failed!");
                raddatabaseRowDescriptionDelete (row);
                raddatabaseClose (dbdbWork.dbId);
                dbdbWork.dbId = NULL;
                return ERROR;
            }
    
            raddatabaseRowDescriptionDelete (row);
            dbTableExists = TRUE;
        }
        else
        {
            dbTableExists = TRUE;
        }            
    }

    return OK;
}

void dbdatabaseClose (void)
{
    raddatabaseClose (dbdbWork.dbId);
    dbdbWork.dbId = NULL;
    return;
}

    
int dbdatabaseInsertRecord
(
    ArchiveRecord   *newRecord,
    int             year,
    int             month,
    int             day,
    float           dew,
    float           chill,
    float           heat
)
{
    FIELD_ID        field;
    char            *host, dtime[64];
    float           rain, click, wdir;
    RESULT_SET_ID   resultId;
    ROW_ID          row;

    if (!dbdbWork.isEnabled)
        return DBDB_INSERT_ERROR;

    // dbfStoreArchiveRecord sets packed time to 24:00 for midnight, but
    // MYSQL/PostGreSQL doesn't like that
    if (newRecord->packedTime >= (60 * 24))
        newRecord->packedTime = 0;
     
    // build the proper date/time field format
    sprintf (dtime, 
             "%4.4d-%2.2d-%2.2d %2.2d:%2.2d:%2.2d",
             year,
             month,
             day,
             newRecord->packedTime / 60,
             newRecord->packedTime % 60,
             0);

    // first check for a duplicate
    sprintf (dupQuery, "SELECT RecordTime FROM %s WHERE RecordTime='%s'",
             dbdbWork.tableName, dtime);
    if (raddatabaseQuery (dbdbWork.dbId, dupQuery, TRUE) == OK)
    {
        resultId = raddatabaseGetResults (dbdbWork.dbId);
        if (resultId != NULL)
        {
            row = raddatabaseResultsGetFirst (resultId);
            if (row != NULL)
            {
                // looks like a duplicate...
                raddatabaseReleaseResults (dbdbWork.dbId, resultId);
                return DBDB_INSERT_DUPLICATE;
            }
            
            raddatabaseReleaseResults (dbdbWork.dbId, resultId);
        }
    }

    // let's add a row here
    row = raddatabaseTableDescriptionGet (dbdbWork.dbId, dbdbWork.tableName);
    if (row == NULL)
    {
        radMsgLog (PRI_MEDIUM, 
                   "dbdatabaseInsertRecord: databaseTableDescriptionGet failed!");
        return DBDB_INSERT_ERROR;
    }

    field = raddatabaseFieldGet (row, archiveDescription[0].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet DateTime failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    raddatabaseFieldSetDateTimeValue (field, dtime, strlen (dtime));

    field = raddatabaseFieldGet (row, archiveDescription[1].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 1 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    raddatabaseFieldSetIntValue (field, (int)newRecord->archiveInterval);
    
    field = raddatabaseFieldGet (row, archiveDescription[2].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 2 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertFahrenToCelsius ((float)newRecord->outsideTemp/10.0));
    else
        raddatabaseFieldSetFloatValue (field, (float)newRecord->outsideTemp/10.0);
    
    field = raddatabaseFieldGet (row, archiveDescription[3].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 3 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertFahrenToCelsius ((float)newRecord->hiOutsideTemp/10.0));
    else
        raddatabaseFieldSetFloatValue (field, (float)newRecord->hiOutsideTemp/10.0);
    
    field = raddatabaseFieldGet (row, archiveDescription[4].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 4 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertFahrenToCelsius ((float)newRecord->lowOutsideTemp/10.0));
    else
        raddatabaseFieldSetFloatValue (field, (float)newRecord->lowOutsideTemp/10.0);
    
    field = raddatabaseFieldGet (row, archiveDescription[5].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 5 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertFahrenToCelsius ((float)newRecord->insideTemp/10.0));
    else
        raddatabaseFieldSetFloatValue (field, (float)newRecord->insideTemp/10.0);
    
    field = raddatabaseFieldGet (row, archiveDescription[6].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 6 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertBPInchesToMillibars ((float)newRecord->barometer/1000.0));
    else
        raddatabaseFieldSetFloatValue (field, (float)newRecord->barometer/1000.0);
    
    field = raddatabaseFieldGet (row, archiveDescription[7].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 7 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    raddatabaseFieldSetFloatValue (field, (float)newRecord->outsideHum/10.0);
    
    field = raddatabaseFieldGet (row, archiveDescription[8].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 8 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    raddatabaseFieldSetFloatValue (field, (float)newRecord->insideHum/10.0);
    
    field = raddatabaseFieldGet (row, archiveDescription[9].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 9 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if ((newRecord->rain & 0xF000) == 0x0000)
        click = 0.1;
    else if ((newRecord->rain & 0xF000) == 0x1000)
        click = 0.01;
    rain = click * (float)(newRecord->rain & 0x0FFF);
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertRainInchesToCentimeters (rain));
    else
        raddatabaseFieldSetFloatValue (field, rain);
    
    field = raddatabaseFieldGet (row, archiveDescription[10].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 10 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    rain = click * (float)(newRecord->hiRainRate & 0x0FFF);
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertRainInchesToCentimeters (rain));
    else
        raddatabaseFieldSetFloatValue (field, rain);
    
    field = raddatabaseFieldGet (row, archiveDescription[11].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 11 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertMphToKph ((float)newRecord->windSpeed/10.0));
    else
        raddatabaseFieldSetFloatValue (field, (float)newRecord->windSpeed/10.0);
    
    field = raddatabaseFieldGet (row, archiveDescription[12].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 12 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertMphToKph ((float)newRecord->hiWindSpeed/10.0));
    else
        raddatabaseFieldSetFloatValue (field, (float)newRecord->hiWindSpeed/10.0);
    
    field = raddatabaseFieldGet (row, archiveDescription[13].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 13 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (newRecord->windDirection < 16)
        wdir = 22.5 * (float)newRecord->windDirection;
    else
        wdir = -1.0;
    raddatabaseFieldSetIntValue (field, (int)wdir);
    
    field = raddatabaseFieldGet (row, archiveDescription[14].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 14 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (newRecord->hiWindDirection < 16)
        wdir = 22.5 * (float)newRecord->hiWindDirection;
    else
        wdir = -1.0;
    raddatabaseFieldSetIntValue (field, (int)wdir);
    
    field = raddatabaseFieldGet (row, archiveDescription[15].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 15 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertFahrenToCelsius (dew));
    else
        raddatabaseFieldSetFloatValue (field, dew);
    
    field = raddatabaseFieldGet (row, archiveDescription[16].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 16 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertFahrenToCelsius (chill));
    else
        raddatabaseFieldSetFloatValue (field, chill);
    
    field = raddatabaseFieldGet (row, archiveDescription[17].name);
    if (field == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 17 failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    if (dbdbWork.isMetricUnits)
        raddatabaseFieldSetFloatValue (field, 
                                       wvutilsConvertFahrenToCelsius (heat));
    else
        raddatabaseFieldSetFloatValue (field, heat);
    

    // Are we storing extended values too?
    if (dbdbWork.isExtended)
    {
        //  ... add the extended values
        field = raddatabaseFieldGet (row, archiveDescription[18].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 18 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        raddatabaseFieldSetIntValue (field, (int)newRecord->solarRad);
        
        field = raddatabaseFieldGet (row, archiveDescription[19].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 19 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        raddatabaseFieldSetIntValue (field, (int)newRecord->hisolarRad);
        
        field = raddatabaseFieldGet (row, archiveDescription[20].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 20 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->UV == 255)
        {
            //no value
            raddatabaseFieldSetFloatValue (field, -1.0);
        }
        else
        {
            raddatabaseFieldSetFloatValue (field, (float)newRecord->UV/10.0);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[21].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 21 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->hiUV == 255)
        {
            //no value
            raddatabaseFieldSetFloatValue (field, -1.0);
        }
        else
        {
            raddatabaseFieldSetFloatValue (field, (float)newRecord->hiUV/10.0);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[22].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 22 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->ET == 255)
        {
            // no value
            raddatabaseFieldSetFloatValue (field, -1.0);
        }
        else
        {
            if (dbdbWork.isMetricUnits)
                raddatabaseFieldSetFloatValue (field, 
                                               wvutilsConvertRainInchesToCentimeters ((float)newRecord->ET/1000.0));
            else
                raddatabaseFieldSetFloatValue (field, (float)newRecord->ET/1000.0);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[23].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 23 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->leafTemp[0] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            if (dbdbWork.isMetricUnits)
                raddatabaseFieldSetIntValue (field, 
                                             (int)wvutilsConvertFahrenToCelsius ((float)(newRecord->leafTemp[0] - 90)));
            else
                raddatabaseFieldSetIntValue (field, (int)newRecord->leafTemp[0] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[24].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 24 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->leafTemp[1] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            if (dbdbWork.isMetricUnits)
                raddatabaseFieldSetIntValue (field, 
                                             (int)wvutilsConvertFahrenToCelsius ((float)(newRecord->leafTemp[1] - 90)));
            else
                raddatabaseFieldSetIntValue (field, (int)newRecord->leafTemp[1] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[25].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 25 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->leafTemp[2] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            if (dbdbWork.isMetricUnits)
                raddatabaseFieldSetIntValue (field, 
                                             (int)wvutilsConvertFahrenToCelsius ((float)(newRecord->leafTemp[2] - 90)));
            else
                raddatabaseFieldSetIntValue (field, (int)newRecord->leafTemp[2] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[26].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 26 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->leafTemp[3] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            if (dbdbWork.isMetricUnits)
                raddatabaseFieldSetIntValue (field, 
                                             (int)wvutilsConvertFahrenToCelsius ((float)(newRecord->leafTemp[3] - 90)));
            else
                raddatabaseFieldSetIntValue (field, (int)newRecord->leafTemp[3] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[27].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 27 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->leafWetness[0] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->leafWetness[0]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[28].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 28 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->leafWetness[1] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->leafWetness[1]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[29].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 29 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->leafWetness[2] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->leafWetness[2]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[30].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 30 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->leafWetness[3] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->leafWetness[3]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[31].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 31 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilTemp[0] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            if (dbdbWork.isMetricUnits)
                raddatabaseFieldSetIntValue (field, 
                                             (int)wvutilsConvertFahrenToCelsius ((float)(newRecord->soilTemp[0] - 90)));
            else
                raddatabaseFieldSetIntValue (field, (int)newRecord->soilTemp[0] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[32].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 32 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilTemp[1] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            if (dbdbWork.isMetricUnits)
                raddatabaseFieldSetIntValue (field, 
                                             (int)wvutilsConvertFahrenToCelsius ((float)(newRecord->soilTemp[1] - 90)));
            else
                raddatabaseFieldSetIntValue (field, (int)newRecord->soilTemp[1] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[33].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 33 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilTemp[2] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            if (dbdbWork.isMetricUnits)
                raddatabaseFieldSetIntValue (field, 
                                             (int)wvutilsConvertFahrenToCelsius ((float)(newRecord->soilTemp[2] - 90)));
            else
                raddatabaseFieldSetIntValue (field, (int)newRecord->soilTemp[2] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[34].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 34 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilTemp[3] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            if (dbdbWork.isMetricUnits)
                raddatabaseFieldSetIntValue (field, 
                                             (int)wvutilsConvertFahrenToCelsius ((float)(newRecord->soilTemp[3] - 90)));
            else
                raddatabaseFieldSetIntValue (field, (int)newRecord->soilTemp[3] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[35].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 35 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilTemp[4] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            if (dbdbWork.isMetricUnits)
                raddatabaseFieldSetIntValue (field, 
                                             (int)wvutilsConvertFahrenToCelsius ((float)(newRecord->soilTemp[4] - 90)));
            else
                raddatabaseFieldSetIntValue (field, (int)newRecord->soilTemp[4] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[36].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 36 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilTemp[5] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            if (dbdbWork.isMetricUnits)
                raddatabaseFieldSetIntValue (field, 
                                             (int)wvutilsConvertFahrenToCelsius ((float)(newRecord->soilTemp[5] - 90)));
            else
                raddatabaseFieldSetIntValue (field, (int)newRecord->soilTemp[5] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[37].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 37 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilMoisture[0] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->soilMoisture[0]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[38].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 38 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilMoisture[1] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->soilMoisture[1]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[39].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 39 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilMoisture[2] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->soilMoisture[2]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[40].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 40 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilMoisture[3] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->soilMoisture[3]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[41].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 41 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilMoisture[4] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->soilMoisture[4]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[42].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 42 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->soilMoisture[5] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->soilMoisture[5]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[43].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 43 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->extraHum[0] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->extraHum[0]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[44].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 44 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->extraHum[1] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->extraHum[1]);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[45].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 45 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->extraTemp[0] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->extraTemp[0] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[46].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 46 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->extraTemp[1] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->extraTemp[1] - 90);
        }
        
        field = raddatabaseFieldGet (row, archiveDescription[47].name);
        if (field == NULL)
        {
            radMsgLog (PRI_CATASTROPHIC, "dbdatabaseInsertRecord: FieldGet 47 failed!");
            raddatabaseRowDescriptionDelete (row);
            return DBDB_INSERT_ERROR;
        }
        if (newRecord->extraTemp[2] == 255)
        {
            // no value
            raddatabaseFieldSetIntValue (field, -1);
        }
        else
        {
            raddatabaseFieldSetIntValue (field, (int)newRecord->extraTemp[2] - 90);
        }
    }

        
    //  ... insert the row
    if (raddatabaseTableInsertRow (dbdbWork.dbId, dbdbWork.tableName, row) == ERROR)
    {
        radMsgLog (PRI_MEDIUM, "dbdatabaseInsertRecord: TableInsertRow failed!");
        raddatabaseRowDescriptionDelete (row);
        return DBDB_INSERT_ERROR;
    }
    
    raddatabaseRowDescriptionDelete (row);
    return DBDB_INSERT_OK;
}

