/*---------------------------------------------------------------------------
 
  FILENAME:
        sensor.c
 
  PURPOSE:
        Provide the consensus wind averaging API methods.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        1/02/2006       M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2006, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

//  ... System header files
#include <errno.h>
#include <radmsgLog.h>


//  ... Local header files
#include <sensor.h>


//  ... methods

// read the sensor store file, updating highs and lows as necessary
// assumes the "set" has already been initialized from the archives
int sensorImportSavedData (WV_SENSOR set[STF_MAX][SENSOR_MAX], char *path)
{
    FILE            *inFile;
    char            filename[_MAX_PATH];
    int             stf, type, retVal;
    int             currDay, currMonth, currYear;
    WV_SENSOR       tempSensor;
    struct tm       locTime;
    time_t          ntime = time(NULL);

    localtime_r (&ntime, &locTime);
    currDay = locTime.tm_mday;
    currMonth = locTime.tm_mon;
    currYear = locTime.tm_year;

    sprintf (filename, "%s/%s", path, SENSOR_SAVE_FILENAME);
    inFile = fopen (filename, "rb");
    if (inFile == NULL)
    {
        radMsgLog (PRI_STATUS, "sensorImportSavedData: save file not found...");
        return ERROR;
    }

    // loop through each sensor for day, week, month, year
    for (stf = STF_DAY; stf <= STF_YEAR; stf ++)
    {
        for (type = SENSOR_INTEMP; type <= SENSOR_SOLRAD; type ++)
        {
            if (fread (&tempSensor, 1, sizeof(WV_SENSOR), inFile) 
                != sizeof(WV_SENSOR))
            {
                if (feof (inFile))
                {
                    radMsgLog (PRI_HIGH, "sensorImportSavedData: save file is short!");
                    fclose (inFile);
                    return ERROR;
                }
                else
                {
                    radMsgLog (PRI_HIGH, "sensorImportSavedData: file error: %s",
                               strerror(errno));
                    fclose (inFile);
                    return ERROR;
                }
            }

            // skip week values for now
            if (stf == STF_WEEK)
            {
                continue;
            }

            // check for a valid time frame date
            localtime_r (&tempSensor.time_high, &locTime);
            if (stf == STF_DAY && locTime.tm_mday != currDay)
            {
                continue;
            }
            else if (stf == STF_MONTH && locTime.tm_mon != currMonth)
            {
                continue;
            }
            else if (stf == STF_YEAR && locTime.tm_year != currYear)
            {
                continue;
            }

            // do the updates
            if (tempSensor.low <= set[stf][type].low)
            {
                set[stf][type].low = tempSensor.low;
                set[stf][type].time_low = tempSensor.time_low;
            }
            if (tempSensor.high >= set[stf][type].high)
            {
                set[stf][type].high = tempSensor.high;
                set[stf][type].time_high = tempSensor.time_high;
                set[stf][type].when_high = tempSensor.when_high;
            }
        }
    }

    fclose (inFile);
    return OK;
}

// write the sensor store file
int sensorExportSavedData (WV_SENSOR set[STF_MAX][SENSOR_MAX], char *path)
{
    FILE            *outFile;
    char            filename[_MAX_PATH];
    int             stf, type, retVal;
    WV_SENSOR       tempSensor;

    sprintf (filename, "%s/%s", path, SENSOR_SAVE_FILENAME);
    outFile = fopen (filename, "wb");
    if (outFile == NULL)
    {
        radMsgLog (PRI_STATUS, "sensorExportSavedData: save file open failed!");
        return ERROR;
    }

    // loop through each sensor for day, week, month, year
    for (stf = STF_DAY; stf <= STF_YEAR; stf ++)
    {
        for (type = SENSOR_INTEMP; type <= SENSOR_SOLRAD; type ++)
        {
            if (fwrite (&set[stf][type], 1, sizeof(WV_SENSOR), outFile) 
                != sizeof(WV_SENSOR))
            {
                radMsgLog (PRI_HIGH, "sensorExportSavedData: write error: %s",
                           strerror(errno));
                fclose (outFile);
                return ERROR;
            }
        }
    }

    fclose (outFile);
    return OK;
}

int sensorDeleteSavedData (char *path)
{
    char            filename[_MAX_PATH];

    sprintf (filename, "%s/%s", path, SENSOR_SAVE_FILENAME);
    unlink (filename);
    return OK;
}

void sensorInit (WV_SENSOR *sensor)
{
    memset (sensor, 0, sizeof(*sensor));
    sensor->low = 1000000.0;
    sensor->high = -1000000.0;
    return;
}

void sensorInsert 
(
    WV_SENSOR   *sensor, 
    time_t      ntime, 
    float       value, 
    int         samples, 
    int         seconds
)
{
    if (sensor->low > value)
    {
        sensor->low = value;
        sensor->time_low = ntime;
    }
    if (sensor->high < value)
    {
        sensor->high = value;
        sensor->time_high = ntime;
    }
    sensor->cumulative += (value * samples);
    sensor->samples += samples;
    sensor->seconds += seconds;
    return;
}

void sensorInsertHighLow
(
    WV_SENSOR   *sensor,
    time_t      ntime,
    float       value,
    float       high,
    float       low,
    int         samples, 
    int         seconds
)
{
    if (sensor->low > low)
    {
        sensor->low = low;
        sensor->time_low = ntime;
    }
    if (sensor->high < high)
    {
        sensor->high = high;
        sensor->time_high = ntime;
    }
    sensor->cumulative += (value * samples);
    sensor->samples += samples;
    sensor->seconds += seconds;
    return;
}

void sensorInsertWhen
(
    WV_SENSOR   *sensor, 
    time_t      ntime, 
    float       value, 
    int         samples, 
    int         seconds, 
    float       whenVal
)
{
    if (sensor->low > value)
    {
        sensor->low = value;
        sensor->time_low = ntime;
    }
    if (sensor->high < value)
    {
        sensor->high = value;
        sensor->time_high = ntime;
        sensor->when_high = whenVal;
    }
    sensor->cumulative += (value * samples);
    sensor->samples += samples;
    sensor->seconds += seconds;
    return;
}

void sensorUpdate (WV_SENSOR *sensor, float value, int seconds)
{
    if (sensor->low > value)
    {
        sensor->low = value;
        sensor->time_low = time (NULL);
    }
    if (sensor->high < value)
    {
        sensor->high = value;
        sensor->time_high = time (NULL);
    }
    sensor->cumulative += value;
    sensor->samples ++;
    sensor->seconds += seconds;
    return;
}

void sensorUpdateWhen (WV_SENSOR *sensor, float value, int seconds, float whenVal)
{
    if (sensor->low > value)
    {
        sensor->low = value;
        sensor->time_low = time (NULL);
    }
    if (sensor->high < value)
    {
        sensor->high = value;
        sensor->time_high = time (NULL);
        sensor->when_high = whenVal;
    }
    sensor->cumulative += value;
    sensor->samples ++;
    sensor->seconds += seconds;
    return;
}

void sensorUpdateLowValue (WV_SENSOR *sensor, float value)
{
    sensor->low = value;
}

void sensorUpdateHighValue (WV_SENSOR *sensor, float value)
{
    sensor->high = value;
}

void sensorUpdateWhenHighValue (WV_SENSOR *sensor, float value, float whenVal)
{
    sensor->high = value;
    sensor->when_high = whenVal;
}

void sensorUpdateCumulative (WV_SENSOR *sensor, float value)
{
    sensor->cumulative = value;
}

void sensorAddSample (WV_SENSOR *sensor, WV_SENSOR *sample)
{
    if (sensor->low > sample->low)
    {
        sensor->low = sample->low;
        sensor->time_low = sample->time_low;
    }
    if (sensor->high < sample->high)
    {
        sensor->high = sample->high;
        sensor->when_high = sample->when_high;
        sensor->time_high = sample->time_high;
    }

    sensor->cumulative += sample->cumulative;
    sensor->samples += sample->samples;
    sensor->seconds += sample->seconds;

    return;
}

void sensorPropogateSample (WV_SENSOR *set, WV_SENSOR *sample)
{
    int         type;

    for (type = 0; type < SENSOR_MAX; type ++)
    {
        sensorAddSample (&set[type], &sample[type]);
    }

    return;
}

void sensorClearSet (WV_SENSOR *set)
{
    int         type;

    for (type = 0; type < SENSOR_MAX; type ++)
    {
        sensorInit (&set[type]);
    }

    return;
}

float sensorGetLow (WV_SENSOR *sensor)
{
    return sensor->low;
}

char *sensorGetLowTime (WV_SENSOR *sensor, char *store)
{
    struct tm       locTime;

    if (sensor->time_low == (time_t)0)
    {
        sprintf (store, "-----");
    }
    else
    {
        localtime_r (&sensor->time_low, &locTime);
        sprintf (store, "%2.2d:%2.2d", locTime.tm_hour, locTime.tm_min);
    }
    return store;
}

char *sensorGetLowDate (WV_SENSOR *sensor, char *store)
{
    struct tm       locTime;

    if (sensor->time_low == (time_t)0)
    {
        sprintf (store, "----------");
    }
    else
    {
        localtime_r (&sensor->time_low, &locTime);
        sprintf (store, "%2.2d/%2.2d/%4.4d", 
                 locTime.tm_mon+1, locTime.tm_mday, locTime.tm_year+1900);
    }
    return store;
}

char *sensorGetLowDateMetric (WV_SENSOR *sensor, char *store)
{
    struct tm       locTime;

    if (sensor->time_low == (time_t)0)
    {
        sprintf (store, "--------");
    }
    else
    {
        localtime_r (&sensor->time_low, &locTime);
        sprintf (store, "%4.4d%2.2d%2.2d", 
                 locTime.tm_year+1900, locTime.tm_mon+1, locTime.tm_mday);
    }
    return store;
}

float sensorGetHigh (WV_SENSOR *sensor)
{
    return sensor->high;
}

float sensorGetWhenHigh (WV_SENSOR *sensor)
{
    return sensor->when_high;
}

char *sensorGetHighTime (WV_SENSOR *sensor, char *store)
{
    struct tm       locTime;

    if (sensor->time_high == (time_t)0)
    {
        sprintf (store, "-----");
    }
    else
    {
        localtime_r (&sensor->time_high, &locTime);
        sprintf (store, "%2.2d:%2.2d", locTime.tm_hour, locTime.tm_min);
    }
    return store;
}

char *sensorGetHighDate (WV_SENSOR *sensor, char *store)
{
    struct tm       locTime;

    if (sensor->time_high == (time_t)0)
    {
        sprintf (store, "----------");
    }
    else
    {
        localtime_r (&sensor->time_high, &locTime);
        sprintf (store, "%2.2d/%2.2d/%4.4d", 
                 locTime.tm_mon+1, locTime.tm_mday, locTime.tm_year+1900);
    }
    return store;
}

char *sensorGetHighDateMetric (WV_SENSOR *sensor, char *store)
{
    struct tm       locTime;

    if (sensor->time_high == (time_t)0)
    {
        sprintf (store, "--------");
    }
    else
    {
        localtime_r (&sensor->time_high, &locTime);
        sprintf (store, "%4.4d%2.2d%2.2d", 
                 locTime.tm_year+1900, locTime.tm_mon+1, locTime.tm_mday);
    }
    return store;
}

float sensorGetAvg (WV_SENSOR *sensor)
{
    return (sensor->cumulative/sensor->samples);
}

float sensorGetCumulative (WV_SENSOR *sensor)
{
    return sensor->cumulative;
}

int sensorGetSamples (WV_SENSOR *sensor)
{
    return sensor->samples;
}

int sensorGetSeconds (WV_SENSOR *sensor)
{
    return sensor->seconds;
}

// get a daily low value based on the RTF_INTERVAL and RTF_DAY time frames - 
// assumes input of type "set[STF_MAX][SENSOR_MAX]" and "SENSOR_TYPES"
float sensorGetDailyLow (WV_SENSOR set[STF_MAX][SENSOR_MAX], SENSOR_TYPES type)
{
    float       intVal, dayVal;

    intVal = sensorGetLow (&set[STF_INTERVAL][type]);
    dayVal = sensorGetLow (&set[STF_DAY][type]);
    if (intVal < dayVal)
        return intVal;
    else
        return dayVal;
}

char *sensorGetDailyLowTime (WV_SENSOR set[STF_MAX][SENSOR_MAX], SENSOR_TYPES type, char *store)
{
    float       intVal, dayVal;

    intVal = sensorGetLow (&set[STF_INTERVAL][type]);
    dayVal = sensorGetLow (&set[STF_DAY][type]);
    if (intVal < dayVal)
        return sensorGetLowTime (&set[STF_INTERVAL][type], store);
    else
        return sensorGetLowTime (&set[STF_DAY][type], store);
}

// get a daily high value based on the RTF_INTERVAL and RTF_DAY time frames - 
// assumes input of type "set[STF_MAX][SENSOR_MAX]" and "SENSOR_TYPES"
float sensorGetDailyHigh (WV_SENSOR set[STF_MAX][SENSOR_MAX], SENSOR_TYPES type)
{
    float       intVal, dayVal;

    intVal = sensorGetHigh (&set[STF_INTERVAL][type]);
    dayVal = sensorGetHigh (&set[STF_DAY][type]);
    if (intVal > dayVal)
        return intVal;
    else
        return dayVal;
}

char *sensorGetDailyHighTime (WV_SENSOR set[STF_MAX][SENSOR_MAX], SENSOR_TYPES type, char *store)
{
    float       intVal, dayVal;

    intVal = sensorGetHigh (&set[STF_INTERVAL][type]);
    dayVal = sensorGetHigh (&set[STF_DAY][type]);
    if (intVal > dayVal)
        return sensorGetHighTime (&set[STF_INTERVAL][type], store);
    else
        return sensorGetHighTime (&set[STF_DAY][type], store);
}

float sensorGetDailyWhenHigh (WV_SENSOR set[STF_MAX][SENSOR_MAX], SENSOR_TYPES type)
{
    float       intVal, dayVal;

    intVal = sensorGetHigh (&set[STF_INTERVAL][type]);
    dayVal = sensorGetHigh (&set[STF_DAY][type]);
    if (intVal > dayVal)
        return sensorGetWhenHigh (&set[STF_INTERVAL][type]);
    else
        return sensorGetWhenHigh (&set[STF_DAY][type]);
}

// get a daily cumulative value based on the RTF_INTERVAL and RTF_DAY time frames - 
float sensorGetDailyCumulative (WV_SENSOR set[STF_MAX][SENSOR_MAX], SENSOR_TYPES type)
{
    return (sensorGetCumulative(&set[STF_INTERVAL][type]) + sensorGetCumulative(&set[STF_DAY][type]));
}

// some special purpose utilities
float sensorGetWindRun (WV_SENSOR *sensor)
{
    float       retVal;

    retVal = sensor->cumulative/sensor->samples;
    retVal *= sensor->seconds;
    retVal /= 3600;
    return retVal;
}

