#ifndef INC_sysdefsh
#define INC_sysdefsh
/*---------------------------------------------------------------------------
 
  FILENAME:
        sysdefs.h
 
  PURPOSE:
        Define system definitions.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        08/24/03        M.S. Teel       0               Original
        11/02/2004      K. McGuire      1               ARM Mods and 
                                                        CONS_ARCHIVE_INTERVAL
        01/10/2006      M.S. Teel       2               Update for station
                                                        abstraction

  NOTES:


  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

#include <sys/types.h>
#include <sys/stat.h>

#include "config.h"
#include <radsysdefs.h>
#include <radmsgLog.h>



// define the GLOBAL version string here
#define globalWviewVersionStr       PACKAGE_STRING


// define some global debug macros
#define __DEBUG_BUFFERS             FALSE


// define the radlib system ID for all wview daemons
#define WVIEW_SYSTEM_ID             0x01


// define the standard system file locations
#define WVIEW_CONFIG_DIR            "/etc/wview"
#define WVIEW_RUN_DIR               "/var/wview"


// define the process names, etc.

#define WVD_LOCK_FILE_NAME              "wviewd.pid"
#define HTML_LOCK_FILE_NAME             "htmlgend.pid"
#define FTP_LOCK_FILE_NAME              "wviewftpd.pid"
#define SSH_LOCK_FILE_NAME              "wviewsshd.pid"
#define ALARMS_LOCK_FILE_NAME           "wvalarmd.pid"
#define CWOP_LOCK_FILE_NAME             "wvcwopd.pid"
#define WUNDERGROUND_LOCK_FILE_NAME     "wvwunderd.pid"
#define SQLD_LOCK_FILE_NAME             "wviewsqld.pid"

#define PROC_NAME_DAEMON                "wviewd"
#define PROC_NUM_TIMERS_DAEMON          5
#define WVIEW_CONFIG_FILENAME           "wview.conf"

#define PROC_NAME_HTML                  "htmlgend"
#define PROC_NUM_TIMERS_HTML            2
#define HTML_CONFIG_FILENAME            "htmlgen.conf"

#define PROC_NAME_FTP                   "wviewftpd"
#define PROC_NUM_TIMERS_FTP             1
#define FTP_CONFIG_FILENAME             "wviewftp.conf"

#define PROC_NAME_SSH                   "wviewsshd"
#define PROC_NUM_TIMERS_SSH             1
#define SSH_CONFIG_FILENAME             "wviewssh.conf"

#define PROC_NAME_ALARMS                "wvalarmd"
#define PROC_NUM_TIMERS_ALARMS          0
#define ALARMS_CONFIG_FILENAME          "wvalarm.conf"

#define PROC_NAME_CWOP                  "wvcwopd"
#define PROC_NUM_TIMERS_CWOP            0
#define CWOP_CONFIG_FILENAME            "wvcwop.conf"

#define PROC_NAME_WUNDERGROUND          "wvwunderd"
#define PROC_NUM_TIMERS_WUNDERGROUND    0
#define WUNDERGROUND_CONFIG_FILENAME    "wvwunder.conf"

#define PROC_NAME_SQLD                  "wviewsqld"
#define PROC_NUM_TIMERS_SQLD            2


// define all wview events here for consistency
#define STATION_INIT_COMPLETE_EVENT     0x00010000
#define STATION_LOOP_COMPLETE_EVENT     0x00020000
#define SQLD_SYNC_EVENT                 0x00040000


// some general time definitions
#define WV_SECONDS_IN_HOUR              3600
#define WV_SECONDS_IN_DAY               86400
#define WV_SECONDS_IN_MONTH             (28*WV_SECONDS_IN_DAY)
#define WV_SECONDS_IN_YEAR              (365*WV_SECONDS_IN_DAY)
#define WV_MINUTES_IN_DAY               1440
#define WV_MINUTES_IN_YEAR              (365*WV_MINUTES_IN_DAY)

// packed time utilities
#define EXTRACT_PACKED_YEAR(x)          (((x >> 9) & 0x3F) + 2000)
#define EXTRACT_PACKED_MONTH(x)         ((x >> 5) & 0xF)
#define EXTRACT_PACKED_DAY(x)           (x & 0x1F)
#define EXTRACT_PACKED_HOUR(y)          (y / 100)
#define EXTRACT_PACKED_MINUTE(y)        (y % 100)

#define INSERT_PACKED_DATE(y,m,d)       ((((y-2000) & 0x3F) << 9) | ((m & 0xF) << 5) | (d & 0x1F))


#ifndef _MAX_PATH
#define _MAX_PATH                       512
#endif


// VP Icon bit definitions
#define VP_FCAST_ICON_MAX               0x1F
#define VP_FCAST_ICON_RAIN              0x01
#define VP_FCAST_ICON_CLOUD             0x02
#define VP_FCAST_ICON_PART_SUN          0x04
#define VP_FCAST_ICON_SUNNY             0x08
#define VP_FCAST_ICON_SNOW              0x10


// utility prototypes

// define the verbosity bit definitions
enum _verbosity_bits
{
    WV_VERBOSE_WVIEWD       = 0x01,
    WV_VERBOSE_HTMLGEND     = 0x02,
    WV_VERBOSE_WVALARMD     = 0x04,
    WV_VERBOSE_WVIEWFTPD    = 0x08,
    WV_VERBOSE_WVIEWSSHD    = 0x10,
    WV_VERBOSE_WVCWOPD      = 0x20,
    WV_VERBOSE_WVWUNDERD    = 0x40,
    WV_VERBOSE_ALL          = 0x7F
};
    
extern int  wvutilsSetVerbosity (USHORT daemonBitMask);
extern int wvutilsToggleVerbosity (void);
extern void wvutilsLogEvent (int priority, char *format, ...);

extern float wvutilsCalculateHeatIndex (float temp, float humidity);
extern float wvutilsCalculateWindChill (float temp, float windspeed);
extern float wvutilsCalculateDewpoint (float temp, float humidity);

/****************************************************************************
 * US-to-Metric Converters
 ***************************************************************************/
extern float wvutilsConvertFahrenToCelsius (float fahrenValue);
extern float wvutilsConvertDeltaFahrenToCelsius (float fahrenValue);
extern float wvutilsConvertMphToKph (float mph);
extern float wvutilsConvertBPInchesToMillibars (float inches);
extern float wvutilsConvertRainInchesToCentimeters (float inches);
extern float wvutilsConvertRainInchesToMillimeters (float inches);
extern float wvutilsConvertMilesToKilometers (float miles);
extern float wvutilsConvertFeetToMeters (float feet);

//  Determine if it is day or night based on loop data
//  Returns TRUE or FALSE
extern int wvutilsIsDayTime (short timeSunrise, short timeSunset);

//  return a string indicating the moon phase
extern char *lunarPhaseGet (char *increase, char *decrease, char *full);

//  calculate the air density in kg/m^3
extern float wvutilsCalculateAirDensity (float tempF, float bp, float dp);

//  calculate sea level pressure from station pressure
extern float wvutilsCalculateSLP (float SP, float tempF, float elevationFT);

// convert packed date/time to time_t
extern time_t wvutilsPackedTimeToTimeT (USHORT packedDate, USHORT packedTime);

// calculate the time difference in packed date/time format
// does not consider leap years
// returns the delta in minutes
extern int wvutilsCalculatePackedTimeDelta
(
    USHORT      newDate,
    USHORT      newTime,
    USHORT      oldDate,
    USHORT      oldTime
);

// define sun rise and set types
enum _rise_set_types
{
    RS_TYPE_SUN         = 1,
    RS_TYPE_CIVIL,
    RS_TYPE_ASTRO,
    RS_TYPE_MIDDAY
};

// all times returned as packed time (hour*100 + minutes)
// lat and long are floats representing degrees:
//   N  => +
//   S  => -
//   E  => +
//   W  => -

// Return Values:
//   packedRise = packedSet = -1     =>  the sun never sets
//   packedRise = packedSet = -2     =>  the sun never rises
//   otherwise the valid times in packed format
extern void sunTimesGetSunRiseSet
(
    int         year,
    int         month,
    int         day,
    float       latitude,
    float       longitude,
    int         type,       // RS_TYPE_SUN, RS_TYPE_CIVIL, RS_TYPE_ASTRO, RS_TYPE_MIDDAY
    short       *packedRise,
    short       *packedSet
);

// Get day length in packed format
extern short sunTimesGetDayLength
(
    int         year,
    int         month,
    int         day,
    float       latitude,
    float       longitude
);

// produce a float string fixing the truncation annoyance
extern char *wvutilsPrintFloat (float value, int decPlaces);

#endif

