/*---------------------------------------------------------------------------
 
  FILENAME:
        cwop.c
 
  PURPOSE:
        Provide the wview CWOP packet generator entry point.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        07/12/2005      M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2005, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/*  ... Library include files
*/
#include <radsystem.h>

/*  ... Local include files
*/
#include <cwop.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
static WVIEW_CWOP_WORK cwopWork;

enum ConfigIds
{
    CFG_ID_CALL_SIGN            = 0,
    CFG_ID_APRS_SERVER1         = 1,
    CFG_ID_APRS_PORTNO1         = 2,
    CFG_ID_APRS_SERVER2         = 3,
    CFG_ID_APRS_PORTNO2         = 4,
    CFG_ID_APRS_SERVER3         = 5,
    CFG_ID_APRS_PORTNO3         = 6,
    CFG_ID_LATITUDE             = 7,
    CFG_ID_LONGITUDE            = 8,
    CFG_ID_LOG_WX               = 9
};

static char *configIDs[] =
{
    "APRS_CALL_SIGN",
    "APRS_SERVER1",
    "APRS_PORTNO1",
    "APRS_SERVER2",
    "APRS_PORTNO2",
    "APRS_SERVER3",
    "APRS_PORTNO3",
    "LATITUDE",
    "LONGITUDE",
    "LOG_WX_PACKET"
};

/* ... methods
*/

static void processCWOP (WVIEW_MSG_ARCHIVE_NOTIFY *notify)
{
    RADSOCK_ID          socket;
    time_t              ntime;
    struct tm           *gmTime;
    char                cwopBuffer[256], login[64];
    int                 length = 0;
    char                *serv;
    int                 port;
    
    // format the CWOP data
    ntime = time (NULL);
    gmTime = gmtime (&ntime);
    length = sprintf (cwopBuffer, "%s>APRS,TCPXX*,qAX,%s:@", 
                      cwopWork.callSign, cwopWork.callSign);
    length += sprintf (&cwopBuffer[length], "%2.2d%2.2d%2.2dz",
                       gmTime->tm_mday, gmTime->tm_hour, gmTime->tm_min);
    length += sprintf (&cwopBuffer[length], "%s/%s",
                       cwopWork.latitude, cwopWork.longitude);

    // check for any wind registered
    if (notify->wspeed < 0)
    {
        length += sprintf (&cwopBuffer[length], "_...");
    }
    else
    {
        length += sprintf (&cwopBuffer[length], "_%3.3d", notify->winddir);
    }
    
    length += sprintf (&cwopBuffer[length], "/%3.3d", notify->wspeed);
    length += sprintf (&cwopBuffer[length], "g%3.3d", notify->hiwspeed);

    if (notify->temp < 0)
    {
        if (((notify->temp * (-1)) % 10) >= 5)
        {
            notify->temp -= 10;
        }

        length += sprintf (&cwopBuffer[length], "t-%2.2d", 
                           (notify->temp * (-1))/10);
    }
    else
    {
        if ((notify->temp % 10) >= 5)
        {
            notify->temp += 10;
        }

        length += sprintf (&cwopBuffer[length], "t%3.3d", notify->temp/10);
    }
    
    length += sprintf (&cwopBuffer[length], "P%3.3d", notify->rainDay);
    
    if (notify->humidity >= 0 && notify->humidity <= 100)
    {
        length += sprintf (&cwopBuffer[length], "h%2.2d", notify->humidity % 100);
    }
    
    length += sprintf (&cwopBuffer[length], "b%5.5d", 
                       (int)(10 * wvutilsConvertBPInchesToMillibars ((float)notify->stationPressure/1000.0)));
    sprintf (&cwopBuffer[length], ".wview%c%c%c",
             globalWviewVersionStr[6],
             globalWviewVersionStr[8],
             globalWviewVersionStr[10]);


    // connect to the CWOP server - try the primary then secondary then tertiary
    socket = radSocketClientCreate (cwopWork.server1, cwopWork.portNo1);
    if (socket == NULL)
    {
        wvutilsLogEvent (PRI_MEDIUM, "CWOP-connect: failed to connect to %s:%d",
                         cwopWork.server1, cwopWork.portNo1);
        
        // try the secondary server
        socket = radSocketClientCreate (cwopWork.server2, cwopWork.portNo2);
        if (socket == NULL)
        {
            wvutilsLogEvent (PRI_MEDIUM, "CWOP-connect: failed to connect to %s:%d",
                             cwopWork.server2, cwopWork.portNo2);
        
            // try the tertiary server
            socket = radSocketClientCreate (cwopWork.server3, cwopWork.portNo3);
            if (socket == NULL)
            {
                // we are all out of luck this time!
                wvutilsLogEvent (PRI_MEDIUM, "CWOP-connect: failed to connect to %s:%d",
                                 cwopWork.server3, cwopWork.portNo3);
                radMsgLog (PRI_HIGH, "CWOP-connect: failed to connect to all 3 APRS servers!");
                return;
            }
            else
            {
                serv = cwopWork.server3;
                port = cwopWork.portNo3;
            }
        }
        else
        {
            serv = cwopWork.server2;
            port = cwopWork.portNo2;
        }
    }
    else
    {
        serv = cwopWork.server1;
        port = cwopWork.portNo1;
    }
    
    // wait 1 second ...
    radUtilsSleep (1000);
    
    // transmit the data 
    sprintf (login, "user %6s pass -1 vers %s", cwopWork.callSign, globalWviewVersionStr);
    length = strlen (login);
    login[length] = 0x0D;           // tack on the CR and LF
    login[length+1] = 0x0A;
    login[length+2] = 0;
    
    if (radSocketWriteExact (socket, login, length + 2) != (length + 2))
    {
        radMsgLog (PRI_HIGH, "CWOP-error: %d: failed to login to %s:%d!",
                   errno, serv, port);
        radSocketDestroy (socket);
        return;
    }
    
    // wait 3 seconds ...
    radUtilsSleep (3000);
    
    // write the data record
    wvutilsLogEvent (PRI_STATUS, "CWOP-send: %s", cwopBuffer);

    length = strlen (cwopBuffer);
    cwopBuffer[length] = 0x0D;       // tack on the CR and LF
    cwopBuffer[length+1] = 0x0A;
    cwopBuffer[length+2] = 0;
    if (radSocketWriteExact (socket, cwopBuffer, length + 2) != (length + 2))
    {
        radMsgLog (PRI_HIGH, "CWOP-error: %d: failed to write to %s:%d!",
                   errno, serv, port);
        radSocketDestroy (socket);
        return;
    }

    // wait 3 more seconds ...
    radUtilsSleep (3000);

    // close connection and cleanup
    radSocketDestroy (socket);

    return;
}

static int waitForWviewDaemon (void)
{
    WVIEW_MSG_REQUEST       msg;
    char                    srcQName[QUEUE_NAME_LENGTH+1];
    UINT                    msgType;
    UINT                    length;
    void                    *recvBfr;
    int                     retVal, done = FALSE;

    // enable message reception from the radlib router for the archive path
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_ARCHIVE_PATH);

    msg.requestType = WVIEW_RQST_TYPE_ARCHIVE_PATH;

    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_REQUEST, &msg, sizeof(msg)) == ERROR)
    {
        radMsgLog (PRI_HIGH, "waitForWviewDaemon: radMsgRouterMessageSend failed!");
        radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
        return ERROR;
    }

    // now wait for the response here
    while (!done)
    {
        radUtilsSleep (50);
        
        
        if ((retVal = radQueueRecv (radProcessQueueGetID (),
                                    srcQName,
                                    &msgType,
                                    &recvBfr,
                                    &length))
            == FALSE)
        {
            continue;
        }
        else if (retVal == ERROR)
        {
            radMsgLog (PRI_STATUS, "waitForWviewDaemon: queue is closed!");
            radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
            return ERROR;
        }
    
        // is this what we want?
        if (msgType == WVIEW_MSG_TYPE_ARCHIVE_PATH)
        {
            // yes!
            done = TRUE;
        }
    
        // release the received buffer
        radBufferRls (recvBfr);
        }
    
    // disable message reception from the radlib router for the archive path
    radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);

    return OK;
}

/*  ... system initialization
*/
static int cwopSysInit (WVIEW_CWOP_WORK *work)
{
    char            devPath[256], temp[64];
    struct stat     fileData;

    /*  ... exit if our config file is not there
    */
    sprintf (work->configFile, "%s/%s", WVIEW_CONFIG_DIR, CWOP_CONFIG_FILENAME);
    if (stat (work->configFile, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_CWOP, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "%s does not exist - exiting...\n",
                   work->configFile);
        radMsgLogExit ();
        return ERROR_ABORT;
    }
    
    /*  ... check for our daemon's pid file, don't run if it isn't there
    */
    sprintf (devPath, "%s/%s", WVIEW_RUN_DIR, WVD_LOCK_FILE_NAME);
    if (stat (devPath, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_CWOP, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "wview daemon lock file %s does not exist - aborting!",
                   devPath);
        radMsgLogExit ();
        return ERROR;
    }

    sprintf (work->pidFile, "%s/%s", WVIEW_RUN_DIR, CWOP_LOCK_FILE_NAME);
    sprintf (work->fifoFile, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_CWOP);
    sprintf (work->daemonQname, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_DAEMON);
    sprintf (work->wviewdir, "%s", WVIEW_RUN_DIR);

    /*  ... check for our pid file, don't run if it IS there
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        radMsgLogInit (PROC_NAME_CWOP, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "lock file %s exists, older copy may be running - aborting!",
                   work->pidFile);
        radMsgLogExit ();
        return ERROR;
    }

    return OK;
}

/*  ... system exit
*/
static int cwopSysExit (WVIEW_CWOP_WORK *work)
{
    struct stat     fileData;

    /*  ... delete our pid file
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        unlink (work->pidFile);
    }

    return OK;
}

static void defaultSigHandler (int signum)
{
    int         retVal;

    switch (signum)
    {
        case SIGHUP:
            // user wants us to change the verbosity setting
            retVal = wvutilsToggleVerbosity ();
            radMsgLog (PRI_STATUS, "wvcwopd: SIGHUP - toggling log verbosity %s",
                       ((retVal == 0) ? "OFF" : "ON"));

            signal (signum, defaultSigHandler);
            return;

        case SIGPIPE:
            // we have a far end socket disconnection, we'll handle it in the
            // "read/write" code
            signal (signum, defaultSigHandler);
            break;

        case SIGILL:
        case SIGBUS:
        case SIGFPE:
        case SIGSEGV:
        case SIGXFSZ:
        case SIGSYS:
            // unrecoverable signal - we must exit right now!
            radMsgLog (PRI_CATASTROPHIC, "wvcwopd: recv sig %d: bailing out!", signum);
            abort ();
        
        case SIGCHLD:
            wait (NULL);
            signal (signum, defaultSigHandler);
            return;

        default:
            if (cwopWork.exiting)
            {
                signal (signum, defaultSigHandler);
                return;
            }

            cwopWork.exiting = TRUE;
            radProcessSetExitFlag ();

            // can we allow the process to exit normally?
            if (!cwopWork.inMainLoop)
            {
                // NO! - we gotta bail here!
                radMsgLog (PRI_HIGH, "wvcwopd: recv sig %d: exiting now!", signum);
                radMsgRouterExit ();
                cwopSysExit (&cwopWork);
                radProcessExit ();
                radSystemExit (WVIEW_SYSTEM_ID);
                exit (0);                
            }
            
            // we can allow the process to exit normally...
            radMsgLog (PRI_HIGH, "wvcwopd: recv sig %d: exiting!", signum);
        
            signal (signum, defaultSigHandler);
            break;
    }
    
    return;
}

static void msgHandler
(
    char                        *srcQueueName,
    UINT                        msgType,
    void                        *msg,
    UINT                        length,
    void                        *userData
)
{
    WVIEW_MSG_ARCHIVE_NOTIFY    *anMsg;
    
    if (msgType == WVIEW_MSG_TYPE_ARCHIVE_NOTIFY)
    {
        // process cwop data transfer
        anMsg = (WVIEW_MSG_ARCHIVE_NOTIFY *)msg;
        processCWOP (anMsg);
    }
    
    return;
}

static void evtHandler
(
    UINT        eventsRx,
    UINT        rxData,
    void        *userData
)
{
    return;
}

static void timerHandler (void *parm)
{
    return;
}

/*  ... the main entry point for the wvcwopd process
*/
int main (int argc, char *argv[])
{
    void            (*alarmHandler)(int);
    int             retVal;
    FILE            *pidfile;
    char            instance[32], value[_MAX_PATH];
    CF_ID           configFileId;
    struct stat     fileStatus;

    memset (&cwopWork, 0, sizeof (cwopWork));

    /*  ... initialize some system stuff first
    */
    retVal = cwopSysInit (&cwopWork);
    if (retVal == ERROR)
    {
        radMsgLogInit (PROC_NAME_CWOP, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "wvcwopd init failed!\n");
        radMsgLogExit ();
        exit (1);
    }
    else if (retVal == ERROR_ABORT)
    {
        exit (2);
    }


    /*  ... call the global radlib system init function
    */
    if (radSystemInit (WVIEW_SYSTEM_ID) == ERROR)
    {
        radMsgLogInit (PROC_NAME_CWOP, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "radSystemInit failed!");
        radMsgLogExit ();
        exit (1);
    }


    /*  ... call the radlib process init function
    */
    if (radProcessInit (PROC_NAME_CWOP,
                        cwopWork.fifoFile,
                        PROC_NUM_TIMERS_CWOP,
                        TRUE,                      // TRUE for daemon
                        msgHandler,
                        evtHandler,
                        NULL)
        == ERROR)
    {
        printf ("\nradProcessInit failed: %s\n\n", PROC_NAME_CWOP);
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    cwopWork.myPid = getpid ();
    pidfile = fopen (cwopWork.pidFile, "w");
    if (pidfile == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "lock file create failed!\n");
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    fprintf (pidfile, "%d", getpid ());
    fclose (pidfile);


    alarmHandler = radProcessSignalGetHandler (SIGALRM);
    radProcessSignalCatchAll (defaultSigHandler);
    radProcessSignalCatch (SIGALRM, alarmHandler);


    // get the wview verbosity setting
    if (wvutilsSetVerbosity (WV_VERBOSE_WVCWOPD) == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "wvutilsSetVerbosity failed!\n");
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    /*  ... get our configuration values
    */
    if (stat (cwopWork.configFile, &fileStatus) == -1)
    {
        radMsgLog (PRI_CATASTROPHIC, "config file %s missing - use wviewconfig to generate one...\n", 
                   cwopWork.configFile);
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    configFileId = radCfOpen (cwopWork.configFile);
    if (configFileId == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfOpen failed!\n");
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    // get the APRS call sign
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_CALL_SIGN],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s failed!", configIDs[CFG_ID_CALL_SIGN]);
        radCfClose (configFileId);
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        strcpy (cwopWork.callSign, value);
    }

    // get the primary APRS server
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_APRS_SERVER1],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s failed!", configIDs[CFG_ID_APRS_SERVER1]);
        radCfClose (configFileId);
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        strcpy (cwopWork.server1, value);
    }

    // get the primary APRS port number
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_APRS_PORTNO1],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s failed!", configIDs[CFG_ID_APRS_PORTNO1]);
        radCfClose (configFileId);
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        cwopWork.portNo1 = atoi (value);
    }
    // get the secondary APRS server
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_APRS_SERVER2],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s failed!", configIDs[CFG_ID_APRS_SERVER2]);
        radCfClose (configFileId);
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        strcpy (cwopWork.server2, value);
    }

    // get the secondary APRS port number
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_APRS_PORTNO2],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s failed!", configIDs[CFG_ID_APRS_PORTNO2]);
        radCfClose (configFileId);
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        cwopWork.portNo2 = atoi (value);
    }
    // get the tertiary APRS server
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_APRS_SERVER3],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s failed!", configIDs[CFG_ID_APRS_SERVER3]);
        radCfClose (configFileId);
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        strcpy (cwopWork.server3, value);
    }

    // get the tertiary APRS port number
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_APRS_PORTNO3],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s failed!", configIDs[CFG_ID_APRS_PORTNO3]);
        radCfClose (configFileId);
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        cwopWork.portNo3 = atoi (value);
    }

    // get the fine pitch latitude that APRS requires
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_LATITUDE],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s failed!", configIDs[CFG_ID_LATITUDE]);
        radCfClose (configFileId);
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        strcpy (cwopWork.latitude, value);
    }

    // get the fine pitch longitude that APRS requires
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_LONGITUDE],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s failed!", configIDs[CFG_ID_LONGITUDE]);
        radCfClose (configFileId);
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        strcpy (cwopWork.longitude, value);
    }

    // get the WX packet logging preference
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_LOG_WX],
                            instance,
                            value)
        == ERROR)
    {
        // just disable it
        cwopWork.logWXPackets = 0;
    }
    else
    {
        cwopWork.logWXPackets = atoi (value);
    }

    radCfClose (configFileId);

    radMsgLog (PRI_STATUS, "CWOP: configured to submit station %s data to:",
               cwopWork.callSign);
    radMsgLog (PRI_STATUS, "CWOP: Primary:   %s:%d", cwopWork.server1, cwopWork.portNo1);
    radMsgLog (PRI_STATUS, "CWOP: Secondary: %s:%d", cwopWork.server2, cwopWork.portNo2);
    radMsgLog (PRI_STATUS, "CWOP: Tertiary:  %s:%d", cwopWork.server3, cwopWork.portNo3);


    // wait a bit here before continuing
    radUtilsSleep (500);


    //  register with the radlib message router
    if (radMsgRouterInit (WVIEW_RUN_DIR) == ERROR)
    {
        radMsgLog (PRI_HIGH, "radMsgRouterInit failed!");
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    
    // wait for the wview daemon to be ready
    if (waitForWviewDaemon () == ERROR)
    {
        radMsgLog (PRI_HIGH, "waitForWviewDaemon failed");
        radMsgRouterExit ();
        cwopSysExit (&cwopWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }


    // enable message reception from the radlib router for archive notifications
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_ARCHIVE_NOTIFY);


    // enter normal processing
    cwopWork.inMainLoop = TRUE;
    radMsgLog (PRI_STATUS, "running...");
    
    while (!cwopWork.exiting)
    {
        // wait on something interesting
        if (radProcessWait (0) == ERROR)
        {
            cwopWork.exiting = TRUE;
        }
    }


    radMsgLog (PRI_STATUS, "exiting normally...");

    radMsgRouterExit ();
    cwopSysExit (&cwopWork);
    radProcessExit ();
    radSystemExit (WVIEW_SYSTEM_ID);
    exit (0);
}

