/*---------------------------------------------------------------------------
 
  FILENAME:
        ftp.c
 
  PURPOSE:
        Provide the wview ftp generator entry point.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        01/10/04        M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/*  ... Library include files
*/
#include <radsystem.h>

/*  ... Local include files
*/
#include <ftp.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
static FTP_WORK         ftpWork;


/* ... methods
*/
/*  ... system initialization
*/
static int ftpSysInit (FTP_WORK *work)
{
    char            devPath[256], temp[64];
    struct stat     fileData;

    /*  ... exit if our config file is not there
    */
    sprintf (work->configFile, "%s/%s", WVIEW_CONFIG_DIR, FTP_CONFIG_FILENAME);
    if (stat (work->configFile, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_FTP, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "%s does not exist - exiting...\n",
                   work->configFile);
        radMsgLogExit ();
        return ERROR_ABORT;
    }
    
    /*  ... check for our daemon's pid file, don't run if it isn't there
    */
    sprintf (devPath, "%s/%s", WVIEW_RUN_DIR, WVD_LOCK_FILE_NAME);
    if (stat (devPath, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_FTP, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "wview daemon lock file %s does not exist - aborting!",
                   devPath);
        radMsgLogExit ();
        return ERROR;
    }

    sprintf (work->pidFile, "%s/%s", WVIEW_RUN_DIR, FTP_LOCK_FILE_NAME);
    sprintf (work->fifoFile, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_FTP);
    sprintf (work->daemonQname, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_DAEMON);
    sprintf (work->wviewdir, "%s", WVIEW_RUN_DIR);

    /*  ... check for our pid file, don't run if it IS there
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        radMsgLogInit (PROC_NAME_FTP, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "lock file %s exists, older copy may be running - aborting!",
                   work->pidFile);
        radMsgLogExit ();
        return ERROR;
    }

    return OK;
}

/*  ... system exit
*/
static int ftpSysExit (FTP_WORK *work)
{
    struct stat     fileData;

    /*  ... delete our pid file
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        unlink (work->pidFile);
    }

    return OK;
}


static void defaultSigHandler (int signum)
{
    int         retVal;

    switch (signum)
    {
        case SIGHUP:
            // user wants us to change the verbosity setting
            retVal = wvutilsToggleVerbosity ();
            radMsgLog (PRI_STATUS, "wviewftpd: SIGHUP - toggling log verbosity %s",
                       ((retVal == 0) ? "OFF" : "ON"));

            signal (signum, defaultSigHandler);
            return;

        case SIGBUS:
        case SIGFPE:
        case SIGSEGV:
        case SIGXFSZ:
        case SIGSYS:
            // unrecoverable signal - we must exit right now!
            radMsgLog (PRI_CATASTROPHIC, "wviewftpd: recv sig %d: shutting down!", signum);
            abort ();
        
        case SIGCHLD:
            // it is normal behavior to have children finishing up
            wait (NULL);
            signal (signum, defaultSigHandler);
            break;

        default:
            // we can allow the process to exit normally...
            if (ftpWork.exiting)
            {
                signal (signum, defaultSigHandler);
                return;
            }
        
            radMsgLog (PRI_CATASTROPHIC, "wviewftpd: recv sig %d: exiting!", signum);
        
            ftpWork.exiting = TRUE;
            radProcessSetExitFlag ();
        
            signal (signum, defaultSigHandler);
            break;
    }

    return;
}

static void msgHandler
(
    char        *srcQueueName,
    UINT        msgType,
    void        *msg,
    UINT        length,
    void        *userData
)
{
    return;
}

static void evtHandler
(
    UINT        eventsRx,
    UINT        rxData,
    void        *userData
)
{
    return;
}

static void timerHandler (void *parm)
{
    ULONG       msOffset = radTimeGetMSSinceEpoch ();
    int         rules;
    long        netOffset;

    //  ... allow for timer latency
    if (ftpWork.msOffset == 0)
    {
        // first time through
        ftpWork.msOffset = msOffset;
        netOffset = 0;
    }
    else
    {
        netOffset = msOffset - ftpWork.msOffset;
    }

    ftpWork.msOffset += 60000;                  // ALWAYS 1 minute

    radProcessTimerStart (ftpWork.timer, 60000 - netOffset);


    //  ... process the ftp rules
    rules = ftpUtilsSendFiles (ftpWork.ftpId, ftpWork.wviewdir);

    if (rules > 0)
    {
        wvutilsLogEvent (PRI_STATUS, "FTP: %d rules processed", rules);
    }

    return;
}


/*  ... the main entry point for the ftp process
*/
int main (int argc, char *argv[])
{
    void            (*alarmHandler)(int);
    STIM            stim;
    int             i;
    int             seconds;
    time_t          ntime;
    struct tm       locTime;
    int             offset, retVal;
    long            msOffset;
    FILE            *pidfile;

    memset (&ftpWork, 0, sizeof (ftpWork));

    /*  ... initialize some system stuff first
    */
    retVal = ftpSysInit (&ftpWork);
    if (retVal == ERROR)
    {
        radMsgLogInit (PROC_NAME_FTP, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "ftp init failed!\n");
        radMsgLogExit ();
        exit (1);
    }
    else if (retVal == ERROR_ABORT)
    {
        exit (2);
    }


    /*  ... call the global radlib system init function
    */
    if (radSystemInit (WVIEW_SYSTEM_ID) == ERROR)
    {
        radMsgLogInit (PROC_NAME_FTP, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "radSystemInit failed!");
        radMsgLogExit ();
        exit (1);
    }


    /*  ... call the radlib process init function
    */
    if (radProcessInit (PROC_NAME_FTP,
                        ftpWork.fifoFile,
                        PROC_NUM_TIMERS_FTP,
                        TRUE,                      // TRUE for daemon
                        msgHandler,
                        evtHandler,
                        NULL)
        == ERROR)
    {
        printf ("\nradProcessInit failed: %s\n\n", PROC_NAME_FTP);
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    ftpWork.myPid = getpid ();
    pidfile = fopen (ftpWork.pidFile, "w");
    if (pidfile == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "lock file create failed!\n");
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    fprintf (pidfile, "%d", getpid ());
    fclose (pidfile);


    alarmHandler = radProcessSignalGetHandler (SIGALRM);
    radProcessSignalCatchAll (defaultSigHandler);
    radProcessSignalCatch (SIGALRM, alarmHandler);


    // get the wview verbosity setting
    if (wvutilsSetVerbosity (WV_VERBOSE_WVIEWFTPD) == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "wvutilsSetVerbosity failed!\n");
        ftpSysExit (&ftpWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    

    ftpWork.timer = radTimerCreate (NULL, timerHandler, NULL);
    if (ftpWork.timer == NULL)
    {
        radMsgLog (PRI_HIGH, "radTimerCreate failed");
        ftpSysExit (&ftpWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    //  ... initialize the ftp utilities
    ftpWork.ftpId = ftpUtilsInit (ftpWork.configFile);
    if (ftpWork.ftpId == NULL)
    {
        radMsgLog (PRI_HIGH, "ftpUtilsInit failed");
        radTimerDelete (ftpWork.timer);
        ftpSysExit (&ftpWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }


    //  ... start THE timer
    ntime = time (NULL);
    localtime_r (&ntime, &locTime);
    seconds = locTime.tm_sec - 15;         // start at 15 secs past

    /*  ... start the ftp timer to go off 1 min past the next 5 minute mark
    */
    offset = locTime.tm_min % 5;
    if (offset)
        offset = 6 - offset;
    else
        offset = 1;

    radMsgLog (PRI_HIGH,
               "starting ftp timer for %d mins %d secs",
               ((seconds > 0) ? ((offset > 0) ? offset-1 : offset) : offset),
                       (seconds > 0) ? 60 - seconds : (-1) * seconds);

    msOffset = radTimeGetMSSinceEpoch () % 1000;
    msOffset -= 250;                        // land on 250 ms mark


    ftpWork.msOffset = 0;
    radProcessTimerStart (ftpWork.timer,
                          ((((offset * 60) - seconds) * 1000)) - msOffset);



    while (!ftpWork.exiting)
    {
        /*  ... wait on timers, events, file descriptors, msgs, everything!
        */
        if (radProcessWait (0) == ERROR)
        {
            ftpWork.exiting = TRUE;
        }
    }


    radMsgLog (PRI_STATUS, "exiting normally...");

    if (ftpWork.ftpId != NULL)
    {
        ftpUtilsExit (ftpWork.ftpId);
    }

    radTimerDelete (ftpWork.timer);
    ftpSysExit (&ftpWork);
    radProcessExit ();
    radSystemExit (WVIEW_SYSTEM_ID);
    exit (0);
}

