/*---------------------------------------------------------------------------
 
  FILENAME:
        ftpUtils.c
 
  PURPOSE:
        Provide the ftp utilities.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        01/08/04        M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/*  ... Library include files
*/
#include <radsysutils.h>

/*  ... Local include files
*/
#include <ftpUtils.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/



FTP_ID ftpUtilsInit (char *confFilePath)
{
    FTP_RULE_ID rule;
    FTP_ID      newId;
    FILE        *file;
    char        temp[1536];
    char        *token;
    int         i;
    FileStates  state = FS_HOST;

    newId = (FTP_ID)malloc (sizeof (*newId));
    if (newId == NULL)
    {
        return NULL;
    }

    strcpy (newId->confFile, confFilePath);

    radListReset (&newId->rules);

    file = fopen (confFilePath, "r");
    if (file == NULL)
    {
        free (newId);
        return NULL;
    }

    while (fgets (temp, 1536, file) != NULL)
    {
        if (temp[0] == ' ' || temp[0] == '\n' || temp[0] == '#')
        {
            // comment or whitespace
            continue;
        }

        token = strtok (temp, " \t");
        if (token == NULL)
        {
            continue;
        }

        switch (state)
        {
        case FS_HOST:
            if (strcmp (token, "host"))
                continue;

            token = strtok (NULL, " \t\n");
            if (token == NULL)
            {
                continue;
            }

            strcpy (newId->host, token);
            state = FS_USER;
            break;

        case FS_USER:
            if (strcmp (token, "user"))
                continue;

            token = strtok (NULL, " \t\n");
            if (token == NULL)
            {
                continue;
            }

            strcpy (newId->user, token);
            state = FS_PASS;
            break;

        case FS_PASS:
            if (strcmp (token, "pass"))
                continue;

            token = strtok (NULL, " \t\n");
            if (token == NULL)
            {
                continue;
            }

            strcpy (newId->pass, token);
            state = FS_DIRECTORY;
            break;

        case FS_DIRECTORY:
            if (isdigit((int)token[0]))
            {
                // no directory entry given, set to empty and start rule processing
                radMsgLog (PRI_MEDIUM, "FTP: INIT: no relative transfer path given - "
                           "defaulting to your FTP login directory...");
                newId->directory[0] = 0;

                // now fall through to FS_RULE state...
                state = FS_RULE;
            }
            else
            {
                if (strcmp (token, "directory"))
                    continue;

                token = strtok (NULL, " \t\n");
                if (token == NULL)
                {
                    continue;
                }
    
                strcpy (newId->directory, token);
                state = FS_RULE;
                break;
            }

        case FS_RULE:
            rule = (FTP_RULE_ID)malloc (sizeof (*rule));
            if (rule == NULL)
            {
                for (rule = (FTP_RULE_ID)radListRemoveFirst (&newId->rules);
                     rule != NULL;
                     rule = (FTP_RULE_ID)radListRemoveFirst (&newId->rules))
                {
                    free (rule);
                }

                free (newId);
                return NULL;
            }
            rule->interval = atoi (token);
            rule->iteration = rule->interval - 1;       // make run first time

            token = strtok (NULL, " \t\n");
            if (token == NULL)
            {
                free (rule);
                continue;
            }
            strcpy (rule->src, token);

            radListAddToEnd (&newId->rules, (NODE_PTR)rule);
            break;
        }
    }

    fclose (file);

    if (state != FS_RULE)
    {
        radMsgLog (PRI_MEDIUM, "FTP: INIT: missing parameters in %s - check it against the distro example!",
                   confFilePath);
        return NULL;
    }

    radMsgLog (PRI_MEDIUM, "FTP: INIT: %d rules added",
               radListGetNumberOfNodes (&newId->rules));

    return newId;
}

int ftpUtilsSendFiles (FTP_ID id, char *workdir)
{
    FTP_RULE_ID     rule;
    int             retVal, numRules = 0;
    char            *args[5 + radListGetNumberOfNodes(&id->rules)];
    char            url[256];

    args[0] = "/usr/bin/ftp";
    args[1] = "-AiV";
    args[2] = "-u";
    if (strlen(id->directory) == 0)
    {
        sprintf (url, "ftp://%s:%s@%s/",
                 id->user, id->pass, id->host);
    }
    else
    {
        sprintf (url, "ftp://%s:%s@%s/%s/",
                 id->user, id->pass, id->host, id->directory);
    }
    args[3] = url;

    for (rule = (FTP_RULE_ID)radListGetFirst (&id->rules);
            rule != NULL;
            rule = (FTP_RULE_ID)radListGetNext (&id->rules, (NODE_PTR)rule))
    {
        // is it time to send this file?
        if (++rule->iteration != rule->interval)
        {
            // nope!
            continue;
        }

        rule->iteration = 0;

        args[4+numRules] = rule->src;
        numRules ++;
    }

    if (numRules == 0)
    {
        // nothing to do!
        return 0;
    }

    args[4+numRules] = 0;

    retVal = fork ();
    if (retVal == -1)
    {
        radMsgLog (PRI_MEDIUM, "ftpUtilsSendFiles: fork failed");
    }
    else if (retVal == 0)
    {
        // we are the child!
        chdir (workdir);
        execv ("/usr/bin/ftp", args);
        exit (0);
    }

    return numRules;
}


void ftpUtilsExit (FTP_ID id)
{
    FTP_RULE_ID rule;

    for (rule = (FTP_RULE_ID)radListRemoveFirst (&id->rules);
         rule != NULL;
         rule = (FTP_RULE_ID)radListRemoveFirst (&id->rules))
    {
        free (rule);
    }

    free (id);
}


