/*---------------------------------------------------------------------------
 
  FILENAME:
        glchart.c
 
  PURPOSE:
        Provide the graphics lib chart utilities.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        09/05/03        M.S. Teel       0               Original
        01/02/05        M.S. Teel       1               Improve labeling and
                                                        chart limits calc

  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>

/*  ... Library include files
*/
#include <sysdefs.h>
#include <radmsgLog.h>
#include "gd.h"
#include "gdfonts.h"
#include "gdfontt.h"
#include "gdfontmb.h"
#include "gdfontg.h"
#include "gdfontl.h"


/*  ... Local include files
*/
#include <glchart.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
#define MAX_STEP_MULTIPLIER         9

static CHART nonReentrantChart;

static double stepSizeMultipliers[MAX_STEP_MULTIPLIER] =
    {
        2.0, 5.0, 10.0, 20.0, 50.0, 100.0, 200.0, 500.0, 1000.0
    };



static int createImage (CHART_ID id)
{
    if (id->height < (10 * gdFontMediumBold->h))
    {
        return -1;
    }

    if (id->im)
    {
        gdImageDestroy (id->im);
    }

    id->im = gdImageCreate (id->width, id->height);
    if (id->im == NULL)
    {
        return -1;
    }

    //  ... allocate our colors
    id->bgcolor = gdImageColorAllocate (id->im,
                                        GLC_RED(id->bgcolor),
                                        GLC_GREEN(id->bgcolor),
                                        GLC_BLUE(id->bgcolor));
    id->chartcolor = gdImageColorAllocate (id->im,
                                           GLC_RED(id->chartcolor),
                                           GLC_GREEN(id->chartcolor),
                                           GLC_BLUE(id->chartcolor));
    id->gridcolor = gdImageColorAllocate (id->im,
                                          GLC_RED(id->gridcolor),
                                          GLC_GREEN(id->gridcolor),
                                          GLC_BLUE(id->gridcolor));
    id->linecolor = gdImageColorAllocate (id->im,
                                          GLC_RED(id->linecolor),
                                          GLC_GREEN(id->linecolor),
                                          GLC_BLUE(id->linecolor));
    id->titleFGcolor = gdImageColorAllocate (id->im,
                                             GLC_RED(id->titleFGcolor),
                                             GLC_GREEN(id->titleFGcolor),
                                             GLC_BLUE(id->titleFGcolor));
    id->titleBGcolor = gdImageColorAllocate (id->im,
                                             GLC_RED(id->titleBGcolor),
                                             GLC_GREEN(id->titleBGcolor),
                                             GLC_BLUE(id->titleBGcolor));
    id->textcolor = gdImageColorAllocate (id->im,
                                          GLC_RED(id->textcolor),
                                          GLC_GREEN(id->textcolor),
                                          GLC_BLUE(id->textcolor));

    return 0;
}


static void drawGrid (CHART_ID id)
{
    register int    i;
    int             xlabelwidth = 0, ylabelwidth = 0;
    double          units, step;
    int             dataPoints, xnumhash, xhashindexlen;
    double          hashwidth, xhashwidth;
    int             numhash, x, y, temp1, temp2;
    char            text[64][32], ylabelfmt[24];

    //  build labels and calculate the maximum x-label width
    for (i = 0; i < id->numpoints; i ++)
    {
        if (strlen (id->pointnames[i]) > xlabelwidth)
            xlabelwidth = strlen (id->pointnames[i]);
    }

    //  calculate the maximum number of hash marks and the units per hash
    //  for the x-axis
    xhashwidth = (xlabelwidth*gdFontSmall->h)/2;
    dataPoints = id->numpoints - 1;
    
   // see if a suggestion was given
    if (id->xnumhashes != 0)
    {
        xnumhash = id->xnumhashes;
    }
    else
    {
        xnumhash =  (id->imbx - id->imtx)/xhashwidth;
    
        // try to normalize the number of hashmarks
        // try with the endpoint removed
        for (i = 1; i < dataPoints/2; i ++)
        {
            if (dataPoints % i != 0)
                continue;
    
            if (dataPoints/i <= xnumhash)
            {
                xnumhash = dataPoints/i;
                break;
            }
        }
    }
    
    xhashwidth = (double)(id->imbx - id->imtx)/(double)xnumhash;
    id->pointpixels = (double)(id->imbx - id->imtx)/(double)dataPoints;
    xhashindexlen = dataPoints/xnumhash;


    //  ... calculate the maximum number of hash marks and the units per hash
    //  ... for the y-axis
    hashwidth = (gdFontSmall->h * 3)/2;

    numhash = (id->imby - id->imty)/hashwidth;
    if (numhash > 64)
        numhash = 64;

    units = (id->max - id->min)/id->ystepSize;
    step = id->ystepSize;
    i = 0;
    while ((int)units > numhash && i < MAX_STEP_MULTIPLIER)
    {
        step = id->ystepSize * stepSizeMultipliers[i];
        units = (id->max - id->min)/step;
        i ++;
    }

    id->ystepSize = step;
    numhash = (int)units;
    
    temp1 = (int)(id->max * 10000);
    temp2 = (int)((id->min + (id->ystepSize * numhash)) * 10000);
    temp2 ++;
    
    if (temp2 < temp1)
    {
        numhash ++;
    }

    hashwidth = (double)(id->imby - id->imty)/(double)numhash;
    id->max = id->min + (id->ystepSize * numhash);
    id->ypixelconstant = (double)(id->imby - id->imty)/(double)(id->max - id->min);
    numhash ++;


    //  build labels and calculate the maximum y-label width
    sprintf (ylabelfmt, "%%.%1.1df", id->ydecPlaces);
    for (i = 0; i < numhash; i ++)
    {
        sprintf (text[i], ylabelfmt, id->min + (i * id->ystepSize));

        if (strlen (text[i]) > ylabelwidth)
            ylabelwidth = strlen (text[i]);
    }

    //  now we know the chart area, draw it
    gdImageFilledRectangle (id->im,
                            id->imtx - GLC_CONTENTS_OFFSET,
                            id->imty - GLC_CONTENTS_OFFSET,
                            id->imbx + GLC_CONTENTS_OFFSET,
                            id->imby + GLC_CONTENTS_OFFSET,
                            id->chartcolor);

    gdImageRectangle (id->im,
                      id->imtx,
                      id->imty,
                      id->imbx,
                      id->imby,
                      id->gridcolor);

    //  draw the y-axis
    gdImageSetThickness (id->im, 1);
    for (i = 0; i < numhash; i ++)
    {
        y = id->imby - (i * hashwidth);

        gdImageLine (id->im,
                     id->imtx,
                     y,
                     id->imbx,
                     y,
                     id->gridcolor);

        y -= gdFontSmall->h/2;
        gdImageString (id->im,
                       gdFontSmall,
                       (id->imtx - 5) - (strlen(text[i]) * gdFontSmall->h/2),
                       y,
                       (UCHAR *)text[i],
                       id->textcolor);
    }

    //  draw the x-axis
    gdImageSetThickness (id->im, 1);

    for (i = 0; i <= xnumhash; i ++)            // one more for right-most
    {
        int         dataPoint = i * xhashindexlen;

        x = id->imtx + (i * xhashwidth);

        gdImageLine (id->im,
                     x,
                     id->imty,
                     x,
                     id->imby,
                     id->gridcolor);

        if (strlen(id->pointnames[dataPoint]) < 2)
            x -= gdFontSmall->h/4;

        gdImageString (id->im,
                       gdFontSmall,
                       x - ((gdFontSmall->h/2) *
                            (strlen(id->pointnames[dataPoint])/2)),
                       (id->height - (3 * gdFontMediumBold->h)) + 4,
                       (UCHAR *)id->pointnames[dataPoint],
                       id->textcolor);
    }


    //  finally, draw date
    gdImageString (id->im,
                   gdFontSmall,
                   ((id->width - ((gdFontSmall->h/2) * strlen (id->datetime)))/2) - 2,
                   id->height - (gdFontMediumBold->h + 2),
                   (UCHAR *)id->datetime,
                   id->textcolor);
}

static void drawTitle (CHART_ID id)
{
    gdImageFilledRectangle (id->im,
                            0, 0,
                            id->width, gdFontMediumBold->h,
                            id->titleBGcolor);

    gdImageString (id->im,
                   gdFontMediumBold,
                   2,
                   0,
                   (UCHAR *)id->title,
                   id->linecolor);

    gdImageString (id->im,
                   gdFontMediumBold,
                   id->width - (((gdFontMediumBold->h/2)+2) * strlen (id->units)),
                   0,
                   (UCHAR *)id->units,
                   id->titleFGcolor);
}

static void drawLine (CHART_ID id)
{
    register int    i;
    int             x1, x2, y1, y2;

    gdImageSetThickness (id->im, 2);
    for (i = 0; i < id->numpoints - 1; i ++)
    {
        if (id->valueset[i] <= GLC_VALUE_NONE ||
            id->valueset[i+1] <= GLC_VALUE_NONE)
        {
            continue;
        }

        x1 = id->imtx + (int)((double)i * id->pointpixels);
        x2 = id->imtx + (int)((double)(i+1) * id->pointpixels);
        y1 = id->imby - (int)(id->ypixelconstant * (id->valueset[i] - id->min));
        y2 = id->imby - (int)(id->ypixelconstant * (id->valueset[i+1] - id->min));
        gdImageLine (id->im,
                     x1,
                     y1,
                     x2,
                     y2,
                     id->linecolor);
    }
}


static void drawBars (CHART_ID id)
{
    register int    i;
    int             x1, x2, y1, y2;

    for (i = 0; i < id->numpoints - 1; i ++)
    {
        if (id->valueset[i] == GLC_VALUE_NONE)
        {
            continue;
        }

        x1 = id->imtx + (int)((double)i * id->pointpixels);
        x2 = id->imtx + (int)((double)(i+1) * id->pointpixels);
        x1 += 1;
        x2 -= 1;
        y1 = id->imby - (int)(id->ypixelconstant * (id->valueset[i] - id->min));
        y2 = id->imby;
        gdImageFilledRectangle (id->im,
                                x1, y1,
                                x2, y2,
                                id->linecolor);
    }
}

static void normalizeMinMax (CHART_ID id, double newMIN, double newMAX)
{
    register double     temp, factor = 1.0;
    register int        i;
    
    // compute factor based on number of decimal places defined
    // (avoid use of pow function)
    for (i = 0; i < id->ydecPlaces; i ++)
    {
        factor *= 10.0;
    }

    // determine chart min
    id->min = newMIN;
    id->min *= factor;
    
    // make sure we don't lop off decimal places "below" since we are 
    // computing a MIN here
    temp = (int)id->min;
    if (temp - id->min > 0)
        temp -= 1.0;
    id->min = temp;
    while (((int)id->min % 5) != 0)
    {
        id->min -= 1.0;
    }
    id->min /= factor;
    
    // determine chart max
    id->max = newMAX;
    id->max *= factor;
    id->max *= 10;
    id->max = (int)id->max;
    id->max /= 10;

    // make sure we don't lop off decimal places "above" since we are 
    // computing a MAX here
    temp = (int)id->max;
    if (id->max - temp > 0)
        temp += 1.0;
    id->max = temp;
    while (((int)id->max % 5) != 0)
    {
        id->max += 1.0;
    }
    id->max /= factor;
    
    return;
}



/*  ... API methods
*/

CHART_ID chartCreate 
(
    int                 width, 
    int                 height, 
    char                *title, 
    char                *units, 
    int                 isBarChart
)
{
    register CHART_ID   newId;

    newId = (CHART_ID)&nonReentrantChart;

    memset (newId, 0, sizeof (*newId));

    newId->isBarChart = isBarChart;
    newId->width = width;
    newId->height = height;
    strcpy (newId->title, title);
    strcpy (newId->units, units);

    //  ... now set the contents coords
    newId->imtx = 5 * gdFontSmall->h/2;
    //  ... SSM: move the right edge of the graph 5 pixels left to make
    //  ...      room for last label
    newId->imbx = (newId->width - 1) - gdFontMediumBold->h - 5;
    newId->imty = 2 * gdFontMediumBold->h;
    newId->imby = newId->height - (3 * gdFontMediumBold->h);

    newId->bgcolor      = GLC_DFLT_BG;
    newId->chartcolor   = GLC_DFLT_CHART;
    newId->gridcolor    = GLC_DFLT_GRID;
    newId->linecolor    = GLC_DFLT_LINE;
    newId->titleFGcolor = GLC_DFLT_TITLEFG;
    newId->titleBGcolor = GLC_DFLT_TITLEBG;
    newId->textcolor    = GLC_DFLT_TEXT;

    return newId;
}

void chartSetXScale (CHART_ID id, int decPlaces)
{
    id->xdecPlaces = decPlaces;

    return;
}

void chartSetXHashes (CHART_ID id, int numHashes)
{
    id->xnumhashes = numHashes;
    
    return;
}

void chartSetYScale (CHART_ID id, double min, double max, double step, int decPlaces)
{
    id->min = min;
    id->max = max;
    id->ystepSize = step;
    id->ydecPlaces = decPlaces;

    normalizeMinMax (id, min, max);
    return;
}

void chartAddPoint (CHART_ID id, double value, char *name)
{
    if (id->numpoints >= MAX_GRAPH_POINTS)
    {
        return;
    }

    id->valueset[id->numpoints] = value;
    strncpy (id->pointnames[id->numpoints], name, 16);
    id->numpoints ++;

    if (value <= GLC_VALUE_NONE)
    {
        return;
    }

    if (value < id->min)
        normalizeMinMax (id, value, id->max);
    else if (value > id->max)
        normalizeMinMax (id, id->min, value);

    return;
}

void chartSetDateTime (CHART_ID id, char *datetime)
{
    strcpy (id->datetime, datetime);
    return;
}

void chartSetBGColor (CHART_ID id, int color)
{
    id->bgcolor = color;
    return;
}

void chartSetChartColor (CHART_ID id, int color)
{
    id->chartcolor = color;
    return;
}

void chartSetGridColor (CHART_ID id, int color)
{
    id->gridcolor = color;
    return;
}

void chartSetLineColor (CHART_ID id, int color)
{
    id->linecolor = color;
    return;
}

void chartSetTitleColors (CHART_ID id, int fg, int bg)
{
    id->titleFGcolor = fg;
    id->titleBGcolor = bg;
    return;
}

void chartSetTextColor (CHART_ID id, int color)
{
    id->textcolor = color;
    return;
}

int chartRender (CHART_ID id)
{
    if (createImage (id) == -1)
    {
        return -1;
    }

    drawTitle (id);
    drawGrid (id);

    if (id->isBarChart)
    {
        drawBars (id);
    }
    else
    {
        drawLine (id);
    }

    return 0;
}

int chartSave (CHART_ID id, char *name)
{
    FILE    *out;

    if ((out = fopen (name, "wb")) != NULL)
    {
        gdImagePng (id->im, out);
        fclose (out);
        return 0;
    }
    else
    {
        return -1;
    }
}

void chartDestroy (CHART_ID id)
{
    if (id->im)
        gdImageDestroy (id->im);

    return;
}
