/*---------------------------------------------------------------------------
 
  FILENAME:
         html.c
 
  PURPOSE:
        Provide the wview html generator entry point.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        08/27/03        M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/*  ... Library include files
*/
#include <radsystem.h>
#include <radconffile.h>

/*  ... Local include files
*/
#include <html.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
static HTML_WORK        htmlWork;

enum ConfigIds
{
    CFG_ID_STATION_NAME         = 0,
    CFG_ID_STATION_CITY         = 1,
    CFG_ID_STATION_STATE        = 2,
    CFG_ID_IMAGE_PATH           = 3,
    CFG_ID_HTML_PATH            = 4,
    CFG_ID_START_OFFSET         = 5,
    CFG_ID_GENERATE_INTERVAL    = 6,
    CFG_ID_METRIC_UNITS         = 7,
    CFG_ID_METRIC_RAIN_MM       = 8,
    CFG_ID_EXTENDED_DATA        = 9,
    CFG_ID_ARC_RECS_TO_KEEP     = 10,
    CFG_ID_MPHASE_INCREASE      = 11,
    CFG_ID_MPHASE_DECREASE      = 12,
    CFG_ID_MPHASE_FULL          = 13
};

static char             *configIDs[] =
{
    "STATION_NAME",
    "STATION_CITY",
    "STATION_STATE",
    "IMAGE_PATH",
    "HTML_PATH",
    "START_OFFSET",
    "GENERATE_INTERVAL",
    "METRIC_UNITS",
    "METRIC_USE_RAIN_MM",
    "EXTENDED_DATA",
    "ARCHIVE_BROWSER_FILES_TO_KEEP",
    "MPHASE_INCREASE",
    "MPHASE_DECREASE",
    "MPHASE_FULL"
};


/* ... methods
*/
/*  ... system initialization
*/
static int htmlSysInit (HTML_WORK *work)
{
    char            devPath[256];
    struct stat     fileData;

    /*  ... exit if our config file is not there
    */
    sprintf (work->configFile, "%s/%s", WVIEW_CONFIG_DIR, HTML_CONFIG_FILENAME);
    if (stat (work->configFile, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_HTML, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "Cannot locate config file (did you run 'make install-env'?): %s - aborting!",
                   work->configFile);
        radMsgLogExit ();
        return ERROR;
    }
    
    /*  ... check for our daemon's pid file, don't run if it isn't there
    */
    sprintf (devPath, "%s/%s", WVIEW_RUN_DIR, WVD_LOCK_FILE_NAME);
    if (stat (devPath, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_HTML, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "wview daemon lock file %s does not exist - aborting!",
                   devPath);
        radMsgLogExit ();
        return ERROR;
    }

    sprintf (work->pidFile, "%s/%s", WVIEW_RUN_DIR, HTML_LOCK_FILE_NAME);
    sprintf (work->fifoFile, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_HTML);
    sprintf (work->daemonQname, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_DAEMON);

    
    /*  ... check for our html template directory and bail if it is not there
    */
    sprintf (devPath, "%s/html", WVIEW_CONFIG_DIR);
    if (stat (devPath, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_HTML, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "HTML template directory %s is missing...aborting!",
                   devPath);
        radMsgLogExit ();
        return ERROR;
    }

    /*  ... check for our pid file, don't run if it is there
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        radMsgLogInit (PROC_NAME_HTML, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "lock file %s exists, older copy may be running - aborting!",
                   work->pidFile);
        radMsgLogExit ();
        return ERROR;
    }

    /*  ... create our device directory if it is not there
    */
    sprintf (devPath, "%s/dev", WVIEW_RUN_DIR);
    if (stat (devPath, &fileData) != 0)
    {
        if (mkdir (devPath, 0755) != 0)
        {
            radMsgLogInit (PROC_NAME_HTML, FALSE, TRUE);
            radMsgLog (PRI_CATASTROPHIC, "Cannot create device dir: %s...",
                       devPath);
            radMsgLogExit ();
            return ERROR;
        }
    }

    /*  ... create our image directory if it is not there
    */
    sprintf (devPath, "%s/img", WVIEW_RUN_DIR);
    if (stat (devPath, &fileData) != 0)
    {
        if (mkdir (devPath, 0755) != 0)
        {
            radMsgLogInit (PROC_NAME_HTML, FALSE, TRUE);
            radMsgLog (PRI_CATASTROPHIC, "Cannot create image dir: %s...",
                       devPath);
            radMsgLogExit ();
            return ERROR;
        }
    }

    return 0;
}

/*  ... system exit
*/
static int htmlSysExit (HTML_WORK *work)
{
    struct stat     fileData;

    /*  ... delete our pid file
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        unlink (work->pidFile);
    }

    return 0;
}

static void defaultSigHandler (int signum)
{
    int         retVal;

    switch (signum)
    {
        case SIGHUP:
            // user wants us to reload some config files
            retVal = wvutilsToggleVerbosity ();
            radMsgLog (PRI_STATUS, "htmlgend: SIGHUP - toggling log verbosity %s",
                       ((retVal == 0) ? "OFF" : "ON"));
            radMsgLog (PRI_STATUS, "htmlgend: SIGHUP - "
                       "re-reading image/html config files");
            if (htmlWork.mgrId != NULL)
            {
                htmlmgrReReadImageFiles (htmlWork.mgrId, WVIEW_CONFIG_DIR);
            }
            signal (signum, defaultSigHandler);
            return;

        case SIGBUS:
        case SIGFPE:
        case SIGSEGV:
        case SIGXFSZ:
        case SIGSYS:
            // unrecoverable signal - we must exit right now!
            radMsgLog (PRI_CATASTROPHIC, "htmlgend: recv sig %d: shutting down!", signum);
            abort ();
        
        case SIGCHLD:
            wait (NULL);
            signal (signum, defaultSigHandler);
            return;

        default:
            // we can allow the process to exit normally...
            if (htmlWork.exiting)
            {
                signal (signum, defaultSigHandler);
                return;
            }
        
            radMsgLog (PRI_CATASTROPHIC, "htmlgend: recv sig %d: exiting!", signum);
        
            htmlWork.exiting = TRUE;
            radProcessSetExitFlag ();
        
            signal (signum, defaultSigHandler);
            break;
    }

    return;
}

static void msgHandler
(
    char        *srcQueueName,
    UINT        msgType,
    void        *msg,
    UINT        length,
    void        *userData
)
{
    STIM        stim;

    stim.type           = STIM_QMSG;
    stim.srcQueueName   = srcQueueName;
    stim.msgType        = msgType;
    stim.msg            = msg;
    stim.length         = length;

    radStatesProcess (htmlWork.stateMachine, &stim);

    return;
}

static void evtHandler
(
    UINT        eventsRx,
    UINT        rxData,
    void        *userData
)
{
    STIM        stim;

    stim.type           = STIM_EVENT;
    stim.eventsRx       = eventsRx;
    stim.eventData      = rxData;

    radStatesProcess (htmlWork.stateMachine, &stim);

    return;
}

static void timerHandler (void *parm)
{
    STIM            stim;
    struct timeval  timenow;
    long            netOffset;

    // get current time
    gettimeofday (&timenow, NULL);
    
    //  bump our expected next generation time
    htmlWork.nextGenerationTime.tv_sec += (htmlWork.timerInterval / 1000);
    
    // compute the next timer period
    netOffset = (htmlWork.nextGenerationTime.tv_sec - timenow.tv_sec) * 1000;
    netOffset += ((htmlWork.nextGenerationTime.tv_usec - timenow.tv_usec) / 1000);

    radProcessTimerStart (htmlWork.timer, (int)netOffset);

    memset (&stim, 0, sizeof (stim));

    stim.type           = STIM_TIMER;
    stim.timerNumber    = TIMER_GENERATE;

    radStatesProcess (htmlWork.stateMachine, &stim);

    return;
}

static void dstTimerHandler (void *parm)
{
    STIM            stim;
    struct timeval  timenow;
    long            netOffset;

    memset (&stim, 0, sizeof (stim));

    stim.type           = STIM_TIMER;
    stim.timerNumber    = TIMER_DST;

    radStatesProcess (htmlWork.stateMachine, &stim);

    return;
}


/*  ... the main entry point for the html process
*/
int main (int argc, char *argv[])
{
    void            (*alarmHandler)(int);
    STIM            stim;
    int             i, verbose;
    char            instance[32], value[_MAX_PATH];
    CF_ID           configFileId;
    struct stat     fileStatus;
    FILE            *pidfile;
    int             isMetricUnits;

    memset (&htmlWork, 0, sizeof (htmlWork));

    /*  ... initialize some system stuff first
    */
    if (htmlSysInit (&htmlWork) == -1)
    {
        radMsgLogInit (PROC_NAME_HTML, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "system init failed!\n");
        radMsgLogExit ();
        exit (1);
    }


    /*  ... call the global radlib system init function
    */
    if (radSystemInit (WVIEW_SYSTEM_ID) == ERROR)
    {
        radMsgLogInit (PROC_NAME_HTML, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "radSystemInit failed!");
        radMsgLogExit ();
        exit (1);
    }


    /*  ... call the radlib process init function
    */
    if (radProcessInit (PROC_NAME_HTML,
                        htmlWork.fifoFile,
                        PROC_NUM_TIMERS_HTML,
                        TRUE,                      // TRUE for daemon
                        msgHandler,
                        evtHandler,
                        NULL)
        == ERROR)
    {
        printf ("\nradProcessInit failed: %s\n\n", PROC_NAME_HTML);
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    htmlWork.myPid = getpid ();
    pidfile = fopen (htmlWork.pidFile, "w");
    if (pidfile == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "lock file create failed!\n");
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    fprintf (pidfile, "%d", getpid ());
    fclose (pidfile);


    alarmHandler = radProcessSignalGetHandler (SIGALRM);
    radProcessSignalCatchAll (defaultSigHandler);
    radProcessSignalCatch (SIGALRM, alarmHandler);


    // get the wview verbosity setting
    if (wvutilsSetVerbosity (WV_VERBOSE_HTMLGEND) == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "wvutilsSetVerbosity failed!\n");
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    
    
    /*  ... get our configuration values
    */
    if (stat (htmlWork.configFile, &fileStatus) == -1)
    {
        radMsgLog (PRI_CATASTROPHIC, "config file %s missing - use wviewconfig to generate one...\n", 
                   htmlWork.configFile);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    configFileId = radCfOpen (htmlWork.configFile);
    if (configFileId == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfOpen failed!\n");
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    // get the metric units flag first
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_METRIC_UNITS],
                            instance,
                            value)
        == ERROR)
    {
        // just assume US units
        htmlWork.isMetricUnits = 0;
    }
    else
    {
        htmlWork.isMetricUnits = atoi (value);
        if (htmlWork.isMetricUnits)
            radMsgLog (PRI_STATUS, "!! configured for metric units/conversion !!");

        // try to get the CM/MM rain setting here too...
        if (radCfGetFirstEntry (configFileId,
                                configIDs[CFG_ID_METRIC_RAIN_MM],
                                instance,
                                value)
            == ERROR)
        {
            // just assume CM units
            htmlWork.isMetricRainMM = 0;
        }
        else
        {
            htmlWork.isMetricRainMM = atoi (value);
            if (htmlWork.isMetricRainMM)
                radMsgLog (PRI_STATUS, "!! Rain units will be mm !!");
        }
    }

    // get the archive browser files-to-keep value
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_ARC_RECS_TO_KEEP],
                            instance,
                            value)
        == ERROR)
    {
        // just assume it is disabled
        htmlWork.arcrecDaysToKeep = -1;
    }
    else
    {
        htmlWork.arcrecDaysToKeep = atoi (value);
    }

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_MPHASE_INCREASE],
                            instance,
                            value)
        == ERROR)
    {
        strncpy (htmlWork.mphaseIncrease, "Waxing", 31);
    }
    else
    {
        strncpy (htmlWork.mphaseIncrease, value, 31);
    }

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_MPHASE_DECREASE],
                            instance,
                            value)
        == ERROR)
    {
        strncpy (htmlWork.mphaseDecrease, "Waning", 31);
    }
    else
    {
        strncpy (htmlWork.mphaseDecrease, value, 31);
    }

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_MPHASE_FULL],
                            instance,
                            value)
        == ERROR)
    {
        strncpy (htmlWork.mphaseFull, "Full", 31);
    }
    else
    {
        strncpy (htmlWork.mphaseFull, value, 31);
    }

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_STATION_NAME],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfGetFirstEntry failed!\n");
        radCfClose (configFileId);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    strcpy (htmlWork.stationName, value);

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_STATION_CITY],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfGetFirstEntry failed!\n");
        radCfClose (configFileId);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    strcpy (htmlWork.stationCity, value);

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_STATION_STATE],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfGetFirstEntry failed!\n");
        radCfClose (configFileId);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    strcpy (htmlWork.stationState, value);

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_IMAGE_PATH],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfGetFirstEntry failed!\n");
        radCfClose (configFileId);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    strcpy (htmlWork.imagePath, value);
    radMsgLog (PRI_STATUS, "generating to %s", htmlWork.imagePath);


    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_HTML_PATH],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfGetFirstEntry failed!\n");
        radCfClose (configFileId);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    strcpy (htmlWork.htmlPath, value);

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_START_OFFSET],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfGetFirstEntry failed!\n");
        radCfClose (configFileId);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    htmlWork.startOffset = atoi (value);

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_GENERATE_INTERVAL],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfGetFirstEntry failed!\n");
        radCfClose (configFileId);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    htmlWork.timerInterval = 60000 * atoi (value);

    // if the extended data parm is there, use it
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_EXTENDED_DATA],
                            instance,
                            value)
        != ERROR)
    {
        htmlWork.isExtendedData = atoi (value);
    }

    radCfClose (configFileId);


    htmlWork.stateMachine = radStatesInit (&htmlWork);
    if (htmlWork.stateMachine == NULL)
    {
        radMsgLog (PRI_HIGH, "radStatesInit failed");
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    if (radStatesAddHandler (htmlWork.stateMachine, HTML_STATE_IDLE,
                             htmlIdleState)
            == ERROR)
    {
        radMsgLog (PRI_HIGH, "radStatesAddHandler failed");
        radStatesExit (htmlWork.stateMachine);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    if (radStatesAddHandler (htmlWork.stateMachine, HTML_STATE_ARCHIVE,
                             htmlArchiveState)
            == ERROR)
    {
        radMsgLog (PRI_HIGH, "radStatesAddHandler failed");
        radStatesExit (htmlWork.stateMachine);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    if (radStatesAddHandler (htmlWork.stateMachine, HTML_STATE_RUN,
                             htmlRunState)
            == ERROR)
    {
        radMsgLog (PRI_HIGH, "radStatesAddHandler failed");
        radStatesExit (htmlWork.stateMachine);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    if (radStatesAddHandler (htmlWork.stateMachine, HTML_STATE_DATA,
                             htmlDataState)
            == ERROR)
    {
        radMsgLog (PRI_HIGH, "radStatesAddHandler failed");
        radStatesExit (htmlWork.stateMachine);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    if (radStatesAddHandler (htmlWork.stateMachine, HTML_STATE_ERROR,
                             htmlErrorState)
            == ERROR)
    {
        radMsgLog (PRI_HIGH, "radStatesAddHandler failed");
        radStatesExit (htmlWork.stateMachine);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    radStatesSetState (htmlWork.stateMachine, HTML_STATE_IDLE);

    
    htmlWork.timer = radTimerCreate (NULL, timerHandler, NULL);
    if (htmlWork.timer == NULL)
    {
        radMsgLog (PRI_HIGH, "radTimerCreate failed");
        radStatesExit (htmlWork.stateMachine);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    htmlWork.dstTimer = radTimerCreate (NULL, dstTimerHandler, NULL);
    if (htmlWork.dstTimer == NULL)
    {
        radMsgLog (PRI_HIGH, "radTimerCreate failed");
        radTimerDelete (htmlWork.timer);
        radStatesExit (htmlWork.stateMachine);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    // wait a bit here before continuing
    radUtilsSleep (500);

    //  register with the radlib message router
    if (radMsgRouterInit (WVIEW_RUN_DIR) == ERROR)
    {
        radMsgLog (PRI_HIGH, "radMsgRouterInit failed!");
        radTimerDelete (htmlWork.timer);
        radStatesExit (htmlWork.stateMachine);
        htmlSysExit (&htmlWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    // enable message reception from the radlib router for worker requests
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_HILOW_DATA);
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_LOOP_DATA);
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_ARCHIVE_NOTIFY);


    radMsgLog (PRI_STATUS, "running...");


    /*  ... dummy up a stimulus to get the state machine running
    */
    stim.type = STIM_DUMMY;
    radStatesProcess (htmlWork.stateMachine, &stim);


    while (!htmlWork.exiting)
    {
        /*  ... wait on timers, events, file descriptors, msgs, everything!
        */
        if (radProcessWait (0) == ERROR)
        {
            htmlWork.exiting = TRUE;
        }
    }


    radMsgLog (PRI_STATUS, "exiting normally...");


    radMsgRouterExit ();
    radTimerDelete (htmlWork.dstTimer);
    radTimerDelete (htmlWork.timer);
    radStatesExit (htmlWork.stateMachine);
    
    if (htmlWork.mgrId != NULL)
        htmlmgrExit (htmlWork.mgrId);

    htmlSysExit (&htmlWork);

    radProcessExit ();
    radSystemExit (WVIEW_SYSTEM_ID);
    exit (0);
}

