/*---------------------------------------------------------------------------
 
  FILENAME:
        htmlMgr.c
 
  PURPOSE:
        Provide the wview html generator utilities.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        08/30/03        M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <termios.h>

/*  ... Library include files
*/
#include <radtimeUtils.h>

/*  ... Local include files
*/
#include <services.h>
#include <html.h>
#include <htmlMgr.h>
#include <htmlGenerate.h>
#include <glbucket.h>
#include <glchart.h>
#include <images.h>
#include <images-user.h>


/*  ... global memory declarations
*/
char sampleLabels[MAX_DAILY_NUM_VALUES-1][8];

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
static HTML_MGR mgrWork;


//  ... local utilities

static int readImageConfFile (HTML_MGR *mgr, char *filename, int isUser)
{
    HTML_IMG        *img;
    FILE            *file;
    int             i, done;
    char            *token;
    char            temp[1536];

    file = fopen (filename, "r");
    if (file == NULL)
    {
        radMsgLog (PRI_HIGH, "htmlmgrInit: %s does not exist!",
                   filename);
        return ERROR_ABORT;
    }

    while (fgets (temp, 1536, file) != NULL)
    {
        if (temp[0] == ' ' || temp[0] == '\n' || temp[0] == '#')
        {
            // comment or whitespace
            continue;
        }

        token = strtok (temp, " \t");
        if (token == NULL)
        {
            continue;
        }

        img = (HTML_IMG *)malloc (sizeof (*img));
        if (img == NULL)
        {
            for (img = (HTML_IMG *)radListRemoveFirst (&mgr->imgList);
                    img != NULL;
                    img = (HTML_IMG *)radListRemoveFirst (&mgr->imgList))
            {
                free (img);
            }

            fclose (file);
            return ERROR;
        }
        memset (img, 0, sizeof (*img));

        img->mgrWork    = mgr;
        
        
        //  ... if legacy image type is found, skip it
        if (!strncmp (token, "IMG_TYPE_", 9))
        {
            // get the next token
            token = strtok (NULL, " \t");
            if (token == NULL)
            {
                free (img);
                continue;
            }
        }

        //  ... do the file name
        strcpy (img->fname, token);

        //  ... now the label
        token = strtok (NULL, " \t");
        if (token == NULL)
        {
            free (img);
            continue;
        }
        if (token[strlen(token)-1] == '"')
        {
            // we're done!
            strncpy (img->title, &token[1], strlen(token)-2);   // lose quotes
        }
        else
        {
            done = FALSE;
            strcpy (img->title, &token[1]);
            while (!done)
            {
                token = strtok (NULL, " ");
                if (token == NULL)
                {
                    free (img);
                    continue;
                }
                if (token[strlen(token)-1] == '"')
                {
                    done = TRUE;
                    token[strlen(token)-1] = 0;
                }
                strcat (img->title, " ");
                strcat (img->title, token);
            }
        }

        //  ... now the units label (no spaces allowed)
        token = strtok (NULL, " \t");
        if (token == NULL)
        {
            free (img);
            continue;
        }
        strcpy (img->units, token);

        //  ... now the decimal places
        token = strtok (NULL, " \t");
        if (token == NULL)
        {
            free (img);
            continue;
        }
        img->decimalPlaces = atoi (token);

        //  ... finally, the generator index
        token = strtok (NULL, " \t\n");
        if (token == NULL)
        {
            free (img);
            continue;
        }
        i = atoi (token);
        if (isUser)
        {
            img->generator = user_generators[i];
        }
        else
        {
            img->generator = images_generators[i];
        }

        radListAddToEnd (&mgr->imgList, (NODE_PTR)img);
    }

    fclose (file);
    return OK;
}

static int readHtmlTemplateFile (HTML_MGR *mgr, char *filename)
{
    HTML_TMPL       *html;
    FILE            *file;
    char            *token;
    char            temp[HTML_MAX_LINE_LENGTH];

    file = fopen (filename, "r");
    if (file == NULL)
    {
        radMsgLog (PRI_HIGH, "htmlmgrInit: %s does not exist!",
                   filename);
        return ERROR_ABORT;
    }

    while (fgets (temp, HTML_MAX_LINE_LENGTH, file) != NULL)
    {
        if (temp[0] == ' ' || temp[0] == '\n' || temp[0] == '#')
        {
            // comment or whitespace
            continue;
        }

        html = (HTML_TMPL *)malloc (sizeof (*html));
        if (html == NULL)
        {
            for (html = (HTML_TMPL *)radListRemoveFirst (&mgr->templateList);
                 html != NULL;
                 html = (HTML_TMPL *)radListRemoveFirst (&mgr->templateList))
            {
                free (html);
            }

            fclose (file);
            return ERROR;
        }

        // do the template file name
        token = strtok (temp, " \t\n");
        if (token == NULL)
        {
            free (html);
            continue;
        }
        strcpy (html->fname, token);

        radListAddToEnd (&mgr->templateList, (NODE_PTR)html);
    }

    fclose (file);
    return OK;
}

static void cleanupForecastRules (HTML_MGR *mgr)
{
    int             i;
    
    for (i = 0; i <= HTML_MAX_FCAST_RULE; i ++)
    {
        if (mgr->ForecastRuleText[i] != NULL)
        {
            free (mgr->ForecastRuleText[i]);
            mgr->ForecastRuleText[i] = NULL;
        }
    }

    for (i = 1; i <= VP_FCAST_ICON_MAX; i ++)
    {
        if (mgr->ForecastIconFile[i] != NULL)
        {
            free (mgr->ForecastIconFile[i]);
            mgr->ForecastIconFile[i] = NULL;
        }
    }
    
    return;
}       

static int readForecastRuleConfigFile (HTML_MGR *mgr, char *filename)
{
    FILE            *file;
    char            *token;
    char            temp[HTML_MAX_LINE_LENGTH];
    int             iconNo = 1, ruleNo = 0;

    file = fopen (filename, "r");
    if (file == NULL)
    {
        return ERROR_ABORT;
    }

    //  read each line starting with "ICON", assigning to the corresponding 
    //  forecast icon
    while (fgets (temp, HTML_MAX_LINE_LENGTH-1, file) != NULL)
    {
        // does the line begin with "ICON"?
        if (strncmp (temp, "ICON", 4))
        {
            // nope
            continue;
        }
        
        // get the memory to store the string
        mgr->ForecastIconFile[iconNo] = (char *) malloc (strlen (temp) + 1);
        if (mgr->ForecastIconFile[iconNo] == NULL)
        {
            radMsgLog (PRI_HIGH, "readForecastRuleConfigFile: cannot allocate memory for forecast icon #%d!",
                       iconNo);
            cleanupForecastRules (mgr);
            return ERROR;
        }
        
        // skip the ICON keyword
        token = strtok (temp, " \t");
        if (token == NULL)
        {
            free (mgr->ForecastIconFile[iconNo]);
            mgr->ForecastIconFile[iconNo] = NULL;
            continue;
        }
        token = strtok (NULL, " \t\n");
        if (token == NULL)
        {
            free (mgr->ForecastIconFile[iconNo]);
            mgr->ForecastIconFile[iconNo] = NULL;
            continue;
        }

        strcpy (mgr->ForecastIconFile[iconNo], token);
        if (mgr->ForecastIconFile[iconNo][strlen (temp) - 1] == '\n')
            mgr->ForecastIconFile[iconNo][strlen (temp) - 1] = 0;
        
        if (++iconNo == (VP_FCAST_ICON_MAX + 1))
        {
            // we're done!
            break;
        }
    }

    
    // reset to the beginning of the file
    fseek (file, 0, SEEK_SET);
    

    // throw away all lines until we hit the keyword "RULES"
    while (fgets (temp, HTML_MAX_LINE_LENGTH-1, file) != NULL)
    {
        if (!strncmp (temp, "<RULES>", 7))
        {
            // begin the rule dance
            break;
        }
    }

    //  read each line, assigning to the corresponding forecast rule
    while (fgets (temp, HTML_MAX_LINE_LENGTH-1, file) != NULL)
    {
        // get the memory to store the string
        mgr->ForecastRuleText[ruleNo] = (char *) malloc (strlen (temp) + 1);
        if (mgr->ForecastRuleText[ruleNo] == NULL)
        {
            radMsgLog (PRI_HIGH, "readForecastRuleConfigFile: cannot allocate memory for forecast rule #%d!",
                       ruleNo);
            cleanupForecastRules (mgr);
            return ERROR;
        }
        
        // copy the string
        strcpy (mgr->ForecastRuleText[ruleNo], temp);
        if (mgr->ForecastRuleText[ruleNo][strlen (temp) - 1] == '\n')
            mgr->ForecastRuleText[ruleNo][strlen (temp) - 1] = 0;
        
        if (++ruleNo == (HTML_MAX_FCAST_RULE + 1))
        {
            // we're done!
            break;
        }
    }
    
    // did we find them all?
    if (ruleNo != (HTML_MAX_FCAST_RULE + 1))
    {
        // nope, let's fuss about it
        radMsgLog (PRI_MEDIUM, "readForecastRuleConfigFile: NOT all forecast rules are assigned in %s: "
                               "the last %d are missing and will be empty ...",
                   filename,
                   (HTML_MAX_FCAST_RULE + 1) - ruleNo);
    }
    if (iconNo != (VP_FCAST_ICON_MAX + 1))
    {
        // nope, let's fuss about it
        radMsgLog (PRI_MEDIUM, "readForecastRuleConfigFile: NOT all forecast icons are assigned in %s: "
                               "the last %d are missing and will be empty ...",
                   filename,
                   (VP_FCAST_ICON_MAX + 1) - iconNo);
    }

    radMsgLog (PRI_STATUS, "%d icon definitions, %d forecast rules found ...",
               iconNo - 1,
               ruleNo);

    fclose (file);
    return OK;
}

static void emptyWorkLists
(
    HTML_MGR_ID     id
)
{
    NODE_PTR        nptr;

    for (nptr = radListRemoveFirst (&id->imgList);
         nptr != NULL;
         nptr = radListRemoveFirst (&id->imgList))
    {
        free (nptr);
    }

    for (nptr = radListRemoveFirst (&id->templateList);
         nptr != NULL;
         nptr = radListRemoveFirst (&id->templateList))
    {
        free (nptr);
    }

    return;
}

//  ... API methods

HTML_MGR_ID htmlmgrInit
(
    char            *installPath,
    int             isMetricUnits,
    int             isMetricRainMM,
    char            *imagePath,
    char            *htmlPath,
    int             arcInterval,
    int             isExtendedData,
    char            *city,
    char            *state,
    short           elevation,
    short           latitude,
    short           longitude,
    char            *mphaseIncrease,
    char            *mphaseDecrease,
    char            *mphaseFull
)
{
    HTML_MGR_ID     newId;
    int             i, numNodes;
    char            confFilePath[256];

    newId = &mgrWork;
    memset (newId, 0, sizeof (mgrWork));

    radListReset (&newId->imgList);
    radListReset (&newId->templateList);
    
    newId->isMetricUnits    = isMetricUnits;
    newId->isMetricRainMM   = isMetricRainMM;
    newId->archiveInterval  = arcInterval;
    newId->isExtendedData   = isExtendedData;
    newId->imagePath        = imagePath;
    newId->htmlPath         = htmlPath;
    newId->stationElevation = elevation;
    newId->stationLatitude  = latitude;
    newId->stationLongitude = longitude;
    strncpy (newId->stationCity, city, 64);
    strncpy (newId->stationState, state, 64);
    strncpy (newId->mphaseIncrease, mphaseIncrease, 31);
    strncpy (newId->mphaseDecrease, mphaseDecrease, 31);
    strncpy (newId->mphaseFull, mphaseFull, 31);
    
    //  ... initialize the newArchiveMask
    newId->newArchiveMask = NEW_ARCHIVE_ALL;

    
    //  ... first, the built-in generators
    if (isMetricUnits)
    {
        if (isMetricRainMM)
            sprintf (confFilePath, "%s/images-metric-mm.conf", installPath);
        else
            sprintf (confFilePath, "%s/images-metric.conf", installPath);
    }
    else
    {
        sprintf (confFilePath, "%s/images.conf", installPath);
    }
    
    if (readImageConfFile (newId, confFilePath, FALSE) == ERROR)
    {
        return NULL;
    }
    else
    {
        numNodes = radListGetNumberOfNodes (&newId->imgList);
        radMsgLog (PRI_STATUS, "htmlmgrInit: %d built-in image definitions added",
                   numNodes);
    }

    //  ... next, the user-defined generators
    sprintf (confFilePath, "%s/images-user.conf", installPath);
    i = readImageConfFile (newId, confFilePath, TRUE);
    if (i == ERROR_ABORT)
    {
        radMsgLog (PRI_STATUS, "htmlmgrInit: no user image definitions found");
    }
    else if (i == ERROR)
    {
        return NULL;
    }
    else
    {
        radMsgLog (PRI_STATUS, "htmlmgrInit: %d user image definitions added",
                   radListGetNumberOfNodes (&newId->imgList) - numNodes);
    }

    //  ... now initialize our html template list
    sprintf (confFilePath, "%s/html-templates.conf", installPath);
    if (readHtmlTemplateFile (newId, confFilePath) == ERROR)
    {
        return NULL;
    }
    else
    {
        radMsgLog (PRI_STATUS, "htmlmgrInit: %d html/xml templates added",
                   radListGetNumberOfNodes (&newId->templateList));
    }

    //  ... now initialize our forecast rule text list
    sprintf (confFilePath, "%s/forecast.conf", installPath);
    if (readForecastRuleConfigFile (newId, confFilePath) != OK)
    {
        radMsgLog (PRI_STATUS, "htmlmgrInit: forecast html tags are disabled - %s not found...",
                   confFilePath);
    }

    //  ... initialize the sample label array
    for (i = 0; i < DAILY_NUM_VALUES(newId) - 1; i ++)
    {
        sprintf (sampleLabels[i], 
                 "%d:%2.2d", 
                 i/(60/newId->archiveInterval), 
                 (i%(60/newId->archiveInterval))*newId->archiveInterval);
    }

    //  ... initialize our dial palette to conserve CPU cycles
    if (htmlGenPngDialInit (newId->imagePath) == ERROR)
    {
        radMsgLog (PRI_HIGH, "htmlmgrInit: htmlGenPngDialInit failed!");
        return NULL;
    }

    return newId;
}

int htmlmgrReReadImageFiles
(
    HTML_MGR_ID     id,
    char            *installPath
)
{
    int             i, numNodes;
    char            confFilePath[256];

    // cleanup the old definitions
    cleanupForecastRules (id);
    emptyWorkLists (id);
    radListReset (&id->imgList);
    radListReset (&id->templateList);
    
    //  re-read the built-in generators
    if (id->isMetricUnits)
    {
        sprintf (confFilePath, "%s/images-metric.conf", installPath);
    }
    else
    {
        sprintf (confFilePath, "%s/images.conf", installPath);
    }
    
    if (readImageConfFile (id, confFilePath, FALSE) == ERROR)
    {
        return ERROR;
    }
    else
    {
        numNodes = radListGetNumberOfNodes (&id->imgList);
        radMsgLog (PRI_STATUS, "htmlmgrReReadImageFiles: %d built-in image definitions added",
                   numNodes);
    }

    //  re-read the user-defined generators
    sprintf (confFilePath, "%s/images-user.conf", installPath);
    i = readImageConfFile (id, confFilePath, TRUE);
    if (i == ERROR_ABORT)
    {
        radMsgLog (PRI_STATUS, "htmlmgrReReadImageFiles: no user image definitions found");
    }
    else if (i == ERROR)
    {
        return ERROR;
    }
    else
    {
        radMsgLog (PRI_STATUS, "htmlmgrReReadImageFiles: %d user image definitions added",
                   radListGetNumberOfNodes (&id->imgList) - numNodes);
    }

    //  reinitialize our html template list
    sprintf (confFilePath, "%s/html-templates.conf", installPath);
    if (readHtmlTemplateFile (id, confFilePath) == ERROR)
    {
        return ERROR;
    }
    else
    {
        radMsgLog (PRI_STATUS, "htmlmgrReReadImageFiles: %d html/xml templates added",
                   radListGetNumberOfNodes (&id->templateList));
    }

    //  reinitialize our forecast rule text list
    sprintf (confFilePath, "%s/forecast.conf", installPath);
    if (readForecastRuleConfigFile (id, confFilePath) != OK)
    {
        radMsgLog (PRI_STATUS, "htmlmgrReReadImageFiles: forecast html tags are disabled - %s not found...",
                   confFilePath);
    }

    return OK;
}

void htmlmgrExit
(
    HTML_MGR_ID     id
)
{
    cleanupForecastRules (id);

    emptyWorkLists (id);

    return;
}


int htmlmgrBPTrendInit
(
    HTML_MGR_ID         id,
    int                 timerIntervalMINs
)
{
    int                 i;
    
    //  ... initialize the barometric pressure trend array
    for (i = 0; i < BP_MAX_VALUES; i ++)
    {
        id->baromTrendValues[i] = 0;
    }
    
    id->baromTrendNumValues = 0;
    id->baromTrendIndex = 0;
    id->baromTrendIndexMax = 60/timerIntervalMINs;      // samples per hour
    id->baromTrendIndexMax *= 4;                        // 4 hours
    id->baromTrendIndicator = 1;                        // steady
    
    return OK;
}


static int computeBPTrend
(
    HTML_MGR_ID         id
)
{
    register int        i;
    register float      avg, sum = 0;
    float               currVal = id->loopStore.barometer;
    
    //  ... compute the barometric pressure trend
    //  ... we just use a circular queue since the order of the values
    //  ... doesn't really matter, and it is more efficient
    
    //  ... save the new value
    id->baromTrendValues[id->baromTrendIndex] = currVal;
    
    //  ... do we need to wrap the index?
    if (++ id->baromTrendIndex >= id->baromTrendIndexMax)
        id->baromTrendIndex = 0;
    
    //  ... manage the number of values
    if (id->baromTrendNumValues < id->baromTrendIndexMax)
        id->baromTrendNumValues ++;
    
    //  ... compute the average over all values
    for (i = 0; i < id->baromTrendNumValues; i ++)
    {
        sum += id->baromTrendValues[i];
    }
    avg = sum/id->baromTrendNumValues;
    
    if (currVal < avg)
        id->baromTrendIndicator = 0;
    else if (currVal > avg)
        id->baromTrendIndicator = 2;
    else
        id->baromTrendIndicator = 1;
    
    return OK;
}



int htmlmgrGenerate
(
    HTML_MGR_ID         id
)
{
    register HTML_IMG   *img;
    int                 retVal, imgs = 0, htmls = 0;
    char                temp[256];
#if __DEBUG
    ULONG               ntime = radTimeGetMSSinceEpoch ();
#endif

#if __DEBUG_BUFFERS
    radMsgLog (PRI_STATUS, "DBG BFRS: HTML BEGIN: %u of %u available",
               buffersGetAvailable (),
               buffersGetTotal ());
#endif


    //  ... compute the Barometric Pressure trend
    computeBPTrend (id);
    
    //  ... generate the weather images
    for (img = (HTML_IMG *)radListGetFirst (&id->imgList);
         img != NULL;
         img = (HTML_IMG *)radListGetNext (&id->imgList, (NODE_PTR)img))
    {
        retVal = (*img->generator) (img);
        if (retVal == OK)
        {
            imgs ++;
        }
        else if (retVal != ERROR_ABORT)
        {
            sprintf (temp, "%s/%s", id->imagePath, img->fname);
            radMsgLog (PRI_HIGH, "%s generation failed - is the IMAGE_PATH correct in htmlgen.conf?",
                       temp);
            radMsgLog (PRI_HIGH, "IMAGE_PATH must be local to the wview server...");
        }
    }

    //  ... clear the archiveAvailable flag (must be after generator loop)
    id->newArchiveMask = 0;

    //  ... now generate the HTML
    if ((htmls = htmlgenHTMLFiles (id)) == ERROR)
    {
        return ERROR;
    }

    wvutilsLogEvent (PRI_STATUS, "Generated: %d ms: %d images, %d html/xml files",
                   radTimeGetMSSinceEpoch() - ntime, imgs, htmls);

#if __DEBUG_BUFFERS
    radMsgLog (PRI_STATUS, "DBG BFRS: HTML END: %u of %u available",
               buffersGetAvailable (),
               buffersGetTotal ());
#endif

    return OK;
}


//  ... read archive files to initialize our historical arrays
int htmlmgrHistoryInit (HTML_MGR_ID id, char *archivePath, time_t arcTime)
{
    HISTORY_DATA    data;
    int             startmin, starthour, startday, startmonth, startyear;
    time_t          ntime, baseTime;
    struct tm       locTime;
    int             i, j, retVal;

    // first, figure out our start label indexes
    ntime = arcTime;
    localtime_r (&ntime, &locTime);
    id->dayStart = ((60 * locTime.tm_hour) + (locTime.tm_min))/id->archiveInterval;


    // do the samples in the last day
    baseTime = ntime;
    for (i = 0; i < DAILY_NUM_VALUES(id); i ++)
    {
        ntime = baseTime;
        ntime -= (WV_SECONDS_IN_DAY - (i*SECONDS_IN_INTERVAL(id->archiveInterval)));
        localtime_r (&ntime, &locTime);
        startmin = (locTime.tm_min/id->archiveInterval) * id->archiveInterval;  // lose any remainder
        starthour = locTime.tm_hour;
        startday = locTime.tm_mday;
        startmonth = locTime.tm_mon + 1;
        startyear = locTime.tm_year + 1900;

        //  ... because the archive records span 00:<archive interval> to 24:00
        if (starthour == 0 && startmin == 0)
        {
            starthour = 24;
            ntime -= WV_SECONDS_IN_DAY;
            localtime_r (&ntime, &locTime);
            startday = locTime.tm_mday;
            startmonth = locTime.tm_mon + 1;
            startyear = locTime.tm_year + 1900;
        }

        retVal = dbfGetAverages (id->isMetricUnits,
                                 id->isMetricRainMM,
                                 id->archiveInterval,
                                 archivePath, 
                                 &data,
                                 startmin, 
                                 starthour, 
                                 startday,
                                 startmonth, 
                                 startyear, 
                                 1);

        if (retVal == ERROR || data.samples == 0)
        {
            for (j = 0; j < HIST_TYPE_MAX(id->isExtendedData); j ++)
            {
                id->dayValues[j][i] = GLC_VALUE_NONE;
            }
        }
        else
        {
            for (j = 0; j < HIST_TYPE_MAX(id->isExtendedData); j ++)
            {
                if (j == HIST_TYPE_RAIN || j == HIST_TYPE_WDIR || j == HIST_TYPE_ET)
                {
                    id->dayValues[j][i] = data.values[j];
                }
                else
                {
                    id->dayValues[j][i] = data.values[j]/data.samples;
                }
            }
        }
    }

    //  ... do the hours in the last month

    ntime = arcTime;
    ntime -= WV_SECONDS_IN_MONTH;
    id->monthStartTime_T = ntime;

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &locTime);
        startmin = id->archiveInterval;
        starthour = locTime.tm_hour;
        startday = locTime.tm_mday;
        startmonth = locTime.tm_mon + 1;
        startyear = locTime.tm_year + 1900;

        retVal = dbfGetAverages (id->isMetricUnits,
                                 id->isMetricRainMM,
                                 id->archiveInterval,
                                 archivePath, 
                                 &data,
                                 startmin, 
                                 starthour, 
                                 startday,
                                 startmonth, 
                                 startyear,
                                 WV_SECONDS_IN_HOUR/SECONDS_IN_INTERVAL(id->archiveInterval));

        if (retVal == ERROR || data.samples == 0.0)
        {
            for (j = 0; j < HIST_TYPE_MAX(id->isExtendedData); j ++)
            {
                id->monthValues[j][i] = GLC_VALUE_NONE;
            }
        }
        else
        {
            for (j = 0; j < HIST_TYPE_MAX(id->isExtendedData); j ++)
            {
                if (j == HIST_TYPE_RAIN || j == HIST_TYPE_WDIR || j == HIST_TYPE_ET)
                {
                    id->monthValues[j][i] = data.values[j];
                }
                else
                {
                    id->monthValues[j][i] = data.values[j]/data.samples;
                }
            }
        }

        ntime += WV_SECONDS_IN_HOUR;
    }

    //  ... do the days in the last year

    ntime = arcTime;
    ntime -= WV_SECONDS_IN_YEAR;
    id->yearStartTime_T = ntime;

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &locTime);
        startmin = id->archiveInterval;
        starthour = 0;
        startday = locTime.tm_mday;
        startmonth = locTime.tm_mon + 1;
        startyear = locTime.tm_year + 1900;

        retVal = dbfGetAverages (id->isMetricUnits,
                                 id->isMetricRainMM,
                                 id->archiveInterval,
                                 archivePath, 
                                 &data,
                                 startmin, 
                                 starthour, 
                                 startday,
                                 startmonth, 
                                 startyear,
                                 WV_SECONDS_IN_DAY/SECONDS_IN_INTERVAL(id->archiveInterval));

        if (retVal == ERROR || data.samples == 0)
        {
            for (j = 0; j < HIST_TYPE_MAX(id->isExtendedData); j ++)
            {
                id->yearValues[j][i] = GLC_VALUE_NONE;
            }
        }
        else
        {
            for (j = 0; j < HIST_TYPE_MAX(id->isExtendedData); j ++)
            {
                if (j == HIST_TYPE_RAIN || j == HIST_TYPE_WDIR || j == HIST_TYPE_ET)
                {
                    id->yearValues[j][i] = data.values[j];
                }
                else
                {
                    id->yearValues[j][i] = data.values[j]/data.samples;
                }
            }
        }

        ntime += WV_SECONDS_IN_DAY;
    }


    return OK;
}

int htmlmgrAddSampleValue (HTML_MGR_ID id, HISTORY_DATA *data)
{
    register int    i, j;

    for (i = 0; i < HIST_TYPE_MAX(id->isExtendedData); i ++)
    {
        for (j = 0; j < DAILY_NUM_VALUES(id) - 1; j ++)
        {
            id->dayValues[i][j] = id->dayValues[i][j+1];
        }
    }

    if (++ mgrWork.dayStart >= (DAILY_NUM_VALUES(id) - 1))
    {
        mgrWork.dayStart = 0;
    }

    for (i = 0; i < HIST_TYPE_MAX(id->isExtendedData); i ++)
    {
        if (i == HIST_TYPE_RAIN || i == HIST_TYPE_WDIR || i == HIST_TYPE_ET)
        {
            id->dayValues[i][DAILY_NUM_VALUES(id)-1] = data->values[i];
        }
        else
        {
            id->dayValues[i][DAILY_NUM_VALUES(id)-1] = data->values[i]/data->samples;
        }
    }

    return OK;
}

int htmlmgrAddHourValue (HTML_MGR_ID id, HISTORY_DATA *data)
{
    register int    i, j;

    for (i = 0; i < HIST_TYPE_MAX(id->isExtendedData); i ++)
    {
        for (j = 0; j < MONTHLY_NUM_VALUES-1; j ++)
        {
            id->monthValues[i][j] = id->monthValues[i][j+1];
        }
    }

    mgrWork.monthStartTime_T += WV_SECONDS_IN_HOUR;

    for (i = 0; i < HIST_TYPE_MAX(id->isExtendedData); i ++)
    {
        if (i == HIST_TYPE_RAIN || i == HIST_TYPE_WDIR || i == HIST_TYPE_ET)
        {
            id->monthValues[i][MONTHLY_NUM_VALUES-1] = data->values[i];
        }
        else
        {
            id->monthValues[i][MONTHLY_NUM_VALUES-1] = data->values[i]/data->samples;
        }
    }

    return OK;
}

int htmlmgrAddDayValue (HTML_MGR_ID id, HISTORY_DATA *data)
{
    register int    i, j;

    for (i = 0; i < HIST_TYPE_MAX(id->isExtendedData); i ++)
    {
        for (j = 0; j < YEARLY_NUM_VALUES-1; j ++)
        {
            id->yearValues[i][j] = id->yearValues[i][j+1];
        }
    }

    mgrWork.yearStartTime_T += WV_SECONDS_IN_DAY;

    for (i = 0; i < HIST_TYPE_MAX(id->isExtendedData); i ++)
    {
        if (i == HIST_TYPE_RAIN || i == HIST_TYPE_WDIR || i == HIST_TYPE_ET)
        {
            id->yearValues[i][YEARLY_NUM_VALUES-1] = data->values[i];
        }
        else
        {
            id->yearValues[i][YEARLY_NUM_VALUES-1] = data->values[i]/data->samples;
        }
    }

    return OK;
}
