/*---------------------------------------------------------------------------
 
  FILENAME:
        htmlStates.c
 
  PURPOSE:
        Provide the wview HTML generator state machine handlers.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        08/30/03        M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <termios.h>

/*  ... Library include files
*/

/*  ... Local include files
*/
#include <services.h>
#include <html.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/

static void processDSTTimer (HTML_WORK *work)
{
    WVIEW_MSG_REQUEST   msg;

    //  ... update the time zone info
    tzset ();

    // only reinitialize the history data if we "sprang" forward
    if (work->dstChangeFlag == 1)
    {
        if (htmlmgrHistoryInit (work->mgrId, work->archivePath, time (NULL)) 
            == ERROR)
        {
            radMsgLog (PRI_HIGH, 
                       "processDSTTimer: htmlmgrHistoryInit failed for DST change!");
        }
        else
        {
            //  ... set the archiveAvailable flags for generation...
            work->mgrId->newArchiveMask |= NEW_ARCHIVE_SAMPLE;
            work->mgrId->newArchiveMask |= NEW_ARCHIVE_HOUR;
            
            radMsgLog (PRI_STATUS, 
                       "DST state change detected - historical data synchronized...");
        }
    }

    //  ... always notify the daemon about the DST change
    msg.requestType = WVIEW_RQST_TYPE_DST_ANNOUNCE;
    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_REQUEST, &msg, sizeof(msg)) == ERROR)
    {
        radMsgLog (PRI_HIGH, "processDSTTimer: radMsgRouterMessageSend failed!");
        return;
    }

    return;
}

static void processNewArchiveRecord (HTML_WORK *work, WVIEW_MSG_ARCHIVE_NOTIFY *armsg)
{
    HISTORY_DATA        data;
    int                 currHour, currDay, currMonth, currYear;
    int                 startmin, starthour, startday, startmonth, startyear;
    short               tempShort;
    time_t              ntime;
    struct tm           locTime;

    //  ... generate the mesonet file
    htmlgenMesonetFile (work->mgrId, armsg);


    //  ... check to see if a DST change has occured
    work->dstChangeFlag = htmlutilsDetectDSTChange ();
    if (work->dstChangeFlag != 0)
    {
        // wait 90 seconds for adjkerntz, etc. to get the GMT offset fixed
        radProcessTimerStart (work->dstTimer, HTML_DST_TIMER_INTERVAL);
    }

    //  ... update our data by adding the new sample
    ntime = time (NULL);
    localtime_r (&ntime, &locTime);

    currHour    = locTime.tm_hour;
    currDay     = locTime.tm_mday;
    currMonth   = locTime.tm_mon + 1;
    currYear    = locTime.tm_year + 1900;

    //  ... determine time to start for last sample
    ntime = time (NULL);
    localtime_r (&ntime, &locTime);
    startmin = (locTime.tm_min/work->archiveInterval) * work->archiveInterval;
    starthour = locTime.tm_hour;
    startday = locTime.tm_mday;
    startmonth = locTime.tm_mon + 1;
    startyear = locTime.tm_year + 1900;


    //  ... confirm the last archive record date matches our computed date
    if (startmin != EXTRACT_PACKED_MINUTE(armsg->archiveTime) ||
        starthour != EXTRACT_PACKED_HOUR(armsg->archiveTime) ||
        startday != EXTRACT_PACKED_DAY(armsg->archiveDate) ||
        startmonth != EXTRACT_PACKED_MONTH(armsg->archiveDate) ||
        startyear != EXTRACT_PACKED_YEAR(armsg->archiveDate))
    {
        radMsgLog (PRI_MEDIUM, "processNewArchiveRecord: "
                               "RX date %4.4d%2.2d%2.2d %2.2d:%2.2d "
                               "does NOT match "
                               "computed date %4.4d%2.2d%2.2d %2.2d:%2.2d",
                   EXTRACT_PACKED_YEAR(armsg->archiveDate),
                   EXTRACT_PACKED_MONTH(armsg->archiveDate),
                   EXTRACT_PACKED_DAY(armsg->archiveDate),
                   EXTRACT_PACKED_HOUR(armsg->archiveTime),
                   EXTRACT_PACKED_MINUTE(armsg->archiveTime),
                   startyear,
                   startmonth,
                   startday,
                   starthour,
                   startmin);
        return;
    }


    //  ... because the archive records span 00:<archive interval> to 24:00
    if (starthour == 0 && startmin == 0)
    {
        starthour = 24;
        ntime -= WV_SECONDS_IN_DAY;
        localtime_r (&ntime, &locTime);
        startday = locTime.tm_mday;
        startmonth = locTime.tm_mon + 1;
        startyear = locTime.tm_year + 1900;
    }

    dbfGetAverages (work->mgrId->isMetricUnits,
                    work->mgrId->isMetricRainMM,
                    work->archiveInterval,
                    work->archivePath,
                    &data,
                    startmin,
                    starthour,
                    startday,
                    startmonth,
                    startyear,
                    1);

    if (data.samples > 0)
    {
        wvutilsLogEvent (PRI_STATUS, "Adding %d minute sample...", work->archiveInterval);

        //  ... set the archiveAvailable flag for generation...
        work->mgrId->newArchiveMask |= NEW_ARCHIVE_SAMPLE;

        // add the RX check value to the data here
        data.values[HIST_TYPE_RXCHECK] = armsg->rxPercent;

        htmlmgrAddSampleValue (work->mgrId, &data);
        
        // generate an ASCII entry in the day's browser file (if enabled)
        arcrecGenerate (&data);
    }

    if (work->histLastHour != currHour)
    {
        //  ... determine times to start for last hour
        ntime = time (NULL);
        ntime -= WV_SECONDS_IN_HOUR;
        localtime_r (&ntime, &locTime);
        startmin = work->archiveInterval;
        starthour = locTime.tm_hour;
        startday = locTime.tm_mday;
        startmonth = locTime.tm_mon + 1;
        startyear = locTime.tm_year + 1900;

        dbfGetAverages (work->mgrId->isMetricUnits,
                        work->mgrId->isMetricRainMM,
                        work->archiveInterval,
                        work->archivePath,
                        &data,
                        startmin,
                        starthour,
                        startday,
                        startmonth,
                        startyear,
                        WV_SECONDS_IN_HOUR/SECONDS_IN_INTERVAL(work->archiveInterval));

        if (data.samples > 0)
        {
            wvutilsLogEvent (PRI_STATUS, "Adding hour sample...");
    
            //  ... set the archiveAvailable flag for generation...
            work->mgrId->newArchiveMask |= NEW_ARCHIVE_HOUR;

            htmlmgrAddHourValue (work->mgrId, &data);
        }

        work->histLastHour = currHour;
    }

    if (work->histLastDay != currDay)
    {
        //  ... determine times to start for last day
        ntime = time (NULL);
        ntime -= WV_SECONDS_IN_DAY;
        localtime_r (&ntime, &locTime);
        startmin = work->archiveInterval;
        starthour = 0;
        startday = locTime.tm_mday;
        startmonth = locTime.tm_mon + 1;
        startyear = locTime.tm_year + 1900;

        dbfGetAverages (work->mgrId->isMetricUnits,
                        work->mgrId->isMetricRainMM,
                        work->archiveInterval,
                        work->archivePath,
                        &data,
                        startmin,
                        starthour,
                        startday,
                        startmonth,
                        startyear,
                        WV_SECONDS_IN_DAY/SECONDS_IN_INTERVAL(work->archiveInterval));

        if (data.samples > 0)
        {
            wvutilsLogEvent (PRI_STATUS, "Adding day sample...");
    
            //  ... set the archiveAvailable flag for generation...
            work->mgrId->newArchiveMask |= NEW_ARCHIVE_DAY;
            htmlmgrAddDayValue (work->mgrId, &data);
        }

        work->histLastDay = currDay;


        //  ... do some NOAA work here (if we are not in metric mode)
        if (!work->isMetricUnits)
        {
            noaaGenerate (work->noaaId, currDay, currMonth, currYear);
        }

        //  ... update the sun times
        sunTimesGetSunRiseSet (currYear, currMonth, currDay,
                               (float)work->mgrId->stationLatitude/10,
                               (float)work->mgrId->stationLongitude/10,
                               RS_TYPE_SUN,
                               &work->mgrId->sunrise,
                               &work->mgrId->sunset);
        sunTimesGetSunRiseSet (currYear, currMonth, currDay,
                               (float)work->mgrId->stationLatitude/10,
                               (float)work->mgrId->stationLongitude/10,
                               RS_TYPE_CIVIL,
                               &work->mgrId->civilrise,
                               &work->mgrId->civilset);
        sunTimesGetSunRiseSet (currYear, currMonth, currDay,
                               (float)work->mgrId->stationLatitude/10,
                               (float)work->mgrId->stationLongitude/10,
                               RS_TYPE_ASTRO,
                               &work->mgrId->astrorise,
                               &work->mgrId->astroset);
        sunTimesGetSunRiseSet (currYear, currMonth, currDay,
                               (float)work->mgrId->stationLatitude/10,
                               (float)work->mgrId->stationLongitude/10,
                               RS_TYPE_MIDDAY,
                               &work->mgrId->midday,
                               &tempShort);
        work->mgrId->dayLength = 
                sunTimesGetDayLength (currYear, currMonth, currDay,
                                      (float)work->mgrId->stationLatitude/10,
                                      (float)work->mgrId->stationLongitude/10);
    }

    return;
}

static int requestDataPackets (HTML_WORK *work)
{
    WVIEW_MSG_REQUEST   msg;

    msg.requestType = WVIEW_RQST_TYPE_LOOP_DATA;
    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_REQUEST, &msg, sizeof(msg)) == ERROR)
    {
        radMsgLog (PRI_HIGH, "requestDataPackets: radMsgRouterMessageSend failed!");
        return ERROR;
    }

    msg.requestType = WVIEW_RQST_TYPE_HILOW_DATA;
    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_REQUEST, &msg, sizeof(msg)) == ERROR)
    {
        radMsgLog (PRI_HIGH, "requestDataPackets: radMsgRouterMessageSend failed!");
        return ERROR;
    }

    return OK;
}



int htmlIdleState (int state, void *stimulus, void *data)
{
    STIM                *stim = (STIM *)stimulus;
    HTML_WORK           *work = (HTML_WORK *)data;
    USHORT              year, month, day, hour, minute, second;
    WVIEW_MSG_REQUEST   msg;

    switch (stim->type)
    {
    case STIM_DUMMY:
        // this one starts this state machine - send the archive request
        msg.requestType = WVIEW_RQST_TYPE_ARCHIVE_PATH;
        if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_REQUEST, &msg, sizeof(msg)) == ERROR)
        {
            radMsgLog (PRI_HIGH, "htmlIdleState: radMsgRouterMessageSend failed!");
            return HTML_STATE_ERROR;
        }

        // initialize the DST state change detector here
        htmlutilsDetectDSTInit ();
        
        return HTML_STATE_ARCHIVE;
    }

    return state;
}

int htmlArchiveState (int state, void *stimulus, void *data)
{
    STIM                        *stim = (STIM *)stimulus;
    HTML_WORK                   *work = (HTML_WORK *)data;
    int                         seconds;
    WVIEW_MSG_ARCHIVE_PATH      *msg = (WVIEW_MSG_ARCHIVE_PATH *)stim->msg;
    time_t                      ntime;
    struct tm                   locTime;
    int                         offset, msOffset;
    int                         oldSecs, newSecs;
    int                         currDay, currMonth, currYear;
    short                       tempShort;

    ntime = time (NULL);
    localtime_r (&ntime, &locTime);
    currDay     = locTime.tm_mday;
    currMonth   = locTime.tm_mon + 1;
    currYear    = locTime.tm_year + 1900;

    switch (stim->type)
    {
    case STIM_QMSG:
        if (stim->msgType == WVIEW_MSG_TYPE_ARCHIVE_PATH)
        {
            radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
            ntime = msg->lastArcTime;
            localtime_r (&ntime, &locTime);
            radMsgLog (PRI_STATUS, 
                       "received archive path %s from wviewd: "
                       "%4.4d%2.2d%2.2d %2.2d:%2.2d:%2.2d",
                       msg->archivePath, 
                       locTime.tm_year + 1900,
                       locTime.tm_mon + 1,
                       locTime.tm_mday,
                       locTime.tm_hour,
                       locTime.tm_min,
                       locTime.tm_sec);

            strcpy (work->archivePath, msg->archivePath);
            work->archiveInterval = msg->archiveInterval;
            

            // initialize the htmlMgr now that we know the archive interval
            work->mgrId = htmlmgrInit (WVIEW_CONFIG_DIR, 
                                       work->isMetricUnits,
                                       work->isMetricRainMM,
                                       work->imagePath,
                                       work->htmlPath,
                                       work->archiveInterval,
                                       work->isExtendedData,
                                       work->stationCity,
                                       work->stationState,
                                       msg->elevation,
                                       msg->latitude,
                                       msg->longitude,
                                       work->mphaseIncrease,
                                       work->mphaseDecrease,
                                       work->mphaseFull);
            if (work->mgrId == NULL)
            {
                radMsgLog (PRI_HIGH, "htlmgrInit failed!\n");
                return HTML_STATE_ERROR;
            }
        
            //  ... initialize the sun times now
            sunTimesGetSunRiseSet (currYear, currMonth, currDay,
                                   (float)work->mgrId->stationLatitude/10,
                                   (float)work->mgrId->stationLongitude/10,
                                   RS_TYPE_SUN,
                                   &work->mgrId->sunrise,
                                   &work->mgrId->sunset);
            sunTimesGetSunRiseSet (currYear, currMonth, currDay,
                                   (float)work->mgrId->stationLatitude/10,
                                   (float)work->mgrId->stationLongitude/10,
                                   RS_TYPE_CIVIL,
                                   &work->mgrId->civilrise,
                                   &work->mgrId->civilset);
            sunTimesGetSunRiseSet (currYear, currMonth, currDay,
                                   (float)work->mgrId->stationLatitude/10,
                                   (float)work->mgrId->stationLongitude/10,
                                   RS_TYPE_ASTRO,
                                   &work->mgrId->astrorise,
                                   &work->mgrId->astroset);
            sunTimesGetSunRiseSet (currYear, currMonth, currDay,
                                   (float)work->mgrId->stationLatitude/10,
                                   (float)work->mgrId->stationLongitude/10,
                                   RS_TYPE_MIDDAY,
                                   &work->mgrId->midday,
                                   &tempShort);
            work->mgrId->dayLength = 
                    sunTimesGetDayLength (currYear, currMonth, currDay,
                                          (float)work->mgrId->stationLatitude/10,
                                          (float)work->mgrId->stationLongitude/10);


            /*  ... initialize the barometric pressure trend algorithm
            */
            if (htmlmgrBPTrendInit (work->mgrId, work->timerInterval/60000)
                == ERROR)
            {
                radMsgLog (PRI_HIGH, "htmlmgrBPTrendInit failed!\n");
                return HTML_STATE_ERROR;
            }

           
            /*  ... load up our historical stores for the history charts
            */
            if (htmlmgrHistoryInit (work->mgrId, work->archivePath, msg->lastArcTime) 
                == ERROR)
            {
                radMsgLog (PRI_HIGH, "htmlArchiveState: htmlmgrHistoryInit failed!");
                return HTML_STATE_ERROR;
            }

            // store the "last" values
            ntime = msg->lastArcTime;
            localtime_r (&ntime, &locTime);
            work->histLastHour = locTime.tm_hour;
            work->histLastDay = locTime.tm_mday;

            
            // if we are in metric mode - no NOAA required
            if (!work->isMetricUnits)
            {
                // initialize the NOAA report generator
                work->noaaId = noaaGenerateInit (work->archivePath,
                                                 work->mgrId->imagePath,
                                                 msg->latitude,
                                                 msg->longitude,
                                                 msg->elevation,
                                                 work->stationName,
                                                 work->stationCity,
                                                 work->stationState,
                                                 msg->lastArcTime);
                if (work->noaaId == NULL)
                {
                    radMsgLog (PRI_CATASTROPHIC, "noaaGenerateInit failed!\n");
                    return HTML_STATE_ERROR;
                }
            }
            
            // initialize the archive record browser generator
            if (arcrecGenerateInit (work->archivePath,
                                    work->mgrId->imagePath,
                                    work->arcrecDaysToKeep,
                                    work->isMetricUnits,
                                    work->isMetricRainMM,
                                    work->archiveInterval)
                == ERROR)
            {
                radMsgLog (PRI_CATASTROPHIC, "arcrecGenerateInit failed!\n");
                return HTML_STATE_ERROR;
            }


            /*  ... initial data request then start the data acquisition timer
            */
            if (requestDataPackets (work) == ERROR)
            {
                radMsgLog (PRI_HIGH, "htmlArchiveState: requestDataPackets failed!");
                return HTML_STATE_ERROR;
            }


            // wait here until a new second arrives
            ntime = time (NULL);
            localtime_r (&ntime, &locTime);
            oldSecs = newSecs = locTime.tm_sec;
            while (oldSecs == newSecs)
            {
                ntime = time (NULL);
                localtime_r (&ntime, &locTime);
                newSecs = locTime.tm_sec;
            }
            
            
            /*  ... start the data timer to go off at the next 5 minute
                ... interval plus the user offset
            */
            ntime = time (NULL);
            localtime_r (&ntime, &locTime);
            seconds = locTime.tm_sec - 10;          // start at 10 secs past

            offset = locTime.tm_min % 5;            // where are we in period?
            offset = 5 - offset;                    // how many mins to get to "0"?
            offset %= 5;                            // don't want "5"
            offset += work->startOffset;            // user start in period
            if (offset == 0)                        // prevent negative timeout
                seconds -= 60;                      // make neg so timeout is pos

            if (seconds < -60)
            {
                offset += 1;
                seconds += 60;
            }

            radMsgLog (PRI_STATUS,
                       "starting html generation in %d mins %d secs",
                       ((seconds > 0) ? ((offset > 0) ? offset - 1 : offset) : offset),
                               (seconds > 0) ? 60 - seconds : (-1) * seconds);

            work->nextGenerationTime.tv_sec = (long)ntime + ((offset * 60) - seconds);
            work->nextGenerationTime.tv_usec = 250 * 1000;
            
            msOffset = 250;                         // land on 250 ms mark

            radProcessTimerStart (work->timer,
                                  ((((offset * 60) - seconds) * 1000)) + msOffset);


            // 20041224 - Add an immediate generation cycle so longer 
            // duration archive intervals don't have to wait up to 30 minutes
            // for the first generation
            radMsgLog (PRI_STATUS, "doing initial html generation now...");
            work->numDataReceived = 0;
            return HTML_STATE_DATA;
        }

        break;
    }

    return state;
}

int htmlRunState (int state, void *stimulus, void *data)
{
    STIM                        *stim = (STIM *)stimulus;
    HTML_WORK                   *work = (HTML_WORK *)data;
    WVIEW_MSG_ARCHIVE_NOTIFY    *arcNot = (WVIEW_MSG_ARCHIVE_NOTIFY *)stim->msg;

    switch (stim->type)
    {
        case STIM_QMSG:
            if (stim->msgType == WVIEW_MSG_TYPE_LOOP_DATA)
            {
                memcpy (&work->mgrId->loopStore,
                        &((WVIEW_MSG_LOOP_DATA *)stim->msg)->loopData,
                        sizeof (LOOP_PKT));

                return state;
            }
            else if (stim->msgType == WVIEW_MSG_TYPE_HILOW_DATA)
            {
                memcpy (&work->mgrId->hilowStore,
                        &((WVIEW_MSG_HILOW_DATA *)stim->msg)->hilowData,
                        sizeof (SENSOR_STORE));

                return state;
            }
            else if (stim->msgType == WVIEW_MSG_TYPE_ARCHIVE_NOTIFY)
            {
                processNewArchiveRecord (work, arcNot);
                return state;
            }

        break;

        case STIM_TIMER:
            if (stim->timerNumber == TIMER_GENERATE)
            {
                //  ... request the next batch of data
                if (requestDataPackets (work) == ERROR)
                {
                    radMsgLog (PRI_HIGH, "htmlRunState: requestDataPackets failed!");
                    return HTML_STATE_ERROR;
                }
    
                work->numDataReceived = 0;
    
                return HTML_STATE_DATA;
            }
            else if (stim->timerNumber == TIMER_DST)
            {
                processDSTTimer (work);
                return state;
            }
    
            return state;
        }

    return state;
}


int htmlDataState (int state, void *stimulus, void *data)
{
    STIM                        *stim = (STIM *)stimulus;
    HTML_WORK                   *work = (HTML_WORK *)data;
    WVIEW_MSG_LOOP_DATA         *loop = (WVIEW_MSG_LOOP_DATA *)stim->msg;
    WVIEW_MSG_HILOW_DATA        *hilow = (WVIEW_MSG_HILOW_DATA *)stim->msg;
    WVIEW_MSG_ARCHIVE_NOTIFY    *arcNot = (WVIEW_MSG_ARCHIVE_NOTIFY *)stim->msg;

    switch (stim->type)
    {
        case STIM_QMSG:
            if (stim->msgType == WVIEW_MSG_TYPE_LOOP_DATA)
            {
                memcpy (&work->mgrId->loopStore, &loop->loopData, sizeof (LOOP_PKT));
                if (++ work->numDataReceived == 2)
                {
                    // we've got the data, generate some html
                    htmlmgrGenerate (work->mgrId);
    
                    return HTML_STATE_RUN;
                }
            }
            else if (stim->msgType == WVIEW_MSG_TYPE_HILOW_DATA)
            {
                memcpy (&work->mgrId->hilowStore, &hilow->hilowData, sizeof (SENSOR_STORE));
                if (++ work->numDataReceived == 2)
                {
                    // we've got the data, generate some html
                    htmlmgrGenerate (work->mgrId);
    
                    return HTML_STATE_RUN;
                }
            }
            else if (stim->msgType == WVIEW_MSG_TYPE_ARCHIVE_NOTIFY)
            {
                processNewArchiveRecord (work, arcNot);
                return state;
            }
    
            return state;

        case STIM_TIMER:
            if (stim->timerNumber == TIMER_DST)
            {
                processDSTTimer (work);
                return state;
            }
    
            return state;

        default:
            return state;
    }

    return state;
}

int htmlErrorState (int state, void *stimulus, void *data)
{
    STIM                *stim = (STIM *)stimulus;
    HTML_WORK           *work = (HTML_WORK *)data;


    switch (stim->type)
    {
    case STIM_QMSG:
    case STIM_TIMER:
        radMsgLog (PRI_STATUS,
                   "htmlErrorState: received stimulus %d", 
                   stim->type);
        break;
    }

    return state;
}
