/*---------------------------------------------------------------------------
 
  FILENAME:
        images.c
 
  PURPOSE:
        Provide the built-in wview image generator utilities.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        01/20/04        M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <termios.h>

/*  ... Library include files
*/
#include <radmsgLog.h>
#include <radtimeUtils.h>

/*  ... Local include files
*/
#include <services.h>
#include <html.h>
#include <htmlMgr.h>
#include <htmlGenerate.h>
#include <glbucket.h>
#include <glchart.h>
#include <glmultichart.h>
#include <images.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/
extern char sampleLabels[MAX_DAILY_NUM_VALUES-1][8];

/*  ... (local) memory declarations
*/
static char *labels[MONTHLY_NUM_VALUES];

static char *monthLabels[12] =
    {
        "Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
    };


//  ... !!!  Define the image generator engines  !!! ...
//  ....................................................

static int generateTemp (HTML_IMG *img)
{
    char        temp[256];

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->isMetricUnits)
    {
        return (htmlGenPngBucket (temp,
                          1,
                          wvutilsConvertFahrenToCelsius (img->mgrWork->loopStore.outTemp),
                          wvutilsConvertFahrenToCelsius (sensorGetDailyLow(img->mgrWork->hilowStore.sensor, SENSOR_OUTTEMP)),
                          wvutilsConvertFahrenToCelsius (sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_OUTTEMP)),
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          -20.0,
                          40.0));
    }
    else
    {
        return (htmlGenPngBucket (temp,
                          0,
                          img->mgrWork->loopStore.outTemp,
                          sensorGetDailyLow(img->mgrWork->hilowStore.sensor, SENSOR_OUTTEMP),
                          sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_OUTTEMP),
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          0.0,
                          99.0));
    }
}

static int generateHumidity (HTML_IMG *img)
{
    char        temp[256];

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    return (htmlGenPngPercentBucket (temp,
                             img->mgrWork->isMetricUnits,
                             (float)img->mgrWork->loopStore.outHumidity,
                             sensorGetDailyLow(img->mgrWork->hilowStore.sensor, SENSOR_OUTHUMID),
                             sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_OUTHUMID),
                             img->title,
                             img->units,
                             img->decimalPlaces));
}

static int generateDewpoint (HTML_IMG *img)
{
    char        temp[256];

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->isMetricUnits)
    {
        return (htmlGenPngBucket (temp,
                          1,
                          wvutilsConvertFahrenToCelsius (img->mgrWork->loopStore.dewpoint),
                          wvutilsConvertFahrenToCelsius (sensorGetDailyLow(img->mgrWork->hilowStore.sensor, SENSOR_DEWPOINT)),
                          wvutilsConvertFahrenToCelsius (sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_DEWPOINT)),
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          -20.0,
                          40.0));
    }
    else
    {
        return (htmlGenPngBucket (temp,
                          0,
                          img->mgrWork->loopStore.dewpoint,
                          sensorGetDailyLow(img->mgrWork->hilowStore.sensor, SENSOR_DEWPOINT),
                          sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_DEWPOINT),
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          0.0,
                          99.0));
    }
}

static int generateWindChill (HTML_IMG *img)
{
    char        temp[256];
    float       low = MIN(img->mgrWork->loopStore.windchill,
                          sensorGetDailyLow(img->mgrWork->hilowStore.sensor, SENSOR_WCHILL));

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->isMetricUnits)
    {
        return (htmlGenPngBucket (temp,
                          1,
                          wvutilsConvertFahrenToCelsius (img->mgrWork->loopStore.windchill),
                          wvutilsConvertFahrenToCelsius (low),
                          (float)GLB_HILOW_NONE,
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          -20.0,
                          40.0));
    }
    else
    {
        return (htmlGenPngBucket (temp,
                          0,
                          img->mgrWork->loopStore.windchill,
                          low,
                          (float)GLB_HILOW_NONE,
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          0.0,
                          99.0));
    }
}

static int generateHeatIndex (HTML_IMG *img)
{
    char        temp[256];
    float       hi = MAX(img->mgrWork->loopStore.heatindex,
                         sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_HINDEX));

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->isMetricUnits)
    {
        return (htmlGenPngBucket (temp,
                          1,
                          wvutilsConvertFahrenToCelsius(img->mgrWork->loopStore.heatindex),
                          (float)GLB_HILOW_NONE,
                          wvutilsConvertFahrenToCelsius (hi),
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          -20.0,
                          40.0));
    }
    else
    {
        return (htmlGenPngBucket (temp,
                          0,
                          img->mgrWork->loopStore.heatindex,
                          (float)GLB_HILOW_NONE,
                          hi,
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          0.0,
                          99.0));
    }
}

static int generateBarometer (HTML_IMG *img)
{
    char        temp[256];

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->isMetricUnits)
    {
        return (htmlGenMetricPngBaromBucket (temp,
                                     wvutilsConvertBPInchesToMillibars (img->mgrWork->loopStore.barometer),
                                     wvutilsConvertBPInchesToMillibars (sensorGetDailyLow(img->mgrWork->hilowStore.sensor, SENSOR_BP)),
                                     wvutilsConvertBPInchesToMillibars (sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_BP)),
                                     img->title,
                                     img->units,
                                     img->decimalPlaces));
    }
    else
    {
        return (htmlGenPngBaromBucket (temp,
                               img->mgrWork->loopStore.barometer,
                               sensorGetDailyLow(img->mgrWork->hilowStore.sensor, SENSOR_BP),
                               sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_BP),
                               img->title,
                               img->units,
                               img->decimalPlaces));
    }
}

static int generateDayRain (HTML_IMG *img)
{
    char        temp[256];
    float       tempfloat;

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->isMetricUnits)
    {
        if (img->mgrWork->isMetricRainMM)
            tempfloat = wvutilsConvertRainInchesToMillimeters (sensorGetCumulative(&img->mgrWork->hilowStore.sensor[STF_DAY][SENSOR_RAIN]));
        else
            tempfloat = wvutilsConvertRainInchesToCentimeters (sensorGetCumulative(&img->mgrWork->hilowStore.sensor[STF_DAY][SENSOR_RAIN]));

        return (htmlGenPngRainBucket (temp,
                              1,
                              tempfloat,
                              (float)GLB_HILOW_NONE,
                              (float)GLB_HILOW_NONE,
                              img->title,
                              img->units,
                              img->decimalPlaces,
                              0.0,
                              0.1));
    }
    else
    {
        return (htmlGenPngRainBucket (temp,
                              0,
                              sensorGetCumulative(&img->mgrWork->hilowStore.sensor[STF_DAY][SENSOR_RAIN]),
                              (float)GLB_HILOW_NONE,
                              (float)GLB_HILOW_NONE,
                              img->title,
                              img->units,
                              img->decimalPlaces,
                              0.0,
                              0.1));
    }
}

static int generateStormRain (HTML_IMG *img)
{
    char        temp[256];
    float       tempfloat;

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->isMetricUnits)
    {
        if (img->mgrWork->isMetricRainMM)
            tempfloat = wvutilsConvertRainInchesToMillimeters (img->mgrWork->loopStore.stormRain);
        else
            tempfloat = wvutilsConvertRainInchesToCentimeters (img->mgrWork->loopStore.stormRain);

        return (htmlGenPngRainBucket (temp,
                              1,
                              tempfloat,
                              (float)GLB_HILOW_NONE,
                              (float)GLB_HILOW_NONE,
                              img->title,
                              img->units,
                              img->decimalPlaces,
                              0.0,
                              0.1));
    }
    else
    {
        return (htmlGenPngRainBucket (temp,
                              0,
                              img->mgrWork->loopStore.stormRain,
                              (float)GLB_HILOW_NONE,
                              (float)GLB_HILOW_NONE,
                              img->title,
                              img->units,
                              img->decimalPlaces,
                              0.0,
                              0.1));
    }
}

static int generateRainRate (HTML_IMG *img)
{
    char        temp[256];
    float       tempfloat, tempfloat2;

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->isMetricUnits)
    {
        if (img->mgrWork->isMetricRainMM)
        {
            tempfloat = wvutilsConvertRainInchesToMillimeters (img->mgrWork->loopStore.rainRate);
            tempfloat2 = wvutilsConvertRainInchesToMillimeters (sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_RAINRATE));
        }
        else
        {
            tempfloat = wvutilsConvertRainInchesToCentimeters (img->mgrWork->loopStore.rainRate);
            tempfloat2 = wvutilsConvertRainInchesToCentimeters (sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_RAINRATE));
        }

        return (htmlGenPngRainBucket (temp,
                              1,
                              tempfloat,
                              (float)GLB_HILOW_NONE,
                              tempfloat2,
                              img->title,
                              img->units,
                              img->decimalPlaces,
                              0.0,
                              0.1));
    }
    else
    {
        return (htmlGenPngRainBucket (temp,
                              0,
                              img->mgrWork->loopStore.rainRate,
                              (float)GLB_HILOW_NONE,
                              sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_RAINRATE),
                              img->title,
                              img->units,
                              img->decimalPlaces,
                              0.0,
                              0.1));
    }
}

static int generateMonthRain (HTML_IMG *img)
{
    char        temp[256];
    float       tempfloat;

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->isMetricUnits)
    {
        if (img->mgrWork->isMetricRainMM)
            tempfloat = wvutilsConvertRainInchesToMillimeters (sensorGetCumulative(&img->mgrWork->hilowStore.sensor[STF_MONTH][SENSOR_RAIN]));
        else
            tempfloat = wvutilsConvertRainInchesToCentimeters (sensorGetCumulative(&img->mgrWork->hilowStore.sensor[STF_MONTH][SENSOR_RAIN]));

        return (htmlGenPngRainBucket (temp,
                              1,
                              tempfloat,
                              (float)GLB_HILOW_NONE,
                              (float)GLB_HILOW_NONE,
                              img->title,
                              img->units,
                              img->decimalPlaces,
                              0.0,
                              0.1));
    }
    else
    {
        return (htmlGenPngRainBucket (temp,
                              0,
                              sensorGetCumulative(&img->mgrWork->hilowStore.sensor[STF_MONTH][SENSOR_RAIN]),
                              (float)GLB_HILOW_NONE,
                              (float)GLB_HILOW_NONE,
                              img->title,
                              img->units,
                              img->decimalPlaces,
                              0.0,
                              0.1));
    }
}

static int generateYearRain (HTML_IMG *img)
{
    char        temp[256];
    float       tempfloat;

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->isMetricUnits)
    {
        if (img->mgrWork->isMetricRainMM)
            tempfloat = wvutilsConvertRainInchesToMillimeters (sensorGetCumulative(&img->mgrWork->hilowStore.sensor[STF_YEAR][SENSOR_RAIN]));
        else
            tempfloat = wvutilsConvertRainInchesToCentimeters (sensorGetCumulative(&img->mgrWork->hilowStore.sensor[STF_YEAR][SENSOR_RAIN]));

        return (htmlGenPngBucket (temp,
                          1,
                          tempfloat,
                          (float)GLB_HILOW_NONE,
                          (float)GLB_HILOW_NONE,
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          0.0,
                          0.5));
    }
    else
    {
        return (htmlGenPngBucket (temp,
                          0,
                          sensorGetCumulative(&img->mgrWork->hilowStore.sensor[STF_YEAR][SENSOR_RAIN]),
                          (float)GLB_HILOW_NONE,
                          (float)GLB_HILOW_NONE,
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          0.0,
                          0.5));
    }
}

static int generateWind (HTML_IMG *img)
{
    char        temp[256];
    int         retVal;

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    retVal = htmlGenPngDialWind (temp,
                    img->mgrWork->isMetricUnits,
                    (int)img->mgrWork->loopStore.windDir,
                    (int)sensorGetDailyWhenHigh(img->mgrWork->hilowStore.sensor, SENSOR_WGUST),
                    (float)img->mgrWork->loopStore.windSpeed,
                    sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_WGUST),
                    img->title);

    return retVal;
}

static int generateTempDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_TEMP],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateTempMonth (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_TEMP],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateTempYear (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_TEMP],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateHumidDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }

    return (htmlGenPngPercentChart (temp,
                            img->mgrWork->isMetricUnits,
                            img->mgrWork->dayValues[HIST_TYPE_HUMID],
                            6,
                            labels,
                            DAILY_NUM_VALUES(img->mgrWork),
                            img->title));
}

static int generateHumidMonth (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngPercentChart (temp,
                            img->mgrWork->isMetricUnits,
                            img->mgrWork->monthValues[HIST_TYPE_HUMID],
                            14,
                            labels,
                            MONTHLY_NUM_VALUES,
                            img->title));
}

static int generateHumidYear (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngPercentChart (temp,
                            img->mgrWork->isMetricUnits,
                            img->mgrWork->yearValues[HIST_TYPE_HUMID],
                            4,
                            labels,
                            YEARLY_NUM_VALUES,
                            img->title));
}

static int generateDewDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_DEW],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateDewMonth (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_DEW],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateDewYear (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_DEW],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateWSpeedDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_WSPEED],
                     6,
                     0,
                     5,
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateWSpeedMonth (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_WSPEED],
                     14,
                     0,
                     5,
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateWSpeedYear (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_WSPEED],
                     4,
                     0,
                     5,
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateWDirDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }

    return (htmlGenPngWDIRChart (temp,
                         img->mgrWork->isMetricUnits,
                         img->mgrWork->dayValues[HIST_TYPE_WDIR],
                         6,
                         labels,
                         DAILY_NUM_VALUES(img->mgrWork),
                         img->title,
                         img->units,
                         img->decimalPlaces));
}

static int generateWDirMonth (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngWDIRChart (temp,
                         img->mgrWork->isMetricUnits,
                         img->mgrWork->monthValues[HIST_TYPE_WDIR],
                         14,
                         labels,
                         MONTHLY_NUM_VALUES,
                         img->title,
                         img->units,
                         img->decimalPlaces));
}

static int generateWDirYear (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngWDIRChart (temp,
                         img->mgrWork->isMetricUnits,
                         img->mgrWork->yearValues[HIST_TYPE_WDIR],
                         4,
                         labels,
                         YEARLY_NUM_VALUES,
                         img->title,
                         img->units,
                         img->decimalPlaces));
}

static int generateHiWSpeedDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_HIWSPEED],
                     6,
                     0,
                     5,
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateHiWSpeedMonth (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_HIWSPEED],
                     14,
                     0,
                     5,
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateHiWSpeedYear (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_HIWSPEED],
                     4,
                     0,
                     5,
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateBaromDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }

    if (img->mgrWork->isMetricUnits)
    {
        return (htmlGenPngChart (temp,
                         1,
                         img->mgrWork->dayValues[HIST_TYPE_BAROM],
                         6,
                         1000.0,
                         1039.9,
                         10.0,
                         labels,
                         DAILY_NUM_VALUES(img->mgrWork),
                         img->title,
                         img->units,
                         img->decimalPlaces));
    }
    else
    {
        return (htmlGenPngChart (temp,
                         0,
                         img->mgrWork->dayValues[HIST_TYPE_BAROM],
                         6,
                         29.7,
                         30.3,
                         0.1,
                         labels,
                         DAILY_NUM_VALUES(img->mgrWork),
                         img->title,
                         img->units,
                         img->decimalPlaces));
    }
}

static int generateBaromMonth (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    if (img->mgrWork->isMetricUnits)
    {
        return (htmlGenPngChart (temp,
                         1,
                         img->mgrWork->monthValues[HIST_TYPE_BAROM],
                         14,
                         1000.0,
                         1039.9,
                         10.0,
                         labels,
                         MONTHLY_NUM_VALUES,
                         img->title,
                         img->units,
                         img->decimalPlaces));
    }
    else
    {
        return (htmlGenPngChart (temp,
                         0,
                         img->mgrWork->monthValues[HIST_TYPE_BAROM],
                         14,
                         29.7,
                         30.3,
                         0.1,
                         labels,
                         MONTHLY_NUM_VALUES,
                         img->title,
                         img->units,
                         img->decimalPlaces));
    }
}

static int generateBaromYear (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    if (img->mgrWork->isMetricUnits)
    {
        return (htmlGenPngChart (temp,
                         1,
                         img->mgrWork->yearValues[HIST_TYPE_BAROM],
                         4,
                         1000.0,
                         1039.9,
                         10.0,
                         labels,
                         YEARLY_NUM_VALUES,
                         img->title,
                         img->units,
                         img->decimalPlaces));
    }
    else
    {
        return (htmlGenPngChart (temp,
                         0,
                         img->mgrWork->yearValues[HIST_TYPE_BAROM],
                         4,
                         29.7,
                         30.3,
                         0.1,
                         labels,
                         YEARLY_NUM_VALUES,
                         img->title,
                         img->units,
                         img->decimalPlaces));
    }
}

static int generateRainDay (HTML_IMG *img)
{
    int         i, j, skipNo;
    char        temp[256];
    float       max = 0;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    skipNo = img->mgrWork->dayStart;
    skipNo %= (60/img->mgrWork->archiveInterval);
    skipNo = (60/img->mgrWork->archiveInterval) - skipNo;

    j = img->mgrWork->dayStart;
    for (i = 0; i < skipNo; i ++)
    {
        if (++ j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;
    }

    for (i = 0; i < DAILY_NUM_VALUES(img->mgrWork)-skipNo; i ++)
    {
        labels[i] = sampleLabels[j];

        if (img->mgrWork->dayValues[HIST_TYPE_RAIN][i+skipNo] > max)
            max = img->mgrWork->dayValues[HIST_TYPE_RAIN][i+skipNo];

        if (++ j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;
    }
    
    // tack on the extra label
    for (i = 0; i < skipNo - 1; i ++)
    {
        if (++ j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;
    }
    labels[DAILY_NUM_VALUES(img->mgrWork)-skipNo] = sampleLabels[j];
    
    return (htmlGenPngBarChart (temp,
                        img->mgrWork->isMetricUnits,
                        &img->mgrWork->dayValues[HIST_TYPE_RAIN][skipNo+1],
                        6,
                        0,
                        MAX(0.1,max),
                        0.01,
                        labels,
                        DAILY_NUM_VALUES(img->mgrWork)-skipNo,
                        img->title,
                        img->units,
                        img->decimalPlaces,
                        60/img->mgrWork->archiveInterval,
                        24,
                        60/img->mgrWork->archiveInterval));
}

static int generateRainMonth (HTML_IMG *img)
{
    int         i, skipNo;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;
    float       max = 0;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, skipNo = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        if (loctime.tm_hour == 0)
        {
            break;
        }
        else
        {
            skipNo ++;
            ntime += WV_SECONDS_IN_HOUR;
        }
    }

    for (i = 0; i < MONTHLY_NUM_VALUES-skipNo; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];

        ntime += WV_SECONDS_IN_HOUR;
        if (img->mgrWork->monthValues[HIST_TYPE_RAIN][i] > max)
            max = img->mgrWork->monthValues[HIST_TYPE_RAIN][i];
    }

    // tack on the extra label
    if (skipNo > 0)
    {
        ntime += WV_SECONDS_IN_DAY;
        localtime_r (&ntime, &loctime);
        sprintf (lbls[MONTHLY_NUM_VALUES-skipNo], "%d", loctime.tm_mday);
        labels[MONTHLY_NUM_VALUES-skipNo] = lbls[MONTHLY_NUM_VALUES-skipNo];
    }

    return (htmlGenPngBarChart (temp,
                        img->mgrWork->isMetricUnits,
                        &img->mgrWork->monthValues[HIST_TYPE_RAIN][skipNo],
                        14,
                        0,
                        MAX(0.1,max),
                        0.1,
                        labels,
                        MONTHLY_NUM_VALUES-skipNo,
                        img->title,
                        img->units,
                        img->decimalPlaces,
                        24,
                        28,
                        24));
}

static int generateRainYear (HTML_IMG *img)
{
    int         i, j, skipNo;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;
    float       max = 0;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    // move forward 1 day to land on 52 weeks ago
    ntime += WV_SECONDS_IN_DAY;
    skipNo = 1;
    
    // if we are sitting on Sunday, bump it another day
    localtime_r (&ntime, &loctime);
    if (loctime.tm_wday == 0)
    {
        ntime += WV_SECONDS_IN_DAY;
        skipNo ++;
    }
    
    // move to the next Sunday
    for (i = 0; i < 7; i ++)
    {
        localtime_r (&ntime, &loctime);
        if (loctime.tm_wday == 0)
        {
            break;
        }
        else
        {
            skipNo ++;
            ntime += WV_SECONDS_IN_DAY;
        }
    }

    for (i = 0; i < YEARLY_NUM_VALUES-skipNo; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];

        ntime += WV_SECONDS_IN_DAY;
        if (img->mgrWork->yearValues[HIST_TYPE_RAIN][i] > max)
            max = img->mgrWork->yearValues[HIST_TYPE_RAIN][i];
    }

    for (j = 0; j < 7; j ++)
    {
        localtime_r (&ntime, &loctime);
        if (loctime.tm_wday == 0)
        {
            break;
        }
        else
        {
            ntime += WV_SECONDS_IN_DAY;
        }
    }

    localtime_r (&ntime, &loctime);
    sprintf (lbls[YEARLY_NUM_VALUES-skipNo], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
    labels[YEARLY_NUM_VALUES-skipNo] = lbls[YEARLY_NUM_VALUES-skipNo];

    return (htmlGenPngBarChart (temp,
                        img->mgrWork->isMetricUnits,
                        &img->mgrWork->yearValues[HIST_TYPE_RAIN][skipNo],
                        4,
                        0,
                        MAX(0.1,max),
                        0.1,
                        labels,
                        YEARLY_NUM_VALUES-skipNo,
                        img->title,
                        img->units,
                        img->decimalPlaces,
                        7,
                        52,
                        7));
}

static int generateWChillDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_WCHILL],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateWChillMonth (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_WCHILL],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateWChillYear (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_WCHILL],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateHIndexDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_HINDEX],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateHIndexMonth (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_HINDEX],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateHIndexYear (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_HINDEX],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


////////////////////////////////////////////////////////////////////////////
//  Composite Charts

// utility to determine max of 4 values
static void setMax(float *maxStore, float v1, float v2, float v3, float v4)
{
    if (v1 > *maxStore)
        *maxStore = v1;
    if (v2 > *maxStore)
        *maxStore = v2;
    if (v3 > *maxStore)
        *maxStore = v3;
    if (v4 > *maxStore)
        *maxStore = v4;
}

// utility to determine min of 4 values
static void setMin(float *minStore, float v1, float v2, float v3, float v4)
{
    if (v1 < *minStore)
        *minStore = v1;
    if (v2 < *minStore)
        *minStore = v2;
    if (v3 < *minStore)
        *minStore = v3;
    if (v4 < *minStore)
        *minStore = v4;
}


static int generateCompositeTempDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];
    MC_DATASET  datasets[2];
    float       min = 10000.0, max = -10000.0;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    datasets[0].lineColor = CHART_DFLT_LINE;
    datasets[1].lineColor = CHART_DFLT_2ND_LINE;
    
    memset (datasets[0].legend, 0, 16);
    memset (datasets[1].legend, 0, 16);
    for (i = 0; i < 15 && i < strlen (img->title); i ++)
    {
        if (img->title[i] == '/')
            break;
    
        datasets[0].legend[i] = img->title[i];
    }
    if (i < strlen (img->title) - 1)
    {
        strncpy (datasets[1].legend, &img->title[i+1], 15);
    }
    
    for (i = 0; i < DAILY_NUM_VALUES(img->mgrWork); i ++)
    {
        datasets[0].valueset[i] = img->mgrWork->dayValues[HIST_TYPE_TEMP][i];
        datasets[1].valueset[i] = img->mgrWork->dayValues[HIST_TYPE_DEW][i];

        if (img->mgrWork->dayValues[HIST_TYPE_TEMP][i] == GLC_VALUE_NONE ||
            img->mgrWork->dayValues[HIST_TYPE_DEW][i] == GLC_VALUE_NONE ||
            img->mgrWork->dayValues[HIST_TYPE_WCHILL][i] == GLC_VALUE_NONE ||
            img->mgrWork->dayValues[HIST_TYPE_HINDEX][i] == GLC_VALUE_NONE)
        {
            continue;
        }
                
        setMin (&min, 
                img->mgrWork->dayValues[HIST_TYPE_TEMP][i],
                img->mgrWork->dayValues[HIST_TYPE_DEW][i],
                img->mgrWork->dayValues[HIST_TYPE_WCHILL][i],
                img->mgrWork->dayValues[HIST_TYPE_HINDEX][i]);
                
        setMax (&max, 
                img->mgrWork->dayValues[HIST_TYPE_TEMP][i],
                img->mgrWork->dayValues[HIST_TYPE_DEW][i],
                img->mgrWork->dayValues[HIST_TYPE_WCHILL][i],
                img->mgrWork->dayValues[HIST_TYPE_HINDEX][i]);
    }

    // if there is no data yet, pick default min/max
    if (min > max)
    {
        min = CHART_MIN_TEMP(img->mgrWork->isMetricUnits);
        max = CHART_MAX_TEMP(img->mgrWork->isMetricUnits);
    }

    return (htmlGenPngMultiChart (temp,
                          img->mgrWork->isMetricUnits,
                          datasets,
                          2,
                          6,
                          min,
                          max,
                          1.0,
                          labels,
                          DAILY_NUM_VALUES(img->mgrWork),
                          img->units,
                          img->decimalPlaces));
}

static int generateCompositeHIndexWChillDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];
    MC_DATASET  datasets[2];
    float       min = 10000.0, max = -10000.0;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    datasets[0].lineColor = CHART_DFLT_LINE;
    datasets[1].lineColor = CHART_DFLT_2ND_LINE;
    
    memset (datasets[0].legend, 0, 16);
    memset (datasets[1].legend, 0, 16);
    for (i = 0; i < 15 && i < strlen (img->title); i ++)
    {
        if (img->title[i] == '/')
            break;
    
        datasets[0].legend[i] = img->title[i];
    }
    if (i < strlen (img->title) - 1)
    {
        strncpy (datasets[1].legend, &img->title[i+1], 15);
    }
    
    for (i = 0; i < DAILY_NUM_VALUES(img->mgrWork); i ++)
    {
        datasets[0].valueset[i] = img->mgrWork->dayValues[HIST_TYPE_WCHILL][i];
        datasets[1].valueset[i] = img->mgrWork->dayValues[HIST_TYPE_HINDEX][i];
        
        if (img->mgrWork->dayValues[HIST_TYPE_TEMP][i] == GLC_VALUE_NONE ||
            img->mgrWork->dayValues[HIST_TYPE_DEW][i] == GLC_VALUE_NONE ||
            img->mgrWork->dayValues[HIST_TYPE_WCHILL][i] == GLC_VALUE_NONE ||
            img->mgrWork->dayValues[HIST_TYPE_HINDEX][i] == GLC_VALUE_NONE)
        {
            continue;
        }
        
        setMin (&min, 
                img->mgrWork->dayValues[HIST_TYPE_TEMP][i],
                img->mgrWork->dayValues[HIST_TYPE_DEW][i],
                img->mgrWork->dayValues[HIST_TYPE_WCHILL][i],
                img->mgrWork->dayValues[HIST_TYPE_HINDEX][i]);
                
        setMax (&max, 
                img->mgrWork->dayValues[HIST_TYPE_TEMP][i],
                img->mgrWork->dayValues[HIST_TYPE_DEW][i],
                img->mgrWork->dayValues[HIST_TYPE_WCHILL][i],
                img->mgrWork->dayValues[HIST_TYPE_HINDEX][i]);
    }

    // if there is no data yet, pick default min/max
    if (min > max)
    {
        min = CHART_MIN_TEMP(img->mgrWork->isMetricUnits);
        max = CHART_MAX_TEMP(img->mgrWork->isMetricUnits);
    }

    return (htmlGenPngMultiChart (temp,
                          img->mgrWork->isMetricUnits,
                          datasets,
                          2,
                          6,
                          min,
                          max,
                          1.0,
                          labels,
                          DAILY_NUM_VALUES(img->mgrWork),
                          img->units,
                          img->decimalPlaces));
}

static int generateCompositeInTempHumidDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];
    MC_DATASET  datasets[2];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    datasets[0].lineColor = CHART_DFLT_LINE;
    datasets[1].lineColor = CHART_DFLT_2ND_LINE;
    
    memset (datasets[0].legend, 0, 16);
    memset (datasets[1].legend, 0, 16);
    for (i = 0; i < 15 && i < strlen (img->title); i ++)
    {
        if (img->title[i] == '/')
            break;
    
        datasets[0].legend[i] = img->title[i];
    }
    if (i < strlen (img->title) - 1)
    {
        strncpy (datasets[1].legend, &img->title[i+1], 15);
    }
    
    for (i = 0; i < DAILY_NUM_VALUES(img->mgrWork); i ++)
    {
        datasets[0].valueset[i] = img->mgrWork->dayValues[HIST_TYPE_IN_TEMP][i];
        datasets[1].valueset[i] = img->mgrWork->dayValues[HIST_TYPE_IN_HUMID][i];
    }

    return (htmlGenPngMultiChart (temp,
                          img->mgrWork->isMetricUnits,
                          datasets,
                          2,
                          6,
                          CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                          CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                          1.0,
                          labels,
                          DAILY_NUM_VALUES(img->mgrWork),
                          img->units,
                          img->decimalPlaces));
}

static int generateCompositeTempMonth (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];
    char        lbls[MONTHLY_NUM_VALUES][8];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;
    float       min = 10000.0, max = -10000.0;
    MC_DATASET  datasets[2];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }
    
    datasets[0].lineColor = CHART_DFLT_LINE;
    datasets[1].lineColor = CHART_DFLT_2ND_LINE;
    
    memset (datasets[0].legend, 0, 16);
    memset (datasets[1].legend, 0, 16);
    for (i = 0; i < 15 && i < strlen (img->title); i ++)
    {
        if (img->title[i] == '/')
            break;
    
        datasets[0].legend[i] = img->title[i];
    }
    if (i < strlen (img->title) - 1)
    {
        strncpy (datasets[1].legend, &img->title[i+1], 15);
    }
    
    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        datasets[0].valueset[i] = img->mgrWork->monthValues[HIST_TYPE_TEMP][i];
        datasets[1].valueset[i] = img->mgrWork->monthValues[HIST_TYPE_DEW][i];

        if (img->mgrWork->monthValues[HIST_TYPE_TEMP][i] == GLC_VALUE_NONE ||
            img->mgrWork->monthValues[HIST_TYPE_DEW][i] == GLC_VALUE_NONE ||
            img->mgrWork->monthValues[HIST_TYPE_WCHILL][i] == GLC_VALUE_NONE ||
            img->mgrWork->monthValues[HIST_TYPE_HINDEX][i] == GLC_VALUE_NONE)
        {
            continue;
        }
        
        setMin (&min, 
                img->mgrWork->monthValues[HIST_TYPE_TEMP][i],
                img->mgrWork->monthValues[HIST_TYPE_DEW][i],
                img->mgrWork->monthValues[HIST_TYPE_WCHILL][i],
                img->mgrWork->monthValues[HIST_TYPE_HINDEX][i]);
                
        setMax (&max, 
                img->mgrWork->monthValues[HIST_TYPE_TEMP][i],
                img->mgrWork->monthValues[HIST_TYPE_DEW][i],
                img->mgrWork->monthValues[HIST_TYPE_WCHILL][i],
                img->mgrWork->monthValues[HIST_TYPE_HINDEX][i]);
    }

    // if there is no data yet, pick default min/max
    if (min > max)
    {
        min = CHART_MIN_TEMP(img->mgrWork->isMetricUnits);
        max = CHART_MAX_TEMP(img->mgrWork->isMetricUnits);
    }

    return (htmlGenPngMultiChart (temp,
                          img->mgrWork->isMetricUnits,
                          datasets,
                          2,
                          14,
                          min,
                          max,
                          1.0,
                          labels,
                          MONTHLY_NUM_VALUES,
                          img->units,
                          img->decimalPlaces));
}

static int generateCompositeHIndexWChillMonth (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];
    char        lbls[MONTHLY_NUM_VALUES][8];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;
    float       min = 10000.0, max = -10000.0;
    MC_DATASET  datasets[2];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }
    
    datasets[0].lineColor = CHART_DFLT_LINE;
    datasets[1].lineColor = CHART_DFLT_2ND_LINE;
    
    memset (datasets[0].legend, 0, 16);
    memset (datasets[1].legend, 0, 16);
    for (i = 0; i < 15 && i < strlen (img->title); i ++)
    {
        if (img->title[i] == '/')
            break;
    
        datasets[0].legend[i] = img->title[i];
    }
    if (i < strlen (img->title) - 1)
    {
        strncpy (datasets[1].legend, &img->title[i+1], 15);
    }
    
    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        datasets[0].valueset[i] = img->mgrWork->monthValues[HIST_TYPE_WCHILL][i];
        datasets[1].valueset[i] = img->mgrWork->monthValues[HIST_TYPE_HINDEX][i];
        
        if (img->mgrWork->monthValues[HIST_TYPE_TEMP][i] == GLC_VALUE_NONE ||
            img->mgrWork->monthValues[HIST_TYPE_DEW][i] == GLC_VALUE_NONE ||
            img->mgrWork->monthValues[HIST_TYPE_WCHILL][i] == GLC_VALUE_NONE ||
            img->mgrWork->monthValues[HIST_TYPE_HINDEX][i] == GLC_VALUE_NONE)
        {
            continue;
        }
        
        setMin (&min, 
                img->mgrWork->monthValues[HIST_TYPE_TEMP][i],
                img->mgrWork->monthValues[HIST_TYPE_DEW][i],
                img->mgrWork->monthValues[HIST_TYPE_WCHILL][i],
                img->mgrWork->monthValues[HIST_TYPE_HINDEX][i]);
                
        setMax (&max, 
                img->mgrWork->monthValues[HIST_TYPE_TEMP][i],
                img->mgrWork->monthValues[HIST_TYPE_DEW][i],
                img->mgrWork->monthValues[HIST_TYPE_WCHILL][i],
                img->mgrWork->monthValues[HIST_TYPE_HINDEX][i]);
    }

    // if there is no data yet, pick default min/max
    if (min > max)
    {
        min = CHART_MIN_TEMP(img->mgrWork->isMetricUnits);
        max = CHART_MAX_TEMP(img->mgrWork->isMetricUnits);
    }

    return (htmlGenPngMultiChart (temp,
                          img->mgrWork->isMetricUnits,
                          datasets,
                          2,
                          14,
                          min,
                          max,
                          1.0,
                          labels,
                          MONTHLY_NUM_VALUES,
                          img->units,
                          img->decimalPlaces));
}

static int generateCompositeInTempHumidMonth (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];
    char        lbls[MONTHLY_NUM_VALUES][8];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;
    MC_DATASET  datasets[2];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }
    
    datasets[0].lineColor = CHART_DFLT_LINE;
    datasets[1].lineColor = CHART_DFLT_2ND_LINE;
    
    memset (datasets[0].legend, 0, 16);
    memset (datasets[1].legend, 0, 16);
    for (i = 0; i < 15 && i < strlen (img->title); i ++)
    {
        if (img->title[i] == '/')
            break;
    
        datasets[0].legend[i] = img->title[i];
    }
    if (i < strlen (img->title) - 1)
    {
        strncpy (datasets[1].legend, &img->title[i+1], 15);
    }
    
    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        datasets[0].valueset[i] = img->mgrWork->monthValues[HIST_TYPE_IN_TEMP][i];
        datasets[1].valueset[i] = img->mgrWork->monthValues[HIST_TYPE_IN_HUMID][i];
    }

    return (htmlGenPngMultiChart (temp,
                          img->mgrWork->isMetricUnits,
                          datasets,
                          2,
                          14,
                          CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                          CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                          1.0,
                          labels,
                          MONTHLY_NUM_VALUES,
                          img->units,
                          img->decimalPlaces));
}


//////////////////////////////////////////////////////////////////////////////
//////////////  E X T E N D E D   D A T A   B E G I N  ///////////////////////
//////////////////////////////////////////////////////////////////////////////

// first a few buckets for common extended items

static int generateUV (HTML_IMG *img)
{
    char        temp[256];

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    return (htmlGenPngBucket (temp,
                      0,
                      (float)img->mgrWork->loopStore.UV/10.0,
                      (float)GLB_HILOW_NONE,
                      (float)GLB_HILOW_NONE,
                      img->title,
                      img->units,
                      img->decimalPlaces,
                      0.0,
                      4.0));
}

static int generateRadiation (HTML_IMG *img)
{
    char        temp[256];

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    return (htmlGenPngBucket (temp,
                      0,
                      (float)img->mgrWork->loopStore.radiation,
                      (float)GLB_HILOW_NONE,
                      (float)GLB_HILOW_NONE,
                      img->title,
                      img->units,
                      img->decimalPlaces,
                      0.0,
                      4.0));
}

static int generateET (HTML_IMG *img)
{
    char        temp[256];
    float       tempfloat;

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->isMetricUnits)
    {
        if (img->mgrWork->isMetricRainMM)
            tempfloat = wvutilsConvertRainInchesToMillimeters (img->mgrWork->loopStore.dayET);
        else
            tempfloat = wvutilsConvertRainInchesToCentimeters (img->mgrWork->loopStore.dayET);

        return (htmlGenPngBucket (temp,
                          1,
                          tempfloat,
                          (float)GLB_HILOW_NONE,
                          (float)GLB_HILOW_NONE,
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          0.0,
                          0.01));
    }
    else
    {
        return (htmlGenPngBucket (temp,
                          0,
                          img->mgrWork->loopStore.dayET,
                          (float)GLB_HILOW_NONE,
                          (float)GLB_HILOW_NONE,
                          img->title,
                          img->units,
                          img->decimalPlaces,
                          0.0,
                          0.01));
    }
}

static int generateRadiationDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_RADIATION],
                     6,
                     0,
                     10,
                     10,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateRadiationMonth (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_RADIATION],
                     14,
                     0,
                     100,
                     100,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateRadiationYear (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_RADIATION],
                     4,
                     0,
                     100,
                     100,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateUVDay (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_UV],
                     6,
                     0,
                     5,
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateUVMonth (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_UV],
                     14,
                     0,
                     5,
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateUVYear (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_UV],
                     4,
                     0,
                     5,
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateETDay (HTML_IMG *img)
{
    int         i, j, skipNo;
    char        temp[256];
    float       max = 0;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    skipNo = img->mgrWork->dayStart;
    skipNo %= (60/img->mgrWork->archiveInterval);
    skipNo = (60/img->mgrWork->archiveInterval) - skipNo;
    
    j = img->mgrWork->dayStart;
    for (i = 0; i < skipNo; i ++)
    {
        if (++ j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;
    }

    for (i = 0; i < DAILY_NUM_VALUES(img->mgrWork)-skipNo; i ++)
    {
        labels[i] = sampleLabels[j];

        if (img->mgrWork->dayValues[HIST_TYPE_ET][i+skipNo] > max)
            max = img->mgrWork->dayValues[HIST_TYPE_ET][i+skipNo];

        if (++ j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;
    }
    
    // tack on the extra label
    for (i = 0; i < skipNo - 1; i ++)
    {
        if (++ j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;
    }
    labels[DAILY_NUM_VALUES(img->mgrWork)-skipNo] = sampleLabels[j];
    
    // add 2 to the skipNo since the VP console burps out ET at :00 -OR- :05 !!!
    return (htmlGenPngBarChart (temp,
                        img->mgrWork->isMetricUnits,
                        &img->mgrWork->dayValues[HIST_TYPE_ET][skipNo+2],
                        6,
                        0,
                        MAX(0.03,max),
                        0.01,
                        labels,
                        DAILY_NUM_VALUES(img->mgrWork)-skipNo,
                        img->title,
                        img->units,
                        img->decimalPlaces,
                        60/img->mgrWork->archiveInterval,
                        24,
                        60/img->mgrWork->archiveInterval));
}

static int generateETMonth (HTML_IMG *img)
{
    int         i, skipNo;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;
    float       max = 0;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, skipNo = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        if (loctime.tm_hour == 0)
        {
            break;
        }
        else
        {
            skipNo ++;
            ntime += WV_SECONDS_IN_HOUR;
        }
    }

    for (i = 0; i < MONTHLY_NUM_VALUES-skipNo; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];

        ntime += WV_SECONDS_IN_HOUR;
        if (img->mgrWork->monthValues[HIST_TYPE_ET][i] > max)
            max = img->mgrWork->monthValues[HIST_TYPE_ET][i];
    }

    // tack on the extra label
    if (skipNo > 0)
    {
        ntime += WV_SECONDS_IN_DAY;
        localtime_r (&ntime, &loctime);
        sprintf (lbls[MONTHLY_NUM_VALUES-skipNo], "%d", loctime.tm_mday);
        labels[MONTHLY_NUM_VALUES-skipNo] = lbls[MONTHLY_NUM_VALUES-skipNo];
    }

    return (htmlGenPngBarChart (temp,
                        img->mgrWork->isMetricUnits,
                        &img->mgrWork->monthValues[HIST_TYPE_ET][skipNo],
                        14,
                        0,
                        MAX(0.03,max),
                        0.01,
                        labels,
                        MONTHLY_NUM_VALUES-skipNo,
                        img->title,
                        img->units,
                        img->decimalPlaces,
                        24,
                        28,
                        24));
}

static int generateETYear (HTML_IMG *img)
{
    int         i, j, skipNo;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;
    float       max = 0;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    // move forward 1 day to land on 52 weeks ago
    ntime += WV_SECONDS_IN_DAY;
    skipNo = 1;
    
    // if we are sitting on Sunday, bump it another day
    localtime_r (&ntime, &loctime);
    if (loctime.tm_wday == 0)
    {
        ntime += WV_SECONDS_IN_DAY;
        skipNo ++;
    }
    
    // move to the next Sunday
    for (i = 0; i < 7; i ++)
    {
        localtime_r (&ntime, &loctime);
        if (loctime.tm_wday == 0)
        {
            break;
        }
        else
        {
            skipNo ++;
            ntime += WV_SECONDS_IN_DAY;
        }
    }

    for (i = 0; i < YEARLY_NUM_VALUES-skipNo; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];

        ntime += WV_SECONDS_IN_DAY;
        if (img->mgrWork->yearValues[HIST_TYPE_ET][i] > max)
            max = img->mgrWork->yearValues[HIST_TYPE_ET][i];
    }

    for (j = 0; j < 7; j ++)
    {
        localtime_r (&ntime, &loctime);
        if (loctime.tm_wday == 0)
        {
            break;
        }
        else
        {
            ntime += WV_SECONDS_IN_DAY;
        }
    }
    
    localtime_r (&ntime, &loctime);
    sprintf (lbls[YEARLY_NUM_VALUES-skipNo], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
    labels[YEARLY_NUM_VALUES-skipNo] = lbls[YEARLY_NUM_VALUES-skipNo];

    return (htmlGenPngBarChart (temp,
                        img->mgrWork->isMetricUnits,
                        &img->mgrWork->yearValues[HIST_TYPE_ET][skipNo],
                        4,
                        0,
                        MAX(0.3,max),
                        0.1,
                        labels,
                        YEARLY_NUM_VALUES-skipNo,
                        img->title,
                        img->units,
                        img->decimalPlaces,
                        7,
                        52,
                        7));
}


static int generateLeafTemp1Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_LEAFTEMP1],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateLeafTemp1Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_LEAFTEMP1],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateLeafTemp1Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_LEAFTEMP1],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateLeafTemp2Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_LEAFTEMP2],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateLeafTemp2Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_LEAFTEMP2],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateLeafTemp2Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_LEAFTEMP2],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateLeafWetness1Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_LEAFWET1],
                     6,
                     0,
                     5,
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateLeafWetness1Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_LEAFWET1],
                     14,
                     0,
                     5,
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));

    return OK;
}

static int generateLeafWetness1Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_LEAFWET1],
                     4,
                     0,
                     5,
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateLeafWetness2Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_LEAFWET2],
                     6,
                     0,
                     5,
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateLeafWetness2Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_LEAFWET2],
                     14,
                     0,
                     5,
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateLeafWetness2Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_LEAFWET2],
                     4,
                     0,
                     5,
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateSoilTemp1Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_SOILTEMP1],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilTemp1Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_SOILTEMP1],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilTemp1Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_SOILTEMP1],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateSoilTemp2Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_SOILTEMP2],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilTemp2Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_SOILTEMP2],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilTemp2Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_SOILTEMP2],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateSoilTemp3Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_SOILTEMP3],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilTemp3Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_SOILTEMP3],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilTemp3Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_SOILTEMP3],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateSoilTemp4Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_SOILTEMP4],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilTemp4Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_SOILTEMP4],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilTemp4Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_SOILTEMP4],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateExtraHumid1Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }

    return (htmlGenPngPercentChart (temp,
                            img->mgrWork->isMetricUnits,
                            img->mgrWork->dayValues[HIST_TYPE_EXTRAHUMID1],
                            6,
                            labels,
                            DAILY_NUM_VALUES(img->mgrWork),
                            img->title));
}

static int generateExtraHumid1Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngPercentChart (temp,
                            img->mgrWork->isMetricUnits,
                            img->mgrWork->monthValues[HIST_TYPE_EXTRAHUMID1],
                            14,
                            labels,
                            MONTHLY_NUM_VALUES,
                            img->title));
}

static int generateExtraHumid1Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngPercentChart (temp,
                            img->mgrWork->isMetricUnits,
                            img->mgrWork->yearValues[HIST_TYPE_EXTRAHUMID1],
                            4,
                            labels,
                            YEARLY_NUM_VALUES,
                            img->title));
}


static int generateExtraHumid2Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }

    return (htmlGenPngPercentChart (temp,
                            img->mgrWork->isMetricUnits,
                            img->mgrWork->dayValues[HIST_TYPE_EXTRAHUMID2],
                            6,
                            labels,
                            DAILY_NUM_VALUES(img->mgrWork),
                            img->title));
}

static int generateExtraHumid2Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngPercentChart (temp,
                            img->mgrWork->isMetricUnits,
                            img->mgrWork->monthValues[HIST_TYPE_EXTRAHUMID2],
                            14,
                            labels,
                            MONTHLY_NUM_VALUES,
                            img->title));
}

static int generateExtraHumid2Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngPercentChart (temp,
                            img->mgrWork->isMetricUnits,
                            img->mgrWork->yearValues[HIST_TYPE_EXTRAHUMID2],
                            4,
                            labels,
                            YEARLY_NUM_VALUES,
                            img->title));
}


static int generateExtraTemp1Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_EXTRATEMP1],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateExtraTemp1Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_EXTRATEMP1],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateExtraTemp1Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_EXTRATEMP1],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateExtraTemp2Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_EXTRATEMP2],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateExtraTemp2Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_EXTRATEMP2],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateExtraTemp2Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_EXTRATEMP2],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateExtraTemp3Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_EXTRATEMP3],
                     6,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateExtraTemp3Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_EXTRATEMP3],
                     14,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateExtraTemp3Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_EXTRATEMP3],
                     4,
                     CHART_MIN_TEMP(img->mgrWork->isMetricUnits),
                     CHART_MAX_TEMP(img->mgrWork->isMetricUnits),
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateSoilMoisture1Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_SOILMOIST1],
                     6,
                     0,
                     5,
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilMoisture1Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_SOILMOIST1],
                     14,
                     0,
                     5,
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilMoisture1Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_SOILMOIST1],
                     4,
                     0,
                     5,
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateSoilMoisture2Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_SOILMOIST2],
                     6,
                     0,
                     5,
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilMoisture2Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_SOILMOIST2],
                     14,
                     0,
                     5,
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilMoisture2Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_SOILMOIST2],
                     4,
                     0,
                     5,
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateSoilMoisture3Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_SOILMOIST3],
                     6,
                     0,
                     5,
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilMoisture3Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_SOILMOIST3],
                     14,
                     0,
                     5,
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilMoisture3Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_SOILMOIST3],
                     4,
                     0,
                     5,
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}


static int generateSoilMoisture4Day (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }
    
    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->dayValues[HIST_TYPE_SOILMOIST4],
                     6,
                     0,
                     5,
                     1.0,
                     labels,
                     DAILY_NUM_VALUES(img->mgrWork),
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilMoisture4Month (HTML_IMG *img)
{
    int         i;
    char        lbls[MONTHLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->monthStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_HOUR))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < MONTHLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%d", loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_HOUR;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->monthValues[HIST_TYPE_SOILMOIST4],
                     14,
                     0,
                     5,
                     1.0,
                     labels,
                     MONTHLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

static int generateSoilMoisture4Year (HTML_IMG *img)
{
    int         i;
    char        lbls[YEARLY_NUM_VALUES][8];
    char        temp[256];
    time_t      ntime = img->mgrWork->yearStartTime_T;
    struct tm   loctime;

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_DAY))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0; i < YEARLY_NUM_VALUES; i ++)
    {
        localtime_r (&ntime, &loctime);
        sprintf (lbls[i], "%s %d", monthLabels[loctime.tm_mon], loctime.tm_mday);
        labels[i] = lbls[i];
        ntime += WV_SECONDS_IN_DAY;
    }

    return (htmlGenPngChart (temp,
                     img->mgrWork->isMetricUnits,
                     img->mgrWork->yearValues[HIST_TYPE_SOILMOIST4],
                     4,
                     0,
                     5,
                     1.0,
                     labels,
                     YEARLY_NUM_VALUES,
                     img->title,
                     img->units,
                     img->decimalPlaces));
}

//////////////////////////////////////////////////////////////////////////////
//////////////  E X T E N D E D   D A T A   E N D      ///////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
//////////////  D I A L   I M A G E S   S T A R T      ///////////////////////
//////////////////////////////////////////////////////////////////////////////

static int generateDialTemp (HTML_IMG *img)
{
    char        temp[256];
    float       feelslike;

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    if (img->mgrWork->loopStore.windchill < img->mgrWork->loopStore.outTemp)
    {
        feelslike = img->mgrWork->loopStore.windchill;
    }
    else if (img->mgrWork->loopStore.heatindex != img->mgrWork->loopStore.outTemp)
    {
        feelslike = img->mgrWork->loopStore.heatindex;
    }
    else
    {
        feelslike = img->mgrWork->loopStore.outTemp;
    }
    
    return (htmlGenPngDialTemperature (temp,
                                       img->mgrWork->isMetricUnits,
                                       img->mgrWork->loopStore.outTemp,
                                       sensorGetDailyLow(img->mgrWork->hilowStore.sensor, SENSOR_OUTTEMP),
                                       sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_OUTTEMP),
                                       feelslike,
                                       img->title,
                                       img->units));
}

static int generateDialHumidity (HTML_IMG *img)
{
    char        temp[256];

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    return (htmlGenPngDialHumidity (temp,
                                    (float)img->mgrWork->loopStore.outHumidity,
                                    sensorGetDailyLow(img->mgrWork->hilowStore.sensor, SENSOR_OUTHUMID),
                                    sensorGetDailyHigh(img->mgrWork->hilowStore.sensor, SENSOR_OUTHUMID),
                                    img->title));
}


static int generateDialNetRainDay (HTML_IMG *img)
{
    char        temp[256];

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    return (htmlGenPngDialNetRain (temp,
                                   img->mgrWork->isMetricUnits,
                                   img->mgrWork->isMetricRainMM,
                                   img->mgrWork->loopStore.dayRain,
                                   img->mgrWork->loopStore.dayET,
                                   img->title,
                                   img->units));
}


static int generateDialNetRainMonth (HTML_IMG *img)
{
    char        temp[256];

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    return (htmlGenPngDialNetRain (temp,
                                   img->mgrWork->isMetricUnits,
                                   img->mgrWork->isMetricRainMM,
                                   img->mgrWork->loopStore.monthRain,
                                   img->mgrWork->loopStore.monthET,
                                   img->title,
                                   img->units));
}


static int generateDialNetRainYear (HTML_IMG *img)
{
    char        temp[256];

    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    return (htmlGenPngDialNetRain (temp,
                                   img->mgrWork->isMetricUnits,
                                   img->mgrWork->isMetricRainMM,
                                   img->mgrWork->loopStore.yearRain,
                                   img->mgrWork->loopStore.yearET,
                                   img->title,
                                   img->units));
}



//////////////////////////////////////////////////////////////////////////////
//////////////  D I A L   I M A G E S   S T O P        ///////////////////////
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////
//////////////  D I A G N O S T I C S   S T A R T      ///////////////////////
//////////////////////////////////////////////////////////////////////////////

static int generateRxCheck (HTML_IMG *img)
{
    int         i, j;
    char        temp[256];

    if (!(img->mgrWork->newArchiveMask & NEW_ARCHIVE_SAMPLE))
    {
        // nothing for us here
        return ERROR_ABORT;
    }
    
    sprintf (temp, "%s/%s", img->mgrWork->imagePath, img->fname);

    for (i = 0, j = img->mgrWork->dayStart; i < DAILY_NUM_VALUES(img->mgrWork); i ++, j ++)
    {
        if (j >= DAILY_NUM_VALUES(img->mgrWork)-1)
            j = 0;

        labels[i] = sampleLabels[j];
    }

    return (htmlGenPngPercentChart (temp,
                            img->mgrWork->isMetricUnits,
                            img->mgrWork->dayValues[HIST_TYPE_RXCHECK],
                            6,
                            labels,
                            DAILY_NUM_VALUES(img->mgrWork),
                            img->title));
}

//////////////////////////////////////////////////////////////////////////////
//////////////  D I A G N O S T I C S   S T O P        ///////////////////////
//////////////////////////////////////////////////////////////////////////////

//  ... this is the generator jump table
//  ... indexes from images.conf are used to index into this table
//  ... Thus, order is VERY important
int (*images_generators[]) (HTML_IMG *img) =
    {
        generateTemp,                           // 0
        generateHumidity,
        generateDewpoint,
        generateWindChill,
        generateHeatIndex,
        generateBarometer,
        generateDayRain,
        generateStormRain,
        generateRainRate,
        generateMonthRain,
        generateYearRain,                       // 10
        generateWind,
        generateTempDay,
        generateTempMonth,
        generateTempYear,
        generateHumidDay,
        generateHumidMonth,
        generateHumidYear,
        generateDewDay,
        generateDewMonth,
        generateDewYear,                        // 20
        generateWSpeedDay,
        generateWSpeedMonth,
        generateWSpeedYear,
        generateWDirDay,
        generateWDirMonth,
        generateWDirYear,
        generateHiWSpeedDay,
        generateHiWSpeedMonth,
        generateHiWSpeedYear,
        generateBaromDay,                       // 30
        generateBaromMonth,
        generateBaromYear,
        generateRainDay,
        generateRainMonth,
        generateRainYear,
        generateWChillDay,
        generateWChillMonth,
        generateWChillYear,
        generateHIndexDay,
        generateHIndexMonth,                    // 40
        generateHIndexYear,
        generateCompositeTempDay,
        generateCompositeHIndexWChillDay,
        generateCompositeTempMonth,
        generateCompositeHIndexWChillMonth,
        generateRadiationDay,
        generateRadiationMonth,
        generateRadiationYear,
        generateUVDay,
        generateUVMonth,                        // 50
        generateUVYear,
        generateETDay,
        generateETMonth,
        generateETYear,
        generateLeafTemp1Day,
        generateLeafTemp1Month,
        generateLeafTemp1Year,
        generateLeafTemp2Day,
        generateLeafTemp2Month,
        generateLeafTemp2Year,                  // 60
        generateLeafWetness1Day,
        generateLeafWetness1Month,
        generateLeafWetness1Year,
        generateLeafWetness2Day,
        generateLeafWetness2Month,
        generateLeafWetness2Year,
        generateSoilTemp1Day,
        generateSoilTemp1Month,
        generateSoilTemp1Year,
        generateSoilTemp2Day,                   // 70
        generateSoilTemp2Month,
        generateSoilTemp2Year,
        generateSoilTemp3Day,
        generateSoilTemp3Month,
        generateSoilTemp3Year,
        generateSoilTemp4Day,
        generateSoilTemp4Month,
        generateSoilTemp4Year,
        generateExtraHumid1Day,
        generateExtraHumid1Month,               // 80
        generateExtraHumid1Year,
        generateExtraHumid2Day,
        generateExtraHumid2Month,
        generateExtraHumid2Year,
        generateExtraTemp1Day,
        generateExtraTemp1Month,
        generateExtraTemp1Year,
        generateExtraTemp2Day,
        generateExtraTemp2Month,
        generateExtraTemp2Year,                 // 90
        generateExtraTemp3Day,
        generateExtraTemp3Month,
        generateExtraTemp3Year,
        generateSoilMoisture1Day,
        generateSoilMoisture1Month,
        generateSoilMoisture1Year,
        generateSoilMoisture2Day,
        generateSoilMoisture2Month,
        generateSoilMoisture2Year,
        generateSoilMoisture3Day,               // 100
        generateSoilMoisture3Month,
        generateSoilMoisture3Year,
        generateSoilMoisture4Day,
        generateSoilMoisture4Month,
        generateSoilMoisture4Year,
        generateUV,
        generateRadiation,
        generateET,
        generateDialTemp,
        generateDialHumidity,                   // 110
        generateRxCheck,
        generateDialNetRainDay,
        generateDialNetRainMonth,
        generateDialNetRainYear,
        generateCompositeInTempHumidDay,
        generateCompositeInTempHumidMonth,
        NULL
    };


