/*---------------------------------------------------------------------------
 
  FILENAME:
        noaaGenerate.c
 
  PURPOSE:
        Provide the wview NOAA generator utilities.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        01/25/04        M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/

/*  ... Library include files
*/

/*  ... Local include files
*/
#include <noaaGenerate.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
static NOAA_WORK        noaaWork;

static char             *monthStr[13] =
    {
        "NAA",
        "JAN",
        "FEB",
        "MAR",
        "APR",
        "MAY",
        "JUN",
        "JUL",
        "AUG",
        "SEP",
        "OCT",
        "NOV",
        "DEC"
    };

static char             *windirStr[16] =
    {
        "N",
        "NNE",
        "NE",
        "ENE",
        "E",
        "ESE",
        "SE",
        "SSE",
        "S",
        "SSW",
        "SW",
        "WSW",
        "W",
        "WNW",
        "NW",
        "NNW"
    };


/*  ... local functions
*/

#if 0
static float adjustAccuracy (float value, int decPlacesOfAccuracy)
{
    value *= powf (10.0, (float)decPlacesOfAccuracy);
    value = truncf (value);
    value *= powf (10.0, (float)(0 - decPlacesOfAccuracy));

    return value;
}
#endif

static int AreDaysEqual (int day1, int mon1, int year1, int day2, int mon2, int year2)
{
    if (day1 != day2 || mon1 != mon2 || year1 != year2)
        return FALSE;

    return TRUE;
}

static void subtractDay (int *day, int *month, int *year)
{
    time_t          ntime;
    struct tm       locTime, newTime;
    time_t          time;

    memset (&locTime, 0, sizeof (locTime));

    locTime.tm_mday     = *day;
    locTime.tm_mon      = *month - 1;
    locTime.tm_year     = *year - 1900;
    locTime.tm_hour     = 1;

    if ((time = mktime (&locTime)) == (time_t)-1)
    {
        return;
    }

    time -= WV_SECONDS_IN_DAY;
    localtime_r (&time, &newTime);

    *day    = newTime.tm_mday;
    *month  = newTime.tm_mon + 1;
    *year   = newTime.tm_year + 1900;

    return;
}

static void addDay (int *day, int *month, int *year)
{
    time_t          ntime;
    struct tm       locTime, newTime;
    time_t          time;

    memset (&locTime, 0, sizeof (locTime));

    locTime.tm_mday     = *day;
    locTime.tm_mon      = *month - 1;
    locTime.tm_year     = *year - 1900;
    locTime.tm_hour     = 1;

    if ((time = mktime (&locTime)) == (time_t)-1)
    {
        return;
    }

    time += WV_SECONDS_IN_DAY;
    localtime_r (&time, &newTime);

    *day    = newTime.tm_mday;
    *month  = newTime.tm_mon + 1;
    *year   = newTime.tm_year + 1900;

    return;
}


static int parseNoaaRecord (char *temp, NOAA_DAY_REC *rec)
{
    char        *token;

    //  ... suck in the data
    token = strtok (temp, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->year = atoi (token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->month = atoi (token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->day = atoi (token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->meanTemp = (float)atof (token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->highTemp = (float)atof (token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    strcpy (rec->highTempTime, token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->lowTemp = (float)atof (token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    strcpy (rec->lowTempTime, token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->heatDegDays = (float)atof (token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->coolDegDays = (float)atof (token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->rain = (float)atof (token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->avgWind = (float)atof (token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->highWind = (float)atof (token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    strcpy (rec->highWindTime, token);

    token = strtok (NULL, " \t\n");
    if (token == NULL)
    {
        return ERROR;
    }
    rec->domWindDir = atoi (token);

    return OK;
}

static char buffer[256];
static char *createFileRecord (NOAA_DAY_REC *rec)
{
    sprintf (buffer,
             "%4.4d  %2.2d  %2.2d  %3.1f  %3.1f  %s  %3.1f  %s  %3.1f  %3.1f  %2.2f  %3.1f  %3.1f  %s  %d\n",
             rec->year,
             rec->month,
             rec->day,
             rec->meanTemp,
             rec->highTemp,
             rec->highTempTime,
             rec->lowTemp,
             rec->lowTempTime,
             rec->heatDegDays,
             rec->coolDegDays,
             rec->rain,
             rec->avgWind,
             rec->highWind,
             rec->highWindTime,
             rec->domWindDir);

    return buffer;
}

static int getFirstNoaaRecord (FILE *file, NOAA_DAY_REC *rec)
{
    char            temp[512];

    fseek (file, 0, SEEK_SET);

    while (fgets (temp, 512, file) != NULL)
    {
        if (temp[0] == ' ' || temp[0] == '\n' || temp[0] == '#')
        {
            // comment or whitespace
            continue;
        }

        if (parseNoaaRecord (temp, rec) == ERROR)
        {
            return ERROR;
        }
        else
        {
            return OK;
        }
    }

    return ERROR;
}

static int getNextNoaaRecord (FILE *file, NOAA_DAY_REC *rec)
{
    char            temp[512];

    while (fgets (temp, 512, file) != NULL)
    {
        if (temp[0] == ' ' || temp[0] == '\n' || temp[0] == '#')
        {
            // comment or whitespace
            continue;
        }

        if (parseNoaaRecord (temp, rec) == ERROR)
        {
            return ERROR;
        }
        else
        {
            return OK;
        }
    }

    return ERROR;
}

static int getLastNoaaRecord (FILE *file, NOAA_DAY_REC *rec)
{
    char            temp[512];
    int             retVal = ERROR;

    fseek (file, 0, SEEK_SET);

    while (fgets (temp, 512, file) != NULL)
    {
        if (temp[0] == ' ' || temp[0] == '\n' || temp[0] == '#')
        {
            // comment or whitespace
            continue;
        }

        if (parseNoaaRecord (temp, rec) == ERROR)
        {
            return ERROR;
        }

        retVal = OK;
    }

    return retVal;
}

static void computeNorms (char *fname, float *temps, float *rains, float *yearTemp, float *yearRain)
{
    FILE            *infile, *outfile = NULL;
    struct stat     fileStatus;
    char            temp[512];
    NOAA_DAY_REC    record;
    int             i, done, numDays, numMonths;
    float           tempSum, rainSum;
    int             thismonth, thisyear;
    time_t          ntime;
    struct tm       locTime;

    ntime = time (NULL);
    localtime_r (&ntime, &locTime);
    thismonth = locTime.tm_mon + 1;
    thisyear = locTime.tm_year + 1900;

    if (stat (fname, &fileStatus) == -1)
    {
        radMsgLog (PRI_HIGH, "noaaGenerate: %s does not exist!", fname);
        return;
    }

    infile = fopen (fname, "r");
    if (infile == NULL)
    {
        radMsgLog (PRI_HIGH, "noaaGenerate: %s open error!", fname);
        return;
    }

    memset (temps, 0, 13 * sizeof (float));
    memset (rains, 0, 13 * sizeof (float));
    *yearTemp = *yearRain = 0.0;

    tempSum = rainSum = 0.0;

    for (i = 1; i < 13; i ++)
    {
        fseek (infile, 0, SEEK_SET);
        done = FALSE;
        numDays = numMonths = 0;
        while (!done)
        {
            if (getNextNoaaRecord (infile, &record) == ERROR)
            {
                done = TRUE;
                record.month = 0xFFFF;
            }
            else if (record.year == thisyear && record.month == thismonth)
            {
                done = TRUE;
                record.month = 0xFFFF;
            }

            if (record.month != i)
            {
                if (numDays > 0)
                {
                    temps[i] += tempSum/numDays;
                    rains[i] += rainSum;
                    numMonths ++;
                }

                numDays = 0;
                tempSum = rainSum = 0.0;
                continue;
            }

            if (!done)
            {
                tempSum += record.meanTemp;
                rainSum += record.rain;
                numDays ++;
            }
        }

        if (numMonths == 0)
        {
            temps[i] = -10001.0;
            rains[i] = -10001.0;
        }
        else
        {
            temps[i] /= numMonths;
            rains[i] /= numMonths;
        }
    }

    fclose (infile);

    //  ... finally, compute the yearly norms
    for (i = 1; i < 13; i ++)
    {
        *yearTemp += temps[i];
        *yearRain += rains[i];
    }

    *yearTemp /= 12;

    return;
}

static void writeNoaaMonthFiles (NOAA_ID work, const char *datFileName)
{
    char            *destPath = work->htmlPath;
    FILE            *infile, *outfile = NULL;
    struct stat     fileStatus;
    char            temp[512];
    NOAA_DAY_REC    record;
    int             month;
    int             numDays, done;
    float           avgMean, avgHigh, avgLow, heatDays, coolDays;
    float           rain, avgWind, avgHighWind;
    float           highTemp, lowTemp, highWind;
    int             highTempDay, lowTempDay, highWindDay;
    int             daysMaxOver90, daysMaxUnder32, daysMinUnder32, daysMinUnder0;
    int             daysRainOver001, daysRainOver010, daysRainOver100;
    float           maxDayRain;
    char            maxDayRainDate[12];
    WAVG            windAvg;

    if (stat (datFileName, &fileStatus) == -1)
    {
        radMsgLog (PRI_HIGH, "noaaGenerate: %s does not exist!", datFileName);
        return;
    }

    infile = fopen (datFileName, "r");
    if (infile == NULL)
    {
        radMsgLog (PRI_HIGH, "noaaGenerate: %s open error!", datFileName);
        return;
    }

    month = done = 0;

    //  ... reset to the beginning
    fseek (infile, 0, SEEK_SET);

    //  ... do the monthly reports
    while (!done)
    {
        if (getNextNoaaRecord (infile, &record) == ERROR)
        {
            done = TRUE;
            record.month = 0xFFFF;
        }

        if (month != record.month)
        {
            month = record.month;

            // time for a new file, close out the old one and open the new one
            if (outfile)
            {
                // write the footer
                fprintf (outfile, "---------------------------------------------------------------------------------------\n");
                fprintf (outfile, "     %5.1f  %5.1f    %2d   %5.1f    %2d   %5.1f  %5.1f   %4.2f  %4.1f   %5.1f %5d    %s\n\n",
                         avgMean/numDays,
                         highTemp,
                         highTempDay,
                         lowTemp,
                         lowTempDay,
                         heatDays,
                         coolDays,
                         rain,
                         avgWind/numDays,
                         highWind,
                         highWindDay,
                         windirStr[(int)(windAverageCompute(&windAvg)/22.5)]);

                fprintf (outfile, "Max >=  90.0:  %d\n", daysMaxOver90);
                fprintf (outfile, "Max <=  32.0:  %d\n", daysMaxUnder32);
                fprintf (outfile, "Min <=  32.0:  %d\n", daysMinUnder32);
                fprintf (outfile, "Min <=   0.0:  %d\n", daysMinUnder0);
                fprintf (outfile, "Max Rain    :  %2.2f ON %s\n",
                         maxDayRain, maxDayRainDate);
                fprintf (outfile, "Days of Rain: %d (> 0.01 in) %d (> 0.10 in) %d (> 1.00 in)\n",
                         daysRainOver001, daysRainOver010, daysRainOver100);

                fclose (outfile);
                outfile = NULL;
            }

            if (!done)
            {
                sprintf (temp, "%s/NOAA-%4.4d-%2.2d.txt",
                         destPath, record.year, record.month);
                outfile = fopen (temp, "w");
                if (outfile == NULL)
                {
                    radMsgLog (PRI_HIGH, "noaaGenerate: %s open error!", temp);
                    fclose (infile);
                    return;
                }

                windAverageReset (&windAvg);
                numDays = 0;
                avgMean = heatDays = coolDays = 0.0;
                rain = avgWind = 0.0;
                highTemp = -1000.0;
                lowTemp = 1000.0;
                highWind = -100.0;
                highTempDay = lowTempDay = highWindDay = 0;
                daysMaxOver90 = daysMaxUnder32 = daysMinUnder32 = daysMinUnder0 = 0;
                daysRainOver001 = daysRainOver010 = daysRainOver100 = 0;
                maxDayRain = 0.0;
                strcpy (maxDayRainDate, "--/--/----");

                // create the header
                fprintf (outfile, "                   MONTHLY CLIMATOLOGICAL SUMMARY for %s %4.4d\n\n\n",
                         monthStr[record.month], record.year);
                fprintf (outfile, "NAME: %-20s  CITY: %-20s  STATE: %-20s\n",
                         work->stationName, work->stationCity, work->stationState);
                fprintf (outfile, "ELEV: %d ft    LAT: %4.1f %c    LONG: %4.1f %c\n\n\n",
                         work->elevation,
                         (float)abs(work->latitude)/10.0,
                         ((work->latitude < 0) ? 'S' : 'N'),
                         (float)abs(work->longitude)/10.0,
                         ((work->longitude < 0) ? 'W' : 'E'));

                fprintf (outfile, "                   TEMPERATURE (F), RAIN (in), WIND SPEED (mph)\n\n");
                fprintf (outfile, "                                         HEAT   COOL         AVG\n");
                fprintf (outfile, "      MEAN                               DEG    DEG          WIND                  DOM\n");
                fprintf (outfile, "DAY   TEMP   HIGH  TIME    LOW   TIME    DAYS   DAYS   RAIN  SPEED   HIGH  TIME    DIR\n");
                fprintf (outfile, "---------------------------------------------------------------------------------------\n");
            }
        }

        if (!done)
        {
            numDays ++;

            // compute some stuff for the total line
            if (record.highTemp > highTemp)
            {
                highTemp = record.highTemp;
                highTempDay = record.day;
            }
            if (record.lowTemp < lowTemp)
            {
                lowTemp = record.lowTemp;
                lowTempDay = record.day;
            }
            if (record.highWind > highWind)
            {
                highWind = record.highWind;
                highWindDay = record.day;
            }
            avgMean     += record.meanTemp;
            heatDays    += record.heatDegDays;
            coolDays    += record.coolDegDays;
            rain        += record.rain;
            avgWind     += record.avgWind;
            windAverageAddValue (&windAvg, record.domWindDir);

            if (record.highTemp >= 90.0)
                daysMaxOver90 ++;
            if (record.highTemp <= 32.0)
                daysMaxUnder32 ++;
            if (record.lowTemp <= 32.0)
                daysMinUnder32 ++;
            if (record.lowTemp <= 0.0)
                daysMinUnder0 ++;
            if (record.rain > maxDayRain)
            {
                maxDayRain = record.rain;
                sprintf (maxDayRainDate, "%2.2d/%2.2d/%4.4d",
                         record.month, record.day, record.year);
            }
            if (record.rain > 0.01)
                daysRainOver001 ++;
            if (record.rain > 0.10)
                daysRainOver010 ++;
            if (record.rain > 1.00)
                daysRainOver100 ++;

            // finally, write out the record
            fprintf (outfile, "%2.2d   %5.1f  %5.1f  %5s  %5.1f  %5s  %5.1f  %5.1f   %4.2f  %4.1f   %5.1f  %5s   %s\n",
                     record.day,
                     record.meanTemp,
                     record.highTemp,
                     record.highTempTime,
                     record.lowTemp,
                     record.lowTempTime,
                     record.heatDegDays,
                     record.coolDegDays,
                     record.rain,
                     record.avgWind,
                     record.highWind,
                     record.highWindTime,
                     windirStr[record.domWindDir]);
        }
    }

    fclose (infile);
    if (outfile)
        fclose (outfile);

    return;
}


static char     tempMonths[13][256];
static char     rainMonths[13][256];
static char     windMonths[13][256];
static void writeNoaaYearFiles (NOAA_ID work, char *datFileName)
{
    char            *destPath = work->htmlPath;
    FILE            *infile, *outfile = NULL;
    struct stat     fileStatus;
    char            temp[512];
    float           normalTemp[13], normalRain[13];
    float           normYearTemp, normYearRain;
    NOAA_DAY_REC    record;
    int             i, month, year;
    int             numDays, numMonths, done;


    float           monthAvgMaxTemp, monthAvgMinTemp, monthAvgTemp, monthDeparture;
    float           monthHeatDays, monthCoolDays;
    float           monthHighTemp, monthLowTemp;
    int             monthHighTempDate, monthLowTempDate;
    int             monthDaysMaxOver90, monthDaysMaxUnder32;
    int             monthDaysMinUnder32, monthDaysMinUnder0;

    float           monthRain, monthHighRainDay;
    int             monthHighRainDayDate;
    int             monthDaysRainOver001, monthDaysRainOver010, monthDaysRainOver100;

    float           monthAvgWind, monthHighWind;
    WAVG            windAvgMonth;
    int             monthHighWindDate;


    float           yearAvgMaxTemp, yearAvgMinTemp, yearAvgTemp;
    float           yearHeatDays, yearCoolDays;
    float           yearHighTemp, yearLowTemp;
    int             yearHighTempDate, yearLowTempDate;
    int             yearDaysMaxOver90, yearDaysMaxUnder32;
    int             yearDaysMinUnder32, yearDaysMinUnder0;

    float           yearRain, yearHighRainDay;
    int             yearHighRainDayDate;
    int             yearDaysRainOver001, yearDaysRainOver010, yearDaysRainOver100;

    float           yearAvgWind, yearHighWind;
    WAVG            windAvgYear;
    int             yearHighWindDate;

    windAverageReset (&windAvgMonth);
    windAverageReset (&windAvgYear);

    if (stat (datFileName, &fileStatus) == -1)
    {
        radMsgLog (PRI_HIGH, "noaaGenerate: %s does not exist!", datFileName);
        return;
    }

    //  ... compute the "normal" values for each month
    computeNorms (datFileName, normalTemp, normalRain, &normYearTemp, &normYearRain);


    //  ... now, down to business
    infile = fopen (datFileName, "r");
    if (infile == NULL)
    {
        radMsgLog (PRI_HIGH, "noaaGenerate: %s open error!", datFileName);
        return;
    }

    month = year = done = 0;

    //  ... reset to the beginning
    fseek (infile, 0, SEEK_SET);

    //  ... do the yearly reports
    while (!done)
    {
        if (getNextNoaaRecord (infile, &record) == ERROR)
        {
            done = TRUE;
            record.month = 0xFFFF;      // force a flush for the last file
            record.year = 0xFFFF;       // to avoid the initial match
        }

        if (month != record.month)
        {
            if (outfile)
            {
                monthDeparture = ((normalTemp[month] <= -10000.0) ? 0.0 : ((monthAvgTemp/numDays) - normalTemp[month]));
                if (monthDeparture > -0.1 && monthDeparture < 0.1)
                    monthDeparture = 0.0;

                // save some month "record"s
                sprintf (tempMonths[month], "%4.4d %2.2d  %5.1f %5.1f %5.1f %5.1f  %6.1f  %6.1f  %5.1f  %2d  %5.1f  %2d     %2d   %2d   %2d   %2d\n",
                         year,
                         month,
                         monthAvgMaxTemp/numDays,
                         monthAvgMinTemp/numDays,
                         monthAvgTemp/numDays,
                         monthDeparture,
                         monthHeatDays,
                         monthCoolDays,
                         monthHighTemp,
                         monthHighTempDate,
                         monthLowTemp,
                         monthLowTempDate,
                         monthDaysMaxOver90,
                         monthDaysMaxUnder32,
                         monthDaysMinUnder32,
                         monthDaysMinUnder0);

                sprintf (rainMonths[month], "%4.4d %2.2d  %5.2f  %6.2f  %5.2f   %2d     %3d    %3d    %3d\n",
                         year,
                         month,
                         monthRain,
                         ((normalRain[month] <= -10000.0) ? 0.0 : (monthRain - normalRain[month])),
                         monthHighRainDay,
                         monthHighRainDayDate,
                         monthDaysRainOver001,
                         monthDaysRainOver010,
                         monthDaysRainOver100);

                sprintf (windMonths[month], "%4.4d %2.2d  %5.1f  %5.1f    %2d   %3s\n",
                         year,
                         month,
                         monthAvgWind/numDays,
                         monthHighWind,
                         monthHighWindDate,
                         windirStr[(int)(windAverageCompute(&windAvgMonth)/22.5)]);

                numMonths ++;

                // compute some stuff for the year totals
                yearAvgMaxTemp      += monthAvgMaxTemp/numDays;
                yearAvgMinTemp      += monthAvgMinTemp/numDays;
                yearAvgTemp         += monthAvgTemp/numDays;
                yearHeatDays        += monthHeatDays;
                yearCoolDays        += monthCoolDays;
                yearRain            += monthRain;
                yearAvgWind         += monthAvgWind/numDays;
                windAverageAddValue (&windAvgYear,
                                     (int)(windAverageCompute(&windAvgMonth)/22.5));

                if (monthHighTemp > yearHighTemp)
                {
                    yearHighTemp = monthHighTemp;
                    yearHighTempDate = month;
                }
                if (monthLowTemp < yearLowTemp)
                {
                    yearLowTemp = monthLowTemp;
                    yearLowTempDate = month;
                }
                if (monthHighRainDay > yearHighRainDay)
                {
                    yearHighRainDay = monthHighRainDay;
                    yearHighRainDayDate = month;
                }
                if (monthHighWind > yearHighWind)
                {
                    yearHighWind = monthHighWind;
                    yearHighWindDate = month;
                }
                yearDaysMaxOver90 += monthDaysMaxOver90;
                yearDaysMaxUnder32 += monthDaysMaxUnder32;
                yearDaysMinUnder32 += monthDaysMinUnder32;
                yearDaysMinUnder0 += monthDaysMinUnder0;
                yearDaysRainOver001 += monthDaysRainOver001;
                yearDaysRainOver010 += monthDaysRainOver010;
                yearDaysRainOver100 += monthDaysRainOver100;
            }

            if (!done)
            {
                // reset some sums, etc.
                numDays = 0;
                month = record.month;

                windAverageReset (&windAvgMonth);

                monthAvgMaxTemp = monthAvgMinTemp = monthAvgTemp =
                monthHeatDays = monthCoolDays =
                monthHighTempDate = monthLowTempDate =
                monthRain = monthHighRainDay =
                monthAvgWind = monthHighWind = 0.0;

                monthHighTemp = -1000.0;
                monthLowTemp = 1000.0;

                monthDaysMaxOver90 = monthDaysMaxUnder32 =
                monthDaysMinUnder32 = monthDaysMinUnder0 =
                monthHighRainDayDate = monthDaysRainOver001 =
                monthDaysRainOver010 = monthDaysRainOver100 =
                monthHighWindDate = 0;
            }
        }

        if (year != record.year)
        {
            // time for a new file, write the old one and open the new one
            if (outfile)
            {
                // write the temp header
                fprintf (outfile, "                   ANNUAL CLIMATOLOGICAL SUMMARY for %4.4d\n\n\n",
                         year);
                fprintf (outfile, "NAME: %-20s  CITY: %-20s  STATE: %-20s\n",
                         work->stationName, work->stationCity, work->stationState);
                fprintf (outfile, "ELEV: %d ft    LAT: %4.1f %c    LONG: %4.1f %c\n\n\n",
                         work->elevation,
                         (float)abs(work->latitude)/10.0,
                         ((work->latitude < 0) ? 'S' : 'N'),
                         (float)abs(work->longitude)/10.0,
                         ((work->longitude < 0) ? 'W' : 'E'));

                fprintf (outfile, "                 TEMPERATURE (F), RAIN (in), WIND SPEED (mph)\n\n");
                fprintf (outfile, "                            DEP.    HEAT    COOL\n");
                fprintf (outfile, "          MEAN  MEAN        FROM    DEG     DEG                          MAX  MAX  MIN   MIN\n");
                fprintf (outfile, " YR  MO   MAX   MIN   MEAN  NORM    DAYS    DAYS     HI DATE   LOW DATE  >=90 <=32 <=32  <=0\n");
                fprintf (outfile, "---------------------------------------------------------------------------------------------\n");

                for (i = 1; i < 13; i ++)
                {
                    fputs (tempMonths[i], outfile);
                }

                // write the temp footer
                fprintf (outfile, "---------------------------------------------------------------------------------------------\n");
                fprintf (outfile, "         %5.1f %5.1f %5.1f %5.1f  %6.1f  %6.1f  %5.1f %3s  %5.1f %3s     %2d   %2d   %2d   %2d\n\n\n",
                         yearAvgMaxTemp/numMonths,
                         yearAvgMinTemp/numMonths,
                         yearAvgTemp/numMonths,
                         ((normYearTemp < 0.0) ? 0.0 : ((yearAvgTemp/numMonths) - normYearTemp)),
                         yearHeatDays,
                         yearCoolDays,
                         yearHighTemp,
                         monthStr[yearHighTempDate],
                         yearLowTemp,
                         monthStr[yearLowTempDate],
                         yearDaysMaxOver90,
                         yearDaysMaxUnder32,
                         yearDaysMinUnder32,
                         yearDaysMinUnder0);

                // write the rain header
                fprintf (outfile, "                            PRECIPITATION (in)\n\n");
                fprintf (outfile, "                  DEP.    MAX         ---DAYS OF RAIN---\n");
                fprintf (outfile, "                  FROM    OBS.                OVER\n");
                fprintf (outfile, " YR  MO  TOTAL    NORM    DAY  DATE   0.01   0.10   1.00\n");
                fprintf (outfile, "---------------------------------------------------------\n");

                for (i = 1; i < 13; i ++)
                {
                    fputs (rainMonths[i], outfile);
                }

                // write the rain footer
                fprintf (outfile, "---------------------------------------------------------\n");
                fprintf (outfile, "         %5.2f  %6.2f  %5.2f  %3s     %3d    %3d    %3d\n\n\n",
                         yearRain,
                         ((normYearRain < 0.0) ? 0.0 : (yearRain - normYearRain)),
                         yearHighRainDay,
                         monthStr[yearHighRainDayDate],
                         yearDaysRainOver001,
                         yearDaysRainOver010,
                         yearDaysRainOver100);

                // write the wind header
                fprintf (outfile, "                              WIND SPEED (mph)\n\n");
                fprintf (outfile, "                              DOM\n");
                fprintf (outfile, " YR  MO    AVG     HI   DATE  DIR\n");
                fprintf (outfile, "----------------------------------\n");

                for (i = 1; i < 13; i ++)
                {
                    fputs (windMonths[i], outfile);
                }

                // write the wind footer
                fprintf (outfile, "----------------------------------\n");
                fprintf (outfile, "          %4.1f  %5.1f   %3s   %3s\n",
                         yearAvgWind/numMonths,
                         yearHighWind,
                         monthStr[yearHighWindDate],
                         windirStr[(int)(windAverageCompute(&windAvgYear)/22.5)]);

                fclose (outfile);
                outfile = NULL;
            }

            if (!done)
            {
                year = record.year;
                numMonths = 0;
                sprintf (temp, "%s/NOAA-%4.4d.txt", destPath, record.year);
                outfile = fopen (temp, "w");
                if (outfile == NULL)
                {
                    radMsgLog (PRI_HIGH, "noaaGenerate: %s open error!", temp);
                    fclose (infile);
                    return;
                }

                windAverageReset (&windAvgYear);

                // reset all the record strings
                for (i = 1; i < 13; i ++)
                {
                    sprintf (tempMonths[i], "%2.2d %2.2d\n", year, i);
                    sprintf (rainMonths[i], "%2.2d %2.2d\n", year, i);
                    sprintf (windMonths[i], "%2.2d %2.2d\n", year, i);
                }

                yearAvgMaxTemp = yearAvgMinTemp = yearAvgTemp =
                yearHeatDays = yearCoolDays =
                yearRain = yearHighRainDay =
                yearDaysRainOver001 = yearDaysRainOver010 =
                yearDaysRainOver100 = yearAvgWind = yearHighWind = 0.0;

                yearHighTemp = -1000.0;
                yearLowTemp = 1000.0;

                yearHighTempDate = yearLowTempDate =
                yearDaysMaxOver90 = yearDaysMaxUnder32 =
                yearDaysMinUnder32 = yearDaysMinUnder0 =
                yearHighRainDayDate = yearHighWindDate = 0;
            }
        }

        if (!done)
        {
            numDays ++;

            // compute some stuff for the current month
            monthAvgMaxTemp     += record.highTemp;
            monthAvgMinTemp     += record.lowTemp;
            monthAvgTemp        += record.meanTemp;
            monthHeatDays       += record.heatDegDays;
            monthCoolDays       += record.coolDegDays;
            monthRain           += record.rain;
            monthAvgWind        += record.avgWind;
            windAverageAddValue (&windAvgMonth, record.domWindDir);

            if (record.highTemp > monthHighTemp)
            {
                monthHighTemp = record.highTemp;
                monthHighTempDate = record.day;
            }
            if (record.lowTemp < monthLowTemp)
            {
                monthLowTemp = record.lowTemp;
                monthLowTempDate = record.day;
            }
            if (record.rain > monthHighRainDay)
            {
                monthHighRainDay = record.rain;
                monthHighRainDayDate = record.day;
            }
            if (record.highWind > monthHighWind)
            {
                monthHighWind = record.highWind;
                monthHighWindDate = record.day;
            }
            if (record.highTemp >= 90.0)
                monthDaysMaxOver90 ++;
            if (record.highTemp <= 32.0)
                monthDaysMaxUnder32 ++;
            if (record.lowTemp <= 32.0)
                monthDaysMinUnder32 ++;
            if (record.lowTemp <= 0.0)
                monthDaysMinUnder0 ++;
            if (record.rain > 0.01)
                monthDaysRainOver001 ++;
            if (record.rain > 0.1)
                monthDaysRainOver010 ++;
            if (record.rain > 1.0)
                monthDaysRainOver100 ++;
        }
    }

    fclose (infile);
    if (outfile)
        fclose (outfile);

    return;
}


/*  ... API methods
*/
NOAA_ID noaaGenerateInit
(
    char            *archivePath,
    char            *htmlPath,
    short           latitude,
    short           longitude,
    short           elevation,
    char            *name,
    char            *city,
    char            *state,
    time_t          arcTime
)
{
    NOAA_ID         newId;
    FILE            *infile;
    struct stat     fileStatus;
    char            temp[512];
    NOAA_DAY_REC    record;
    int             curyear, curmonth, curday;
    int             year, month, day;
    int             firstyear, firstmonth, firstday;
    time_t          ntime;
    struct tm       locTime;

    ntime = arcTime;
    localtime_r (&ntime, &locTime);
    curday = locTime.tm_mday;
    curmonth = locTime.tm_mon + 1;
    curyear = locTime.tm_year + 1900;

    newId = &noaaWork;
    memset (newId, 0, sizeof (*newId));

    strcpy (newId->archivePath, archivePath);
    strcpy (newId->htmlPath, htmlPath);
    newId->latitude = latitude;
    newId->longitude = longitude;
    newId->elevation = elevation;
    strcpy (newId->stationName, name);
    strcpy (newId->stationCity, city);
    strcpy (newId->stationState, state);

    //  ... bring noaa.dat up-to-date (if necessary)
    sprintf (temp, "%s/noaa/noaa.dat", WVIEW_RUN_DIR);
    if (stat (temp, &fileStatus) == -1)
    {
        /*  ... data file does not exist, create it
        */
        infile = fopen (temp, "w");
        if (infile == NULL)
        {
            radMsgLog (PRI_HIGH, "noaaGenerateInit: noaa.dat open error!");
            return NULL;
        }

        if (fputs ("#\n", infile) == EOF)
        {
            radMsgLog (PRI_HIGH, "noaaGenerateInit: noaa.dat write error!");
            fclose (infile);
            return NULL;
        }

        fclose (infile);

        //  ... set these to a date before any archive records could exist
        firstday        = 1;
        firstmonth      = 1;
        firstyear       = 1990;
        day             = 1;
        month           = 1;
        year            = 1990;
    }
    else
    {
        infile = fopen (temp, "r");
        if (infile == NULL)
        {
            radMsgLog (PRI_HIGH, "noaaGenerateInit: noaa.dat open2 error!");
            return NULL;
        }

        //      ... find the first record
        if (getFirstNoaaRecord (infile, &record) == ERROR)
        {
            radMsgLog (PRI_STATUS, "noaaGenerateInit: noaa.dat has no records yet...");

            //  ... set these to a date before any archive records could exist
            firstday        = 1;
            firstmonth      = 1;
            firstyear       = 1990;
            day             = 1;
            month           = 1;
            year            = 1990;

            fclose (infile);
        }
        else
        {
            firstday        = record.day;
            firstmonth      = record.month;
            firstyear       = record.year;
    
            //  ... find the last record
            if (getLastNoaaRecord (infile, &record) == ERROR)
            {
                radMsgLog (PRI_HIGH, "noaaGenerateInit: noaa.dat getLast error!");
                fclose (infile);
                return NULL;
            }
    
            day             = record.day;
            month           = record.month;
            year            = record.year;
    
            fclose (infile);
        }
    }


    newId->lastDay      = curday;
    newId->lastMonth    = curmonth;
    newId->lastYear     = curyear;

    //  ... day to start looping on
    addDay (&day, &month, &year);


    //  ... open the dat file for writing
    infile = fopen (temp, "a");
    if (infile == NULL)
    {
        radMsgLog (PRI_HIGH, "noaaGenerateInit: cannot open %s!",
                   temp);
        return NULL;
    }

    //  ... do i need to do any work here?
    while (!AreDaysEqual (day, month, year, curday, curmonth, curyear))
    {
        if (dbfGetNoaaDay (archivePath, &record, year, month, day) == OK)
        {
            if (firstyear == 1990)
            {
                firstday    = record.day;
                firstmonth  = record.month;
                firstyear   = record.year;
            }

            //  ... add a record to noaa.dat
            if (fputs (createFileRecord (&record), infile) == EOF)
            {
                radMsgLog (PRI_HIGH, "noaaGenerateInit: fputs error for %d/%d/%d!",
                           month, day, year);
                fclose (infile);
                return NULL;
            }
        }

        addDay (&day, &month, &year);
    }

    fclose (infile);

    
    // do we have any records?
    if (firstyear == 1990)
    {
        // nope - just return happily
        return newId;
    }
    
#if __DEBUG_BUFFERS
    radMsgLog (PRI_STATUS, "DBG BFRS: NOAA BEGIN: %u of %u available",
               buffersGetAvailable (),
               buffersGetTotal ());
#endif


    //  ... write all NOAA reports
    radMsgLog (PRI_STATUS, "NOAA Generate: creating monthly NOAA reports starting %d-%d",
               firstmonth, firstyear);
    writeNoaaMonthFiles (newId, temp);

    radMsgLog (PRI_STATUS, "NOAA Generate: creating yearly NOAA reports starting %d",
               firstyear);
    writeNoaaYearFiles (newId, temp);


#if __DEBUG_BUFFERS
    radMsgLog (PRI_STATUS, "DBG BFRS: NOAA END: %u of %u available",
               buffersGetAvailable (),
               buffersGetTotal ());
#endif


    return newId;
}


int noaaGenerate (NOAA_ID id, int day, int month, int year)
{
    NOAA_DAY_REC    record;
    FILE            *infile;
    char            temp[512];

    sprintf (temp, "%s/noaa/noaa.dat", WVIEW_RUN_DIR);

    subtractDay (&day, &month, &year);

    if (dbfGetNoaaDay (id->archivePath, &record, year, month, day) == OK)
    {
        infile = fopen (temp, "a");
        if (infile == NULL)
        {
            radMsgLog (PRI_HIGH, "noaaGenerate: cannot open %s!", temp);
            return ERROR;
        }

        //      ... add a record to noaa.dat
        if (fputs (createFileRecord (&record), infile) == EOF)
        {
            radMsgLog (PRI_HIGH, "noaaGenerate: fputs error for %d/%d/%d!",
                       month, day, year);
            fclose (infile);
            return ERROR;
        }

        fclose (infile);
    }
    else
    {
        radMsgLog (PRI_HIGH, "noaaGenerate: dbfGetNoaaDay failed!");
        return ERROR;
    }


#if __DEBUG_BUFFERS
    radMsgLog (PRI_STATUS, "DBG BFRS: NOAA BEGIN: %u of %u available",
               buffersGetAvailable (),
               buffersGetTotal ());
#endif


    //  ... write all NOAA reports
    wvutilsLogEvent (PRI_STATUS, "NOAA Generate: creating monthly NOAA reports up to %4.4d%2.2d%2.2d",
                   year, month, day);
    writeNoaaMonthFiles (id, temp);

    wvutilsLogEvent (PRI_STATUS, "NOAA Generate: creating yearly NOAA reports up to %4.4d%2.2d%2.2d",
                   year, month, day);
    writeNoaaYearFiles (id, temp);


#if __DEBUG_BUFFERS
    radMsgLog (PRI_STATUS, "DBG BFRS: NOAA END: %u of %u available",
               buffersGetAvailable (),
               buffersGetTotal ());
#endif


    addDay (&day, &month, &year);

    id->lastDay     = day;
    id->lastMonth   = month;
    id->lastYear    = year;

    return OK;
}

