/*---------------------------------------------------------------------------
 
  FILENAME:
        sqld.c
 
  PURPOSE:
        Provide the wview SQL daemon entry point.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        12/22/2005      M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2005, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/*  ... Library include files
*/
#include <radsystem.h>

/*  ... Local include files
*/
#include <sqld.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
static WVIEW_SQLD_WORK  sqldWork;

enum ConfigIds
{
    CFG_ID_SQLDB_ENABLE         = 0,
    CFG_ID_SQLDB_EXTENDED       = 1,
    CFG_ID_SQLDB_METRIC         = 2,
    CFG_ID_SQLDB_HOST           = 3,
    CFG_ID_SQLDB_USERNAME       = 4,
    CFG_ID_SQLDB_PASSWORD       = 5,
    CFG_ID_SQLDB_DB_NAME        = 6,
    CFG_ID_SQLDB_TABLE_NAME     = 7
};

static char             *configIDs[] =
{
    "SQLDB_ENABLE",
    "SQLDB_EXTENDED_INFO",
    "SQLDB_STORE_METRIC",
    "SQLDB_HOST",
    "SQLDB_USERNAME",
    "SQLDB_PASSWORD",
    "SQLDB_DB_NAME",
    "SQLDB_TABLE_NAME"
};

static char             instanceStr[] = {'1', 0};

/* ... methods
*/

// get the next new record out of the WLK file and insert to the SQL table
static void sychronizeDB (void)
{
    ArchiveRecord       workRec;
    USHORT              tempDate, tempTime;
    int                 retVal;
    float               chill, dew, heat;

    if (!sqldWork.sqldbEnable)
    {
        return;
    }

    switch (sqldWork.state)
    {
        case SQLD_STATE_IDLE:
        {
            if (dbdatabaseOpen() == ERROR)
            {
                radMsgLog (PRI_HIGH, "dbdatabaseOpen failed!");
                radTimerStart (sqldWork.syncTimer, SQLD_TIMER_DBOPEN_VALUE);
                return;
            }

            sqldWork.state = SQLD_STATE_SYNC;
            // fall through to next case ...
        }
        case SQLD_STATE_SYNC:
        {
            tempDate = sqldWork.lastDate;
            tempTime = sqldWork.lastTime;
            if (dbfGetNextArchiveRecord (sqldWork.archivePath,
                                         &tempDate,
                                         &tempTime,
                                         &workRec)
                == ERROR)
            {
                // there is nothing for us to do...
                dbdatabaseClose ();
                if (sqldWork.initialSync)
                {
                    radMsgLog (PRI_STATUS, "SQLDB: initial database sync complete:");
                    radMsgLog (PRI_STATUS, "SQLDB: %u inserts, %u duplicates, %u errors",
                               sqldWork.inserts, sqldWork.duplicates, sqldWork.errors);
                    sqldWork.initialSync = FALSE;
                }
                sqldWork.state = SQLD_STATE_IDLE;
                return;
            }

            // OK, we have an arc record, try to insert it
            chill = wvutilsCalculateWindChill ((float)workRec.outsideTemp/10,
                                               (float)workRec.windSpeed/10);
            dew = wvutilsCalculateDewpoint ((float)workRec.outsideTemp/10,
                                            (float)workRec.outsideHum/10);
            heat = wvutilsCalculateHeatIndex ((float)workRec.outsideTemp/10,
                                              (float)workRec.outsideHum/10);

            retVal = dbdatabaseInsertRecord (&workRec,
                                             EXTRACT_PACKED_YEAR(tempDate),
                                             EXTRACT_PACKED_MONTH(tempDate),
                                             EXTRACT_PACKED_DAY(tempDate),
                                             dew,
                                             chill,
                                             heat);
            if (retVal == DBDB_INSERT_ERROR)
            {
                // there is nothing for us to do now, try again later
                dbdatabaseClose ();
                sqldWork.errors ++;
                sqldWork.state = SQLD_STATE_IDLE;
                return;
            }
            else if (retVal == DBDB_INSERT_DUPLICATE)
            {
                // a duplicate, keep on truckin'
                sqldWork.lastDate = tempDate;
                sqldWork.lastTime = tempTime;
                sqldWork.duplicates ++;
                radProcessEventsSend (NULL, SQLD_SYNC_EVENT, 0);
                return;
            }
            else
            {
                // an insertion, keep on truckin'
                sqldWork.lastDate = tempDate;
                sqldWork.lastTime = tempTime;
                sqldWork.inserts ++;
                radProcessEventsSend (NULL, SQLD_SYNC_EVENT, 0);
                return;
            }
        }
    }

    return;
}


static int waitForWviewDaemon (void)
{
    WVIEW_MSG_REQUEST       msg;
    char                    srcQName[QUEUE_NAME_LENGTH+1];
    UINT                    msgType;
    UINT                    length;
    void                    *recvBfr;
    int                     retVal, done = FALSE;
    WVIEW_MSG_ARCHIVE_PATH  *arcPath;

    // enable message reception from the radlib router for the archive path
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_ARCHIVE_PATH);

    msg.requestType = WVIEW_RQST_TYPE_ARCHIVE_PATH;

    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_REQUEST, &msg, sizeof(msg)) == ERROR)
    {
        radMsgLog (PRI_HIGH, "waitForWviewDaemon: radMsgRouterMessageSend failed!");
        radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
        return ERROR;
    }

    // now wait for the response here
    while (!done)
    {
        radUtilsSleep (50);
        
        
        if ((retVal = radQueueRecv (radProcessQueueGetID (),
                                    srcQName,
                                    &msgType,
                                    &recvBfr,
                                    &length))
            == FALSE)
        {
            continue;
        }
        else if (retVal == ERROR)
        {
            radMsgLog (PRI_STATUS, "waitForWviewDaemon: queue is closed!");
            radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
            return ERROR;
        }
    
        // is this what we want?
        if (msgType == WVIEW_MSG_TYPE_ARCHIVE_PATH)
        {
            // yes!
            arcPath = (WVIEW_MSG_ARCHIVE_PATH *)recvBfr;
            strcpy (sqldWork.archivePath, arcPath->archivePath);
            done = TRUE;
        }
    
        // release the received buffer
        radBufferRls (recvBfr);
        }
    
    // disable message reception from the radlib router for the archive path
    radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);

    return OK;
}

/*  ... system initialization
*/
static int sqldSysInit (WVIEW_SQLD_WORK *work)
{
    char            devPath[256], temp[64];
    struct stat     fileData;

    /*  ... exit if our config file is not there
    */
    sprintf (work->configFile, "%s/%s", WVIEW_CONFIG_DIR, WVIEW_CONFIG_FILENAME);
    if (stat (work->configFile, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_SQLD, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "%s does not exist - exiting...\n",
                   work->configFile);
        radMsgLogExit ();
        return ERROR_ABORT;
    }
    
    /*  ... check for our daemon's pid file, don't run if it isn't there
    */
    sprintf (devPath, "%s/%s", WVIEW_RUN_DIR, WVD_LOCK_FILE_NAME);
    if (stat (devPath, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_SQLD, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "wview daemon lock file %s does not exist - aborting!",
                   devPath);
        radMsgLogExit ();
        return ERROR;
    }

    sprintf (work->pidFile, "%s/%s", WVIEW_RUN_DIR, SQLD_LOCK_FILE_NAME);
    sprintf (work->fifoFile, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_SQLD);
    sprintf (work->wviewdir, "%s", WVIEW_RUN_DIR);

    /*  ... check for our pid file, don't run if it IS there
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        radMsgLogInit (PROC_NAME_SQLD, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "lock file %s exists, older copy may be running - aborting!",
                   work->pidFile);
        radMsgLogExit ();
        return ERROR;
    }

    return OK;
}

/*  ... system exit
*/
static int sqldSysExit (WVIEW_SQLD_WORK *work)
{
    struct stat     fileData;

    /*  ... delete our pid file
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        unlink (work->pidFile);
    }

    return OK;
}

static void defaultSigHandler (int signum)
{
    switch (signum)
    {
        case SIGPIPE:
            // we have a far end socket disconnection, we'll handle it in the
            // "read/write" code
            signal (signum, defaultSigHandler);
            break;

        case SIGILL:
        case SIGBUS:
        case SIGFPE:
        case SIGSEGV:
        case SIGXFSZ:
        case SIGSYS:
            // unrecoverable signal - we must exit right now!
            radMsgLog (PRI_CATASTROPHIC, "wviewsqld: recv sig %d: bailing out!", signum);
            abort ();
        
        case SIGCHLD:
            wait (NULL);
            signal (signum, defaultSigHandler);
            return;

        default:
            if (sqldWork.exiting)
            {
                signal (signum, defaultSigHandler);
                return;
            }

            sqldWork.exiting = TRUE;
            radProcessSetExitFlag ();

            // can we allow the process to exit normally?
            if (!sqldWork.inMainLoop)
            {
                // NO! - we gotta bail here!
                radMsgLog (PRI_HIGH, "wviewsqld: recv sig %d: exiting now!", signum);
                sqldSysExit (&sqldWork);
                radProcessExit ();
                radSystemExit (WVIEW_SYSTEM_ID);
                exit (0);                
            }
            
            // we can allow the process to exit normally...
            radMsgLog (PRI_HIGH, "wviewsqld: recv sig %d: exiting!", signum);
        
            signal (signum, defaultSigHandler);
            break;
    }
    
    return;
}

static void msgHandler
(
    char                        *srcQueueName,
    UINT                        msgType,
    void                        *msg,
    UINT                        length,
    void                        *userData
)
{
    if (msgType == WVIEW_MSG_TYPE_ARCHIVE_NOTIFY)
    {
        // start the delay timer if we are idle
        if (sqldWork.state == SQLD_STATE_IDLE)
        {
            radTimerStart (sqldWork.syncTimer, SQLD_TIMER_DELAY_VALUE);
        }
    }
    
    return;
}

static void evtHandler
(
    UINT        eventsRx,
    UINT        rxData,
    void        *userData
)
{
    if (eventsRx == SQLD_SYNC_EVENT)
    {
        // we need to start/continue the sync process
        sychronizeDB ();
    }
    return;
}

static void syncTimerHandler (void *parm)
{
    // we need to start/continue the sync process
    sychronizeDB ();
    return;
}

/*  ... the main entry point for the wviewsqld process
*/
int main (int argc, char *argv[])
{
    void            (*alarmHandler)(int);
    int             retVal;
    FILE            *pidfile;
    char            instance[32], value[_MAX_PATH];
    CF_ID           configFileId;
    struct stat     fileStatus;

    memset (&sqldWork, 0, sizeof (sqldWork));

    /*  ... initialize some system stuff first
    */
    retVal = sqldSysInit (&sqldWork);
    if (retVal == ERROR)
    {
        radMsgLogInit (PROC_NAME_SQLD, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "wviewsqld init failed!\n");
        radMsgLogExit ();
        exit (1);
    }
    else if (retVal == ERROR_ABORT)
    {
        exit (2);
    }


    /*  ... call the global radlib system init function
    */
    if (radSystemInit (WVIEW_SYSTEM_ID) == ERROR)
    {
        radMsgLogInit (PROC_NAME_SQLD, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "radSystemInit failed!");
        radMsgLogExit ();
        exit (1);
    }


    /*  ... call the radlib process init function
    */
    if (radProcessInit (PROC_NAME_SQLD,
                        sqldWork.fifoFile,
                        PROC_NUM_TIMERS_SQLD,
                        TRUE,                      // TRUE for daemon
                        msgHandler,
                        evtHandler,
                        NULL)
        == ERROR)
    {
        printf ("\nradProcessInit failed: %s\n\n", PROC_NAME_SQLD);
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    sqldWork.myPid = getpid ();
    pidfile = fopen (sqldWork.pidFile, "w");
    if (pidfile == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "lock file create failed!\n");
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    fprintf (pidfile, "%d", getpid ());
    fclose (pidfile);


    alarmHandler = radProcessSignalGetHandler (SIGALRM);
    radProcessSignalCatchAll (defaultSigHandler);
    radProcessSignalCatch (SIGALRM, alarmHandler);


    /*  ... get our configuration values
    */
    if (stat (sqldWork.configFile, &fileStatus) == -1)
    {
        radMsgLog (PRI_CATASTROPHIC, "config file %s missing - use wviewconfig to generate one...\n", 
                   sqldWork.configFile);
        sqldSysExit (&sqldWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    configFileId = radCfOpen (sqldWork.configFile);
    if (configFileId == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfOpen 2 failed!\n");
        sqldSysExit (&sqldWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }


    //  ... get SQLDB config (if any)
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_SQLDB_ENABLE],
                            instance,
                            value)
        != ERROR)
    {
        sqldWork.sqldbEnable = atoi (value);
    }
    
    //  ... is SQLDB enabled?
    if (!sqldWork.sqldbEnable)
    {
        radMsgLog (PRI_MEDIUM, "SQL archiving disabled in wview.conf - exiting...");
        radCfClose (configFileId);
        sqldSysExit (&sqldWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    //  ... get the rest of the SQL parameters
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_SQLDB_EXTENDED],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_MEDIUM, 
                   "SQLDB enabled but %s not found - exiting...",
                   configIDs[CFG_ID_SQLDB_EXTENDED]);
        sqldWork.sqldbEnable = 0;
        goto sqldbDone;
    }
    sqldWork.sqldbExtended = atoi (value);

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_SQLDB_METRIC],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_MEDIUM, 
                   "SQLDB enabled but %s not found - exiting...",
                   configIDs[CFG_ID_SQLDB_METRIC]);
        sqldWork.sqldbEnable = 0;
        goto sqldbDone;
    }
    sqldWork.sqldbMetric = atoi (value);

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_SQLDB_HOST],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_MEDIUM, 
                   "SQLDB enabled but %s not found - exiting...",
                   configIDs[CFG_ID_SQLDB_HOST]);
        sqldWork.sqldbEnable = 0;
        goto sqldbDone;
    }
    strcpy (sqldWork.sqldbHost, value);

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_SQLDB_USERNAME],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_MEDIUM, 
                   "SQLDB enabled but %s not found - exiting...",
                   configIDs[CFG_ID_SQLDB_USERNAME]);
        sqldWork.sqldbEnable = 0;
        goto sqldbDone;
    }
    strcpy (sqldWork.sqldbUser, value);

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_SQLDB_PASSWORD],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_MEDIUM, 
                   "SQLDB enabled but %s not found - exiting...",
                   configIDs[CFG_ID_SQLDB_PASSWORD]);
        sqldWork.sqldbEnable = 0;
        goto sqldbDone;
    }
    strcpy (sqldWork.sqldbPassword, value);

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_SQLDB_DB_NAME],
                            instance,
                            value)
        == ERROR)
    {
        // just assume the old hard-coded default
        strcpy (sqldWork.sqldatabase, "wviewDB");
    }
    else
    {
        strcpy (sqldWork.sqldatabase, value);
    }

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_SQLDB_TABLE_NAME],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_MEDIUM, 
                   "SQLDB enabled but %s not found - exiting...",
                   configIDs[CFG_ID_SQLDB_TABLE_NAME]);
        sqldWork.sqldbEnable = 0;
        goto sqldbDone;
    }
    strcpy (sqldWork.sqldbTable, value);

sqldbDone:

    radCfClose (configFileId);

    //  was everything OK?
    if (!sqldWork.sqldbEnable)
    {
        radMsgLog (PRI_MEDIUM, "SQL parameter missing/error in wview.conf - exiting...");
        sqldSysExit (&sqldWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    radMsgLog (PRI_STATUS, "SQLDB: %s %s archiving enabled -",
               ((sqldWork.sqldbExtended) ? "extended" : "basic"),
               ((sqldWork.sqldbMetric) ? "metric" : "US"));
    radMsgLog (PRI_STATUS, "SQLDB: to %s table %s.%s",
               sqldWork.sqldbHost,
               sqldWork.sqldatabase,
               sqldWork.sqldbTable);

    //  ... initialize the SQLDB utility (it stores whether it is enabled)
    dbdatabaseInit (sqldWork.sqldbEnable,
                    sqldWork.sqldbExtended,
                    sqldWork.sqldbHost,
                    sqldWork.sqldbUser,
                    sqldWork.sqldbPassword,
                    sqldWork.sqldatabase,
                    sqldWork.sqldbTable,
                    sqldWork.sqldbMetric);


    // wait a bit here before continuing
    radUtilsSleep (500);


    //  register with the radlib message router
    if (radMsgRouterInit (WVIEW_RUN_DIR) == ERROR)
    {
        radMsgLog (PRI_HIGH, "radMsgRouterInit failed!");
        sqldSysExit (&sqldWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    
    // wait for the wview daemon to be ready
    if (waitForWviewDaemon () == ERROR)
    {
        radMsgLog (PRI_HIGH, "waitForWviewDaemon failed");
        radMsgRouterExit ();
        sqldSysExit (&sqldWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }


    // enable message reception from the radlib router for archive notifications
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_ARCHIVE_NOTIFY);

    sqldWork.syncTimer = radTimerCreate (NULL, syncTimerHandler, NULL);
    if (sqldWork.syncTimer == NULL)
    {
        radMsgLog (PRI_HIGH, "radTimerCreate failed");
        radMsgRouterExit ();
        sqldSysExit (&sqldWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    // add the sync event
    radProcessEventsAdd (SQLD_SYNC_EVENT);


    // start the initial SQL table sync
    sqldWork.state = SQLD_STATE_IDLE;
    sqldWork.initialSync = TRUE;
    radMsgLog (PRI_STATUS, "SQLDB: beginning initial database sync (this may take some time)...");
    radProcessEventsSend (NULL, SQLD_SYNC_EVENT, 0);


    // enter normal processing
    sqldWork.inMainLoop = TRUE;
    radMsgLog (PRI_STATUS, "running...");
    
    while (!sqldWork.exiting)
    {
        // wait on something interesting
        if (radProcessWait (0) == ERROR)
        {
            sqldWork.exiting = TRUE;
        }
    }


    radMsgLog (PRI_STATUS, "exiting normally...");

    radMsgRouterExit ();
    radTimerDelete (sqldWork.syncTimer);
    sqldSysExit (&sqldWork);
    radProcessExit ();
    radSystemExit (WVIEW_SYSTEM_ID);
    exit (0);
}

