/*---------------------------------------------------------------------------
 
  FILENAME:
        sshUtils.c
 
  PURPOSE:
        Provide the ssh utilities.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        05/28/2005      M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/*  ... Library include files
*/
#include <radsysutils.h>

/*  ... Local include files
*/
#include <sshUtils.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
static char     *rsyncPaths[] =
{
    "/usr/bin/rsync",
    "/usr/local/bin/rsync",
    "/usr/sbin/rsync",
    "/usr/local/sbin/rsync",
    NULL
};


SSH_ID sshUtilsInit (char *confFilePath)
{
    SSH_RULE_ID     rule;
    SSH_ID          newId;
    FILE            *file;
    char            temp[1536];
    char            *token;
    int             i;
    struct stat     fileData;

    // first, let's find rsync - if it isn't there, error out
    for (i = 0; rsyncPaths[i] != NULL; i ++)
    {
        if (stat (rsyncPaths[i], &fileData) == 0)
        {
            // found him!
            strcpy (temp, rsyncPaths[i]);
            break;
        }
    }

    if (rsyncPaths[i] == NULL)
    {
        // we no findey, rsync must not be installed!
        radMsgLog (PRI_HIGH, 
                   "sshUtilsInit: cannot locate rsync - is it installed?");
        return NULL;
    }
    
    // OK, we can proceed
    newId = (SSH_ID) malloc (sizeof (*newId));
    if (newId == NULL)
    {
        return NULL;
    }

    memset (newId, 0, sizeof (*newId));
    radListReset (&newId->rules);
    strcpy (newId->rsyncPath, temp);
    strcpy (newId->confFile, confFilePath);

    file = fopen (confFilePath, "r");
    if (file == NULL)
    {
        free (newId);
        return NULL;
    }

    while (fgets (temp, 1536, file) != NULL)
    {
        if (temp[0] == ' ' || temp[0] == '\n' || temp[0] == '#')
        {
            // comment or whitespace
            continue;
        }

        token = strtok (temp, " \t");
        if (token == NULL)
        {
            continue;
        }

        // if here, we have something to look at
        rule = (SSH_RULE_ID) malloc (sizeof (*rule));
        if (rule == NULL)
        {
            for (rule = (SSH_RULE_ID)radListRemoveFirst (&newId->rules);
                 rule != NULL;
                 rule = (SSH_RULE_ID)radListRemoveFirst (&newId->rules))
            {
                free (rule);
            }

            free (newId);
            return NULL;
        }
        rule->interval = atoi (token);

        token = strtok (NULL, " \t");
        if (token == NULL)
        {
            free (rule);
            continue;
        }
        strcpy (rule->src, token);

        token = strtok (NULL, " \t");
        if (token == NULL)
        {
            free (rule);
            continue;
        }
        strcpy (rule->host, token);

        token = strtok (NULL, " \t\n");
        if (token == NULL)
        {
            free (rule);
            continue;
        }
        strcpy (rule->dest, token);

        radMsgLog (PRI_STATUS, "rsync: updating %s ==> %s:%s every %d minutes",
                   rule->src, rule->host, rule->dest, rule->interval);

        radListAddToEnd (&newId->rules, (NODE_PTR)rule);
    }

    fclose (file);
    return newId;
}

int sshUtilsSendFiles (SSH_ID id, char *workdir)
{
    SSH_RULE_ID     rule;
    int             retVal, isUpdated = FALSE;
    char            *args[6];
    char            rsync[64], sshStr[16], src[128], dest[256];

    // loop through all of our rules here
    for (rule = (SSH_RULE_ID)radListGetFirst (&id->rules);
         rule != NULL;
         rule = (SSH_RULE_ID)radListGetNext (&id->rules, (NODE_PTR)rule))
    {
        // our time to run?
        if (rule->currentCount > 0)
        {
            rule->currentCount --;
            continue;
        }
        
        // if we have already started one update, hold off a bit to be polite
        if (isUpdated)
        {
            radUtilsSleep (50);
        }
            
        // we have to place all of these on the stack so the child can use them
        strcpy (rsync, id->rsyncPath);
        args[0] = rsync;
        args[1] = "-aqz";
        sprintf (sshStr, "--rsh=ssh");
        args[2] = sshStr;
        sprintf (src, "%s/", rule->src);
        args[3] = src;
        sprintf (dest, "%s:%s",
                 rule->host, rule->dest);
        args[4] = dest;
        args[5] = 0;
    
        
        // fork off a child to run rsync...
        retVal = fork ();
        if (retVal == -1)
        {
            radMsgLog (PRI_MEDIUM, "sshUtilsSendFiles: fork failed");
        }
        else if (retVal == 0)
        {
            // we are the child!
            chdir (workdir);
            execv (args[0], args);
            exit (0);
        }
        else
        {
            rule->currentCount = rule->interval - 1;
            isUpdated = TRUE;
            wvutilsLogEvent (PRI_STATUS, "Updating: %s ==> %s:%s", 
                             rule->src, rule->host, rule->dest);
        }
    }
    
    return OK;
}


void sshUtilsExit (SSH_ID id)
{
    SSH_RULE_ID rule;

    for (rule = (SSH_RULE_ID)radListRemoveFirst (&id->rules);
         rule != NULL;
         rule = (SSH_RULE_ID)radListRemoveFirst (&id->rules))
    {
        free (rule);
    }

    free (id);
}

