/*---------------------------------------------------------------------
 
 FILE NAME:
        vpconfig.c
 
 PURPOSE:
        Main entry point for the Vantage Pro configuration utility.
 
 REVISION HISTORY:
    Date        Programmer  Revision    Function
    04/13/2005  M.S. Teel   0           Original
 
 ASSUMPTIONS:
 None.
 
------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/select.h>

#include <datadefs.h>
#include <daemon.h>
#include <station.h>
#include <vproInterface.h>

/*  ... global references
*/

/*  ... local memory
*/
// initial console wakeup number of attempts
#define VPC_INITIAL_WAKEUP_TRIES        10

static WVIEWD_WORK      wvWork;


/*  ... methods
*/

static void dumpConfig (void)
{
    int         numTries = 0;
    
    // initial wakeup of the VP console
    while ((vpifWakeupConsole (&wvWork) == ERROR) && 
           (numTries < VPC_INITIAL_WAKEUP_TRIES))
    {
        printf ("VP console WAKEUP failed - retry\n");
        radUtilsSleep (1000);
        numTries ++;
    }
    if (numTries == VPC_INITIAL_WAKEUP_TRIES)
    {
        printf ("VP console WAKEUP failed - aborting\n");
        return;
    }

    // get the archive interval
    if (vpconfigGetArchiveInterval (&wvWork) == ERROR)
    {
        printf ("vpconfigGetArchiveInterval failed - aborting\n");
        return;
    }
    
    // get the RXCheck string
    if (vpifGetRXCheck (&wvWork) == ERROR)
    {
        printf ("vpifGetRXCheck failed - aborting\n");
        return;
    }
    
    // get the version string
    if (vpconfigGetFWVersion (&wvWork) == ERROR)
    {
        printf ("vpconfigGetFWVersion failed - aborting\n");
        return;
    }
    
    // get the Lat and Long
    if (vpifGetLatandLong (&wvWork) == ERROR)
    {
        printf ("vpifGetLatandLong failed - aborting\n");
        return;
    }
    
    // get the Rain Season Start
    if (vpconfigGetRainSeasonStart (&wvWork) == ERROR)
    {
        printf ("vpconfigGetRainSeasonStart failed - aborting\n");
        return;
    }
    
    printf ("\n");
    printf ("Firmware Version:        %s\n",
            ((VP_IF_DATA *)wvWork.stationData)->fwVersion);

    printf ("Station Location:        ");
    if (wvWork.latitude < 0)
        printf ("%.1f S, ", (float)(-wvWork.latitude)/10.0);
    else
        printf ("%.1f N, ", (float)wvWork.latitude/10.0);
    if (wvWork.longitude < 0)
        printf ("%.1f W, ", (float)(-wvWork.longitude)/10.0);
    else
        printf ("%.1f E, ", (float)wvWork.longitude/10.0);
    printf ("%d feet\n", (int)wvWork.elevation);
        
    printf ("Archive Interval:        %d minutes\n", 
            wvWork.archiveInterval);

    printf ("Rain Season Start Month: %d\n", 
            ((VP_IF_DATA *)wvWork.stationData)->rainSeasonStart);
    
    printf ("RX Check Stats:          %s recvd:missed:resyncs:good-pkts-in-a-row:CRC errors\n\n",
            ((VP_IF_DATA *)wvWork.stationData)->rxCheck);
    
    return;
}

static int setInterval (int value)
{
    if (value != 5 &&
        value != 10 &&
        value != 15 &&
        value != 30)
    {
        printf ("archive interval must be one of: 5, 10, 15, 30\n");
        return ERROR;
    }
    
    if (vpconfigSetInterval (&wvWork, value) == ERROR)
    {
        printf ("failed to set archive interval\n");
        return ERROR;
    }
    
    sleep (5);
    
    vpconfigClearArchiveMemory (&wvWork);
    
    return OK;
}



static void USAGE (void)
{
    printf ("Usage: vpconfig <station_device> <command> [cmnd_args]\n\n");
    printf ("    station_device     serial device the VP console is connected to:\n");
    printf ("                           FreeBSD:   /dev/cuaa0 - /dev/cuaa4\n");
    printf ("                           Linux:     /dev/ttyS0 - /dev/ttyS4\n");
    printf ("                           Linux USB: /dev/ttyUSB0\n");
    printf ("                           Ethernet:  host:port\n\n");
    printf ("    command            command to execute, one of:\n");
    printf ("                           show\n");
    printf ("                             - retrieve and display VP console config\n");
    printf ("                           cleararchive\n");
    printf ("                             - clear the archive memory without changing the interval\n");
    printf ("                           setinterval <interval in minutes>\n");
    printf ("                             - set the archive interval (this clears the archive memory)\n");
    printf ("                           setelevation <elevation in feet>\n");
    printf ("                             - set the station elevation (feet) - use this to adjust the barometer\n");
    printf ("                           setgain <0 for off, 1 for on>\n");
    printf ("                             - sets the gain of the radio receiver\n");
    printf ("                           setlatlong <latitude (negative for S)> <longitude (negative for W)>\n");
    printf ("                             - set the station latitude and longitude in tenths of a degree\n");
    printf ("                           setrainseasonstart <month (1-12)>\n");
    printf ("                             - set the month that the yearly rain total begins\n");
    printf ("                           setyeartodaterain <0.00 - 200.00>\n");
    printf ("                             - set the yearly rain total to date\n");
    printf ("                           setyeartodateET <0.00 - 200.00>\n");
    printf ("                             - set the yearly Evapotranspiration total to date\n");
    
    return;
}

/*  ... THE entry point
*/
int main (int argc, char *argv[])
{
    int         temp, temp1;
    float       tempf;
    char        *tptr, host[128], port[16];
    ULONG       (*radsysFptr) (UCHAR systemID);
    
    if (argc < 3)
    {
        USAGE ();
        exit (1);
    }
    
    // make sure we pull in radSystem.o when we link
    radsysFptr = radSystemGetUpTimeSEC;
    
    memset (&wvWork, 0, sizeof (wvWork));
    
    // do we need to parse an ethernet string here?
    if (argv[1][0] != '/')
    {
        // guess so, did they give us host and port?
        tptr = strstr (argv[1], ":");
        if (tptr == NULL)
        {
            printf ("vpconfig: invalid interface %s given - ", argv[1]);
            printf ("not device and not host:port format - aborting!\n");
            exit (1);
        }
        
        strcpy (wvWork.stationInterface, "ethernet");
        strncpy (wvWork.stationHost, argv[1], tptr - argv[1]);
        wvWork.stationPort = atoi (tptr + 1);
    }
    else
    {
        // normal device specification
        strcpy (wvWork.stationInterface, "serial");
        strcpy (wvWork.stationDevice, argv[1]);
    }   


    // initialize the station abstraction
    if (stationInit (&wvWork, NULL) == ERROR)
    {
        printf ("stationInit failed\n");
        exit (1);
    }


    // process command
    if (!strcmp (argv[2], "show"))
    {
        // dump configuration
        dumpConfig ();
    }
    else if (!strcmp (argv[2], "setinterval"))
    {
        // set the archive interval
        if (argc < 4)
        {
            printf ("no archive interval specified\n");
            (*(wvWork.medium.exit)) (&wvWork.medium);
            exit (1);
        }
        else
        {
            if (setInterval (atoi (argv[3])) == ERROR)
            {
                printf ("set failed!\n");
                (*(wvWork.medium.exit)) (&wvWork.medium);
                exit (1);
            }
            else
            {
                printf ("archive memory cleared and interval set to %d minutes\n",
                        atoi (argv[3]));
            }
        }
    }
    else if (!strcmp (argv[2], "cleararchive"))
    {
        // clear archive memory
        vpifWakeupConsole (&wvWork);
        if (vpconfigClearArchiveMemory (&wvWork) == ERROR)
        {
            printf ("failed to clear archive memory\n");
            (*(wvWork.medium.exit)) (&wvWork.medium);
            exit (1);
        }
        else
        {
            printf ("archive memory cleared\n");
        }
    }
    else if (!strcmp (argv[2], "setelevation"))
    {
        // set the station elevation
        if (argc < 4)
        {
            printf ("no elevation specified\n");
            (*(wvWork.medium.exit)) (&wvWork.medium);
            exit (1);
        }
        else
        {
            vpifWakeupConsole (&wvWork);
            if (vpconfigSetElevation (&wvWork, atoi (argv[3])) == ERROR)
            {
                printf ("set elevation failed!\n");
                (*(wvWork.medium.exit)) (&wvWork.medium);
                exit (1);
            }
            else
            {
                printf ("elevation set to %d feet\n", atoi (argv[3]));
            }
        }
    }
    else if (!strcmp (argv[2], "setgain"))
    {
        // set the station receiver gain
        if (argc < 4)
        {
            printf ("no gain flag specified\n");
            (*(wvWork.medium.exit)) (&wvWork.medium);
            exit (1);
        }
        else
        {
            temp = atoi (argv[3]);
            if (temp != 0)
                temp = 1;
            vpifWakeupConsole (&wvWork);
            if (vpconfigSetGain (&wvWork, temp) == ERROR)
            {
                printf ("set gain failed!\n");
                printf ("gain cannot be set on VP 2 (is this a VP 2?)\n");
                (*(wvWork.medium.exit)) (&wvWork.medium);
                exit (1);
            }
            else
            {
                printf ("gain turned %s\n", ((temp) ? "on" : "off"));
            }
        }
    }
    else if (!strcmp (argv[2], "setlatlong"))
    {
        // set the station latitude and longitude
        if (argc < 5)
        {
            printf ("latitude and/or longitude value missing\n");
            (*(wvWork.medium.exit)) (&wvWork.medium);
            exit (1);
        }
        else
        {
            temp = atoi (argv[3]);
            temp1 = atoi (argv[4]);
            if (temp < -900 || temp > 900 || temp1 < -1799 || temp1 > 1800)
            {
                printf ("invalid latitude or longitude given\n");
                (*(wvWork.medium.exit)) (&wvWork.medium);
                exit (1);
            }
            else
            {
                vpifWakeupConsole (&wvWork);
                if (vpconfigSetLatandLong (&wvWork, temp, temp1) == ERROR)
                {
                    printf ("set latlong failed!\n");
                    (*(wvWork.medium.exit)) (&wvWork.medium);
                    exit (1);
                }
                else
                {
                    printf ("latitude set to %.1f, longitude set to %.1f\n", 
                            (float)temp/10.0, (float)temp1/10.0);
                }
            }
        }
    }
    else if (!strcmp (argv[2], "setrainseasonstart"))
    {
        // set the station rain season start month
        if (argc < 4)
        {
            printf ("no start month specified\n");
            (*(wvWork.medium.exit)) (&wvWork.medium);
            exit (1);
        }
        else
        {
            temp = atoi (argv[3]);
            if (temp < 1 || temp > 12)
            {
                printf ("invalid start month given\n");
                (*(wvWork.medium.exit)) (&wvWork.medium);
                exit (1);
            }
            else
            {
                vpifWakeupConsole (&wvWork);
                if (vpconfigSetRainSeasonStart (&wvWork, temp) == ERROR)
                {
                    printf ("set rain season start month failed!\n");
                    (*(wvWork.medium.exit)) (&wvWork.medium);
                    exit (1);
                }
                else
                {
                    printf ("rain season start month set to %d\n", temp);
                }
            }
        }
    }
    else if (!strcmp (argv[2], "setyeartodaterain"))
    {
        // set the station rain year-to-date
        if (argc < 4)
        {
            printf ("no rain amount specified\n");
            (*(wvWork.medium.exit)) (&wvWork.medium);
            exit (1);
        }
        else
        {
            tempf = (float)atof (argv[3]);
            if (tempf < 0.0 || tempf > 200.0)
            {
                printf ("invalid rain amount given\n");
                (*(wvWork.medium.exit)) (&wvWork.medium);
                exit (1);
            }
            else
            {
                vpifWakeupConsole (&wvWork);
                if (vpconfigSetRainYearToDate (&wvWork, tempf) == ERROR)
                {
                    printf ("set rain year-to-date failed!\n");
                    (*(wvWork.medium.exit)) (&wvWork.medium);
                    exit (1);
                }
                else
                {
                    printf ("rain year-to-date set to %.2f\n", tempf);
                }
            }
        }
    }
    else if (!strcmp (argv[2], "setyeartodateET"))
    {
        // set the station ET year-to-date
        if (argc < 4)
        {
            printf ("no ET value specified\n");
            (*(wvWork.medium.exit)) (&wvWork.medium);
            exit (1);
        }
        else
        {
            tempf = (float)atof (argv[3]);
            if (tempf < 0.0 || tempf > 200.0)
            {
                printf ("invalid ET value given\n");
                (*(wvWork.medium.exit)) (&wvWork.medium);
                exit (1);
            }
            else
            {
                vpifWakeupConsole (&wvWork);
                if (vpconfigSetETYearToDate (&wvWork, tempf) == ERROR)
                {
                    printf ("set ET year-to-date failed!\n");
                    (*(wvWork.medium.exit)) (&wvWork.medium);
                    exit (1);
                }
                else
                {
                    printf ("ET year-to-date set to %.2f\n", tempf);
                }
            }
        }
    }
    else
    {
        printf ("unknown command %s given!\n", argv[2]);
        (*(wvWork.medium.exit)) (&wvWork.medium);
        exit (1);
    }

    (*(wvWork.medium.exit)) (&wvWork.medium);
    exit (0);
}
