/*---------------------------------------------------------------------------
 
  FILENAME:
        vproInterface.c
 
  PURPOSE:
        Provide the non-medium-specific interface utilities.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        06/06/2005      M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2005, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/

/*  ... Library include files
*/

/*  ... Local include files
*/
#include <vproInterface.h>
#include <Ccitt.h>

/*  ... global memory declarations
*/

/*  ... local memory
*/

#if HOST_IS_BIGENDIAN
#define SHORT_SWAP(x)           (((x << 8) & 0xFF00) | ((x >> 8) & 0x00FF))
#else
#define SHORT_SWAP(x)           (x)
#endif

static VP_IF_DATA   vpWorkData;

static void portConfig (int fd);
static void (*ArchiveIndicator) (ARCHIVE_RECORD *newRecord);


////////////****////****  S T A T I O N   A P I  ****////****////////////
/////  Must be provided by each supported wview station interface  //////

// station-supplied init function
// -- Can Be Asynchronous - event indication required --
//
// MUST:
//   - set the 'stationGeneratesArchives' flag in WVIEWD_WORK:
//     if the station generates archive records (TRUE) or they should be 
//     generated automatically by the daemon from the sensor readings (FALSE)
//   - Initialize the 'stationData' store for station work area
//   - Initialize the interface medium
//   - do initial LOOP acquisition
//   - do any catch-up on archive records if there is a data logger
//   - 'work->runningFlag' can be used for start up synchronization but should 
//     not be modified by the station interface code
//   - indicate init is done by sending the STATION_INIT_COMPLETE_EVENT event to
//     this process (radProcessEventsSend (NULL, STATION_INIT_COMPLETE_EVENT, 0))
//
// OPTIONAL:
//   - Initialize a state machine or any other construct required for the 
//     station interface - these should be stored in the 'stationData' store
//
// 'archiveIndication' - indication callback used to pass back an archive record
//   generated as a result of 'stationGetArchive' being called; should receive a
//   NULL pointer for 'newRecord' if no record available; only used if 
//   'stationGeneratesArchives' flag is set to TRUE by the station interface
//
// Returns: OK or ERROR
//
int stationInit
(
    WVIEWD_WORK     *work,
    void            (*archiveIndication)(ARCHIVE_RECORD *newRecord)
)
{
    STIM            stim;

    memset (&vpWorkData, 0, sizeof(vpWorkData));
    vpWorkData.sampleRain = -1;
    vpWorkData.sampleET = -1;

    // save the archive indication callback
    ArchiveIndicator = archiveIndication;

    // set our work data pointer
    work->stationData = &vpWorkData;

    // set the Archive Generation flag to indicate the VP generates them
    work->stationGeneratesArchives = TRUE;


    // initialize the medium abstraction based on user configuration
    if (!strcmp (work->stationInterface, "serial"))
    {
        if (serialMediumInit (&work->medium, portConfig) == ERROR)
        {
            radMsgLog (PRI_HIGH, "stationInit: serial MediumInit failed");
            return ERROR;
        }
    }
    else if (!strcmp (work->stationInterface, "ethernet"))
    {
        if (ethernetMediumInit (&work->medium, work->stationHost, work->stationPort) 
            == ERROR)
        {
            radMsgLog (PRI_HIGH, "stationInit: ethernet MediumInit failed");
            return ERROR;
        }
    }
    else
    {
        radMsgLog (PRI_HIGH, "stationInit: medium %s not supported",
                   work->stationInterface);
        return ERROR;
    }

    // initialize the VP interface using the media specific routine
    if ((*(work->medium.init)) (&work->medium, work->stationDevice) == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationInit: medium setup failed");
        return ERROR;
    }

    vpifWakeupConsole (work);
    vpifWakeupConsole (work);


#ifndef _VP_CONFIG_ONLY
    if (!strcmp (work->stationInterface, "serial"))
    {
        radMsgLog (PRI_STATUS, "Vantage Pro on %s opened ...", 
                   work->stationDevice);
    }
    else if (!strcmp (work->stationInterface, "ethernet"))
    {
        radMsgLog (PRI_STATUS, "Vantage Pro on %s:%d opened ...", 
                   work->stationHost, work->stationPort);
    }

    // get VP-specific configuration
    if (stationGetConfigValueInt (work, 
                                  VP_PARM_DO_RXCHECK, 
                                  &vpWorkData.doRXCheck) 
        == ERROR)
    {
        // just default to disabled
        vpWorkData.doRXCheck = 0;
    }


    // now initialize the state machine
    vpWorkData.stateMachine = radStatesInit (work);
    if (vpWorkData.stateMachine == NULL)
    {
        radMsgLog (PRI_HIGH, "stationInit: radStatesInit failed");
        (*(work->medium.exit)) (&work->medium);
        return ERROR;
    }

    if (radStatesAddHandler (vpWorkData.stateMachine, 
                             VPRO_STATE_STARTPROC,
                             vproStartProcState)
        == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationInit: radStatesAddHandler failed");
        radStatesExit (vpWorkData.stateMachine);
        (*(work->medium.exit)) (&work->medium);
        return ERROR;
    }
    if (radStatesAddHandler (vpWorkData.stateMachine, 
                             VPRO_STATE_RUN,
                             vproRunState)
        == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationInit: radStatesAddHandler failed");
        radStatesExit (vpWorkData.stateMachine);
        (*(work->medium.exit)) (&work->medium);
        return ERROR;
    }
    if (radStatesAddHandler (vpWorkData.stateMachine, 
                             VPRO_STATE_DMPAFT_RQST,
                             vproDumpAfterState)
        == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationInit: radStatesAddHandler failed");
        radStatesExit (vpWorkData.stateMachine);
        (*(work->medium.exit)) (&work->medium);
        return ERROR;
    }
    if (radStatesAddHandler (vpWorkData.stateMachine, 
                             VPRO_STATE_DMPAFT_ACK,
                             vproDumpAfterAckState)
        == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationInit: radStatesAddHandler failed");
        radStatesExit (vpWorkData.stateMachine);
        (*(work->medium.exit)) (&work->medium);
        return ERROR;
    }
    if (radStatesAddHandler (vpWorkData.stateMachine, 
                             VPRO_STATE_RECV_ARCH,
                             vproReceiveArchiveState)
        == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationInit: radStatesAddHandler failed");
        radStatesExit (vpWorkData.stateMachine);
        (*(work->medium.exit)) (&work->medium);
        return ERROR;
    }
    if (radStatesAddHandler (vpWorkData.stateMachine, 
                             VPRO_STATE_LOOP_RQST,
                             vproLoopState)
        == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationInit: radStatesAddHandler failed");
        radStatesExit (vpWorkData.stateMachine);
        (*(work->medium.exit)) (&work->medium);
        return ERROR;
    }
    if (radStatesAddHandler (vpWorkData.stateMachine, 
                             VPRO_STATE_READ_RECOVER,
                             vproReadRecoverState)
        == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationInit: radStatesAddHandler failed");
        radStatesExit (vpWorkData.stateMachine);
        (*(work->medium.exit)) (&work->medium);
        return ERROR;
    }
    if (radStatesAddHandler (vpWorkData.stateMachine, 
                             VPRO_STATE_ERROR,
                             vproErrorState)
        == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationInit: radStatesAddHandler failed");
        radStatesExit (vpWorkData.stateMachine);
        (*(work->medium.exit)) (&work->medium);
        return ERROR;
    }

    radStatesSetState (vpWorkData.stateMachine, VPRO_STATE_STARTPROC);

    // dummy up a stimulus to get the state machine running
    stim.type = STIM_DUMMY;
    radStatesProcess (vpWorkData.stateMachine, &stim);

#endif

    return OK;
}

// station-supplied exit function
//
// Returns: N/A
//
void stationExit (WVIEWD_WORK *work)
{
    radStatesExit (vpWorkData.stateMachine);
    (*(work->medium.exit)) (&work->medium);

    return;
}

// station-supplied function to retrieve positional info (lat, long, elev) -
// should populate 'work' fields: latitude, longitude, elevation
// -- Synchronous --
//
// - If station does not store these parameters, they can be retrieved from the
//   wview.conf file (see daemon.c for example conf file use) - user must choose
//   station type "Generic" when running the wviewconfig script
//
// Returns: OK or ERROR
//
int stationGetPosition (WVIEWD_WORK *work)
{
    //  wakeup the console
    if (vpifWakeupConsole (work) == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationGetPosition: WAKEUP failed");
        return ERROR;
    }

    // get the station latitude and longitude and elevation
    if (vpifGetLatandLong (work) == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationGetPosition: failed");
        return ERROR;
    }
    else
    {
        vpWorkData.elevation = work->elevation;

        radMsgLog (PRI_STATUS, "station location: elevation: %d feet",
                   work->elevation);

        radMsgLog (PRI_STATUS, "station location: latitude: %3.1f %c  longitude: %3.1f %c",
                   (float)abs(work->latitude)/10.0,
                   ((work->latitude < 0) ? 'S' : 'N'),
                   (float)abs(work->longitude)/10.0,
                   ((work->longitude < 0) ? 'W' : 'E'));

        return OK;
    }
}

// station-supplied function to indicate a time sync should be performed if the
// station maintains time, otherwise may be safely ignored
// -- Can Be Asynchronous --
//
// Returns: OK or ERROR
//
int stationSyncTime (WVIEWD_WORK *work)
{
    // let state machine do it when convenient
    vpWorkData.timeSyncFlag = TRUE;

    return OK;
}

// station-supplied function to indicate sensor readings should be performed -
// should populate 'work' struct: loopPkt (see datadefs.h for minimum field reqs)
// -- Can Be Asynchronous --
//
// - indicate readings are complete by sending the STATION_LOOP_COMPLETE_EVENT
//   event to this process (radProcessEventsSend (NULL, STATION_LOOP_COMPLETE_EVENT, 0))
//
// Returns: OK or ERROR
//
int stationGetReadings (WVIEWD_WORK *work)
{
    STIM        stim;

    stim.type = VP_STIM_READINGS;

    // use the VP_STIM_READINGS stim to kick off the state machine
    radStatesProcess (vpWorkData.stateMachine, &stim);
    
    return OK;
}

// station-supplied function to indicate an archive record should be generated -
// MUST populate an ARCHIVE_RECORD struct and indicate it to 'archiveIndication'
// function passed into 'stationInit'
// -- Asynchronous - callback indication required --
//
// Returns: OK or ERROR
//
// Note: 'archiveIndication' should receive a NULL pointer for the newRecord if
//       no record is available
// Note: This function will only be invoked by the wview daemon if the 
//       'stationInit' function set the 'stationGeneratesArchives' to TRUE
//
int stationGetArchive (WVIEWD_WORK *work)
{
    STIM            stim;

    stim.type = VP_STIM_ARCHIVE;

    // use the VP_STIM_ARCHIVE stim to kick off the state machine
    radStatesProcess (vpWorkData.stateMachine, &stim);
    
    return OK;
}

// station-supplied function to indicate data is available on the station 
// interface medium (serial or ethernet) -
// It is the responsibility of the station interface to read the data from the 
// medium and process appropriately. The data does not have to be read within
// the context of this function, but may be used to stimulate a state machine.
// -- Synchronous --
//
// Returns: N/A
//
void stationDataIndicate (WVIEWD_WORK *work)
{
    STIM            stim;
    
    stim.type = STIM_IO;

    radStatesProcess (vpWorkData.stateMachine, &stim);
    return;
}

// station-supplied function to indicate the interface timer has expired -
// It is the responsibility of the station interface to start/stop the interface
// timer as needed for the particular station requirements.
// The station interface timer is specified by the 'ifTimer' member of the
// WVIEWD_WORK structure. No other timers in that structure should be manipulated
// in any way by the station interface code.
// -- Synchronous --
//
// Returns: N/A
//
void stationIFTimerExpiry (WVIEWD_WORK *work)
{
    STIM            stim;
    
    stim.type = STIM_TIMER;

    radStatesProcess (vpWorkData.stateMachine, &stim);
    
    return;
}

////////////****////  S T A T I O N   A P I   E N D  ////****////////////


/*  ... ----- static (local) methods -----
*/
static void portConfig (int fd)
{
    struct termios  port;

    tcgetattr (fd, &port);

    cfsetispeed (&port, B19200);
    cfsetospeed (&port, B19200);

    /*  ... set port to 8N1
    */
    port.c_cflag &= ~PARENB;
    port.c_cflag &= ~CSTOPB;
    port.c_cflag &= ~CSIZE;
    port.c_cflag |= CS8;
    port.c_cflag |= (CREAD | CLOCAL);
    port.c_cflag |= CRTSCTS;                    /* turn on H/W flow control */

    port.c_iflag &= ~(IXON | IXOFF | IXANY);    /* turn off SW flow control */
    
    port.c_iflag &= ~(INLCR | ICRNL);           /* turn off other input magic */

    port.c_oflag = 0;                           /* NO output magic wanted */

    port.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

    tcsetattr (fd, TCSAFLUSH, &port);

    return;
}


#ifndef _VP_CONFIG_ONLY

#if HOST_IS_BIGENDIAN
static void swapArchiveRecord (ARCHIVE_RECORD *rec)
{
    rec->date = SHORT_SWAP(rec->date);
    rec->time = SHORT_SWAP(rec->time);
    rec->outTemp = SHORT_SWAP(rec->outTemp);
    rec->highOutTemp = SHORT_SWAP(rec->highOutTemp);
    rec->lowOutTemp = SHORT_SWAP(rec->lowOutTemp);
    rec->rain = SHORT_SWAP(rec->rain);
    rec->highRainRate = SHORT_SWAP(rec->highRainRate);
    rec->barometer = SHORT_SWAP(rec->barometer);
    rec->radiation = SHORT_SWAP(rec->radiation);
    rec->windSamples = SHORT_SWAP(rec->windSamples);
    rec->inTemp = SHORT_SWAP(rec->inTemp);
    
    return;
}
#else
static void swapArchiveRecord (ARCHIVE_RECORD *rec)
{
    return;
}
#endif


static void storeLoopPkt (LOOP_PKT *dest, LOOP_DATA *src)
{
    float       tempfloat;
    USHORT      tempshort;
    int         intTemp;
    float       scaledRain;

    // do this magic to track rainfall and ET per LOOP sample
	intTemp = (int)SHORT_SWAP(src->dayRain);

    if (vpWorkData.sampleRain == -1 && intTemp != 0xFFFF)
    {
        vpWorkData.sampleRain = intTemp;
        dest->sampleRain = 0;
    }
    else if (intTemp != 0xFFFF)
    {
        if (intTemp - vpWorkData.sampleRain >= 0)
        {
            dest->sampleRain = ((float)intTemp - (float)vpWorkData.sampleRain)/100;
        }
        else
        {
            // we had a day rollover, all rain must have come in this sample
            dest->sampleRain = (float)intTemp/100;
        }
        vpWorkData.sampleRain = intTemp;
    }
    else
    {
        dest->sampleRain = 0;
    }

    intTemp = (int)SHORT_SWAP(src->dayET);
    if (vpWorkData.sampleET == -1 && intTemp != 0xFFFF)
    {
        vpWorkData.sampleET = intTemp;
        dest->sampleET = 0;
    }
    else if (intTemp != 0xFFFF)
    {
        if (intTemp - vpWorkData.sampleET >= 0)
        {
            dest->sampleET = ((float)(intTemp - vpWorkData.sampleET))/1000;
        }
        else
        {
            dest->sampleET = (float)intTemp/1000;
        }
        vpWorkData.sampleET = intTemp;
    }
    else
    {
        dest->sampleET = 0;
    }


    // store the RX Check value here
    if (vpWorkData.doRXCheck)
        dest->rxCheckPercent = vpWorkData.rxCheckPercent;

    src->outTemp = SHORT_SWAP(src->outTemp);
    if (src->outTemp != 0x7FFF)
        dest->outTemp = (float)src->outTemp/10;

    if (src->barometer != 0xFFFF)
    {
        // VP produces sea level pressure (SLP)
        tempshort = SHORT_SWAP(src->barometer);
        dest->barometer = (float)tempshort/1000;

        // calculate station pressure by giving a negative elevation 
        tempfloat = wvutilsCalculateSLP (dest->barometer, 
                                         dest->outTemp,
                                         (float)(-vpWorkData.elevation));
        dest->stationPressure = tempfloat;
    }

    src->inTemp = SHORT_SWAP(src->inTemp);
    if (src->inTemp != 0x7FFF)
        dest->inTemp = (float)src->inTemp/10;
    if (src->inHumidity != 0xFF)
        dest->inHumidity = src->inHumidity;
    if (src->outHumidity != 0xFF)
        dest->outHumidity = src->outHumidity;
    if (src->windSpeed != 0xFF)
    {
        dest->windSpeed = src->windSpeed;
        dest->windGust = src->windSpeed;
    }
    tempshort = SHORT_SWAP(src->windDir);
    if (tempshort != 0xFFFF && tempshort != 0x7FFF)
    {
        dest->windDir = tempshort;
        dest->windGustDir = tempshort;
    }
    tempshort = SHORT_SWAP(src->rainRate);
    if (tempshort != 0xFFFF && tempshort != 0x7FFF)
    {
	    dest->rainRate = (float)tempshort/100;
    }
    if (src->UV != 0xFF)
        dest->UV = src->UV;
    if (src->radiation != 0xFFFF)
        dest->radiation = SHORT_SWAP(src->radiation);

    if (src->tenMinuteAvgWindSpeed != 0xFF)
        dest->tenMinuteAvgWindSpeed = src->tenMinuteAvgWindSpeed;
    dest->forecastIcon = src->forecastIcon;
    dest->forecastRule = src->forecastRule;
    dest->txBatteryStatus = src->txBatteryStatus;
    dest->consBatteryVoltage = SHORT_SWAP(src->consBatteryVoltage);
    dest->extraTemp1 = src->extraTemp1;
    dest->extraTemp2 = src->extraTemp2;
    dest->extraTemp3 = src->extraTemp3;
    dest->soilTemp1 = src->soilTemp1;
    dest->soilTemp2 = src->soilTemp2;
    dest->soilTemp3 = src->soilTemp3;
    dest->soilTemp4 = src->soilTemp4;
    dest->leafTemp1 = src->leafTemp1;
    dest->leafTemp2 = src->leafTemp2;
    dest->extraHumid1 = src->extraHumid1;
    dest->extraHumid2 = src->extraHumid2;
    dest->soilMoist1 = src->soilMoist1;
    dest->soilMoist2 = src->soilMoist2;
    if (src->leafWet1 != 0xFF)
        dest->leafWet1 = src->leafWet1;
    if (src->leafWet2 != 0xFF)
        dest->leafWet2 = src->leafWet2;

    
    //  ... now calculate a few
    dest->dewpoint = wvutilsCalculateDewpoint (dest->outTemp, 
                                               (float)dest->outHumidity);
    dest->windchill = wvutilsCalculateWindChill (dest->outTemp, 
                                                 (float)dest->windSpeed);
    dest->heatindex = wvutilsCalculateHeatIndex (dest->outTemp, 
                                                 (float)dest->outHumidity);

    return;
}

static int processArchivePage (WVIEWD_WORK *work, ARCHIVE_PAGE *page)
{
    int                     i, start, numNewRecords = 0;

    start = vpWorkData.archiveRecOffset;
    vpWorkData.archiveRecOffset = 0;

    for (i = start; i < 5; i ++)
    {
        //  ... swap shorts for big endian before we get started... 
        swapArchiveRecord (&page->record[i]);

        if ((page->record[i].date < work->archiveDate) ||
            (page->record[i].date == work->archiveDate && page->record[i].time <= work->archiveTime) ||
            (page->record[i].date == 0xFFFF) ||
            (page->record[i].time == 0xFFFF))
        {
            //  ... this is an old record or uninitialized, skip it
            continue;
        }

        //  ... validate archive record
        if (page->record[i].outTemp == 0x7FFF)
        {
            // we have a bad record, log and set to last known good record ...
            radMsgLog (PRI_MEDIUM, "BAD archive record for %4.4d-%2.2d-%2.2d %2.2d:%2.2d",
                       (page->record[i].date >> 9) + 2000,
                       (page->record[i].date >> 5) & 0xf,
                       page->record[i].date & 0x1f,
                       page->record[i].time/100,
                       page->record[i].time%100);
            radMsgLog (PRI_MEDIUM, "Copying last good archive record for this interval...");

            dbfIncrementConsoleTime (&work->lastGoodArchiveRecord, work->archiveInterval);
            page->record[i] = work->lastGoodArchiveRecord;
        }
        else
        {
            // save it as the last good record
            work->lastGoodArchiveRecord = page->record[i];
        }

        numNewRecords ++;
        work->archiveDate = page->record[i].date;
        work->archiveTime = page->record[i].time;

        // save the high wind speed
        work->lastHighWindSpeed = (int)page->record[i].highWindSpeed;


        // indicate through the station API
        (*ArchiveIndicator) (&page->record[i]);
    }

    vpWorkData.archiveCurrentPage ++;
    return numNewRecords;
}
#endif


static USHORT genCRC (void *data, int length)
{
    UCHAR           *ptr = (UCHAR *)data;
    USHORT          crc = 0;
    register int    i;

    for (i = 0; i < length; i ++)
    {
        crc =  crc_table[(crc >> 8) ^ ptr[i]] ^ (crc << 8);
    }

    crc = ((crc >> 8) & 0xFF) | ((crc << 8) & 0xFF00);
    return crc;
}

static int readWithCRC (WVIEWD_WORK *work, void *bfr, int len, int msTimeout)
{
    int     retVal, index = 0;
    UCHAR   *ptr = (UCHAR *)bfr;
    USHORT  crc = 0;

    retVal = (*work->medium.read) (&work->medium, bfr, len, msTimeout);
    if (retVal != len)
    {
        return ERROR;
    }

    for (index = 0; index < len; index ++)
    {
        crc =  crc_table [(crc >> 8) ^ ptr[index]] ^ (crc << 8);
    }

    return (crc == 0) ? (len) : (ERROR);
}

static int writeWithCRC (WVIEWD_WORK *work, void *buffer, int length)
{
    int     retVal;
    int     wrerrno = 0;
    USHORT  crc;
    
    crc = SHORT_SWAP(genCRC (buffer, length));

    retVal = (*work->medium.write) (&work->medium, buffer, length);
    if (retVal != length)
    {
        if (retVal == -1)
        {
            wrerrno = errno;
        }

        return ERROR;
    }
    retVal = (*work->medium.write) (&work->medium, &crc, 2);
    if (retVal != 2)
    {
        if (retVal == -1)
        {
            wrerrno = errno;
        }

        return ERROR;
    }

    /*  ... now wait for the TX buffer to empty out (this blocks)
    */
    (*work->medium.txdrain) (&work->medium);

    return length;
}

static int wakeupConsole (WVIEWD_WORK *work)
{
    int         retVal;
    UCHAR       bfr[32];

    bfr[0] = VP_CR;
    (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
    retVal = (*work->medium.write) (&work->medium, bfr, 1);
    if (retVal != 1)
    {
        radMsgLog (PRI_MEDIUM, "wakeupConsole: Write ERROR!");
        return ERROR;
    }

    memset (bfr, 0, sizeof (bfr));
    retVal = (*work->medium.read) (&work->medium, bfr, 2, 250);

    if (retVal != 2 || bfr[0] != VP_LF || bfr[1] != VP_CR)
    {
        // hit him again!
        bfr[0] = VP_CR;
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        retVal = (*work->medium.write) (&work->medium, bfr, 1);
        if (retVal != 1)
        {
            radMsgLog (PRI_MEDIUM, "wakeupConsole: Write ERROR!");
            return ERROR;
        }
        memset (bfr, 0, sizeof (bfr));
        retVal = (*work->medium.read) (&work->medium, bfr, 2, 3000);
        if (retVal != 2)
        {
            // radMsgLog (PRI_MEDIUM, "wakeupConsole: Read TIMEOUT!");
            return ERROR;
        }
    }

    (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
    if (bfr[0] == VP_LF && bfr[1] == VP_CR)
    {
        return OK;
    }
    else
    {
        // radMsgLog (PRI_MEDIUM, "wakeupConsole: Invalid data: %2.2X %2.2X",
        //            bfr[0], bfr[1]);
        return ERROR;
    }
}



/*  ... ----- API methods -----
*/
void vpifIndicateStationUp (void)
{
    radProcessEventsSend (NULL, STATION_INIT_COMPLETE_EVENT, 0);
    return;
}    

void vpifIndicateLoopDone (void)
{
    radProcessEventsSend (NULL, STATION_LOOP_COMPLETE_EVENT, 0);
    return;
}    

void vpifFlush (WVIEWD_WORK *work)
{
    (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
    return;
}

int vpifWakeupConsole (WVIEWD_WORK *work)
{
    vpifFlush (work);

    vpWorkData.wakingFlag = 1;
    return wakeupConsole (work);
}

int vpifSynchronizeConsoleClock (WVIEWD_WORK *work)
{
    time_t          ntime;
    struct tm       locTime;
    int             gmtOffset;

    /*  ... wakeup the console
    */
    if (vpifWakeupConsole (work) == ERROR)
    {
        radMsgLog (PRI_HIGH, "vpifSynchronizeConsoleClock: WAKEUP1 failed");
        return ERROR;
    }

    /*  ... set the console GMT offset
    */
    if (vpifSetGMTOffset (work, &gmtOffset) == ERROR)
    {
        radMsgLog (PRI_HIGH, "vpifSynchronizeConsoleClock: GMT failed");
        return ERROR;
    }

    /*  ... wakeup the console
    */
    if (vpifWakeupConsole (work) == ERROR)
    {
        radMsgLog (PRI_HIGH, "vpifSynchronizeConsoleClock: WAKEUP2 failed");
        return ERROR;
    }

    /*  ... set the console date and time
    */
    ntime = time (NULL);
    localtime_r (&ntime, &locTime);

    if (vpifSetTime (work,
                       locTime.tm_year + 1900,
                       locTime.tm_mon + 1,
                       locTime.tm_mday,
                       locTime.tm_hour,
                       locTime.tm_min,
                       locTime.tm_sec)
        == ERROR)
    {
        radMsgLog (PRI_HIGH, "vpifSynchronizeConsoleClock: SETTIME failed");
        return ERROR;
    }

    radMsgLog (PRI_STATUS, "station time synchronized to: %2.2d-%2.2d-%4.4d %2.2d:%2.2d:%2.2d",
               locTime.tm_mon + 1, locTime.tm_mday, locTime.tm_year + 1900,
               locTime.tm_hour, locTime.tm_min, locTime.tm_sec);
    radMsgLog (PRI_STATUS, "station GMT offset synchronized to: %d hours, %d minutes",
               gmtOffset/60, gmtOffset%60);

    // the VP console is cranky after a time reset
    radUtilsSleep (1500);

    return OK;
}

int vpifGetTime
(
    WVIEWD_WORK *work,
    USHORT      *year,
    USHORT      *month,
    USHORT      *day,
    USHORT      *hour,
    USHORT      *minute,
    USHORT      *second
)
{
    int         retVal, len;
    UCHAR       bfr[32];

    strcpy ((char *)bfr, "GETTIME");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    retVal = (*work->medium.write) (&work->medium, bfr, len+1);
    if (retVal != len+1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 1000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    memset (bfr, 0, sizeof (bfr));
    retVal = readWithCRC (work, bfr, 8, 1000);
    if (retVal != 8)
    {
        return ERROR;
    }

    *second = bfr[0];
    *minute = bfr[1];
    *hour   = bfr[2];
    *day    = bfr[3];
    *month  = bfr[4];
    *year   = bfr[5] + 1900;

    return OK;
}

int vpifGetLatandLong
(
    WVIEWD_WORK *work
)
{
    int         retVal, len;
    short       tmp[16];
    UCHAR       *bfr = (UCHAR *)tmp;

    strcpy ((char *)bfr, "EEBRD 0B 2");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, bfr, len + 1) != len + 1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    memset (bfr, 0, sizeof (tmp));
    retVal = readWithCRC (work, bfr, 4, 2000);
    if (retVal != 4)
    {
        return ERROR;
    }

    work->latitude = SHORT_SWAP(tmp[0]);

    strcpy ((char *)bfr, "EEBRD 0D 2");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, bfr, len + 1) != len + 1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    memset (bfr, 0, sizeof (tmp));
    retVal = readWithCRC (work, bfr, 4, 2000);
    if (retVal != 4)
    {
        return ERROR;
    }

    work->longitude = SHORT_SWAP(tmp[0]);

    strcpy ((char *)bfr, "EEBRD 0F 2");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, bfr, len + 1) != len + 1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    memset (bfr, 0, sizeof (tmp));
    retVal = readWithCRC (work, bfr, 4, 2000);
    if (retVal != 4)
    {
        return ERROR;
    }

    work->elevation = SHORT_SWAP(tmp[0]);

    return OK;
}

int vpifGetArchiveInterval (WVIEWD_WORK *work)
{
    USHORT              temp[32];                       // short align
    UCHAR               *ptr = (UCHAR *)temp;
    int                 len, retVal;
    ARCHIVE_INTERVAL    *interval = (ARCHIVE_INTERVAL *)temp;

    strcpy ((char *)ptr, "EEBRD 2D 1");
    len = strlen ((char *)ptr);
    ptr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, ptr, len + 1) != len + 1)
    {
        return ERROR;
    }

    if (vpifGetAck (work, 3000) == ERROR)
    {
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        return ERROR;
    }

    retVal = readWithCRC (work, interval, sizeof (ARCHIVE_INTERVAL), 3000);
    if (retVal != sizeof (ARCHIVE_INTERVAL))
    {
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        return ERROR;
    }

    work->archiveInterval = interval->interval;
    return OK;
}

int vpifSetTime
(
    WVIEWD_WORK *work,
    USHORT      year,
    USHORT      month,
    USHORT      day,
    USHORT      hour,
    USHORT      minute,
    USHORT      second
)
{
    int         retVal, len;
    USHORT      usbfr[16];
    UCHAR       *bfr = (UCHAR *)usbfr;
    USHORT      *crc;

    strcpy ((char *)bfr, "SETTIME");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    retVal = (*work->medium.write) (&work->medium, bfr, len+1);
    if (retVal != len+1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);       // 2 secs
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    bfr[0]  = second;
    bfr[1]  = minute;
    bfr[2]  = hour;
    bfr[3]  = day;
    bfr[4]  = month;
    bfr[5]  = year - 1900;
    crc     = (USHORT *)&bfr[6];
    *crc    = SHORT_SWAP(genCRC (bfr, 6));

    retVal = (*work->medium.write) (&work->medium, bfr, 8);
    if (retVal != 8)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);       // 2 secs
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    return OK;
}

int vpifSetGMTOffset
(
    WVIEWD_WORK *work,
    int         *save
)
{
    int             retVal, len, gmtOffsetHours, gmtOffsetMinutes;
    short           tmp[16];
    UCHAR           *bfr = (UCHAR *)tmp;
    time_t          nowtime = time (NULL);
    struct tm       bknTime;
    long            gmtMinsEast;

    localtime_r (&nowtime, &bknTime);

#ifdef HAVE_STRUCT_TM_TM_ZONE
    gmtMinsEast = bknTime.tm_gmtoff/60;
#else
    gmtMinsEast = -(timezone/60);
#endif

    gmtOffsetHours      = gmtMinsEast/60;
    gmtOffsetMinutes    = gmtMinsEast%60;
    *save               = gmtMinsEast;

    strcpy ((char *)bfr, "EEBWR 12 5");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, bfr, len + 1) != len + 1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    memset (bfr, 0, 8);

    // set dst to manual and turn it off (it is reflected in the GMT offset)
    bfr[0] = 1;
    bfr[1] = 0;

    // set the GMT offset
    tmp[1] = SHORT_SWAP((short)((gmtOffsetHours * 100) + gmtOffsetMinutes));

    // make sure the VP uses the GMT offset
    bfr[4] = 1;

    retVal = writeWithCRC (work, bfr, 5);
    if (retVal != 5)
    {
        return ERROR;
    }

    return OK;
}

/*  ... this guy reads/parses msgs and updates the internal data stores;
    ... returns OK or ERROR (except when ACK expected, then returns
    ... ACK, NAK, or ERROR)
*/
int vpifReadMessage (WVIEWD_WORK *work, int expectACK)
{
    USHORT              temp[VP_BYTE_LENGTH_MAX/2];     // short align
    UCHAR               *chPtr = (UCHAR *)temp;
    ARCHIVE_PAGE        *arcRec = (ARCHIVE_PAGE *)temp;
    ARCHIVE_INTERVAL    *interval = (ARCHIVE_INTERVAL *)temp;
    DMPAFT_HDR          *dmphdr = (DMPAFT_HDR *)temp;
    LOOP_DATA           *loop = (LOOP_DATA *)temp;
    int                 retVal;

    memset (temp, 0, sizeof(temp));
    switch (vpWorkData.reqMsgType)
    {
        case SER_MSG_ACK:
            retVal = (*work->medium.read) (&work->medium, chPtr, 1, 1000);
            if (retVal != 1)
            {
                return ERROR;
            }
            if (chPtr[0] == VP_ACK)
            {
                return VP_ACK;
            }
            else if (chPtr[0] == VP_NAK)
            {
                return VP_NAK;
            }
            else
            {
                return ERROR;
            }
    
#ifndef _VP_CONFIG_ONLY
        case SER_MSG_DMPAFT_HDR:
            if (expectACK)
            {
                if (vpifGetAck (work, 1000) == ERROR)
                {
                    (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
                    return ERROR;
                }
            }
            retVal = readWithCRC (work, dmphdr, sizeof (DMPAFT_HDR), 3000);
            if (retVal != sizeof (DMPAFT_HDR))
            {
                (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
                return ERROR;
            }
    
            vpWorkData.archivePages = SHORT_SWAP(dmphdr->pages);
            vpWorkData.archiveRecOffset = SHORT_SWAP(dmphdr->firstRecIndex);
            return OK;
    
        case SER_MSG_ARCHIVE:
            retVal = readWithCRC (work, arcRec, sizeof (ARCHIVE_PAGE), 5000);
            if (retVal != sizeof (ARCHIVE_PAGE))
            {
                (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
                return ERROR;
            }
    
            /*  ... send ACK before processing to speed up download
            */
            if (vpWorkData.archiveCurrentPage < vpWorkData.archivePages - 1)
            {
                if (vpifSendAck (work) == ERROR)
                {
                    return ERROR;
                }
            }

            return (processArchivePage (work, arcRec));
    
        case SER_MSG_LOOP:
            if (expectACK)
            {
                if (vpifGetAck (work, 1000) == ERROR)
                {
                    (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
                    return ERROR;
                }
            }
            retVal = readWithCRC (work, loop, sizeof (LOOP_DATA), 5000);
            if (retVal != sizeof (LOOP_DATA))
            {
                (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
                return ERROR;
            }
    
            /*  ... store in IPM format
            */
            storeLoopPkt (&work->loopPkt, loop);
    
            return OK;
#endif    
    
        case SER_MSG_NONE:
        default:
            /*  ... just flush the receive buffer, we're not expecting anything!
            */
            (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
            return OK;
    }
}

int vpifGetAck (WVIEWD_WORK *work, int msWait)
{
    UCHAR       temp[4];

    if ((*work->medium.read) (&work->medium, temp, 1, msWait) != 1)
    {
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        return ERROR;
    }

    // this "flushing" of CR/LF's takes care of many problems with terminal 
    // servers who want to echo them or attach them to the end of packets - 
    // while not bothering cleaner true serial interfaces...
    while (temp[0] == VP_LF || temp[0] == VP_CR)
    {
        // extra LF/CR, discard and read again
        if ((*work->medium.read) (&work->medium, temp, 1, msWait) != 1)
        {
            (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
            return ERROR;
        }
    }

    if (temp[0] != VP_ACK)
    {
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        return ERROR;
    }

    return OK;
}


/*  ... send msgs to the console;
    ... will set the "reqMsgType" of the work area appropriately;
    ... returns OK or ERROR
*/
int vpifSendAck (WVIEWD_WORK *work)
{
    USHORT              temp[32];                       // short align
    UCHAR               *ptr = (UCHAR *)temp;

    ptr[0] = VP_ACK;

    if ((*work->medium.write) (&work->medium, ptr, 1) != 1)
    {
        return ERROR;
    }

    return OK;
}

int vpifSendNak (WVIEWD_WORK *work)
{
    USHORT              temp[32];                       // short align
    UCHAR               *ptr = (UCHAR *)temp;

    ptr[0] = VP_NAK;

    if ((*work->medium.write) (&work->medium, ptr, 1) != 1)
    {
        return ERROR;
    }

    return OK;
}

int vpifSendCancel (WVIEWD_WORK *work)
{
    USHORT              temp[32];                       // short align
    UCHAR               *ptr = (UCHAR *)temp;

    ptr[0] = VP_CANCEL;

    if ((*work->medium.write) (&work->medium, ptr, 1) != 1)
    {
        return ERROR;
    }

    return OK;
}

int vpifSendDumpAfterRqst (WVIEWD_WORK *work)
{
    USHORT              temp[32];                       // short align
    UCHAR               *ptr = (UCHAR *)temp;
    int                 len;

    strcpy ((char *)ptr, "DMPAFT");
    len = strlen ((char *)ptr);
    ptr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, ptr, len + 1) != len + 1)
    {
        return ERROR;
    }

    vpWorkData.reqMsgType = SER_MSG_ACK;
    return OK;
}

int vpifSendDumpDateTimeRqst (WVIEWD_WORK *work)
{
    USHORT              temp[32];                       // short align

    temp[0] = SHORT_SWAP(work->archiveDate);
    temp[1] = SHORT_SWAP(work->archiveTime);
    temp[2] = SHORT_SWAP(genCRC (temp, 4));

    if ((*work->medium.write) (&work->medium, temp, 6) != 6)
    {
        return ERROR;
    }

    vpWorkData.reqMsgType = SER_MSG_DMPAFT_HDR;
    return OK;
}

int vpifSendLoopRqst (WVIEWD_WORK *work, int number)
{
    USHORT              temp[32];                       // short align
    UCHAR               *ptr = (UCHAR *)temp;
    int                 len;

    sprintf ((char *)ptr, "LOOP %d", number);
    len = strlen ((char *)ptr);
    ptr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, ptr, len + 1) != len + 1)
    {
        return ERROR;
    }

    vpWorkData.reqMsgType = SER_MSG_LOOP;
    return OK;
}

int vpifGetRXCheck (WVIEWD_WORK *work)
{
    int                 i, len, retVal, done;
    USHORT              temp[VP_BYTE_LENGTH_MAX/2];     // short align
    UCHAR               *ptr = (UCHAR *)temp;
    char                *token;
    int                 rxGood, rxMiss, rxCRC, tempint;

    vpWorkData.rxCheck[0] = 0;
    strcpy ((char *)ptr, "RXCHECK");
    len = strlen ((char *)ptr);
    ptr[len] = VP_CR;

    if ((*work->medium.write) (&work->medium, ptr, len + 1) != len + 1)
    {
        return ERROR;
    }

    // lose the <LF><CR>OK<LF><CR>
    if ((*work->medium.read) (&work->medium, ptr, 6, 2000) != 6)
    {
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        return ERROR;
    }
    
    memset (ptr, 0, 64);
    for (i = 0, done = 0; (done == 0) && (i < 63); i ++)
    {
        retVal = (*work->medium.read) (&work->medium, &ptr[i], 1, 2000);
        if (retVal != 1)
        {
            return ERROR;
        }
        
        if (ptr[i] == VP_CR)
            done = 1;
    }

    if (!done || i < 3)
    {
        return ERROR;
    }

    i -= 2;                     // lose the <CR> and <LF>
    ptr[i] = 0;

    strcpy (vpWorkData.rxCheck, (char *)ptr);

    token = strtok ((char *)ptr, " ");
    if (token == NULL)
    {
        return ERROR;
    }
    rxGood = atoi (token);

    token = strtok (NULL, " ");
    if (token == NULL)
    {
        return ERROR;
    }
    rxMiss = atoi (token);

    // toss re-syncs
    token = strtok (NULL, " ");
    if (token == NULL)
    {
        return ERROR;
    }
    // toss good-in-a-row
    token = strtok (NULL, " ");
    if (token == NULL)
    {
        return ERROR;
    }

    token = strtok (NULL, " ");
    if (token == NULL)
    {
        return ERROR;
    }
    rxCRC = atoi (token);
    if (rxCRC < 0)
        rxCRC += 65536;

    // now we have something to work with
    tempint = rxGood - vpWorkData.rxCheckGood;
    vpWorkData.rxCheckGood = rxGood;
    if (tempint < 0)
    {
        // must have rolled over the counts or manually reset
        vpWorkData.rxCheckMissed = rxMiss;
        vpWorkData.rxCheckCRC = rxCRC;
        return OK;
    }
    else
    {
        rxGood = tempint;
    }
    tempint = rxMiss - vpWorkData.rxCheckMissed;
    vpWorkData.rxCheckMissed = rxMiss;
    if (tempint < 0)
    {
        // rolled over
        vpWorkData.rxCheckCRC = rxCRC;
        return OK;
    }
    else
    {
        rxMiss = tempint;
    }
    tempint = rxCRC - vpWorkData.rxCheckCRC;
    vpWorkData.rxCheckCRC = rxCRC;
    if (tempint < 0)
    {
        // rolled over
        return OK;
    }
    else
    {
        rxCRC = tempint;
    }

    // finally calculate the percentage
    tempint = rxGood + rxMiss + rxCRC;
    if (tempint > 0)
    {
        vpWorkData.rxCheckPercent = (100 * rxGood)/tempint;
    }
    else
    {
        vpWorkData.rxCheckPercent = 100;
    }

    return OK;
}


//////// methods for vpconfig ////////
#ifdef _VP_CONFIG_ONLY

int vpconfigGetArchiveInterval
(
    WVIEWD_WORK *work
)
{
    int                 retVal, len;
    USHORT              temp[VP_BYTE_LENGTH_MAX/2];     // short align
    UCHAR               *ptr = (UCHAR *)temp;
    ARCHIVE_INTERVAL    *interval = (ARCHIVE_INTERVAL *)temp;

    strcpy ((char *)ptr, "EEBRD 2D 1");
    len = strlen ((char *)ptr);
    ptr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, ptr, len + 1) != len + 1)
    {
        return ERROR;
    }

    if (vpifGetAck (work, 2000) == ERROR)
    {
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        return ERROR;
    }
    retVal = readWithCRC (work,
                          interval,
                          sizeof (ARCHIVE_INTERVAL),
                          2000);
    if (retVal != sizeof (ARCHIVE_INTERVAL))
    {
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        return ERROR;
    }

    work->archiveInterval = interval->interval;

    return OK;
}

int vpconfigGetFWVersion
(
    WVIEWD_WORK *work
)
{
    int                 i, retVal, len, done = 0;
    USHORT              temp[VP_BYTE_LENGTH_MAX/2];     // short align
    UCHAR               *indexPtr, *ptr = (UCHAR *)temp;

    strcpy ((char *)ptr, "VER");
    len = strlen ((char *)ptr);
    ptr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, ptr, len + 1) != len + 1)
    {
        return ERROR;
    }

    // lose the <LF><CR>OK<LF><CR>
    if ((*work->medium.read) (&work->medium, ptr, 6, 2000) != 6)
    {
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        return ERROR;
    }
    
    indexPtr = ptr;
    memset (indexPtr, 0, 32);
    while (!done)
    {
        retVal = (*work->medium.read) (&work->medium, indexPtr, 1, 2000);
        if (retVal != 1)
        {
            done = 1;
        }
        
        if (*indexPtr == VP_CR)
            done = 1;
        
        indexPtr ++;
    }
    
    len = indexPtr - ptr;
    len -= 2;                   // lose the <CR> and <LF>
    ptr[len] = 0;


#if 0
printf ("@!@ GetRXCheck: ");
for (i = 0; i < len; i ++)
    printf ("%2.2X, ", ptr[i]);
printf ("\n len = %d\n", len);
#endif

    strcpy (vpWorkData.fwVersion, (char *)ptr);

    return OK;
}

int vpconfigGetRainSeasonStart
(
    WVIEWD_WORK *work
)
{
    int         retVal, len;
    short       tmp[16];
    UCHAR       *bfr = (UCHAR *)tmp;

    strcpy ((char *)bfr, "EEBRD 2C 1");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, bfr, len + 1) != len + 1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    memset (bfr, 0, sizeof (tmp));
    retVal = readWithCRC (work, bfr, 3, 2000);
    if (retVal != 3)
    {
        return ERROR;
    }

    vpWorkData.rainSeasonStart = bfr[0];

    return OK;
}

int vpconfigSetInterval
(
    WVIEWD_WORK *work,
    int         interval
)
{
    int         i, retVal, len, done = 0;
    USHORT      usbfr[16];
    UCHAR       *indexPtr, *bfr = (UCHAR *)usbfr;

    sprintf ((char *)bfr, "SETPER %d", interval);
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    retVal = (*work->medium.write) (&work->medium, bfr, len+1);
    if (retVal != len+1)
    {
        return ERROR;
    }

    // lose the <LF><CR>
    if ((*work->medium.read) (&work->medium, bfr, 2, 5000) != 2)
    {
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        return ERROR;
    }

    indexPtr = bfr;
    memset (indexPtr, 0, 32);
    while (!done)
    {
        retVal = (*work->medium.read) (&work->medium, indexPtr, 1, 2000);
        if (retVal != 1)
        {
            done = 1;
        }
        
        indexPtr ++;
    }

    len = (indexPtr-1) - bfr;
    len -= 2;                   // lose the <CR> and <LF>
    bfr[len] = 0;

    if (strncmp ((char *)bfr, "OK", 2))
    {
        return ERROR;
    }

    sleep (1);
    
    sprintf ((char *)bfr, "NEWSETUP");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    retVal = (*work->medium.write) (&work->medium, bfr, len+1);
    if (retVal != len+1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 5000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }
    return OK;
}

int vpconfigClearArchiveMemory
(
    WVIEWD_WORK *work
)
{
    int         retVal, len;
    USHORT      usbfr[16];
    UCHAR       *bfr = (UCHAR *)usbfr;

    sprintf ((char *)bfr, "CLRLOG");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    retVal = (*work->medium.write) (&work->medium, bfr, len+1);
    if (retVal != len+1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }
    
    return OK;
}

int vpconfigSetElevation
(
    WVIEWD_WORK *work,
    int         value
)
{
    int         retVal, len, done = 0;
    USHORT      usbfr[16];
    UCHAR       *indexPtr, *bfr = (UCHAR *)usbfr;

    sprintf ((char *)bfr, "BAR=0 %d", value);
    len = strlen ((char *)bfr);
    bfr[len] = VP_CR;
    bfr[len+1] = VP_LF;

    retVal = (*work->medium.write) (&work->medium, bfr, len+2);
    if (retVal != len+2)
    {
        return ERROR;
    }

    // lose the <LF><CR>
    if ((*work->medium.read) (&work->medium, bfr, 2, 5000) != 2)
    {
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        return ERROR;
    }

    indexPtr = bfr;
    memset (indexPtr, 0, 32);
    while (!done)
    {
        retVal = (*work->medium.read) (&work->medium, indexPtr, 1, 5000);
        if (retVal != 1)
        {
            done = 1;
        }
        
        if (*indexPtr == VP_CR)
            done = 1;
        
        indexPtr ++;
    }
    
    len = indexPtr - bfr;
    len -= 2;                   // lose the <CR> and <LF>
    bfr[len] = 0;

    if (strncmp ((char *)bfr, "OK", 2))
    {
        return ERROR;
    }

    return OK;
}

int vpconfigSetGain
(
    WVIEWD_WORK *work,
    int         on
)
{
    int                 i, retVal, len, done = 0;
    USHORT              temp[VP_BYTE_LENGTH_MAX/2];     // short align
    UCHAR               *indexPtr, *ptr = (UCHAR *)temp;

    sprintf ((char *)ptr, "GAIN %d", on);
    len = strlen ((char *)ptr);
    ptr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, ptr, len + 1) != len + 1)
    {
        return ERROR;
    }

    // lose the <LF><CR>
    if ((*work->medium.read) (&work->medium, ptr, 2, 2000) != 2)
    {
        (*work->medium.flush) (&work->medium, WV_QUEUE_INPUT);
        return ERROR;
    }
    
    indexPtr = ptr;
    memset (indexPtr, 0, 32);
    while (!done)
    {
        retVal = (*work->medium.read) (&work->medium, indexPtr, 1, 2000);
        if (retVal != 1)
        {
            done = 1;
        }
        
        if (*indexPtr == VP_CR)
            done = 1;
        
        indexPtr ++;
    }
    
    len = indexPtr - ptr;
    len -= 2;                   // lose the <CR> and <LF>
    ptr[len] = 0;

    if (!strncmp ((char *)ptr, "OK", 2))
        return OK;
    else
        return ERROR;

    return OK;
}

int vpconfigSetLatandLong
(
    WVIEWD_WORK *work,
    int         latitude,
    int         longitude
)
{
    int         retVal, len;
    short       tmp[16], lat1, long1;
    UCHAR       *bfr = (UCHAR *)tmp;

    strcpy ((char *)bfr, "EEBWR 0B 4");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, bfr, len + 1) != len + 1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    lat1 = (short)latitude;
    long1 = (short)longitude;
    memset (bfr, 0, 4);
    tmp[0] = SHORT_SWAP(lat1);
    tmp[1] = SHORT_SWAP(long1);
    retVal = writeWithCRC (work, bfr, 4);
    if (retVal != 4)
    {
        return ERROR;
    }

    sleep (3);
    
    sprintf ((char *)bfr, "NEWSETUP");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    retVal = (*work->medium.write) (&work->medium, bfr, len+1);
    if (retVal != len+1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 5000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    return OK;
}

int vpconfigSetRainSeasonStart
(
    WVIEWD_WORK *work,
    int         startMonth
)
{
    int         retVal, len;
    short       tmp[16];
    UCHAR       *bfr = (UCHAR *)tmp;

    strcpy ((char *)bfr, "EEBWR 2C 1");
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, bfr, len + 1) != len + 1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    memset (bfr, 0, sizeof (tmp));
    bfr[0] = startMonth;
    retVal = writeWithCRC (work, bfr, 1);
    if (retVal != 1)
    {
        return ERROR;
    }

    return OK;
}

int vpconfigSetRainYearToDate
(
    WVIEWD_WORK *work,
    float       rainAmount
)
{
    int         retVal, len;
    short       tmp[16];
    UCHAR       *bfr = (UCHAR *)tmp;

    rainAmount *=1000;
    rainAmount = (int)rainAmount;
    rainAmount += 1;
    rainAmount /= 10;
    
    retVal = (int)(rainAmount);              // rain collector = 0.01 inches
    sprintf ((char *)bfr, "PUTRAIN %d", retVal);
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, bfr, len + 1) != len + 1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    return OK;
}

int vpconfigSetETYearToDate
(
    WVIEWD_WORK *work,
    float       etAmount
)
{
    int         retVal, len;
    short       tmp[16];
    UCHAR       *bfr = (UCHAR *)tmp;

    etAmount *=1000;
    etAmount = (int)etAmount;
    etAmount += 1;
    etAmount /= 10;

    retVal = (int)etAmount;                  // rain collector = 0.01 inches
    sprintf ((char *)bfr, "PUTET %d", retVal);
    len = strlen ((char *)bfr);
    bfr[len] = VP_LF;

    if ((*work->medium.write) (&work->medium, bfr, len + 1) != len + 1)
    {
        return ERROR;
    }

    retVal = (*work->medium.read) (&work->medium, bfr, 1, 2000);
    if (retVal != 1)
    {
        return ERROR;
    }
    if (bfr[0] != VP_ACK)
    {
        return ERROR;
    }

    return OK;
}

#endif

