/*---------------------------------------------------------------------------
 
  FILENAME:
        computedData.c
 
  PURPOSE:
        Provide utilities to compute and store HILOW values.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        08/07/2005      M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2005, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <signal.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>
#include <errno.h>
#include <math.h>


/*  ... Library include files
*/
#include <radmsgLog.h>
#include <radsysutils.h>

/*  ... Local include files
*/
#include <services.h>
#include <computedData.h>

/*  ... global memory declarations
*/

/*  ... local memory
*/
static COMPDATA_WORK        cdWork;
static ARCHIVE_RECORD       ArcRecStore;
static HeaderBlock          fileHdr;
static DailySummaryRecord   sumRecord;
static ArchiveRecord        arcRecord;


// compute HILOW value deltas
static int computeDataChanges (WVIEWD_WORK *work)
{
    int                 j, currday;
    FILE                *infile;
    time_t              timenow = time (NULL);
    struct tm           bkntimenow, bknweekago, bkndayago, bknhourago;
    int                 weekago = FALSE, dayago = FALSE, hourago = FALSE;
    char                startName[256];
    SENSOR_STORE        *store = &work->sensors;
    LOOP_PKT            *current = &work->loopPkt;
    HeaderBlock         fileHdr;
    DailySummaryRecord  sumRecord;
    ArchiveRecord       arcRecord;
    float               temp;

    // store broken down representations of our three periods of interest
    timenow = time (NULL);
    timenow -= (WV_SECONDS_IN_DAY * 7);
    localtime_r (&timenow, &bknweekago);
    bknweekago.tm_min -= (bknweekago.tm_min % work->archiveInterval);

    timenow = time (NULL);
    timenow -= WV_SECONDS_IN_DAY;
    localtime_r (&timenow, &bkndayago);
    bkndayago.tm_min -= (bkndayago.tm_min % work->archiveInterval);

    timenow = time (NULL);
    timenow -= WV_SECONDS_IN_HOUR;
    localtime_r (&timenow, &bknhourago);
    bknhourago.tm_min -= (bknhourago.tm_min % work->archiveInterval);

    // initialize the store
    store->hourchangetemp = 0;
    store->hourchangewind = 0;
    store->hourchangewinddir = 0;
    store->hourchangehumid = 0;
    store->hourchangedewpt = 0;
    store->hourchangebarom = 0;
    store->daychangetemp = 0;
    store->daychangewind = 0;
    store->daychangewinddir = 0;
    store->daychangehumid = 0;
    store->daychangedewpt = 0;
    store->daychangebarom = 0;
    store->weekchangetemp = 0;
    store->weekchangewind = 0;
    store->weekchangewinddir = 0;
    store->weekchangehumid = 0;
    store->weekchangedewpt = 0;
    store->weekchangebarom = 0;


    // now on to the record examination...
    timenow = time (NULL);
    timenow -= (work->archiveInterval * 60);
    timenow -= (WV_SECONDS_IN_DAY * 7);

    for (currday = 0; currday <= 7; currday ++)
    {
        localtime_r (&timenow, &bkntimenow);
        
        if (bkntimenow.tm_mday == bknweekago.tm_mday)
        {
            weekago = TRUE;
        }
        if (bkntimenow.tm_mday == bkndayago.tm_mday)
        {
            dayago = TRUE;
        }
        if (bkntimenow.tm_mday == bknhourago.tm_mday)
        {
            hourago = TRUE;
        }
        if (!weekago && !dayago && !hourago)
        {
            // just move along
            timenow += WV_SECONDS_IN_DAY;
            continue;
        }
    
        sprintf (startName, "%s/%4.4d-%2.2d.wlk", 
                 work->archivePath, 
                 bkntimenow.tm_year + 1900, 
                 bkntimenow.tm_mon + 1);
        
        infile = fopen (startName, "r");
        if (infile == NULL)
        {
            timenow += WV_SECONDS_IN_DAY;
            weekago = dayago = hourago = FALSE;
            continue;
        }
    
        if (fread (&fileHdr, sizeof(fileHdr), 1, infile) != 1)
        {
            fclose (infile);
            timenow += WV_SECONDS_IN_DAY;
            weekago = dayago = hourago = FALSE;
            continue;
        }
    
        if (fileHdr.dayIndex[bkntimenow.tm_mday].recordsInDay == 0)
        {
            // no entry for this day 
            fclose (infile);
            timenow += WV_SECONDS_IN_DAY;
            weekago = dayago = hourago = FALSE;
            continue;
        }

        //  ... goto the beginning of the archive recs for this day
        if (fseek (infile,
                   sizeof(fileHdr) +
                   DBFILES_RECORD_SIZE*(fileHdr.dayIndex[bkntimenow.tm_mday].startPos + 2),
                   SEEK_SET)
                == -1)
        {
            fclose (infile);
            timenow += WV_SECONDS_IN_DAY;
            weekago = dayago = hourago = FALSE;
            continue;
        }
    
        for (j = 0; j < fileHdr.dayIndex[bkntimenow.tm_mday].recordsInDay - 2; j ++)
        {
            // read the archive record 
            if (fread (&arcRecord, sizeof(arcRecord), 1, infile) != 1)
            {
                fclose (infile);
                return ERROR;
            }

            if ((USHORT)arcRecord.outsideTemp == 0x8000 ||
                (USHORT)arcRecord.outsideTemp == 0x7FFF)
            {
                // if the temp is bad, toss the entire record
                continue;
            }
            if (arcRecord.outsideHum > 1000)
            {
                // if the humidity is bad, toss the entire record
                continue;
            }
            if (arcRecord.windSpeed > 2000)
            {
                // if the windSpeed is bad, toss the entire record
                continue;
            }
    
            if (weekago)
            {
                if (arcRecord.packedTime/60 == bknweekago.tm_hour &&
                    arcRecord.packedTime%60 == bknweekago.tm_min)
                {
                    // we are in business!
                    store->weekchangetemp = current->outTemp - (float)arcRecord.outsideTemp/10;
                    store->weekchangewind = current->windSpeed - (arcRecord.windSpeed/10);
                    if (arcRecord.windDirection < 16)
                        store->weekchangewinddir = (short)((float)current->windDir - (22.5 * (float)arcRecord.windDirection));
                    store->weekchangehumid = current->outHumidity - (arcRecord.outsideHum/10);
                    temp = wvutilsCalculateDewpoint ((float)arcRecord.outsideTemp/10, 
                                                     (float)arcRecord.outsideHum/10);
                    store->weekchangedewpt = current->dewpoint - temp;
                    store->weekchangebarom = current->barometer - (float)arcRecord.barometer/1000;
                }
            }
            else 
            {
                if (dayago)
                {
                    if (arcRecord.packedTime/60 == bkndayago.tm_hour &&
                        arcRecord.packedTime%60 == bkndayago.tm_min)
                    {
                        // we are in business!
                        store->daychangetemp = current->outTemp - (float)arcRecord.outsideTemp/10;
                        store->daychangewind = current->windSpeed - (arcRecord.windSpeed/10);
                        if (arcRecord.windDirection < 16)
                            store->daychangewinddir = (short)((float)current->windDir - (22.5 * (float)arcRecord.windDirection));
                        store->daychangehumid = current->outHumidity - (arcRecord.outsideHum/10);
                        temp = wvutilsCalculateDewpoint ((float)arcRecord.outsideTemp/10, 
                                                         (float)arcRecord.outsideHum/10);
                        store->daychangedewpt = current->dewpoint - temp;
                        store->daychangebarom = current->barometer - (float)arcRecord.barometer/1000;
                    }
                }

                if (hourago)
                {
                    if (arcRecord.packedTime/60 == bknhourago.tm_hour &&
                        arcRecord.packedTime%60 == bknhourago.tm_min)
                    {
                        // we are in business!
                        store->hourchangetemp = current->outTemp - (float)arcRecord.outsideTemp/10;
                        store->hourchangewind = current->windSpeed - (arcRecord.windSpeed/10);
                        if (arcRecord.windDirection < 16)
                            store->hourchangewinddir = (short)((float)current->windDir - (22.5 * (float)arcRecord.windDirection));
                        store->hourchangehumid = current->outHumidity - (arcRecord.outsideHum/10);
                        temp = wvutilsCalculateDewpoint ((float)arcRecord.outsideTemp/10, 
                                                         (float)arcRecord.outsideHum/10);
                        store->hourchangedewpt = current->dewpoint - temp;
                        store->hourchangebarom = current->barometer - (float)arcRecord.barometer/1000;
                    }
                }
            }
        }
        
        fclose (infile);
        timenow += WV_SECONDS_IN_DAY;
        weekago = dayago = hourago = FALSE;
    }

    return OK;
}


// helper to update a sensor set with an archive file record
static void updateSensorsWithArchive
(
    WVIEWD_WORK         *work,
    SENSOR_TIMEFRAMES   frame, 
    ArchiveRecord       *arcRec,
    int                 year,
    int                 month,
    int                 day,
    int                 rainOnlyFlag
)
{
    SENSOR_STORE        *store = &work->sensors;
    time_t              ntime;
    struct tm           tmtime;
    float               tempfloat;
    int                 intTemp;
    int                 arcSamples = (work->archiveInterval*60)/(work->cdataInterval/1000);

    memset (&tmtime, 0, sizeof(tmtime));
    tmtime.tm_year = year - 1900;
    tmtime.tm_mon = month - 1;
    tmtime.tm_mday = day;
    tmtime.tm_hour = arcRec->packedTime/60;
    tmtime.tm_min = arcRec->packedTime%60;
    ntime = mktime (&tmtime);

    // are we just updating the yearly rain sensors?
    if (rainOnlyFlag)
    {
        sensorInsert (&store->sensor[STF_YEAR][SENSOR_RAIN], 
                      ntime,
                      ((float)(arcRec->rain & 0xFFF))/100,
                      1,
                      work->archiveInterval * 60);
        sensorInsert (&store->sensor[STF_YEAR][SENSOR_RAINRATE], 
                      ntime,
                      (float)arcRec->hiRainRate/100,
                      arcSamples,
                      work->archiveInterval * 60);
        sensorInsert (&store->sensor[STF_YEAR][SENSOR_ET], 
                      ntime,
                      (float)arcRec->ET/1000,
                      1,
                      work->archiveInterval * 60);
        return;
    }

    // calculate these
    tempfloat = wvutilsCalculateDewpoint ((float)arcRec->outsideTemp/10, (float)arcRec->outsideHum/10);
    sensorInsert (&store->sensor[frame][SENSOR_DEWPOINT], 
                  ntime,
                  tempfloat,
                  arcSamples,
                  work->archiveInterval * 60);
    
    tempfloat = wvutilsCalculateWindChill ((float)arcRec->outsideTemp/10, (float)arcRec->windSpeed/10);
    sensorInsert (&store->sensor[frame][SENSOR_WCHILL], 
                  ntime,
                  tempfloat,
                  arcSamples,
                  work->archiveInterval * 60);
    tempfloat = wvutilsCalculateHeatIndex ((float)arcRec->outsideTemp/10, (float)arcRec->outsideHum/10);
    sensorInsert (&store->sensor[frame][SENSOR_HINDEX], 
                  ntime,
                  tempfloat,
                  arcSamples,
                  work->archiveInterval * 60);

    sensorInsert (&store->sensor[frame][SENSOR_INTEMP],
                  ntime,
                  (float)arcRec->insideTemp/10,
                  arcSamples,
                  work->archiveInterval * 60);
    sensorInsertHighLow (&store->sensor[frame][SENSOR_OUTTEMP], 
                         ntime,
                         (float)arcRec->outsideTemp/10,
                         (float)arcRec->hiOutsideTemp/10,
                         (float)arcRec->lowOutsideTemp/10,
                         arcSamples,
                         work->archiveInterval * 60);
    sensorInsert (&store->sensor[frame][SENSOR_INHUMID], 
                  ntime,
                  (float)arcRec->insideHum/10,
                  arcSamples,
                  work->archiveInterval * 60);
    sensorInsert (&store->sensor[frame][SENSOR_OUTHUMID], 
                  ntime,
                  (float)arcRec->outsideHum/10,
                  arcSamples,
                  work->archiveInterval * 60);
    sensorInsert (&store->sensor[frame][SENSOR_BP], 
                  ntime,
                  (float)arcRec->barometer/1000,
                  arcSamples,
                  work->archiveInterval * 60);
    sensorInsert (&store->sensor[frame][SENSOR_WSPEED], 
                  ntime,
                  (float)arcRec->windSpeed/10,
                  arcSamples,
                  work->archiveInterval * 60);
    if (arcRec->hiWindDirection < 16)
    {
        sensorInsertWhen (&store->sensor[frame][SENSOR_WGUST], 
                          ntime,
                          (float)arcRec->hiWindSpeed/10,
                          arcSamples,
                          work->archiveInterval * 60,
                          (float)arcRec->hiWindDirection * 22.5);
    }
    sensorInsert (&store->sensor[frame][SENSOR_RAIN], 
                  ntime,
                  ((float)(arcRec->rain & 0xFFF))/100,
                  1,
                  work->archiveInterval * 60);
    sensorInsert (&store->sensor[frame][SENSOR_RAINRATE], 
                  ntime,
                  (float)arcRec->hiRainRate/100,
                  arcSamples,
                  work->archiveInterval * 60);
    sensorInsert (&store->sensor[frame][SENSOR_ET], 
                  ntime,
                  (float)arcRec->ET/1000,
                  1,
                  work->archiveInterval * 60);
    sensorInsert (&store->sensor[frame][SENSOR_UV], 
                  ntime,
                  (float)arcRec->UV/10,
                  arcSamples,
                  work->archiveInterval * 60);
    sensorInsert (&store->sensor[frame][SENSOR_SOLRAD], 
                  ntime,
                  (float)arcRec->solarRad,
                  arcSamples,
                  work->archiveInterval * 60);

    if (arcRec->windDirection < 16)
    {
        windAverageAddValue (&store->wind[frame], arcRec->windDirection);
    }
}

// compute HILOW values for the current hour
static int computeDataHour (WVIEWD_WORK *work)
{
    int                 j, numrecs = 0;
    time_t              timenow = time (NULL);
    struct tm           bkntimenow;
    char                startName[256];
    SENSOR_STORE        *store = &work->sensors;
    FILE                *infile;
    char                temp[512];
    HeaderBlock         fileHdr;
    DailySummaryRecord  sumRecord;
    ArchiveRecord       arcRecord;

    // initialize the store
    sensorClearSet (store->sensor[STF_HOUR]);
    windAverageReset (&store->wind[STF_HOUR]);

    // do this so we pick up the proper hour/day when mins < archiveInterval
    timenow -= (work->archiveInterval * 60);
    
    localtime_r (&timenow, &bkntimenow);
    sprintf (startName, "%s/%4.4d-%2.2d.wlk", 
             work->archivePath, 
             bkntimenow.tm_year + 1900, 
             bkntimenow.tm_mon + 1);
    
    infile = fopen (startName, "r");
    if (infile == NULL)
    {
        return ERROR;
    }

    if (fread (&fileHdr, sizeof(fileHdr), 1, infile) != 1)
    {
        fclose (infile);
        return ERROR;
    }

    if (fileHdr.dayIndex[bkntimenow.tm_mday].recordsInDay == 0)
    {
        // no entry for this day 
        fclose (infile);
        return ERROR;
    }

    // goto the beginning of the archive recs for this day
    if (fseek (infile,
               sizeof(fileHdr) +
               DBFILES_RECORD_SIZE*(fileHdr.dayIndex[bkntimenow.tm_mday].startPos + 2),
               SEEK_SET)
            == -1)
    {
        fclose (infile);
        return ERROR;
    }

    for (j = 0; j < fileHdr.dayIndex[bkntimenow.tm_mday].recordsInDay - 2; j ++)
    {
        // read the archive record 
        if (fread (&arcRecord, sizeof(arcRecord), 1, infile) != 1)
        {
            fclose (infile);
            return ERROR;
        }
        
        // is this for our hour of interest?
        if ((arcRecord.packedTime/60) < bkntimenow.tm_hour)
        {
            continue;
        }

        if (((arcRecord.packedTime/60) == bkntimenow.tm_hour) &&
            ((arcRecord.packedTime%60) < work->archiveInterval))
        {
            // we only allow the next hour's first record...
            continue;
        }

        if ((USHORT)arcRecord.outsideTemp == 0x8000 ||
            (USHORT)arcRecord.outsideTemp == 0x7FFF)
        {
            // if the temp is bad, toss the entire record
            continue;
        }
        if (arcRecord.outsideHum > 1000)
        {
            // if the humidity is bad, toss the entire record
            continue;
        }
        if (arcRecord.windSpeed > 2000)
        {
            // if the windSpeed is bad, toss the entire record
            continue;
        }

        numrecs ++;

        updateSensorsWithArchive (work, 
                                  STF_HOUR, 
                                  &arcRecord, 
                                  bkntimenow.tm_year + 1900,
                                  bkntimenow.tm_mon + 1,
                                  bkntimenow.tm_mday,
                                  FALSE);
    }

    fclose (infile);

    // were there any recs for this hour?
    if (numrecs == 0)
    {
        return ERROR;
    }
    else
    {
        return OK;
    }
}

// compute HILOW values for the current day
static int computeDataDay (WVIEWD_WORK *work)
{
    int                 j, numrecs = 0;
    FILE                *infile;
    time_t              timenow = time (NULL);
    struct tm           bkntimenow;
    char                startName[256];
    SENSOR_STORE        *store = &work->sensors;
    HeaderBlock         fileHdr;
    DailySummaryRecord  sumRecord;
    ArchiveRecord       arcRecord;

    // initialize the store
    sensorClearSet (store->sensor[STF_DAY]);
    windAverageReset (&store->wind[STF_DAY]);

    // do this so we pick up the proper hour/day when mins < archiveInterval
    timenow -= (work->archiveInterval * 60);
    
    localtime_r (&timenow, &bkntimenow);
    sprintf (startName, "%s/%4.4d-%2.2d.wlk", 
             work->archivePath, 
             bkntimenow.tm_year + 1900, 
             bkntimenow.tm_mon + 1);
    
    infile = fopen (startName, "r");
    if (infile == NULL)
    {
        return ERROR;
    }

    if (fread (&fileHdr, sizeof(fileHdr), 1, infile) != 1)
    {
        fclose (infile);
        return ERROR;
    }

    if (fileHdr.dayIndex[bkntimenow.tm_mday].recordsInDay == 0)
    {
        // no entry for this day 
        fclose (infile);
        return ERROR;
    }

    // goto the beginning of the archive recs for this day
    if (fseek (infile,
               sizeof(fileHdr) +
               DBFILES_RECORD_SIZE*(fileHdr.dayIndex[bkntimenow.tm_mday].startPos + 2),
               SEEK_SET)
            == -1)
    {
        fclose (infile);
        return ERROR;
    }

    for (j = 0; j < fileHdr.dayIndex[bkntimenow.tm_mday].recordsInDay - 2; j ++)
    {
        // read the archive record 
        if (fread (&arcRecord, sizeof(arcRecord), 1, infile) != 1)
        {
            fclose (infile);
            return ERROR;
        }

        if ((USHORT)arcRecord.outsideTemp == 0x8000 ||
            (USHORT)arcRecord.outsideTemp == 0x7FFF ||
            (USHORT)arcRecord.lowOutsideTemp == 0x8000 ||
            (USHORT)arcRecord.lowOutsideTemp == 0x7FFF ||
            (USHORT)arcRecord.hiOutsideTemp == 0x8000 ||
            (USHORT)arcRecord.hiOutsideTemp == 0x7FFF)
        {
            // if the temp is bad, toss the entire record
            continue;
        }
        if (arcRecord.outsideHum > 1000 || arcRecord.outsideHum < 0)
        {
            // if the humidity is bad, toss the entire record
            continue;
        }
        if (arcRecord.windSpeed > 2000 || arcRecord.windSpeed < 0 ||
            arcRecord.hiWindSpeed > 2000 || arcRecord.hiWindSpeed < 0)
        {
            // if the windSpeed is bad, toss the entire record
            continue;
        }

        numrecs ++;

        updateSensorsWithArchive (work, 
                                  STF_DAY, 
                                  &arcRecord, 
                                  bkntimenow.tm_year + 1900,
                                  bkntimenow.tm_mon + 1,
                                  bkntimenow.tm_mday,
                                  FALSE);
    }

    fclose (infile);
    //  were there any recs for this day?
    if (numrecs == 0)
    {
        return ERROR;
    }
    else
    {
        return OK;
    }
}

// compute HILOW values for the last 7 days
static int computeDataWeek (WVIEWD_WORK *work)
{
    int                 j, currday, numrecs = 0;
    FILE                *infile;
    time_t              timenow = time (NULL);
    struct tm           bkntimenow;
    char                startName[256];
    SENSOR_STORE        *store = &work->sensors;
    HeaderBlock         fileHdr;
    DailySummaryRecord  sumRecord;
    ArchiveRecord       arcRecord;

    // do this so we pick up the proper hour/day when mins < archiveInterval
    timenow -= (work->archiveInterval * 60);
    timenow -= (WV_SECONDS_IN_DAY * 7);

    // initialize the store
    sensorClearSet (store->sensor[STF_WEEK]);
    windAverageReset (&store->wind[STF_WEEK]);

    for (currday = 0; currday < 7; currday ++)
    {
        localtime_r (&timenow, &bkntimenow);
    
        sprintf (startName, "%s/%4.4d-%2.2d.wlk", 
                 work->archivePath, 
                 bkntimenow.tm_year + 1900, 
                 bkntimenow.tm_mon + 1);
        
        infile = fopen (startName, "r");
        if (infile == NULL)
        {
            timenow += WV_SECONDS_IN_DAY;
            continue;
        }
    
        if (fread (&fileHdr, sizeof(fileHdr), 1, infile) != 1)
        {
            fclose (infile);
            timenow += WV_SECONDS_IN_DAY;
            continue;
        }
    
        if (fileHdr.dayIndex[bkntimenow.tm_mday].recordsInDay == 0)
        {
            // no entry for this day 
            fclose (infile);
            timenow += WV_SECONDS_IN_DAY;
            continue;
        }

        // goto the beginning of the archive recs for this day
        if (fseek (infile,
                   sizeof(fileHdr) +
                   DBFILES_RECORD_SIZE*(fileHdr.dayIndex[bkntimenow.tm_mday].startPos + 2),
                   SEEK_SET)
                == -1)
        {
            fclose (infile);
            timenow += WV_SECONDS_IN_DAY;
            continue;
        }
    
        for (j = 0; j < fileHdr.dayIndex[bkntimenow.tm_mday].recordsInDay - 2; j ++)
        {
            // read the archive record 
            if (fread (&arcRecord, sizeof(arcRecord), 1, infile) != 1)
            {
                fclose (infile);
                return ERROR;
            }
    
            if ((USHORT)arcRecord.outsideTemp == 0x8000 ||
                (USHORT)arcRecord.outsideTemp == 0x7FFF ||
                (USHORT)arcRecord.lowOutsideTemp == 0x8000 ||
                (USHORT)arcRecord.lowOutsideTemp == 0x7FFF ||
                (USHORT)arcRecord.hiOutsideTemp == 0x8000 ||
                (USHORT)arcRecord.hiOutsideTemp == 0x7FFF)
            {
                // if the temp is bad, toss the entire record
                continue;
            }
            if (arcRecord.outsideHum > 1000 || arcRecord.outsideHum < 0)
            {
                // if the humidity is bad, toss the entire record
                continue;
            }
            if (arcRecord.windSpeed > 2000 || arcRecord.windSpeed < 0 ||
                arcRecord.hiWindSpeed > 2000 || arcRecord.hiWindSpeed < 0)
            {
                // if the windSpeed is bad, toss the entire record
                continue;
            }
    
            numrecs ++;
    
            updateSensorsWithArchive (work, 
                                      STF_WEEK, 
                                      &arcRecord, 
                                      bkntimenow.tm_year + 1900,
                                      bkntimenow.tm_mon + 1,
                                      bkntimenow.tm_mday,
                                      FALSE);
        }
        
        fclose (infile);
        timenow += WV_SECONDS_IN_DAY;
    }

    // were there any recs for this week?
    if (numrecs == 0)
    {
        return ERROR;
    }
    else
    {
        return OK;
    }
}

// compute HILOW values for the given month
// Returns: number of good records processed
static int computeDataForMonth
(
    WVIEWD_WORK         *work,
    int                 month,
    int                 year,
    SENSOR_TIMEFRAMES   frame,
    int                 yearRainFlag
)
{
    int                 i, j, numrecs = 0;
    FILE                *infile;
    char                startName[256];
    HeaderBlock         fileHdr;
    DailySummaryRecord  sumRecord;
    ArchiveRecord       arcRecord;
    float               temp, dewpoint;

    sprintf (startName, "%s/%4.4d-%2.2d.wlk", 
             work->archivePath, 
             year, 
             month);
    
    infile = fopen (startName, "r");
    if (infile == NULL)
    {
        return ERROR;
    }

    if (fread (&fileHdr, sizeof(fileHdr), 1, infile) != 1)
    {
        fclose (infile);
        return ERROR;
    }

    // loop through the month one day at a time
    for (i = 1; i <= 31; i ++)
    {
        if (fileHdr.dayIndex[i].recordsInDay == 0)
        {
            // no entry for this day 
            continue;
        }
    
        // goto the beginning of the archive recs for this day
        if (fseek (infile,
                   sizeof(fileHdr) +
                   DBFILES_RECORD_SIZE*(fileHdr.dayIndex[i].startPos + 2),
                   SEEK_SET)
                == -1)
        {
            fclose (infile);
            return ERROR;
        }
    
        for (j = 0; j < fileHdr.dayIndex[i].recordsInDay - 2; j ++)
        {
            // read the archive record 
            if (fread (&arcRecord, sizeof(arcRecord), 1, infile) != 1)
            {
                fclose (infile);
                return ERROR;
            }
    
            if ((USHORT)arcRecord.outsideTemp == 0x8000 ||
                (USHORT)arcRecord.outsideTemp == 0x7FFF ||
                (USHORT)arcRecord.lowOutsideTemp == 0x8000 ||
                (USHORT)arcRecord.lowOutsideTemp == 0x7FFF ||
                (USHORT)arcRecord.hiOutsideTemp == 0x8000 ||
                (USHORT)arcRecord.hiOutsideTemp == 0x7FFF)
            {
                // if the temp is bad, toss the entire record
                continue;
            }
            if (arcRecord.outsideHum > 1000 || arcRecord.outsideHum < 0)
            {
                // if the humidity is bad, toss the entire record
                continue;
            }
            if (arcRecord.windSpeed > 2000 || arcRecord.windSpeed < 0 ||
                arcRecord.hiWindSpeed > 2000 || arcRecord.hiWindSpeed < 0)
            {
                // if the windSpeed is bad, toss the entire record
                continue;
            }
    
            numrecs ++;

            // if yearRainFlag set, only update the yearly rain data
            updateSensorsWithArchive (work, 
                                      frame, 
                                      &arcRecord, 
                                      year,
                                      month,
                                      i,
                                      yearRainFlag);
        }
    }

    fclose (infile);
    return numrecs;
}

// compute HILOW values for the current month:
// Returns: 0 if OK or ERROR if an error occurs
static int computeDataMonth (WVIEWD_WORK *work)
{
    SENSOR_STORE        *store = &work->sensors;
    int                 retVal;
    time_t              timenow = time (NULL);
    struct tm           bkntimenow;

    // do this so we pick up the proper hour/day when mins < archiveInterval
    timenow -= (work->archiveInterval * 60);
    localtime_r (&timenow, &bkntimenow);
    
    // initialize the store
    sensorClearSet (store->sensor[STF_MONTH]);
    windAverageReset (&store->wind[STF_MONTH]);

    // process the month
    retVal = computeDataForMonth (work, 
                                  bkntimenow.tm_mon + 1, 
                                  bkntimenow.tm_year + 1900,
                                  STF_MONTH,
                                  FALSE);
    if (retVal == ERROR || retVal == 0)
    {
        return ERROR;
    }

    return OK;
}

// compute HILOW values for the current year
static int computeDataYear (WVIEWD_WORK *work)
{
    SENSOR_STORE        *store = &work->sensors;
    int                 i, retVal, numrecs = 0;
    int                 nowmonth, rainmonth, nowyear, rainyear;
    time_t              timenow = time (NULL);
    struct tm           bkntimenow;

    // do this so we pick up the proper hour/day when mins < archiveInterval
    timenow -= (work->archiveInterval * 60);
    
    localtime_r (&timenow, &bkntimenow);
    nowmonth = bkntimenow.tm_mon + 1;
    nowyear = bkntimenow.tm_year + 1900;
    
    // initialize the store
    sensorClearSet (store->sensor[STF_YEAR]);
    windAverageReset (&store->wind[STF_YEAR]);

    // loop through each month this year (so far)
    for (i = 1; i <= nowmonth; i ++)
    {
        // process the month
        retVal = computeDataForMonth (work, i, nowyear, STF_YEAR, FALSE);
        if (retVal == ERROR || retVal == 0)
        {
            // just continue here
            continue;
        }

        numrecs += retVal;
    }
    
    // were there any recs for this year?
    if (numrecs == 0)
    {
        return ERROR;
    }

    // redo the Year rain total to account for rain season start
    sensorInit (&work->sensors.sensor[STF_YEAR][SENSOR_RAIN]);
    sensorInit (&work->sensors.sensor[STF_YEAR][SENSOR_RAINRATE]);
    sensorInit (&work->sensors.sensor[STF_YEAR][SENSOR_ET]);
    nowmonth = bkntimenow.tm_mon + 1;
    rainmonth = work->stationRainSeasonStart;
    rainyear = nowyear;
    if (rainmonth > nowmonth)
    {
        // we need to go back a year...
        rainyear --;
    }

    // now loop till we get to this month
    while ((rainyear < nowyear) || 
           ((rainmonth <= nowmonth) && (rainyear == nowyear)))
    {
        // process the month for yearly rain
        computeDataForMonth (work, rainmonth, rainyear, STF_YEAR, TRUE);

        if (++rainmonth > 12)
        {
            rainmonth = 1;
            rainyear ++;
        }
    }

    return OK;
}

static void addRecordHour (WVIEWD_WORK *work, int clearFlag)
{
    if (clearFlag)
    {
        // initialize the store
        sensorClearSet (work->sensors.sensor[STF_HOUR]);
        windAverageReset (&work->sensors.wind[STF_HOUR]);
    }

    sensorPropogateSample (work->sensors.sensor[STF_HOUR], 
                           work->sensors.sensor[STF_INTERVAL]);
    windAverageAddValue (&work->sensors.wind[STF_HOUR], 
                         (int)windAverageCompute(&work->sensors.wind[STF_INTERVAL])/22.5);
    return;
}

static void addRecordDay (WVIEWD_WORK *work, int clearFlag)
{
    if (clearFlag)
    {
        // initialize the store
        sensorClearSet (work->sensors.sensor[STF_DAY]);
        windAverageReset (&work->sensors.wind[STF_DAY]);
    }

    sensorPropogateSample (work->sensors.sensor[STF_DAY],
                           work->sensors.sensor[STF_INTERVAL]);
    windAverageAddValue (&work->sensors.wind[STF_DAY], 
                         (int)windAverageCompute(&work->sensors.wind[STF_INTERVAL])/22.5);
    return;
}

static void addRecordWeek (WVIEWD_WORK *work)
{
    sensorPropogateSample (work->sensors.sensor[STF_WEEK],
                           work->sensors.sensor[STF_INTERVAL]);
    windAverageAddValue (&work->sensors.wind[STF_WEEK], 
                         (int)windAverageCompute(&work->sensors.wind[STF_INTERVAL])/22.5);
    return;
}

static void addRecordMonth (WVIEWD_WORK *work, int clearFlag)
{
    if (clearFlag)
    {
        // initialize the store
        sensorClearSet (work->sensors.sensor[STF_MONTH]);
        windAverageReset (&work->sensors.wind[STF_MONTH]);
    }

    sensorPropogateSample (work->sensors.sensor[STF_MONTH],
                           work->sensors.sensor[STF_INTERVAL]);
    windAverageAddValue (&work->sensors.wind[STF_MONTH], 
                         (int)windAverageCompute(&work->sensors.wind[STF_INTERVAL])/22.5);
    return;
}

static void addRecordYear (WVIEWD_WORK *work, int clearFlag)
{
    WV_SENSOR       rainSensor, rainrateSensor, etSensor;

    if (clearFlag)
    {
        // initialize the store
        rainSensor = work->sensors.sensor[STF_YEAR][SENSOR_RAIN];
        rainrateSensor = work->sensors.sensor[STF_YEAR][SENSOR_RAINRATE];
        etSensor = work->sensors.sensor[STF_YEAR][SENSOR_ET];
        sensorClearSet (work->sensors.sensor[STF_YEAR]);
        work->sensors.sensor[STF_YEAR][SENSOR_RAIN] = rainSensor;
        work->sensors.sensor[STF_YEAR][SENSOR_RAINRATE] = rainrateSensor;
        work->sensors.sensor[STF_YEAR][SENSOR_ET] = etSensor;
        windAverageReset (&work->sensors.wind[STF_YEAR]);
    }

    sensorPropogateSample (work->sensors.sensor[STF_YEAR],
                           work->sensors.sensor[STF_INTERVAL]);
    windAverageAddValue (&work->sensors.wind[STF_YEAR], 
                         (int)windAverageCompute(&work->sensors.wind[STF_INTERVAL])/22.5);
    return;
}

static void resetRainRecordYear (WVIEWD_WORK *work)
{
    // initialize the store
    sensorInit (&work->sensors.sensor[STF_YEAR][SENSOR_RAIN]);
    sensorInit (&work->sensors.sensor[STF_YEAR][SENSOR_RAINRATE]);
    sensorInit (&work->sensors.sensor[STF_YEAR][SENSOR_ET]);
    sensorAddSample (&work->sensors.sensor[STF_YEAR][SENSOR_RAIN],
                     &work->sensors.sensor[STF_INTERVAL][SENSOR_RAIN]);
    sensorAddSample (&work->sensors.sensor[STF_YEAR][SENSOR_RAINRATE],
                     &work->sensors.sensor[STF_INTERVAL][SENSOR_RAINRATE]);
    sensorAddSample (&work->sensors.sensor[STF_YEAR][SENSOR_ET],
                     &work->sensors.sensor[STF_INTERVAL][SENSOR_ET]);
    return;
}

static void intervalHousekeepingInit (WVIEWD_WORK *work)
{
    float       windAvg, tempAvg;

    // Add the interval average values    
    windAvg = sensorGetAvg (&work->sensors.sensor[STF_HOUR][SENSOR_WSPEED]);
    tempAvg = sensorGetAvg (&work->sensors.sensor[STF_HOUR][SENSOR_OUTTEMP]);
    work->loopPkt.intervalAvgWCHILL = wvutilsCalculateWindChill(tempAvg, windAvg);
    work->loopPkt.intervalAvgWSPEED = (short)windAvg;


    return;
}

static void intervalHousekeeping (WVIEWD_WORK *work)
{
    float       windAvg, tempAvg;

    // do any processing here which needs to be done every archive interval

    // Add the interval average values    
    windAvg = sensorGetAvg (&work->sensors.sensor[STF_INTERVAL][SENSOR_WSPEED]);
    tempAvg = sensorGetAvg (&work->sensors.sensor[STF_INTERVAL][SENSOR_OUTTEMP]);
    work->loopPkt.intervalAvgWCHILL = wvutilsCalculateWindChill(tempAvg, windAvg);
    work->loopPkt.intervalAvgWSPEED = (short)windAvg;


    return;
}

////////////////////////////////////////////////////////////////////////////
/////////////////////////////////   A P I  /////////////////////////////////
////////////////////////////////////////////////////////////////////////////

// store a new data sample for all sensors to the pending archive interval
int computedDataStoreSample (WVIEWD_WORK *work)
{
    WV_SENSOR       sample[SENSOR_MAX];
    int             tempInt;
    float           tempfloat;

    sensorClearSet (sample);

    // convert from LOOP_PKT to a sensor sample
    sensorUpdate (&sample[SENSOR_INTEMP], work->loopPkt.inTemp, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_OUTTEMP], work->loopPkt.outTemp, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_INHUMID], (float)work->loopPkt.inHumidity, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_OUTHUMID], (float)work->loopPkt.outHumidity, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_BP], work->loopPkt.barometer, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_WSPEED], (float)work->loopPkt.windSpeed, work->cdataInterval/1000);
    sensorUpdateWhen (&sample[SENSOR_WGUST], 
                      (float)work->loopPkt.windGust, 
                      work->cdataInterval/1000, 
                      (float)work->loopPkt.windGustDir);
    sensorUpdate (&sample[SENSOR_DEWPOINT], work->loopPkt.dewpoint, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_RAIN], work->loopPkt.sampleRain, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_RAINRATE], work->loopPkt.rainRate, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_WCHILL], work->loopPkt.windchill, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_HINDEX], work->loopPkt.heatindex, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_ET], work->loopPkt.sampleET, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_UV], (float)work->loopPkt.UV/10, work->cdataInterval/1000);
    sensorUpdate (&sample[SENSOR_SOLRAD], (float)work->loopPkt.radiation, work->cdataInterval/1000);

    // store it
    sensorPropogateSample (work->sensors.sensor[STF_INTERVAL], sample);

    // store the wind direction
    tempfloat = (float)work->loopPkt.windDir + 11.24;
    tempfloat /= 22.5;
    tempInt = (int)tempfloat;
    tempInt %= 16;
    windAverageAddValue (&work->sensors.wind[STF_INTERVAL], tempInt);

    return OK;
}

// check interval HILOWs against the station-generated archive record
// (the VP is a goofy animal in terms of protocol)
int computedDataCheckHiLows (WVIEWD_WORK *work, ARCHIVE_RECORD *newRecord)
{
    WV_SENSOR       *sample = work->sensors.sensor[STF_INTERVAL];
    float           chill, dew, heat;

    if (sensorGetHigh(&sample[SENSOR_OUTTEMP]) < ((float)newRecord->highOutTemp/10))
    {
        // update the high value only
        sensorUpdateHighValue (&sample[SENSOR_OUTTEMP], (float)newRecord->highOutTemp/10);
    }

    if (sensorGetLow(&sample[SENSOR_OUTTEMP]) > ((float)newRecord->lowOutTemp/10))
    {
        // update the low value only
        sensorUpdateLowValue (&sample[SENSOR_OUTTEMP], (float)newRecord->lowOutTemp/10);
    }

    if ((newRecord->highWindDir < 16) && 
        (sensorGetHigh(&sample[SENSOR_WGUST]) < (float)newRecord->highWindSpeed))
    {
        // update the high and when high values only
        sensorUpdateWhenHighValue (&sample[SENSOR_WGUST], 
                                   (float)newRecord->highWindSpeed,
                                   (float)newRecord->highWindDir * 22.5);
    }

    if (sensorGetHigh(&sample[SENSOR_RAINRATE]) < ((float)newRecord->highRainRate/100))
    {
        // update the high value only
        sensorUpdateHighValue (&sample[SENSOR_RAINRATE], (float)newRecord->highRainRate/100);
    }

    if ((newRecord->rain == 0) || 
        (sensorGetCumulative(&sample[SENSOR_RAIN]) < ((float)newRecord->rain/100)))
    {
        // update the cumulative value only
        sensorUpdateCumulative (&sample[SENSOR_RAIN], (float)newRecord->rain/100);
    }

    if ((newRecord->ET == 0) || 
        (sensorGetCumulative(&sample[SENSOR_ET]) < ((float)newRecord->ET/1000)))
    {
        // update the cumulative value only
        sensorUpdateCumulative (&sample[SENSOR_ET], (float)newRecord->ET/1000);
    }

    return OK;
}

ARCHIVE_RECORD *computedDataGenerateArchive (WVIEWD_WORK *work)
{
    WV_SENSOR       *sample = work->sensors.sensor[STF_INTERVAL];
    time_t          nowtime = time (NULL);
    struct tm       tmtime;
    int             tempInt;
    float           tempfloat, tempfloat1;

    // have we received any LOOP updates this interval?
    if (sensorGetSamples(&sample[SENSOR_OUTTEMP]) == 0)
    {
        // we have not - squawk about it and exit
        radMsgLog (PRI_MEDIUM, "computedDataGenerateArchive: no samples for this interval!");
        return NULL;
    }

    localtime_r (&nowtime, &tmtime);

    // 0xFF is the "no data" indicator for archive records
    memset (&ArcRecStore, 0xFF, sizeof(ArcRecStore));

    // build this puppy from our sensor store
    ArcRecStore.date          = INSERT_PACKED_DATE(tmtime.tm_year+1900,tmtime.tm_mon+1,tmtime.tm_mday);
    ArcRecStore.time          = (tmtime.tm_hour * 100) + tmtime.tm_min;

    // must store working time here too
    work->archiveDate         = ArcRecStore.date;
    work->archiveTime         = ArcRecStore.time;

    tempfloat = sensorGetAvg (&sample[SENSOR_OUTTEMP]) * 10;
    if (tempfloat >= 0)
        tempfloat += 0.5;
    else
        tempfloat -= 0.5;
    ArcRecStore.outTemp       = (short)tempfloat;

    tempfloat = sensorGetHigh (&sample[SENSOR_OUTTEMP]) * 10;
    if (tempfloat >= 0)
        tempfloat += 0.5;
    else
        tempfloat -= 0.5;
    ArcRecStore.highOutTemp   = (short)tempfloat;

    tempfloat = sensorGetLow (&sample[SENSOR_OUTTEMP]) * 10;
    if (tempfloat >= 0)
        tempfloat += 0.5;
    else
        tempfloat -= 0.5;
    ArcRecStore.lowOutTemp    = (short)tempfloat;

    tempfloat = sensorGetCumulative (&sample[SENSOR_RAIN]) * 100;
    tempfloat += 0.5;
    ArcRecStore.rain          = (USHORT)floorf(tempfloat);

    tempfloat = sensorGetHigh (&sample[SENSOR_RAINRATE]) * 100;
    tempfloat += 0.5;
    ArcRecStore.highRainRate  = (USHORT)floorf(tempfloat);

    tempfloat = sensorGetAvg (&sample[SENSOR_BP]) * 1000;
    tempfloat += 0.5;
    ArcRecStore.barometer     = (USHORT)tempfloat;
    ArcRecStore.radiation     = (USHORT)sensorGetAvg (&sample[SENSOR_SOLRAD]);
    tempfloat = sensorGetAvg (&sample[SENSOR_INTEMP]) * 10;
    tempfloat += 0.5;
    ArcRecStore.inTemp        = (USHORT)tempfloat;
    ArcRecStore.inHumidity    = (UCHAR)sensorGetAvg (&sample[SENSOR_INHUMID]);
    ArcRecStore.outHumidity   = (UCHAR)sensorGetAvg (&sample[SENSOR_OUTHUMID]);
    ArcRecStore.avgWindSpeed  = (UCHAR)sensorGetAvg (&sample[SENSOR_WSPEED]);
    ArcRecStore.highWindSpeed = (UCHAR)sensorGetHigh (&sample[SENSOR_WGUST]);
    tempfloat = sensorGetAvg (&sample[SENSOR_UV]) * 10;
    tempfloat += 0.5;
    ArcRecStore.UV            = (UCHAR)tempfloat;

    tempfloat = sensorGetCumulative (&sample[SENSOR_ET]) * 1000;
    ArcRecStore.ET            = (UCHAR)floorf(tempfloat);

    tempfloat = (float)windAverageCompute(&work->sensors.wind[STF_INTERVAL]) + 11.24;
    tempfloat /= 22.5;
    tempInt = (int)tempfloat;
    tempInt %= 16;
    ArcRecStore.prevWindDir   = (UCHAR)tempInt;

    tempfloat = sensorGetWhenHigh(&sample[SENSOR_WGUST]) + 11.24;
    tempfloat /= 22.5;
    tempInt = (int)tempfloat;
    tempInt %= 16;
    ArcRecStore.highWindDir   = (UCHAR)tempInt;

    return &ArcRecStore;
}

void computedDataClearInterval (WVIEWD_WORK *work, float rainCarry, float etCarry)
{
    // clear our interval store to start the next cycle
    sensorClearSet (work->sensors.sensor[STF_INTERVAL]);

    // add back the accumulator trace amounts
    sensorUpdateCumulative (&work->sensors.sensor[STF_INTERVAL][SENSOR_RAIN], 
                            rainCarry);
    sensorUpdateCumulative (&work->sensors.sensor[STF_INTERVAL][SENSOR_ET], 
                            etCarry);
    
    windAverageReset (&work->sensors.wind[STF_INTERVAL]);
    return;
}


// initialize the computed values from the archive records
int computedDataInit (WVIEWD_WORK *work)
{
    time_t          nowtime = time (NULL);
    struct tm       bknnowtime;

    radMsgLog (PRI_STATUS, "initializing computed data values...");

    memset (&cdWork, 0, sizeof (cdWork));

    // update ALL data types to get started (ignore failures)
    computeDataYear (work);
    computeDataMonth (work);
    computeDataWeek (work);
    computeDataDay (work);
    computeDataHour (work);
    computeDataChanges (work);

    if (sensorImportSavedData (work->sensors.sensor, work->archivePath) == ERROR)
    {
        sensorDeleteSavedData (work->archivePath);
    }

    sensorClearSet (work->sensors.sensor[STF_INTERVAL]);

    // initialize per-interval housekeeping
    intervalHousekeepingInit (work);

    // save the current time so we will know when things need to be updated
    nowtime -= (work->archiveInterval * 60);
    localtime_r (&nowtime, &bknnowtime);
    cdWork.currentHour      = bknnowtime.tm_hour;
    cdWork.currentDay       = bknnowtime.tm_mday;
    cdWork.currentMonth     = bknnowtime.tm_mon;
    cdWork.currentYear      = bknnowtime.tm_year;

    return OK;
}

void computedDataExit (WVIEWD_WORK *work)
{
    // save the sensor highs/lows
    sensorExportSavedData (work->sensors.sensor, work->archivePath);

    return;
}

// update the computed values based on a new archive record
int computedDataUpdate (WVIEWD_WORK *work, ARCHIVE_RECORD *newRecord)
{
    time_t          timenow = time (NULL);
    struct tm       bkntimenow;

    // do some per-interval housekeeping
    intervalHousekeeping (work);

    // subtract an archive interval because the period 0 - (archiveInterval - 1)
    // minutes of each hour is still really the previous hour's data
    timenow -= (work->archiveInterval * 60);
    localtime_r (&timenow, &bkntimenow);

    // always update the "change" values
    computeDataChanges (work);

    // normalize the accumulator values to the archive record - 
    // we'll add back in the trace amounts in the "clear" call
    if (newRecord != NULL)
    {
        sensorUpdateCumulative (&work->sensors.sensor[STF_INTERVAL][SENSOR_RAIN], 
                                (float)(newRecord->rain & 0xFFF)/100);
        sensorUpdateCumulative (&work->sensors.sensor[STF_INTERVAL][SENSOR_ET], 
                                (float)newRecord->ET/1000);
    }

    // start at the lowest timeframe, that way we can bail out early 

    // has the hour changed?
    if (bkntimenow.tm_hour != cdWork.currentHour)
    {
        // save the new hour
        cdWork.currentHour = bkntimenow.tm_hour;

        // update the new hour
        addRecordHour (work, TRUE);

        // has the day changed?
        if (bkntimenow.tm_mday != cdWork.currentDay)
        {
            // save the new day
            cdWork.currentDay = bkntimenow.tm_mday;

            // update the last week
            computeDataWeek (work);

            // update the last day
            addRecordDay (work, TRUE);

            // has the month changed?
            if (bkntimenow.tm_mon != cdWork.currentMonth)
            {
                // save the new month
                cdWork.currentMonth = bkntimenow.tm_mon;

                // update the month
                addRecordMonth (work, TRUE);

                // has the year changed?
                if (bkntimenow.tm_year != cdWork.currentYear)
                {
                    // save the new year
                    cdWork.currentYear = bkntimenow.tm_year;

                    // update the year
                    addRecordYear (work, TRUE);
                }
                else
                {
                    // just add record to year
                    addRecordYear (work, FALSE);
                }

                // has the rain season rolled over?
                if ((cdWork.currentMonth+1) == work->stationRainSeasonStart)
                {
                    // start the new rain season year!
                    resetRainRecordYear (work);
                }
            }
            else
            {
                // just add record to month and year
                addRecordMonth (work, FALSE);
                addRecordYear (work, FALSE);
            }
        }
        else
        {
            // just add record to day, week, month and year
            addRecordDay (work, FALSE);
            addRecordWeek (work);
            addRecordMonth (work, FALSE);
            addRecordYear (work, FALSE);
        }
    }
    else
    {
        // just add record to hour, day, week, month and year
        addRecordHour (work, FALSE);
        addRecordDay (work, FALSE);
        addRecordWeek (work);
        addRecordMonth (work, FALSE);
        addRecordYear (work, FALSE);
    }

    return OK;
}

