/*---------------------------------------------------------------------------
 
  FILENAME:
        daemon.c
 
  PURPOSE:
        Provide the wview daemon entry point.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        08/14/03        M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2004, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>

/*  ... Library include files
*/
#include <radsystem.h>
#include <radconffile.h>

/*  ... Local include files
*/
#include <daemon.h>
#include <station.h>
#include <computedData.h>
#include <stormRain.h>

/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
static WVIEWD_WORK      wviewdWork;

enum ConfigIds
{
    CFG_ID_STATION_INTERFACE    = 0,
    CFG_ID_STATION_DEV          = 1,
    CFG_ID_STATION_HOST         = 2,
    CFG_ID_STATION_PORT         = 3,
    CFG_ID_RAIN_SEASON_START    = 4,
    CFG_ID_RAIN_STORM_START     = 5,
    CFG_ID_RAIN_STORM_STOP      = 6,
    CFG_ID_ARCHIVE_PATH         = 7,
    CFG_ID_POLL_INTERVAL        = 8,
    CFG_ID_PUSH_INTERVAL        = 9,
    CFG_ID_VERBOSE_MODE         = 10
};

static char             *configIDs[] =
{
    "STATION_INTERFACE",
    "STATION_DEV",
    "STATION_HOST",
    "STATION_PORT",
    "STATION_RAIN_SEASON_START",
    "STATION_RAIN_STORM_TRIGGER_START",
    "STATION_RAIN_STORM_IDLE_STOP",
    "ARCHIVE_PATH",
    "POLL_INTERVAL",
    "PUSH_INTERVAL",
    "VERBOSE_MSGS"
};


/* ... methods
*/

static int daemonStationLoopComplete (void)
{
    float           tempf, sampleRain, sampleET;

    if (!wviewdWork.runningFlag)
    {
        return OK;
    }

    // store the results
    computedDataStoreSample (&wviewdWork);

    sampleRain = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_INTERVAL][SENSOR_RAIN]);
    sampleET = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_INTERVAL][SENSOR_ET]);

    // do some post-processing on the LOOP data
    tempf = stormRainGet();
    if (tempf > 0)
        tempf += sampleRain;
    wviewdWork.loopPkt.stormRain        = tempf;
    wviewdWork.loopPkt.stormStart       = stormRainGetStartTimeT();

    tempf = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_DAY][SENSOR_RAIN]);
    tempf += sampleRain;
    wviewdWork.loopPkt.dayRain          = tempf;

    tempf = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_MONTH][SENSOR_RAIN]);
    tempf += sampleRain;
    wviewdWork.loopPkt.monthRain        = tempf;

    tempf = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_YEAR][SENSOR_RAIN]);
    tempf += sampleRain;
    wviewdWork.loopPkt.yearRain         = tempf;

    tempf = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_DAY][SENSOR_ET]);
    tempf += sampleET;
    wviewdWork.loopPkt.dayET            = tempf;

    tempf = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_MONTH][SENSOR_ET]);
    tempf += sampleET;
    wviewdWork.loopPkt.monthET          = tempf;

    tempf = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_YEAR][SENSOR_ET]);
    tempf += sampleET;
    wviewdWork.loopPkt.yearET           = tempf;

    wviewdWork.loopPkt.yearRainMonth    = wviewdWork.stationRainSeasonStart;

    return OK;
}

static int daemonStationInitComplete (int eventData)
{
    time_t              ntime;
    struct tm           locTime;
    ArchiveRecord       newestRecord;

    if (eventData != 0)
    {
        // failed startup!
        radMsgLog (PRI_HIGH, "daemonStationInitComplete: station startup failed!");
        return ERROR;        
    }

    if (!wviewdWork.runningFlag)
    {
        wviewdWork.runningFlag = TRUE;

        // set the positional data
        if (stationGetPosition (&wviewdWork) == ERROR)
        {
            radMsgLog (PRI_HIGH, "daemonStationInitComplete: stationGetPosition failed!");
            return ERROR;
        }

        // !!! the order of these calls is very important !!!
        computedDataInit (&wviewdWork);

        stormRainInit (wviewdWork.archivePath, 
                       wviewdWork.stationRainStormTrigger,
                       wviewdWork.stationRainStormIdleHours);

        computedDataClearInterval (&wviewdWork, 0, 0);
        
        // we know it just finished an initial sensor readings, it is a
        // REQUIREMENT of the stationInit API...
        daemonStationLoopComplete ();

        // do an initial update to propogate the initial readings
        // (so we have some data to start with)
        computedDataUpdate (&wviewdWork, NULL);

        // start the timers...
        stationStartArchiveTimerUniform (&wviewdWork);
        stationStartCDataTimerUniform (&wviewdWork);
        radProcessTimerStart (wviewdWork.pushTimer, 30000);  // first run
        radProcessTimerStart (wviewdWork.syncTimer, 15000);  // first run

        radMsgLog (PRI_STATUS, "-- Station Init Complete --");

        // get the newest archive file record date/time
        ntime = time (NULL);
        localtime_r (&ntime, &locTime);
        dbfGetNewestArchiveTime (wviewdWork.archivePath,
                                 locTime.tm_mon + 1,
                                 locTime.tm_year + 1900,
                                 &wviewdWork.archiveDate,
                                 &wviewdWork.archiveTime,
                                 &newestRecord);

        radMsgLog (PRI_STATUS, "newest archive record: %4.4d-%2.2d-%2.2d %2.2d:%2.2d",
                   EXTRACT_PACKED_YEAR(wviewdWork.archiveDate),
                   EXTRACT_PACKED_MONTH(wviewdWork.archiveDate),
                   EXTRACT_PACKED_DAY(wviewdWork.archiveDate),
                   EXTRACT_PACKED_HOUR(wviewdWork.archiveTime),
                   EXTRACT_PACKED_MINUTE(wviewdWork.archiveTime));

        // finally, answer all the WVIEW_RQST_TYPE_ARCHIVE_PATH requestors 
        // so they can continue initialization - our data is ready
        stationProcessArchiveResponses (&wviewdWork);
    }

    return OK;
}

static void daemonStoreArchiveRecord (ARCHIVE_RECORD *newRecord)
{
    float           carryOverRain, carryOverET;

    if (newRecord == NULL)
    {
        radMsgLog (PRI_MEDIUM, "daemonStoreArchiveRecord: record is NULL!");
        return;
    }

    wvutilsLogEvent (PRI_STATUS, "storing record for %4.4d-%2.2d-%2.2d %2.2d:%2.2d",
                     EXTRACT_PACKED_YEAR(newRecord->date),
                     EXTRACT_PACKED_MONTH(newRecord->date),
                     EXTRACT_PACKED_DAY(newRecord->date),
                     EXTRACT_PACKED_HOUR(newRecord->time),
                     EXTRACT_PACKED_MINUTE(newRecord->time));

    if (dbfStoreArchiveRecord (wviewdWork.archivePath,
                               newRecord,
                               wviewdWork.archiveInterval)
        == ERROR)
    {
        radMsgLog (PRI_MEDIUM, "daemonStoreArchiveRecord: dbfStoreArchiveRecord failed!!!");
        return;
    }

    // if we are running normally (out of init), notify some folks
    if (wviewdWork.runningFlag)
    {
        // save trace accumulator amounts
        carryOverRain = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_INTERVAL][SENSOR_RAIN]);
        carryOverRain -= ((float)(newRecord->rain & 0xFFF)/100);
        carryOverET = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_INTERVAL][SENSOR_ET]);
        carryOverET -= ((float)newRecord->ET/1000);

        // compute HILOW data
        computedDataUpdate (&wviewdWork, newRecord);

        // clear for the next archive period (saving trace amounts)
        computedDataClearInterval (&wviewdWork, carryOverRain, carryOverET);

        // compute storm rain
        stormRainUpdate ((float)newRecord->highRainRate/100, 
                         (float)(newRecord->rain & 0xFFF)/100);

        // sync to sensors
        wviewdWork.loopPkt.stormRain  = stormRainGet();
        wviewdWork.loopPkt.stormStart = stormRainGetStartTimeT();
        wviewdWork.loopPkt.dayRain    = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_DAY][SENSOR_RAIN]);
        wviewdWork.loopPkt.monthRain  = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_MONTH][SENSOR_RAIN]);
        wviewdWork.loopPkt.yearRain   = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_YEAR][SENSOR_RAIN]);
        wviewdWork.loopPkt.dayET      = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_DAY][SENSOR_ET]);
        wviewdWork.loopPkt.monthET    = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_MONTH][SENSOR_ET]);
        wviewdWork.loopPkt.yearET     = sensorGetCumulative(&wviewdWork.sensors.sensor[STF_YEAR][SENSOR_ET]);

        // send archive notification
        stationSendArchiveNotifications (&wviewdWork);
    }

    return;
}

static void daemonArchiveIndication (ARCHIVE_RECORD *newRecord)
{
    if (newRecord != NULL)
    {
        // if we are running normally verify record against LOOP readings
        if (wviewdWork.runningFlag)
        {
            computedDataCheckHiLows (&wviewdWork, newRecord);
        }

        daemonStoreArchiveRecord (newRecord);
    }

    return;
}


/*  ... system initialization
*/
static int daemonSysInit (WVIEWD_WORK *work)
{
    char            temp[256];
    char            *installPath;
    struct stat     fileData;
    FILE            *pidfile;

    /*  ... exit if our config file is not there
    */
    sprintf (work->configFile, "%s/%s", WVIEW_CONFIG_DIR, WVIEW_CONFIG_FILENAME);
    if (stat (work->configFile, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_DAEMON, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "Cannot locate config file (did you run 'make install-env'?): %s - aborting!",
                   work->configFile);
        radMsgLogExit ();
        return -1;
    }

    /*  ... create our run directory if it is not there
    */
    sprintf (temp, "%s", WVIEW_RUN_DIR);
    if (stat (temp, &fileData) != 0)
    {
        if (mkdir (temp, 0755) != 0)
        {
            radMsgLogInit (PROC_NAME_DAEMON, TRUE, TRUE);
            radMsgLog (PRI_CATASTROPHIC, 
                       "Cannot create run directory: %s - aborting!",
                       temp);
            radMsgLogExit ();
            return -1;
        }
    }

    /*  ... create our device directory if it is not there
    */
    sprintf (temp, "%s/dev", WVIEW_RUN_DIR);
    if (stat (temp, &fileData) != 0)
    {
        if (mkdir (temp, 0755) != 0)
        {
            radMsgLogInit (PROC_NAME_DAEMON, TRUE, TRUE);
            radMsgLog (PRI_CATASTROPHIC, 
                       "Cannot create device directory: %s - aborting!",
                       temp);
            radMsgLogExit ();
            return -1;
        }
    }

    sprintf (work->pidFile, "%s/%s", WVIEW_RUN_DIR, WVD_LOCK_FILE_NAME);
    sprintf (work->fifoFile, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_DAEMON);

    sprintf (work->archivePath, "%s/%s", WVIEW_RUN_DIR, "archive");

    /*  ... check for our pid file, don't run if it is there
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        radMsgLogInit (PROC_NAME_DAEMON, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "lock file %s exists, older copy may be running - aborting!",
                   work->pidFile);
        radMsgLogExit ();
        return -1;
    }

    return 0;
}

/*  ... system exit
*/
static int daemonSysExit (WVIEWD_WORK *work)
{
    struct stat     fileData;

    /*  ... delete our pid file
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        unlink (work->pidFile);
    }

    return 0;
}


static void defaultSigHandler (int signum)
{
    int         retVal;

    switch (signum)
    {
        case SIGHUP:
            // user wants us to change the verbosity setting
            retVal = wvutilsToggleVerbosity ();
            radMsgLog (PRI_STATUS, "wviewd: SIGHUP - toggling log verbosity %s",
                       ((retVal == 0) ? "OFF" : "ON"));

            signal (signum, defaultSigHandler);
            return;

        case SIGPIPE:
            // we have a far end socket disconnection, we'll handle it in the
            // "read/write" code
            signal (signum, defaultSigHandler);
            break;

        case SIGBUS:
        case SIGFPE:
        case SIGSEGV:
        case SIGXFSZ:
        case SIGSYS:
            // unrecoverable signal - we must exit right now!
            radMsgLog (PRI_CATASTROPHIC, "wviewd: recv sig %d: shutting down!", signum);
            abort ();

        case SIGCHLD:
            wait (NULL);
            signal (signum, defaultSigHandler);
            return;

        default:
            // we can allow the process to exit normally...
            if (wviewdWork.exiting)
            {
                signal (signum, defaultSigHandler);
                return;
            }

            radMsgLog (PRI_HIGH, "wviewd: recv sig %d: exiting!", signum);

            wviewdWork.exiting = TRUE;
            radProcessSetExitFlag ();

            signal (signum, defaultSigHandler);
            break;
    }

    return;
}

static void msgHandler
(
    char        *srcQueueName,
    UINT        msgType,
    void        *msg,
    UINT        length,
    void        *userData
)
{
    stationProcessIPM (&wviewdWork, srcQueueName, msgType, msg);
    return;
}

static void evtHandler
(
    UINT        eventsRx,
    UINT        rxData,
    void        *userData
)
{
    STIM        stim;

    // bleed off our special events
    if (eventsRx & STATION_INIT_COMPLETE_EVENT)
    {
        // call the init complete handler
        if (daemonStationInitComplete((int)userData) == ERROR)
        {
            wviewdWork.exiting = TRUE;
            return;
        }

        eventsRx &= ~STATION_INIT_COMPLETE_EVENT;
    }

    if (eventsRx & STATION_LOOP_COMPLETE_EVENT)
    {
        // call the loop readings complete handler
        daemonStationLoopComplete ();

        eventsRx &= ~STATION_LOOP_COMPLETE_EVENT;
    }

    return;
}


static void archiveTimerHandler (void *parm)
{
    ARCHIVE_RECORD  *newRec;

    if (wviewdWork.stationGeneratesArchives)
    {
        // tell the station to generate an archive record
        // (he will indicate it back to us)
        stationGetArchive (&wviewdWork);
    }
    else
    {
        // generate it on our own
        newRec = computedDataGenerateArchive (&wviewdWork);
        daemonStoreArchiveRecord (newRec);
    }

    // restart the timer
    stationStartArchiveTimerUniform (&wviewdWork);
    return;
}

static void cdtimerHandler (void *parm)
{
    // tell the station to acquire data
    stationGetReadings (&wviewdWork);

    // restart the timer
    stationStartCDataTimerUniform (&wviewdWork);
    return;
}

static void pushTimerHandler (void *parm)
{
    // ... send to clients
    stationPushDataToClients (&wviewdWork);
    return;
}

static void syncTimerHandler (void *parm)
{
    // tell the station to synchronize the station time (if required)
    stationSyncTime (&wviewdWork);

    // restart the timer
    stationStartSyncTimerUniform (&wviewdWork);

    return;
}

static void ifTimerHandler (void *parm)
{
    // we just pass through the IF timer to the station-specific indication
    stationIFTimerExpiry (&wviewdWork);

    return;
}

static void stationDataCallback (int fd, void *userData)
{
    // we just indicate the IF data to the station-specific function
    stationDataIndicate (&wviewdWork);

    return;
}


/*  ... the main entry point for the daemon process
*/
int main (int argc, char *argv[])
{
    void            (*alarmHandler)(int);
    int             i;
    char            instance[32], value[32];
    CF_ID           configFileId;
    struct stat     fileStatus;
    FILE            *pidfile;

    /*  ... start with a clean slate
    */
    memset (&wviewdWork, 0, sizeof (wviewdWork));

    /*  ... initialize some system stuff first
    */
    if (daemonSysInit (&wviewdWork) == -1)
    {
        radMsgLogInit (PROC_NAME_DAEMON, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "system init failed!\n");
        radMsgLogExit ();
        exit (1);
    }


    /*  ... call the global radlib system init function
    */
    if (radSystemInit (WVIEW_SYSTEM_ID) == ERROR)
    {
        radMsgLogInit (PROC_NAME_DAEMON, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "radSystemInit failed!");
        radMsgLogExit ();
        exit (1);
    }


    /*  ... call the radlib process init function
    */
    if (radProcessInit (PROC_NAME_DAEMON,
                        wviewdWork.fifoFile,
                        PROC_NUM_TIMERS_DAEMON,
                        TRUE,                      // TRUE for daemon
                        msgHandler,
                        evtHandler,
                        NULL)
        == ERROR)
    {
        printf ("\nradProcessInit failed: %s\n\n", PROC_NAME_DAEMON);
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    wviewdWork.myPid = getpid ();
    pidfile = fopen (wviewdWork.pidFile, "w");
    if (pidfile == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "lock file create failed!\n");
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    fprintf (pidfile, "%d", getpid ());
    fclose (pidfile);


    alarmHandler = radProcessSignalGetHandler (SIGALRM);
    radProcessSignalCatchAll (defaultSigHandler);
    radProcessSignalCatch (SIGALRM, alarmHandler);


    // get the wview verbosity setting
    if (wvutilsSetVerbosity (WV_VERBOSE_WVIEWD) == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "wvutilsSetVerbosity failed!\n");
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    radMsgLog (PRI_STATUS, "%s starting ...", globalWviewVersionStr);

    /*  ... get our configuration values
    */
    if (stat (wviewdWork.configFile, &fileStatus) == -1)
    {
        radMsgLog (PRI_CATASTROPHIC, "config file %s missing - use wviewconfig to generate one...\n", 
                   wviewdWork.configFile);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    configFileId = radCfOpen (wviewdWork.configFile);
    if (configFileId == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfOpen failed!\n");
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    ///// STATION_INTERFACE PROCESSING BEGIN /////
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_STATION_INTERFACE],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_MEDIUM, 
                   "no station interface type given, defaulting to 'serial'...");
        strcpy (wviewdWork.stationInterface, "serial");
    }
    else
    {
        strcpy (wviewdWork.stationInterface, value);
    }

    // process the interface type
    if (!strcmp (wviewdWork.stationInterface, "serial"))
    {
        radMsgLog (PRI_MEDIUM, 
                   "station interface: serial ...");

        // we need a device name for serial IFs
        if (radCfGetFirstEntry (configFileId,
                                configIDs[CFG_ID_STATION_DEV],
                                instance,
                                value)
            == ERROR)
        {
            radMsgLog (PRI_CATASTROPHIC, 
                       "no serial device given, aborting...");
            radCfClose (configFileId);
            daemonSysExit (&wviewdWork);
            radProcessExit ();
            radSystemExit (WVIEW_SYSTEM_ID);
            exit (1);
        }
        strcpy (wviewdWork.stationDevice, value);
    }
    else if (!strcmp (wviewdWork.stationInterface, "ethernet"))
    {
        radMsgLog (PRI_MEDIUM, 
                   "station interface: ethernet ...");

        // we need host and port for ethernet
        if (radCfGetFirstEntry (configFileId,
                                configIDs[CFG_ID_STATION_HOST],
                                instance,
                                value)
            == ERROR)
        {
            radMsgLog (PRI_CATASTROPHIC, 
                       "no ethernet host given, aborting...");
            radCfClose (configFileId);
            daemonSysExit (&wviewdWork);
            radProcessExit ();
            radSystemExit (WVIEW_SYSTEM_ID);
            exit (1);
        }
        else
        {
            strcpy (wviewdWork.stationHost, value);
            
            if (radCfGetFirstEntry (configFileId,
                                    configIDs[CFG_ID_STATION_PORT],
                                    instance,
                                    value)
                == ERROR)
            {
                radMsgLog (PRI_CATASTROPHIC, 
                           "no ethernet port given, aborting...");
                radCfClose (configFileId);
                daemonSysExit (&wviewdWork);
                radProcessExit ();
                radSystemExit (WVIEW_SYSTEM_ID);
                exit (1);
            }
            else
            {
                wviewdWork.stationPort = atoi (value);
            }
        }
    }
    else
    {
        // invalid type specified - abort
        radMsgLog (PRI_CATASTROPHIC, 
                   "invalid STATION_INTERFACE %s given, aborting...",
                   wviewdWork.stationInterface);
        radCfClose (configFileId);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }   
    ///// STATION_INTERFACE PROCESSING END /////

    
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_RAIN_SEASON_START],
                            instance,
                            value)
            == ERROR)
    {
        radMsgLog (PRI_MEDIUM, "Rain Season Start Month not found - defaulting to 1 (JAN)...\n");
        wviewdWork.stationRainSeasonStart = 1;
    }
    else
    {
        wviewdWork.stationRainSeasonStart = atoi (value);
        if (wviewdWork.stationRainSeasonStart < 1 ||
            wviewdWork.stationRainSeasonStart > 12)
        {
            radMsgLog (PRI_MEDIUM, "Invalid Rain Season Start Month %d found - defaulting to 1 (JAN)...\n",
                       wviewdWork.stationRainSeasonStart);
            wviewdWork.stationRainSeasonStart = 1;
        }
        else
        {
            radMsgLog (PRI_STATUS, "Rain Season Start Month set to %d\n",
                       wviewdWork.stationRainSeasonStart);
        }
    }

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_RAIN_STORM_START],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_MEDIUM, 
                   "no rain storm start trigger given, defaulting to 0.05 in/hr...");
        wviewdWork.stationRainStormTrigger = 0.05;
    }
    else
    {
        wviewdWork.stationRainStormTrigger = (float)atof (value);
        radMsgLog (PRI_STATUS, "Rain Storm Start Trigger set to %5.2f in/hr\n",
                   wviewdWork.stationRainStormTrigger);
    }

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_RAIN_STORM_STOP],
                            instance,
                            value)
        == ERROR)
    {
        radMsgLog (PRI_MEDIUM, 
                   "no rain storm idle stop time given, defaulting to 12 hours...");
        wviewdWork.stationRainStormIdleHours = 12;
    }
    else
    {
        wviewdWork.stationRainStormIdleHours = atoi (value);
        radMsgLog (PRI_STATUS, "Rain Storm Stop Time set to %d hours\n",
                   wviewdWork.stationRainStormIdleHours);
    }

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_ARCHIVE_PATH],
                            instance,
                            value)
            == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfGetFirstEntry failed!\n");
        radCfClose (configFileId);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    strcpy (wviewdWork.archivePath, value);

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_POLL_INTERVAL],
                            instance,
                            value)
            == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfGetFirstEntry failed - did you run wviewconfig after upgrading?\n");
        radCfClose (configFileId);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    wviewdWork.cdataInterval = atoi (value);
    if (((wviewdWork.cdataInterval % 1000) != 0) ||
        ((wviewdWork.cdataInterval/1000) > 60) || 
        ((60 % (wviewdWork.cdataInterval/1000)) != 0))
    {
        radMsgLog (PRI_MEDIUM, 
                   "station polling interval %d found in wview.conf is invalid:",
                   wviewdWork.cdataInterval);
        radMsgLog (PRI_MEDIUM, 
                   "defaulting to 15 seconds (15000) ...");
        radMsgLog (PRI_MEDIUM, 
                   "Note: station polling interval must be less than 60 seconds");
        radMsgLog (PRI_MEDIUM, 
                   "      and an even divisor of 60 seconds (10000, 15000, 30000)");
        wviewdWork.cdataInterval = 15 * 1000;
    }
    else
    {
        radMsgLog (PRI_STATUS, "station polling interval set to %d seconds",
                   (wviewdWork.cdataInterval/1000));
    }
        

    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_PUSH_INTERVAL],
                            instance,
                            value)
            == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfGetFirstEntry failed!\n");
        radCfClose (configFileId);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    wviewdWork.pushInterval = atoi (value);

    radCfClose (configFileId);


    wviewdWork.archiveTimer = radTimerCreate (NULL, archiveTimerHandler, NULL);
    if (wviewdWork.archiveTimer == NULL)
    {
        radMsgLog (PRI_HIGH, "radTimerCreate failed");
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    wviewdWork.cdataTimer = radTimerCreate (NULL, cdtimerHandler, NULL);
    if (wviewdWork.cdataTimer == NULL)
    {
        radMsgLog (PRI_HIGH, "radTimerCreate failed");
        radTimerDelete (wviewdWork.archiveTimer);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    wviewdWork.pushTimer = radTimerCreate (NULL, pushTimerHandler, NULL);
    if (wviewdWork.pushTimer == NULL)
    {
        radMsgLog (PRI_HIGH, "radTimerCreate failed");
        radTimerDelete (wviewdWork.cdataTimer);
        radTimerDelete (wviewdWork.archiveTimer);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    wviewdWork.syncTimer = radTimerCreate (NULL, syncTimerHandler, NULL);
    if (wviewdWork.syncTimer == NULL)
    {
        radMsgLog (PRI_HIGH, "sync radTimerCreate failed");
        radTimerDelete (wviewdWork.cdataTimer);
        radTimerDelete (wviewdWork.pushTimer);
        radTimerDelete (wviewdWork.archiveTimer);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    wviewdWork.ifTimer = radTimerCreate (NULL, ifTimerHandler, NULL);
    if (wviewdWork.ifTimer == NULL)
    {
        radMsgLog (PRI_HIGH, "sync radTimerCreate failed");
        radTimerDelete (wviewdWork.syncTimer);
        radTimerDelete (wviewdWork.cdataTimer);
        radTimerDelete (wviewdWork.pushTimer);
        radTimerDelete (wviewdWork.archiveTimer);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    radProcessEventsAdd (STATION_INIT_COMPLETE_EVENT);
    radProcessEventsAdd (STATION_LOOP_COMPLETE_EVENT);

    //  register with the radlib message router
    if (radMsgRouterInit (WVIEW_RUN_DIR) == ERROR)
    {
        radMsgLog (PRI_HIGH, "radMsgRouterInit failed!");
        radTimerDelete (wviewdWork.ifTimer);
        radTimerDelete (wviewdWork.syncTimer);
        radTimerDelete (wviewdWork.cdataTimer);
        radTimerDelete (wviewdWork.pushTimer);
        radTimerDelete (wviewdWork.archiveTimer);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    // enable message reception from the radlib router for worker requests
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_REQUEST);


    // initialize the station abstraction
    radMsgLog (PRI_STATUS, "-- Station Init Start --");
    if (stationInit (&wviewdWork, daemonArchiveIndication) == ERROR)
    {
        radMsgLog (PRI_HIGH, "stationInit failed!");
        radMsgRouterExit ();
        radTimerDelete (wviewdWork.ifTimer);
        radTimerDelete (wviewdWork.syncTimer);
        radTimerDelete (wviewdWork.cdataTimer);
        radTimerDelete (wviewdWork.pushTimer);
        radTimerDelete (wviewdWork.archiveTimer);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    if (radProcessIORegisterDescriptor (wviewdWork.medium.fd,
                                        stationDataCallback,
                                        NULL)
            == ERROR)
    {
        radMsgLog (PRI_HIGH, "IORegDescriptor failed");
        radMsgRouterExit ();
        radTimerDelete (wviewdWork.ifTimer);
        radTimerDelete (wviewdWork.syncTimer);
        radTimerDelete (wviewdWork.cdataTimer);
        radTimerDelete (wviewdWork.pushTimer);
        radTimerDelete (wviewdWork.archiveTimer);
        stationExit (&wviewdWork);
        daemonSysExit (&wviewdWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }


    radMsgLog (PRI_STATUS, "running...");

    while (!wviewdWork.exiting)
    {
        // wait on timers, events, file descriptors, msgs
        if (radProcessWait (0) == ERROR)
        {
            wviewdWork.exiting = TRUE;
        }
    }


    radMsgLog (PRI_STATUS, "exiting normally...");

    computedDataExit (&wviewdWork);
    radMsgRouterExit ();
    radTimerDelete (wviewdWork.ifTimer);
    radTimerDelete (wviewdWork.syncTimer);
    radTimerDelete (wviewdWork.pushTimer);
    radTimerDelete (wviewdWork.cdataTimer);
    radTimerDelete (wviewdWork.archiveTimer);
    stationExit (&wviewdWork);
    daemonSysExit (&wviewdWork);

    radProcessExit ();
    radSystemExit (WVIEW_SYSTEM_ID);
    exit (0);
}
