/*---------------------------------------------------------------------------
 
  FILENAME:
        station.c
 
  PURPOSE:
        Provide the station abstraction utility.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        12/31/2005      M.S. Teel       0               Original
 
  NOTES:

  LICENSE:
        Copyright (c) 2005, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/

/*  ... Library include files
*/
#include <radconffile.h>

/*  ... Local include files
*/
#include <station.h>

/*  ... global memory declarations
*/

int stationSendArchiveNotifications (WVIEWD_WORK *work)
{
    WVIEW_MSG_ARCHIVE_NOTIFY    notify;
    float                       tempfloat;

    notify.archiveDate      = work->archiveDate;
    notify.archiveTime      = work->archiveTime;
    notify.intemp           = (int)floorf(work->loopPkt.inTemp * 10);
    notify.inhumidity       = work->loopPkt.inHumidity;
    notify.temp             = (int)floorf(work->loopPkt.outTemp * 10);
    notify.humidity         = work->loopPkt.outHumidity;
    notify.barom            = (int)floorf(work->loopPkt.barometer * 1000);
    notify.stationPressure  = (int)floorf(work->loopPkt.stationPressure * 1000);
    notify.winddir          = work->loopPkt.windDir;
    notify.wspeed           = work->loopPkt.windSpeed;
    notify.dewpoint         = (int)floorf(work->loopPkt.dewpoint * 10);
    notify.hiwspeed         = work->lastHighWindSpeed;
    notify.rxPercent        = work->loopPkt.rxCheckPercent;

    tempfloat = sensorGetCumulative (&work->sensors.sensor[STF_HOUR][SENSOR_RAIN]);
    tempfloat *= 100;
    tempfloat += 0.5;
    notify.rainHour    = (int)tempfloat;

    tempfloat = sensorGetCumulative (&work->sensors.sensor[STF_DAY][SENSOR_RAIN]);
    tempfloat *= 100;
    tempfloat += 0.5;
    notify.rainDay     = (int)tempfloat;

    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_ARCHIVE_NOTIFY, &notify, sizeof(notify))
        == ERROR)
    {
        // can't send!
        radMsgLog (PRI_HIGH, "radMsgRouterMessageSend failed: notify");
        return ERROR;
    }

    return OK;
}

int stationProcessArchiveResponses (WVIEWD_WORK *work)
{
    WVIEW_MSG_ARCHIVE_PATH  apath;

    if (!work->archiveRqstPending)
    {
        return OK;
    }

    strcpy (apath.archivePath, work->archivePath);
    apath.lastArcTime = time (NULL);
    apath.archiveInterval = work->archiveInterval;
    apath.latitude = work->latitude;
    apath.longitude = work->longitude;
    apath.elevation = work->elevation;

    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_ARCHIVE_PATH, &apath, sizeof (apath))
        == ERROR)
    {
        radMsgLog (PRI_HIGH, "radMsgRouterMessageSend failed Archive Path");
        return ERROR;
    }

    work->archiveRqstPending = FALSE;
    return OK;
}

int stationProcessIPM (WVIEWD_WORK *work, char *srcQueueName, int msgType, void *msg)
{
    WVIEW_MSG_REQUEST           *msgRqst;
    WVIEW_MSG_LOOP_DATA         loop;
    WVIEW_MSG_HILOW_DATA        hilow;
    int                         retVal, i;

    switch (msgType)
    {
        case WVIEW_MSG_TYPE_REQUEST:
            msgRqst = (WVIEW_MSG_REQUEST *)msg;
    
            switch (msgRqst->requestType)
            {
                case WVIEW_RQST_TYPE_ARCHIVE_PATH:
                    // flag that a request has been tendered
                    work->archiveRqstPending = TRUE;
        
                    // are we currently in a serial cycle?
                    if (!work->runningFlag)
                    {
                        // yes, just bail out for now
                        return OK;
                    }
                    else
                    {
                        // no, send it now
                        return (stationProcessArchiveResponses (work));
                    }
        
                case WVIEW_RQST_TYPE_LOOP_DATA:
                    memcpy (&loop.loopData, &work->loopPkt, sizeof (LOOP_PKT));

                    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_LOOP_DATA, 
                                                 &loop, 
                                                 sizeof(loop))
                        == ERROR)
                    {
                        radMsgLog (PRI_HIGH, "radMsgRouterMessageSend failed LOOP");
                        return ERROR;
                    }
        
                    return OK;
        
                case WVIEW_RQST_TYPE_HILOW_DATA:
                    memcpy (&hilow.hilowData, &work->sensors, sizeof (SENSOR_STORE));

                    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_HILOW_DATA, 
                                                 &hilow, 
                                                 sizeof(hilow))
                        == ERROR)
                    {
                        radMsgLog (PRI_HIGH, "radMsgRouterMessageSend failed HILOW");
                        return ERROR;
                    }
        
                    return OK;
        
                case WVIEW_RQST_TYPE_DST_ANNOUNCE:
                    radMsgLog (PRI_MEDIUM, "DST change announcement received...");

                    //  update the time zone info
                    tzset ();

                    // synchronize the VP console to the new time/GMT offset
                    return (stationSyncTime (work));
            }
            break;
                
        default:
            radMsgLog (PRI_MEDIUM, "unknown IPM msg type %d received...", msgType);
            break;
    }

    return OK;
}

int stationPushDataToClients (WVIEWD_WORK *work)
{
    WVIEW_MSG_LOOP_DATA loop;

    if (!work->runningFlag)
    {
        return OK;
    }

    memcpy (&loop.loopData, &work->loopPkt, sizeof (loop.loopData));

    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_LOOP_DATA_SVC, &loop, sizeof(loop))
        == ERROR)
    {
        radMsgLog (PRI_HIGH, "radMsgRouterMessageSend failed for loop transmit!");
        return ERROR;
    }

    radProcessTimerStart (work->pushTimer, work->pushInterval);
    return OK;
}

void stationStartArchiveTimerUniform (WVIEWD_WORK *work)
{
    time_t              ntime;
    struct tm           currTime;
    int                 intSECS;

    // get the current time
    ntime = time (NULL);
    localtime_r (&ntime, &currTime);

    // get the next archive interval delta in seconds
    intSECS  = currTime.tm_min/work->archiveInterval;
    intSECS *= work->archiveInterval;
    intSECS += work->archiveInterval;
    intSECS -= currTime.tm_min;                     // delta in minutes
    intSECS *= 60;

    // we try to land 4 seconds after top-of-minute
    intSECS += (4 - currTime.tm_sec);

    // now we have the delta in seconds
    radTimerStart (work->archiveTimer, intSECS*1000);
}

void stationStartCDataTimerUniform (WVIEWD_WORK *work)
{
    time_t              ntime;
    struct tm           locTime;
    int                 moduloVal, cdataintSECS;

    ntime = time (NULL);
    localtime_r (&ntime, &locTime);
    cdataintSECS = work->cdataInterval/1000;

    // try to hit the start of next cdataInterval
    moduloVal = locTime.tm_sec % cdataintSECS;
    moduloVal = cdataintSECS - moduloVal;

    radTimerStart (work->cdataTimer, moduloVal*1000);
}

void stationStartSyncTimerUniform (WVIEWD_WORK *work)
{
    time_t              ntime;
    struct tm           locTime;
    int                 tempVal;

    ntime = time (NULL);
    localtime_r (&ntime, &locTime);

    // try to hit 30 secs into the minute
    tempVal = 30 - locTime.tm_sec;

    radTimerStart (work->syncTimer, WVD_TIME_SYNC_INTERVAL + (tempVal * 1000));
}

int stationVerifyArchiveInterval (WVIEWD_WORK *work)
{
    time_t          ntime;
    struct tm       locTime;
    USHORT          adate, atime;
    ArchiveRecord   tempRec;
    
    // sanity check the archive interval against the most recent record
    ntime = time (NULL);
    localtime_r (&ntime, &locTime);
    memset (&tempRec, 0, sizeof(tempRec));
    if (dbfGetNewestArchiveTime (work->archivePath,
                                 locTime.tm_mon + 1,
                                 locTime.tm_year + 1900,
                                 &adate,
                                 &atime,
                                 &tempRec)
        == ERROR)
    {
        // there must be no archive records - just return OK
        return OK;
    }

    if (tempRec.archiveInterval != work->archiveInterval)
    {
        radMsgLog (PRI_HIGH, 
                   "verifyArchiveInterval: station value of %d does NOT match archive value of %d",
                   work->archiveInterval, tempRec.archiveInterval);
        return ERROR;
    }
    else
    {
        return OK;
    }
}

int stationGetConfigValueInt (WVIEWD_WORK *work, char *configName, int *store)
{
    char            value[32];
    CF_ID           configFileId;
    struct stat     fileStatus;

    *store = 0;

    // check for file existence
    if (stat (work->configFile, &fileStatus) == -1)
    {
        return ERROR;
    }

    configFileId = radCfOpen (work->configFile);
    if (configFileId == NULL)
    {
        return ERROR;
    }

    if (radCfGetFirstEntry(configFileId, configName, NULL, value) == ERROR)
    {
        radCfClose (configFileId);
        return ERROR;
    }

    radCfClose (configFileId);
    *store = atoi (value);
    return OK;
}

int stationGetConfigValueFloat (WVIEWD_WORK *work, char *configName, float *store)
{
    char            value[32];
    CF_ID           configFileId;
    struct stat     fileStatus;
    float           tempfloat;

    *store = 0;

    // check for file existence
    if (stat (work->configFile, &fileStatus) == -1)
    {
        return ERROR;
    }

    configFileId = radCfOpen (work->configFile);
    if (configFileId == NULL)
    {
        return ERROR;
    }

    if (radCfGetFirstEntry(configFileId, configName, NULL, value) == ERROR)
    {
        radCfClose (configFileId);
        return ERROR;
    }

    radCfClose (configFileId);
    tempfloat = (float)atof (value);
    tempfloat *= 100;
    if (tempfloat < 0.0)
        tempfloat -= 0.5;
    else
        tempfloat += 0.5;
    tempfloat /= 100;
    *store = tempfloat;

    return OK;
}

int stationGetConfigValueString
(
    WVIEWD_WORK     *work, 
    char            *configName,
    char            *store,
    int             maxLen
)
{
    char            value[64];
    CF_ID           configFileId;
    struct stat     fileStatus;

    *store = 0;

    // check for file existence
    if (stat (work->configFile, &fileStatus) == -1)
    {
        return ERROR;
    }

    configFileId = radCfOpen (work->configFile);
    if (configFileId == NULL)
    {
        return ERROR;
    }

    if (radCfGetFirstEntry(configFileId, configName, NULL, value) == ERROR)
    {
        radCfClose (configFileId);
        return ERROR;
    }

    radCfClose (configFileId);
    strncpy (store, value, MIN(63,maxLen));
    return OK;
}

