/*---------------------------------------------------------------------------
 
  FILENAME:
        stormRain.c
 
  PURPOSE:
        Provide utilities to compute "storm" rain and start date.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        01/08/2006      M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2006, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <signal.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>
#include <errno.h>
#include <math.h>


/*  ... Library include files
*/
#include <radmsgLog.h>
#include <radsysutils.h>

/*  ... Local include files
*/
#include <stormRain.h>

/*  ... global memory declarations
*/

/*  ... local memory
*/
static STORMRAIN_WORK       stormWork;
static ArchiveRecord        arcRecord;


void stormRainInit (char *archivePath, float startTrigger, int idleHours)
{
    time_t          ntime;
    struct tm       locTime;
    USHORT          packedDate, packedTime;
    USHORT          startTime = 0, startDate = 0, lastTime = 0, lastDate = 0;
    float           rate, rainfall, rainSoFar = 0;
    int             diffMins, inStorm = 0;

    memset (&stormWork, 0, sizeof(stormWork));
    stormWork.startTrigger = startTrigger;
    stormWork.idleHours = idleHours;

    // get our search start time/date
    ntime = time (NULL);
    ntime -= WV_SECONDS_IN_MONTH;           // go back one month
    localtime_r (&ntime, &locTime);
    packedDate = INSERT_PACKED_DATE(locTime.tm_year+1900, 
                                    locTime.tm_mon+1, 
                                    locTime.tm_mday);
    packedTime = (locTime.tm_hour*100) + locTime.tm_min;

    // determine the storm start time - loop until we reach now
    while (dbfGetNextArchiveRecord(archivePath, &packedDate, &packedTime, &arcRecord) != ERROR)
    {
        rainfall = (float)(arcRecord.rain & 0xFFF)/100;
        if (!inStorm)
        {
            rate = (float)arcRecord.hiRainRate/100;
            if (rate >= stormWork.startTrigger)
            {
                // we have a start time
                rainSoFar = rainfall;
                startTime = lastTime = packedTime;
                startDate = lastDate = packedDate;
                inStorm = TRUE;
            }
        }
        else
        {
            if (rainfall != 0)
            {
                // update the rain total, etc.
                rainSoFar += rainfall;
                lastTime = packedTime;
                lastDate = packedDate;
            }
            else
            {
                // see if the storm is over
                diffMins = wvutilsCalculatePackedTimeDelta (packedDate, 
                                                            packedTime,
                                                            lastDate,
                                                            lastTime);
                if (diffMins >= (60*stormWork.idleHours))
                {
                    // storm is over...
                    rainSoFar = 0;
                    startTime = lastTime = 0;
                    startDate = lastDate = 0;
                    inStorm = FALSE;
                }
            }
        }
    }

    // are we currently in a rain storm condition?
    if (!inStorm)
    {
        // set everything to initial values
        stormWork.inStorm       = FALSE;
        stormWork.startTime     = (time_t)0;
        stormWork.lastRainTime  = (time_t)0;
        stormWork.rain          = 0;
    }
    else
    {
        // we know the storm start date and storm rain up to now
        stormWork.inStorm       = TRUE;
        stormWork.startTime     = wvutilsPackedTimeToTimeT (startDate, startTime);
        stormWork.lastRainTime  = wvutilsPackedTimeToTimeT (lastDate, lastTime);
        stormWork.rain          = rainSoFar;
    }

    return;
}

void stormRainUpdate (float rainRate, float rainFall)
{
    time_t          ntime = time(NULL);
    int             diffMins;

    if (!stormWork.inStorm)
    {
        if (rainRate >= stormWork.startTrigger)
        {
            // we have a start time
            stormWork.rain = rainFall;
            stormWork.startTime = stormWork.lastRainTime = ntime;
            stormWork.inStorm = TRUE;
        }
    }
    else
    {
        if (rainFall != 0)
        {
            // update the rain total, etc.
            stormWork.rain += rainFall;
            stormWork.lastRainTime = ntime;
        }
        else
        {
            // see if the storm is over
            diffMins = (int)(ntime - stormWork.lastRainTime);
            diffMins /= 60;
            if (diffMins >= (60*stormWork.idleHours))
            {
                // storm is over...
                stormWork.rain = 0;
                stormWork.startTime = stormWork.lastRainTime = (time_t)0;
                stormWork.inStorm = FALSE;
            }
        }
    }

    return;
}

time_t stormRainGetStartTimeT (void)
{
    return stormWork.startTime;
}

float stormRainGet (void)
{
    return stormWork.rain;
}

