/*---------------------------------------------------------------------------
 
  FILENAME:
        wunderground.c
 
  PURPOSE:
        Provide the wview wunderground packet generator entry point.
 
  REVISION HISTORY:
        Date            Engineer        Revision        Remarks
        08/06/2005      M.S. Teel       0               Original
 
  NOTES:
        
 
  LICENSE:
        Copyright (c) 2005, Mark S. Teel (mark@teel.ws)
  
        This source code is released for free distribution under the terms 
        of the GNU General Public License.
  
----------------------------------------------------------------------------*/

/*  ... System include files
*/
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/*  ... Library include files
*/
#include <radsystem.h>
#include <radprocess.h>
#include <radmsgRouter.h>

/*  ... Local include files
*/
#include <wunderground.h>


/*  ... global memory declarations
*/

/*  ... global memory referenced
*/

/*  ... static (local) memory declarations
*/
static WVIEW_WUNDERGROUND_WORK  wundergroundWork;

static char                     wundergroundBuffer[1024];


enum ConfigIds
{
    CFG_ID_STATIONID            = 0,
    CFG_ID_PASSWORD             = 1
};

static char *configIDs[] =
{
    "STATIONID",
    "PASSWORD"
};

/* ... methods
*/
// curl recv callback
static size_t recv_data (void *buffer, size_t size, size_t nmemb, void *userp)
{
    // we don't care about the recvd data
    return size*nmemb;
}

static void processWUNDERGROUND (WVIEW_MSG_ARCHIVE_NOTIFY *notify)
{
    RADSOCK_ID          socket;
    time_t              ntime;
    struct tm           gmTime;
    int                 length = 0;
    char                *serv;
    int                 port;
    char                version[64];
    CURL                *curl;
    CURLcode            res;
    char                curlError[CURL_ERROR_SIZE];
    char                tempBfr[192];
    
    // format the WUNDERGROUND data
    ntime = time (NULL);
    gmtime_r (&ntime, &gmTime);
    length = sprintf (wundergroundBuffer, "http://weatherstation.wunderground.com/weatherstation/updateweatherstation.php?");
    length += sprintf (&wundergroundBuffer[length], "ID=%s&PASSWORD=%s",
                       wundergroundWork.stationId, wundergroundWork.password);

    length += sprintf (&wundergroundBuffer[length], "&dateutc=%4.4d-%2.2d-%2.2d+%2.2d%%3a%2.2d%%3a%2.2d",
                       gmTime.tm_year + 1900, gmTime.tm_mon + 1, gmTime.tm_mday, 
                       gmTime.tm_hour, gmTime.tm_min, gmTime.tm_sec);

    // check for any wind registered
    if (notify->wspeed >= 0 || notify->hiwspeed >= 0)
    {
        length += sprintf (&wundergroundBuffer[length], "&winddir=%3.3d", notify->winddir);
    }
    
    length += sprintf (&wundergroundBuffer[length], "&windspeedmph=%3.3d", 
                       (notify->wspeed >= 0) ? notify->wspeed : 0);
    length += sprintf (&wundergroundBuffer[length], "&windgustmph=%3.3d", 
                       (notify->hiwspeed >= 0) ? notify->hiwspeed : 0);

    if (notify->humidity >= 0 && notify->humidity <= 100)
    {
        length += sprintf (&wundergroundBuffer[length], "&humidity=%d", notify->humidity);
    }

    length += sprintf (&wundergroundBuffer[length], "&tempf=%3.3d.%1.1d", 
                       notify->temp/10, notify->temp%10);
    
    length += sprintf (&wundergroundBuffer[length], "&rainin=%d.%2.2d", 
                       notify->rainHour/100, notify->rainHour%100);
    
    length += sprintf (&wundergroundBuffer[length], "&baromin=%2.2d.%2.2d", 
                       notify->barom/1000, (notify->barom%1000)/10);

    length += sprintf (&wundergroundBuffer[length], "&dewptf=%2.2d.%3.3d", 
                       notify->dewpoint/10, (notify->dewpoint%10)*100);

    strcpy (version, globalWviewVersionStr);
    version[5] = '-';
    length += sprintf (&wundergroundBuffer[length], "&weather=&clouds=&softwaretype=%s&action=updateraw",
                       version);


    strncpy (tempBfr, wundergroundBuffer, 192);
    wvutilsLogEvent (PRI_STATUS, "WUNDERGROUND-send: %s", tempBfr);
    if (strlen(wundergroundBuffer) > 192)
    {
        strncpy (tempBfr, &wundergroundBuffer[192], 192);
        wvutilsLogEvent (PRI_STATUS, "WUNDERGROUND-send: %s", tempBfr);
    }


    // OK, now use libcurl to execute the HTTP "GET"
    curl = curl_easy_init ();
    if (curl == NULL)
    {
        radMsgLog (PRI_HIGH, "WUNDERGROUND-error: failed to initialize curl!");
        return;
    }
    
    curl_easy_setopt (curl, CURLOPT_URL, wundergroundBuffer);
    curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, recv_data);
    curl_easy_setopt (curl, CURLOPT_ERRORBUFFER, curlError);

    res = curl_easy_perform (curl);
    if (res != 0)
    {
        radMsgLog (PRI_HIGH, "WUNDERGROUND-error: %s", curlError);
    }
    
    curl_easy_cleanup (curl);
    return;
}

static int waitForWviewDaemon (void)
{
    WVIEW_MSG_REQUEST       msg;
    char                    srcQName[QUEUE_NAME_LENGTH+1];
    UINT                    msgType;
    UINT                    length;
    void                    *recvBfr;
    int                     retVal, done = FALSE;

    // enable message reception from the radlib router for the archive path
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_ARCHIVE_PATH);

    msg.requestType = WVIEW_RQST_TYPE_ARCHIVE_PATH;

    if (radMsgRouterMessageSend (WVIEW_MSG_TYPE_REQUEST, &msg, sizeof(msg)) == ERROR)
    {
        radMsgLog (PRI_HIGH, "waitForWviewDaemon: radMsgRouterMessageSend failed!");
        radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
        return ERROR;
    }

    // now wait for the response here
    while (!done)
    {
        radUtilsSleep (50);
        
        
        if ((retVal = radQueueRecv (radProcessQueueGetID (),
                                    srcQName,
                                    &msgType,
                                    &recvBfr,
                                    &length))
            == FALSE)
        {
            continue;
        }
        else if (retVal == ERROR)
        {
            radMsgLog (PRI_STATUS, "waitForWviewDaemon: queue is closed!");
            radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);
            return ERROR;
        }
    
        // is this what we want?
        if (msgType == WVIEW_MSG_TYPE_ARCHIVE_PATH)
        {
            // yes!
            done = TRUE;
        }
    
        // release the received buffer
        radBufferRls (recvBfr);
        }
    
    // disable message reception from the radlib router for the archive path
    radMsgRouterMessageDeregister (WVIEW_MSG_TYPE_ARCHIVE_PATH);

    return OK;
}

/*  ... system initialization
*/
static int wundergroundSysInit (WVIEW_WUNDERGROUND_WORK *work)
{
    char            devPath[256], temp[64];
    struct stat     fileData;

    /*  ... exit if our config file is not there
    */
    sprintf (work->configFile, "%s/%s", WVIEW_CONFIG_DIR, WUNDERGROUND_CONFIG_FILENAME);
    if (stat (work->configFile, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_WUNDERGROUND, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "%s does not exist - exiting...\n",
                   work->configFile);
        radMsgLogExit ();
        return ERROR_ABORT;
    }
    
    /*  ... check for our daemon's pid file, don't run if it isn't there
    */
    sprintf (devPath, "%s/%s", WVIEW_RUN_DIR, WVD_LOCK_FILE_NAME);
    if (stat (devPath, &fileData) != 0)
    {
        radMsgLogInit (PROC_NAME_WUNDERGROUND, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "wview daemon lock file %s does not exist - aborting!",
                   devPath);
        radMsgLogExit ();
        return ERROR;
    }

    sprintf (work->pidFile, "%s/%s", WVIEW_RUN_DIR, WUNDERGROUND_LOCK_FILE_NAME);
    sprintf (work->fifoFile, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_WUNDERGROUND);
    sprintf (work->daemonQname, "%s/dev/%s", WVIEW_RUN_DIR, PROC_NAME_DAEMON);
    sprintf (work->wviewdir, "%s", WVIEW_RUN_DIR);

    /*  ... check for our pid file, don't run if it IS there
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        radMsgLogInit (PROC_NAME_WUNDERGROUND, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, 
                   "lock file %s exists, older copy may be running - aborting!",
                   work->pidFile);
        radMsgLogExit ();
        return ERROR;
    }

    return OK;
}

/*  ... system exit
*/
static int wundergroundSysExit (WVIEW_WUNDERGROUND_WORK *work)
{
    struct stat     fileData;

    /*  ... delete our pid file
    */
    if (stat (work->pidFile, &fileData) == 0)
    {
        unlink (work->pidFile);
    }

    return OK;
}

static void defaultSigHandler (int signum)
{
    int         retVal;

    switch (signum)
    {
        case SIGHUP:
            // user wants us to change the verbosity setting
            retVal = wvutilsToggleVerbosity ();
            radMsgLog (PRI_STATUS, "wvwunderd: SIGHUP - toggling log verbosity %s",
                       ((retVal == 0) ? "OFF" : "ON"));

            signal (signum, defaultSigHandler);
            return;

        case SIGPIPE:
            // we have a far end socket disconnection, we'll handle it in the
            // "read/write" code
            signal (signum, defaultSigHandler);
            break;

        case SIGILL:
        case SIGBUS:
        case SIGFPE:
        case SIGSEGV:
        case SIGXFSZ:
        case SIGSYS:
            // unrecoverable signal - we must exit right now!
            radMsgLog (PRI_CATASTROPHIC, "wvwunderd: recv sig %d: bailing out!", signum);
            abort ();
        
        case SIGCHLD:
            wait (NULL);
            signal (signum, defaultSigHandler);
            return;

        default:
            if (wundergroundWork.exiting)
            {
                signal (signum, defaultSigHandler);
                return;
            }

            wundergroundWork.exiting = TRUE;
            radProcessSetExitFlag ();
        
            // can we allow the process to exit normally?
            if (!wundergroundWork.inMainLoop)
            {
                // NO! - we gotta bail here!
                radMsgLog (PRI_HIGH, "wvwunderd: recv sig %d: exiting now!", signum);
                radMsgRouterExit ();
                wundergroundSysExit (&wundergroundWork);
                radProcessExit ();
                radSystemExit (WVIEW_SYSTEM_ID);
                exit (0);                
            }
            
            // we can allow the process to exit normally...
            radMsgLog (PRI_HIGH, "wvwunderd: recv sig %d: exiting!", signum);
        
            signal (signum, defaultSigHandler);
            break;
    }
    
    return;
}

static void msgHandler
(
    char                        *srcQueueName,
    UINT                        msgType,
    void                        *msg,
    UINT                        length,
    void                        *userData
)
{
    WVIEW_MSG_ARCHIVE_NOTIFY    *anMsg;
    
    if (msgType == WVIEW_MSG_TYPE_ARCHIVE_NOTIFY)
    {
        // process wunderground data transfer
        anMsg = (WVIEW_MSG_ARCHIVE_NOTIFY *)msg;
        processWUNDERGROUND (anMsg);
    }
    
    return;
}

static void evtHandler
(
    UINT        eventsRx,
    UINT        rxData,
    void        *userData
)
{
    return;
}

static void timerHandler (void *parm)
{
    return;
}

/*  ... the main entry point for the wvwunderd process
*/
int main (int argc, char *argv[])
{
    void            (*alarmHandler)(int);
    int             retVal;
    FILE            *pidfile;
    char            instance[32], value[_MAX_PATH];
    CF_ID           configFileId;
    struct stat     fileStatus;

    memset (&wundergroundWork, 0, sizeof (wundergroundWork));

    /*  ... initialize some system stuff first
    */
    retVal = wundergroundSysInit (&wundergroundWork);
    if (retVal == ERROR)
    {
        radMsgLogInit (PROC_NAME_WUNDERGROUND, FALSE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "wvwundergroundd init failed!\n");
        radMsgLogExit ();
        exit (1);
    }
    else if (retVal == ERROR_ABORT)
    {
        exit (2);
    }


    /*  ... call the global radlib system init function
    */
    if (radSystemInit (WVIEW_SYSTEM_ID) == ERROR)
    {
        radMsgLogInit (PROC_NAME_WUNDERGROUND, TRUE, TRUE);
        radMsgLog (PRI_CATASTROPHIC, "radSystemInit failed!");
        radMsgLogExit ();
        exit (1);
    }


    /*  ... call the radlib process init function
    */
    if (radProcessInit (PROC_NAME_WUNDERGROUND,
                        wundergroundWork.fifoFile,
                        PROC_NUM_TIMERS_WUNDERGROUND,
                        TRUE,                      // TRUE for daemon
                        msgHandler,
                        evtHandler,
                        NULL)
        == ERROR)
    {
        printf ("\nradProcessInit failed: %s\n\n", PROC_NAME_WUNDERGROUND);
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    wundergroundWork.myPid = getpid ();
    pidfile = fopen (wundergroundWork.pidFile, "w");
    if (pidfile == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "lock file create failed!");
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    fprintf (pidfile, "%d", getpid ());
    fclose (pidfile);


    alarmHandler = radProcessSignalGetHandler (SIGALRM);
    radProcessSignalCatchAll (defaultSigHandler);
    radProcessSignalCatch (SIGALRM, alarmHandler);


    // get the wview verbosity setting
    if (wvutilsSetVerbosity (WV_VERBOSE_WVWUNDERD) == ERROR)
    {
        radMsgLog (PRI_CATASTROPHIC, "wvutilsSetVerbosity failed!");
        wundergroundSysExit (&wundergroundWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    /*  ... get our configuration values
    */
    if (stat (wundergroundWork.configFile, &fileStatus) == -1)
    {
        radMsgLog (PRI_CATASTROPHIC, "config file %s missing - use wviewconfig to generate one...", 
                   wundergroundWork.configFile);
        wundergroundSysExit (&wundergroundWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    configFileId = radCfOpen (wundergroundWork.configFile);
    if (configFileId == NULL)
    {
        radMsgLog (PRI_CATASTROPHIC, "radCfOpen failed!");
        wundergroundSysExit (&wundergroundWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    // get the STATION ID
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_STATIONID],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s failed!", configIDs[CFG_ID_STATIONID]);
        radCfClose (configFileId);
        wundergroundSysExit (&wundergroundWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        strcpy (wundergroundWork.stationId, value);
    }

    // get the password
    if (radCfGetFirstEntry (configFileId,
                            configIDs[CFG_ID_PASSWORD],
                            instance,
                            value)
        == ERROR)
    {
        // we can't do without this!
        radMsgLog (PRI_CATASTROPHIC, "%s failed!", configIDs[CFG_ID_PASSWORD]);
        radCfClose (configFileId);
        wundergroundSysExit (&wundergroundWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }
    else
    {
        strcpy (wundergroundWork.password, value);
    }

    radCfClose (configFileId);

    radMsgLog (PRI_STATUS, "WUNDERGROUND: configured to submit station %s data to wunderground.com",
               wundergroundWork.stationId);


    // wait a bit here before continuing
    radUtilsSleep (500);


    //  register with the radlib message router
    if (radMsgRouterInit (WVIEW_RUN_DIR) == ERROR)
    {
        radMsgLog (PRI_HIGH, "radMsgRouterInit failed!");
        wundergroundSysExit (&wundergroundWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }

    
    // wait for the wview daemon to be ready
    if (waitForWviewDaemon () == ERROR)
    {
        radMsgLog (PRI_HIGH, "waitForWviewDaemon failed");
        wundergroundSysExit (&wundergroundWork);
        radProcessExit ();
        radSystemExit (WVIEW_SYSTEM_ID);
        exit (1);
    }


    // enable message reception from the radlib router for archive notifications
    radMsgRouterMessageRegister (WVIEW_MSG_TYPE_ARCHIVE_NOTIFY);
    

    // enter normal processing
    wundergroundWork.inMainLoop = TRUE;
    radMsgLog (PRI_STATUS, "running...");
    
    while (!wundergroundWork.exiting)
    {
        // wait on something interesting
        if (radProcessWait (0) == ERROR)
        {
            wundergroundWork.exiting = TRUE;
        }
    }


    radMsgLog (PRI_STATUS, "exiting normally...");

    radMsgRouterExit ();
    wundergroundSysExit (&wundergroundWork);
    radProcessExit ();
    radSystemExit (WVIEW_SYSTEM_ID);
    exit (0);
}

